import * as React from 'react';
export type AutocompleteMode = 'sync' | 'async';
export type AutocompleteTitle = string | React.ReactNode;
export type AutocompletePlaceholder = string | React.ReactNode;
export type AutocompleteNoOptions = string | React.ReactNode;
export type AutocompleteShowAll = string | React.ReactNode;
export type AutocompleteAriaLiveOptions = string | React.ReactNode;
export type AutocompleteIndicatorLabel = string | React.ReactNode;
export type AutocompleteSubmitButtonIcon =
  | string
  | React.ReactNode
  | ((...args: any[]) => any);
export type AutocompleteInputRef = ((...args: any[]) => any) | Object;
export type AutocompleteIconPosition = 'left' | 'right';
export type AutocompleteTrianglePosition = 'left' | 'right';
export type AutocompleteInputIcon =
  | string
  | React.ReactNode
  | ((...args: any[]) => any);
export type AutocompleteLabel =
  | string
  | ((...args: any[]) => any)
  | React.ReactNode;
export type AutocompleteLabelDirection = 'horizontal' | 'vertical';
export type AutocompleteLabelSrOnly = string | boolean;
export type AutocompleteKeepValue = string | boolean;
export type AutocompleteKeepValueAndSelection = string | boolean;
export type AutocompleteStatus =
  | string
  | boolean
  | ((...args: any[]) => any)
  | React.ReactNode;
export type AutocompleteSuffix =
  | string
  | ((...args: any[]) => any)
  | React.ReactNode;
export type AutocompleteDisableFilter = string | boolean;
export type AutocompleteDisableReorder = string | boolean;
export type AutocompleteScrollable = string | boolean;
export type AutocompleteFocusable = string | boolean;
export type AutocompleteDisableHighlighting = string | boolean;
export type AutocompleteDirection = 'auto' | 'top' | 'bottom';
export type AutocompleteSkipPortal = string | boolean;
export type AutocompleteNoAnimation = string | boolean;
export type AutocompleteNoScrollAnimation = string | boolean;
export type AutocompleteShowSubmitButton = string | boolean;
export type AutocompletePreventSelection = string | boolean;
export type AutocompleteSize = 'default' | 'small' | 'medium' | 'large';
export type AutocompleteAlignAutocomplete = 'left' | 'right';
export type AutocompleteOptionsRender =
  | Object
  | ((...args: any[]) => any)
  | React.ReactNode;
export type AutocompleteInputComponent =
  | ((...args: any[]) => any)
  | React.ReactNode;
export type AutocompleteData =
  | string
  | ((...args: any[]) => any)
  | React.ReactNode
  | Object
  | (
      | string
      | React.ReactNode
      | {
          selected_value?: string | React.ReactNode;
          content?: string | React.ReactNode | string[];
        }
    )[];
export type AutocompleteSearchInWordIndex = string | number;
export type AutocompleteDefaultValue = string | number;
export type AutocompleteValue = string | number;
export type AutocompleteIcon = string | React.ReactNode;
export type AutocompleteOpenOnFocus = string | boolean;
export type AutocompletePreventClose = string | boolean;
export type AutocompleteKeepOpen = string | boolean;
export type AutocompleteOpened = string | boolean;
export type AutocompleteDisabled = string | boolean;
export type AutocompleteStretch = string | boolean;
export type AutocompleteSkeleton = string | boolean;
export type AutocompleteSpace =
  | string
  | number
  | boolean
  | {
      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. Will use `margin-top`.
       */
      top?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-right`.
       */
      right?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-bottom`.
       */
      bottom?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-left`.
       */
      left?: string | number | boolean;
    };
export type AutocompleteTop = string | number | boolean;
export type AutocompleteRight = string | number | boolean;
export type AutocompleteBottom = string | number | boolean;
export type AutocompleteLeft = string | number | boolean;
export type AutocompleteChildren =
  | string
  | ((...args: any[]) => any)
  | React.ReactNode
  | Object
  | any[];

/**
 * NB: Do not change the docs (comments) in here. The docs are updated during build time by "generateTypes.js" and "fetchPropertiesFromDocs.js".
 */
export interface AutocompleteProps extends React.HTMLProps<HTMLElement> {
  id?: string;

  /**
   * If set to `async`, it prevents showing the "no options" message during typing / filtering. Defaults to `sync`.
   */
  mode?: AutocompleteMode;

  /**
   * Give a title to let the user know what they has to do. Defaults to `Skriv og få alternativer&#39;` .
   */
  title?: AutocompleteTitle;

  /**
   * Use this to define the pre filled placeholder text in the input. Defaults to `title="Skriv og velg"`.
   */
  placeholder?: AutocompletePlaceholder;

  /**
   * Text show in the "no options" item. Defaults to `Ingen alternativer`.
   */
  no_options?: AutocompleteNoOptions;
  show_all?: AutocompleteShowAll;

  /**
   * Text read out by screen readers. This way users with screen readers know how many options they got during typing. Defaults to `%s alternativer`.
   */
  aria_live_options?: AutocompleteAriaLiveOptions;

  /**
   * Text show on indicator "options" item. Defaults to `Henter data ...`.
   */
  indicator_label?: AutocompleteIndicatorLabel;

  /**
   * Title on submit button. Defaults to `Vis alternativer`.
   */
  submit_button_title?: string;

  /**
   * The icon used in the submit button. Defaults to `chevron_down`.
   */
  submit_button_icon?: AutocompleteSubmitButtonIcon;

  /**
   * Use a React.Ref to get access to the `input` DOM element.
   */
  input_ref?: AutocompleteInputRef;

  /**
   * Change the size of the icon pragmatically.
   */
  icon_size?: string;

  /**
   * Position of icon inside the autocomplete. Set to `left` or `right`. Defaults to `left`.
   */
  icon_position?: AutocompleteIconPosition;

  /**
   * Position of icon arrow / triangle the drawer. Set to `left` or `right`. Defaults to `left`.
   */
  triangle_position?: AutocompleteTrianglePosition;

  /**
   * Same as `icon`.
   */
  input_icon?: AutocompleteInputIcon;

  /**
   * Prepends the Form Label component. If no ID is provided, a random ID is created.
   */
  label?: AutocompleteLabel;

  /**
   * Use `label_direction="vertical"` to change the label layout direction. Defaults to `horizontal`
   */
  label_direction?: AutocompleteLabelDirection;

  /**
   * Use `true` to make the label only readable by screen readers.
   */
  label_sr_only?: AutocompleteLabelSrOnly;

  /**
   * Use `true` to not remove the typed value on input blur, if it is invalid. By default the typed value will disappear / replaced by a selected value from the data list during the input field blur.
   */
  keep_value?: AutocompleteKeepValue;

  /**
   * Like `keep_value` – but would not reset to the selected value during input field blur. Also, the selected value would still be kept.
   */
  keep_value_and_selection?: AutocompleteKeepValueAndSelection;

  /**
   * Text with a status message. The style defaults to an error message. You can use `true` to only get the status color, without a message.
   */
  status?: AutocompleteStatus;

  /**
   * Defines the state of the status. Currently there are two statuses `[error, info]`. Defaults to `error`.
   */
  status_state?: string;
  status_animation?: string;

  /**
   * The `status_id` used for the target <a href="/uilib/components/global-status">GlobalStatus</a>.
   */
  global_status_id?: string;

  /**
   * Text describing the content of the Autocomplete more than the label. You can also send in a React component, so it gets wrapped inside the Autocomplete component.
   */
  suffix?: AutocompleteSuffix;

  /**
   * If set to `true`, word highlighting will still be active, but no options will be filtered out. Defaults to `false`.
   */
  disable_filter?: AutocompleteDisableFilter;

  /**
   * If set to `true`, reordering of search results will be disabled. Defaults to `false`.
   */
  disable_reorder?: AutocompleteDisableReorder;
  scrollable?: AutocompleteScrollable;
  focusable?: AutocompleteFocusable;

  /**
   * If set to `true`, word highlighting will disabled, but the options will still get filtered. Defaults to `false`.
   */
  disable_highlighting?: AutocompleteDisableHighlighting;
  direction?: AutocompleteDirection;
  max_height?: number;

  /**
   * Set to `true` to disable the React Portal behavior. Defaults to `false`.
   */
  skip_portal?: AutocompleteSkipPortal;
  no_animation?: AutocompleteNoAnimation;
  no_scroll_animation?: AutocompleteNoScrollAnimation;

  /**
   * Use `true` to show a Dropdown button to toggle the <a href="/uilib/components/fragments/drawer-list">DrawerList</a>. Defaults to `false`.
   */
  show_submit_button?: AutocompleteShowSubmitButton;

  /**
   * Replace the drop down / submit button with a custom React element. Defaults to the input SubmitButton `import { SubmitButton } from &#39;dnb-ui-lib/components/input/Input&#39;`.
   */
  submit_element?: React.ReactNode;

  /**
   * If set to `true`, no permanent selection will be made. Also, the the typed value will not disappear on input blur (like `keep_value`). Defaults to false`.
   */
  prevent_selection?: AutocompletePreventSelection;

  /**
   * Define the height of the Autocomplete. Can be set to `small`, `default`, `medium` and `large`. Defaults to `default`.
   */
  size?: AutocompleteSize;

  /**
   * Use `right` to change the options alignment direction. Defaults to `left`.
   */
  align_autocomplete?: AutocompleteAlignAutocomplete;
  options_render?: AutocompleteOptionsRender;
  input_component?: AutocompleteInputComponent;
  data?: AutocompleteData;

  /**
   * This gives you the possibility to change the threshold number, which defines from what word on we search "inside words". Defaults to `3`.
   */
  search_in_word_index?: AutocompleteSearchInWordIndex;
  default_value?: AutocompleteDefaultValue;
  value?: AutocompleteValue;
  input_value?: string;

  /**
   * To be included in the autocomplete input.
   */
  icon?: AutocompleteIcon;

  /**
   * Use `true` to auto open the list once the users is entering the input field with the keyboard.
   */
  open_on_focus?: AutocompleteOpenOnFocus;
  prevent_close?: AutocompletePreventClose;
  keep_open?: AutocompleteKeepOpen;
  opened?: AutocompleteOpened;
  disabled?: AutocompleteDisabled;

  /**
   * If set to `true`, then the autocomplete will be 100% in available `width`.
   */
  stretch?: AutocompleteStretch;
  skeleton?: AutocompleteSkeleton;

  /**
   * Define a custom class for the internal drawer-list. This makes it possible more easily customize the drawer-list style with styled-components and the `css` style method. Defaults to `null`.
   */
  drawer_class?: string;

  /**
   * Has to be an object with either: `top`, `right`, `bottom` or `left`. Use spacing values like: `small`, `1rem`, `1` or , `16px`.
   */
  space?: AutocompleteSpace;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. Will use `margin-top`.
   */
  top?: AutocompleteTop;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-right`.
   */
  right?: AutocompleteRight;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-bottom`.
   */
  bottom?: AutocompleteBottom;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-left`.
   */
  left?: AutocompleteLeft;
  class?: string;
  className?: string;
  children?: AutocompleteChildren;
  custom_element?: Object;
  custom_method?: (...args: any[]) => any;

  /**
   * Will be called once the user presses the autocomplete. Returns the data item `{ data, attributes }`.
   */
  on_show?: (...args: any[]) => any;

  /**
   * Will be called for every key change the users makes. Returns an object with the input `value` inside `{ value, event, attributes }` including <a href="/uilib/components/autocomplete/events#dynamically-change-data">these methods</a>.
   */
  on_type?: (...args: any[]) => any;

  /**
   * Will be called on user generated focus action. Returns an object with the input `value` inside `{ value, event, attributes }` including <a href="/uilib/components/autocomplete/events#dynamically-change-data">these methods</a>.
   */
  on_focus?: (...args: any[]) => any;

  /**
   * Will be called on user generated blur action. Returns an object with the input `value` inside `{ value, event, attributes }` including <a href="/uilib/components/autocomplete/events#dynamically-change-data">these methods</a>.
   */
  on_blur?: (...args: any[]) => any;

  /**
   * Will be called once the user presses the autocomplete again, or clicks somewhere else. Returns the data item `{ data, attributes }`.
   */
  on_hide?: (...args: any[]) => any;

  /**
   * Will be called on state changes made by the user. Returns an object with the new selected `data` item `{ data, event, attributes, value }` including <a href="/uilib/components/autocomplete/events#dynamically-change-data">these methods</a>.
   */
  on_change?: (...args: any[]) => any;

  /**
   * Will be called once the users selects an item by a click or keyboard navigation. Returns an object with the new selected `data` item `{ data, event, attributes, value, active_item }` including <a href="/uilib/components/autocomplete/events#dynamically-change-data">these methods</a>. The "active_item" property is the currently selected item by keyboard navigation
   */
  on_select?: (...args: any[]) => any;
  on_state_update?: (...args: any[]) => any;
}
export default class Autocomplete extends React.Component<
  AutocompleteProps,
  any
> {
  static defaultProps: object;
  render(): JSX.Element;
}
