"use strict";

require("core-js/modules/es.array.concat.js");

require("core-js/modules/es.array.fill.js");

require("core-js/modules/es.array.index-of.js");

require("core-js/modules/es.array.map.js");

require("core-js/modules/es.regexp.exec.js");

require("core-js/modules/es.string.replace.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.convertStringToDate = exports.correctV1Format = exports.isDisabled = exports.getMonth = exports.getWeek = exports.toRange = exports.dayOffset = exports.getCalendar = exports.makeDayObject = void 0;

var _subMonths = _interopRequireDefault(require("date-fns/subMonths"));

var _addMonths = _interopRequireDefault(require("date-fns/addMonths"));

var _isWithinInterval = _interopRequireDefault(require("date-fns/isWithinInterval"));

var _isValid = _interopRequireDefault(require("date-fns/isValid"));

var _isAfter = _interopRequireDefault(require("date-fns/isAfter"));

var _isBefore = _interopRequireDefault(require("date-fns/isBefore"));

var _isSameDay = _interopRequireDefault(require("date-fns/isSameDay"));

var _isToday = _interopRequireDefault(require("date-fns/isToday"));

var _isSameMonth = _interopRequireDefault(require("date-fns/isSameMonth"));

var _addDays = _interopRequireDefault(require("date-fns/addDays"));

var _startOfWeek = _interopRequireDefault(require("date-fns/startOfWeek"));

var _getDay = _interopRequireDefault(require("date-fns/getDay"));

var _startOfMonth = _interopRequireDefault(require("date-fns/startOfMonth"));

var _getDaysInMonth = _interopRequireDefault(require("date-fns/getDaysInMonth"));

var _toDate = _interopRequireDefault(require("date-fns/toDate"));

var _parseISO = _interopRequireDefault(require("date-fns/parseISO"));

var _parse = _interopRequireDefault(require("date-fns/parse"));

var _componentHelper = require("../../shared/component-helper");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var makeDayObject = function makeDayObject(date, _ref) {
  var startDate = _ref.startDate,
      endDate = _ref.endDate,
      hoverDate = _ref.hoverDate,
      minDate = _ref.minDate,
      maxDate = _ref.maxDate,
      month = _ref.month;
  var range = getRange(startDate, endDate, hoverDate);
  var isLastMonth = (0, _isSameMonth.default)((0, _subMonths.default)(date, 1), month);
  var isNextMonth = (0, _isSameMonth.default)((0, _addMonths.default)(date, 1), month);
  var isStartDate = isStartDateCalc(date, range);
  var isEndDate = isEndDateCalc(date, range);
  var isWithinSelection = isWithinSelectionCalc(date, startDate, endDate);
  var isPreview = isPreviewCalc(date, startDate, endDate, hoverDate);
  var isDisabled = isDisabledCalc(date, minDate, maxDate);
  return {
    date: date,
    isToday: (0, _isToday.default)(date),
    isLastMonth: isLastMonth,
    isNextMonth: isNextMonth,
    isStartDate: isStartDate,
    isEndDate: isEndDate,
    isWithinSelection: isWithinSelection,
    isPreview: isPreview,
    isDisabled: isDisabled,
    isSelectable: !isLastMonth && !isNextMonth && !isDisabled,
    isInactive: isLastMonth || isNextMonth || isDisabled
  };
};

exports.makeDayObject = makeDayObject;

var getCalendar = function getCalendar(month) {
  var weekStartsOn = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

  var _ref2 = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
      _ref2$onlyMonth = _ref2.onlyMonth,
      onlyMonth = _ref2$onlyMonth === void 0 ? false : _ref2$onlyMonth,
      _ref2$hideNextMonthWe = _ref2.hideNextMonthWeek,
      hideNextMonthWeek = _ref2$hideNextMonthWe === void 0 ? false : _ref2$hideNextMonthWe;

  var thisMonth = getMonth(month);

  if (onlyMonth) {
    return calendarCache[month] = _toConsumableArray(thisMonth);
  }

  if (calendarCache[month]) {
    return calendarCache[month];
  }

  var firstDay = (7 + (0, _getDay.default)((0, _startOfMonth.default)(month)) - weekStartsOn) % 7;
  var lastMonth = getMonth((0, _subMonths.default)(month, 1), (0, _getDaysInMonth.default)((0, _subMonths.default)(month, 1)) - firstDay);
  var fillCount = -1;

  if (hideNextMonthWeek) {
    fillCount = 35 - (thisMonth.length + firstDay);
  }

  if (fillCount < 0) {
    fillCount = 42 - (thisMonth.length + firstDay);
  }

  var nextMonth = getMonth((0, _addMonths.default)(month, 1), 0, fillCount > -1 ? fillCount : 0);
  return calendarCache[month] = [].concat(_toConsumableArray(lastMonth), _toConsumableArray(thisMonth), _toConsumableArray(nextMonth));
};

exports.getCalendar = getCalendar;
var calendarCache = {};

var dayOffset = function dayOffset(dayName) {
  var week = ['sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday'];
  return dayName ? week.indexOf(dayName.toLowerCase()) : 0;
};

exports.dayOffset = dayOffset;

var toRange = function toRange(startDate, endDate) {
  if ((0, _isBefore.default)(endDate, startDate)) {
    var _startDate = startDate;
    startDate = endDate;
    endDate = _startDate;
  }

  return {
    startDate: startDate,
    endDate: endDate
  };
};

exports.toRange = toRange;

var getRange = function getRange(startDate, endDate, hoverDate) {
  if (startDate && endDate) {
    return toRange(startDate, endDate);
  } else if (startDate && hoverDate) {
    return toRange(startDate, hoverDate);
  } else {
    return toRange(startDate, startDate);
  }
};

var getWeek = function getWeek(weekStartsOn) {
  var startDay = (0, _startOfWeek.default)(new Date(), {
    weekStartsOn: weekStartsOn
  });
  return Array(7).fill(1).map(function (value, i) {
    return (0, _addDays.default)(startDay, i);
  });
};

exports.getWeek = getWeek;

var getMonth = function getMonth(month) {
  var skip = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  var limit = arguments.length > 2 ? arguments[2] : undefined;
  var startDay = (0, _startOfMonth.default)(month);
  var size = (0, _getDaysInMonth.default)(month) - skip;
  size = Math.min(Math.max(parseInt(size), 0), limit > -1 ? limit : size);
  size = size < 0 ? 0 : size;
  return Array(size).fill(1).map(function (value, i) {
    return (0, _addDays.default)(startDay, i + skip);
  });
};

exports.getMonth = getMonth;

var isWithinSelectionCalc = function isWithinSelectionCalc(date, startDate, endDate) {
  var _toRange = toRange(startDate, endDate),
      start = _toRange.startDate,
      end = _toRange.endDate;

  return startDate && endDate ? (0, _isValid.default)(start) && (0, _isValid.default)(end) && (0, _isWithinInterval.default)(date, {
    start: start,
    end: end
  }) : false;
};

var isDisabledCalc = function isDisabledCalc(date, minDate, maxDate) {
  return minDate && (0, _isBefore.default)(date, minDate) || maxDate && (0, _isAfter.default)(date, maxDate);
};

exports.isDisabled = isDisabledCalc;

var isStartDateCalc = function isStartDateCalc(date, range) {
  return range.startDate && (0, _isSameDay.default)(date, range.startDate);
};

var isEndDateCalc = function isEndDateCalc(date, range) {
  return range.endDate && (0, _isSameDay.default)(date, range.endDate);
};

var isPreviewCalc = function isPreviewCalc(date, startDate, endDate, hoverDate) {
  var _toRange2 = toRange(startDate, hoverDate),
      start = _toRange2.startDate,
      end = _toRange2.endDate;

  return startDate && !endDate && (0, _isValid.default)(start) && (0, _isValid.default)(end) && (0, _isWithinInterval.default)(date, {
    start: start,
    end: end
  });
};

var correctV1Format = function correctV1Format(date) {
  if (/YYYY/.test(date) && /DD/.test(date)) {
    (0, _componentHelper.warn)('You are using "YYYY-MM-DD" as the date_format or return_format? Please use "yyyy-MM-dd" instead!');
    date = date.replace(/DD/, 'dd').replace(/YYYY/, 'yyyy');
  }

  return date;
};

exports.correctV1Format = correctV1Format;

var convertStringToDate = function convertStringToDate(date) {
  var _ref3 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref3$date_format = _ref3.date_format,
      date_format = _ref3$date_format === void 0 ? null : _ref3$date_format;

  if (date === null) {
    return null;
  }

  var dateObject;
  dateObject = typeof date === 'string' ? (0, _parseISO.default)(date) : (0, _toDate.default)(date);

  if (typeof date === 'string' && date_format && !(0, _isValid.default)(dateObject)) {
    date_format = correctV1Format(date_format);
    dateObject = (0, _parse.default)(date, date_format, new Date());
  }

  if (!(0, _isValid.default)(dateObject)) {
    (0, _componentHelper.warn)('convertStringToDate got invalid date:', date);
    return null;
  }

  return dateObject;
};

exports.convertStringToDate = convertStringToDate;