"use strict";

require("core-js/modules/es.array.filter.js");

require("core-js/modules/es.array.for-each.js");

require("core-js/modules/web.dom-collections.for-each.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _componentHelper = require("../../shared/component-helper");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var Animation = function () {
  function Animation() {
    _classCallCheck(this, Animation);

    this.stack = [];
    this.events = [];
  }

  _createClass(Animation, [{
    key: "add",
    value: function add(animation) {
      if (!animation.type) {
        (0, _componentHelper.warn)('You should define an animation type.');
      }

      var isInProgress = this.isInProgress(animation);

      if (isInProgress) {
        if (typeof animation.onPartial === 'function') {
          animation.onPartial(isInProgress);
        }

        return animation;
      }

      this.stack.push(animation);
      return this.runNext();
    }
  }, {
    key: "unbind",
    value: function unbind() {
      var _this = this;

      this.stack.forEach(function (animation) {
        animation.onReset = null;

        _this.reset(animation);
      });
      this.stack = [];
      this.events = [];
    }
  }, {
    key: "reset",
    value: function reset(animation) {
      if (!animation) {
        return;
      }

      clearTimeout(animation._durationId);
      clearTimeout(animation._delayId);

      if (typeof animation.onReset === 'function') {
        animation.onReset(animation);
      }

      this.stack = this.stack.filter(function (a) {
        return a !== animation;
      });
    }
  }, {
    key: "isInProgress",
    value: function isInProgress(animation) {
      return this.stack.length > 0 && this.stack[0].running && animation.type === this.stack[0].type && this.stack[0];
    }
  }, {
    key: "runNext",
    value: function runNext() {
      var _this2 = this;

      var animation = this.stack[0];

      if (!animation || animation.running) {
        return null;
      }

      animation.running = true;

      var run = function run() {
        if (typeof animation.onStart === 'function') {
          animation.onStart(animation);
        }

        _this2.runGlobalEvents(animation, 'onStart');

        var next = function next() {
          animation.running = false;

          if (typeof animation.onComplete === 'function') {
            animation.onComplete(animation);
          }

          _this2.runGlobalEvents(animation, 'onComplete');

          _this2.stack = _this2.stack.filter(function (a) {
            return a !== animation;
          });

          _this2.runNext();
        };

        clearTimeout(animation._durationId);

        if (animation.duration > 0) {
          animation._durationId = setTimeout(next, animation.duration);
        } else {
          next();
        }
      };

      clearTimeout(animation._delayId);

      if (animation.delay > 0) {
        animation._delayId = setTimeout(run, animation.delay);
      } else {
        run();
      }

      return animation;
    }
  }, {
    key: "onStart",
    value: function onStart(callback) {
      this.events.push({
        callback: callback,
        type: 'onStart'
      });
    }
  }, {
    key: "onComplete",
    value: function onComplete(callback) {
      this.events.push({
        callback: callback,
        type: 'onComplete'
      });
    }
  }, {
    key: "runGlobalEvents",
    value: function runGlobalEvents(animation, type) {
      this.events.forEach(function (event) {
        if (type === event.type && typeof event.callback === 'function') {
          event.callback(animation);
        }
      });
      this.events = [];
    }
  }]);

  return Animation;
}();

exports.default = Animation;