"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Counter = exports.initCounter = void 0;

var _HeadingHelpers = require("./HeadingHelpers");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var initCounter = function initCounter() {
  var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

  if (!_HeadingHelpers.globalHeadingCounter.current) {
    _HeadingHelpers.globalHeadingCounter.current = new Counter({
      group: 'global',
      isGlobal: true
    });
  }

  return props && props.counter || new Counter(props);
};

exports.initCounter = initCounter;

var Counter = function () {
  function Counter() {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

    _classCallCheck(this, Counter);

    this.level = 0;
    this.entry = 0;
    this._isReady = false;
    this.countHeadings = 0;
    this._initCount = 0;
    this.isGlobal = false;
    this.isHeading = false;
    this.bypassChecks = false;
    this.contextCounter = null;
    this.reports = [];
    props = props || {};

    if (props.group) {
      this.group = props.group;
    }

    if (props.isGlobal) {
      this.isGlobal = true;
    }

    this.children = props.children;
  }

  _createClass(Counter, [{
    key: "report",
    value: function report() {
      if (!(process && process.env.NODE_ENV === 'production')) {
        for (var _len = arguments.length, str = new Array(_len), _key = 0; _key < _len; _key++) {
          str[_key] = arguments[_key];
        }

        this.reports.push(str);
      }
    }
  }, {
    key: "useLastReport",
    value: function useLastReport() {
      return this.reports.shift();
    }
  }, {
    key: "enableBypassChecks",
    value: function enableBypassChecks() {
      this.bypassChecks = true;
    }
  }, {
    key: "disableBypassChecks",
    value: function disableBypassChecks() {
      this.bypassChecks = false;
    }
  }, {
    key: "getLevel",
    value: function getLevel() {
      return this.level;
    }
  }, {
    key: "hasEntryLevel",
    value: function hasEntryLevel() {
      return this.entry > 0;
    }
  }, {
    key: "hasCorrection",
    value: function hasCorrection() {
      return this._isReady;
    }
  }, {
    key: "setEntryLevel",
    value: function setEntryLevel() {
      var level = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
      this.entry = parseFloat(level) || 1;
    }
  }, {
    key: "isInContext",
    value: function isInContext() {
      return Boolean(this.contextCounter);
    }
  }, {
    key: "setContextCounter",
    value: function setContextCounter(contextCounter) {
      this.contextCounter = contextCounter;
    }
  }, {
    key: "skipMakeMeReady",
    value: function skipMakeMeReady() {
      this._isReady = true;
      this.entry = 1;
      this.level = 1;
      this.contextCounter.entry = 2;
    }
  }, {
    key: "windup",
    value: function windup() {
      this.contextCounter.countHeadings++;
    }
  }, {
    key: "teardown",
    value: function teardown() {
      if (this.contextCounter.countHeadings > 0) {
        this.contextCounter.level = this.contextCounter.entry;
      }

      this.contextCounter.countHeadings--;
      this.contextCounter._initCount--;
    }
  }, {
    key: "makeMeReady",
    value: function makeMeReady() {
      if (!this.hasCorrection()) {
        if (this.contextCounter.level > 1) {
          this.level = this.contextCounter.level;
        } else if (this.contextCounter._isReady) {
          if (!this.bypassChecks) {
            this.level = 2;
          }
        } else if (!this.contextCounter._isReady) {
          this.contextCounter._isReady = true;

          if (!this.bypassChecks) {
            this.level = 1;
          }
        }

        if (this.entry === 0) {
          this.entry = this.level;

          if (this.isHeading) {
            this.contextCounter._initCount++;
          }
        }
      }
    }
  }, {
    key: "factorCheck",
    value: function factorCheck(_ref) {
      var action = _ref.action,
          level = _ref.level,
          current = _ref.current,
          report = _ref.report;

      if (!this.bypassChecks && level - current > 1) {
        report && report.push("Heading levels can only be changed by factor one! Got:", level, 'and had before', current);

        if (level > current) {
          action = 'increment';
        }

        switch (action) {
          case 'increment':
            level = current + 1;
            break;

          case 'decrement':
            level = current - 1;
            break;

          default:
            level = current === 0 ? 1 : current;
            break;
        }
      }

      return level;
    }
  }, {
    key: "setLevel",
    value: function setLevel(level) {
      var _globalSyncCounter$cu;

      var action = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'set';
      level = parseFloat(level);
      var report = [];

      if (!this._isReady && this.level === 1 && (!_HeadingHelpers.globalSyncCounter.current || _HeadingHelpers.globalSyncCounter.current.level < 2)) {
          return;
        }

      if (((_globalSyncCounter$cu = _HeadingHelpers.globalSyncCounter.current) === null || _globalSyncCounter$cu === void 0 ? void 0 : _globalSyncCounter$cu.level) > 0) {
        level = this.factorCheck({
          report: report,
          action: action,
          level: level,
          current: _HeadingHelpers.globalSyncCounter.current.level
        });
      }

      level = this.factorCheck({
        action: action,
        level: level,
        current: this.level,
        report: report
      });

      if (level > 6) {
        report.push("Can not [".concat(action, "] heading level higher than 6! Got:"), level, 'and had before', this.level);
        level = 6;
      } else if (level < 1 && this.level !== -1) {
        report.push("Can not [".concat(action, "] heading level lower than 1! Got:"), level, 'and had before', this.level);
        level = 1;
      } else if (!this.bypassChecks && level === 1 && this.level === 1) {
        report.push("Can not set ([".concat(action, "]) heading level 1 several times! Got:"), level, 'and had before', this.level);
        level = 2;
      } else if (!this.bypassChecks && level < 2 && this.level === 2) {
        report.push('Can not decrease to heading level 1! Had before', this.level);
        level = this.level;
      }

      if (report.length > 0) {
        report.push('- The new level is', level);
        this.report.apply(this, report);
      }

      this.level = level;

      if (this.isHeading) {
        this.contextCounter.level = level;
      } else {
        this.entry = level;
      }
    }
  }, {
    key: "increment",
    value: function increment() {
      this.setLevel(this.level + 1, 'increment');
    }
  }, {
    key: "decrement",
    value: function decrement() {
      this.setLevel(this.level - 1, 'decrement');
    }
  }, {
    key: "force",
    value: function force() {
      var level = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
      this.bypassChecks = true;
      this.setLevel(level);
      this.bypassChecks = false;
    }
  }, {
    key: "reset",
    value: function reset() {
      var toLevel = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
      toLevel = parseFloat(toLevel) || 2;
      this.level = this.entry = this.lastResetLevel = toLevel;

      if (this.contextCounter) {
        this.contextCounter.level = this.contextCounter.entry;
      }

      if (toLevel === 1 && _HeadingHelpers.globalHeadingCounter.current) {
        _HeadingHelpers.globalHeadingCounter.current.level = 2;
      }
    }
  }]);

  return Counter;
}();

exports.Counter = Counter;