import * as React from 'react';
export type NumberValue = number | string;
export type NumberPrefix = React.ReactNode | ((...args: any[]) => any);
export type NumberSuffix = React.ReactNode | ((...args: any[]) => any);
export type NumberCurrency = string | boolean;
export type NumberCurrencyPosition = 'auto' | 'before' | 'after';
export type NumberBan = string | boolean;
export type NumberNin = string | boolean;
export type NumberPhone = string | boolean;
export type NumberOrg = string | boolean;
export type NumberLink = string | boolean;
export type NumberOptions = Object | string;
export type NumberDecimals = number | string;
export type NumberSelectall = string | boolean;
export type NumberCopySelection = string | boolean;
export type NumberOmitRounding = string | boolean;
export type NumberClean = string | boolean;
export type NumberElement = string | boolean;
export type NumberSpace =
  | string
  | number
  | boolean
  | {
      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. Will use `margin-top`.
       */
      top?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-right`.
       */
      right?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-bottom`.
       */
      bottom?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-left`.
       */
      left?: string | number | boolean;
    };
export type NumberTop = string | number | boolean;
export type NumberRight = string | number | boolean;
export type NumberBottom = string | number | boolean;
export type NumberLeft = string | number | boolean;
export type NumberChildren = React.ReactNode | ((...args: any[]) => any);

/**
 * NB: Do not change the docs (comments) in here. The docs are updated during build time by "generateTypes.js" and "fetchPropertiesFromDocs.js".
 */
export interface NumberProps extends React.HTMLProps<HTMLElement> {
  id?: string;

  /**
   * A number.
   */
  value?: NumberValue;

  /**
   * Use a <a href="https://www.sitepoint.com/iso-2-letter-language-codes/">2 Letter Language Codes</a> or an extended code such as `nb-NO`. Use `auto` to detect the locale from the browser (`navigator.language`). Defaults to the Norwegian locale: `nb-NO`.
   */
  locale?: string;

  /**
   * Add a string or React component before the number, including a white space.
   */
  prefix?: NumberPrefix;

  /**
   * Appends a string or React component after the number, including a white space.
   */
  suffix?: NumberSuffix;

  /**
   * Currency code (ISO 4217) or `true` to use the default `NOK`. Uses two decimals by default.
   */
  currency?: NumberCurrency;

  /**
   * Use either `code` (NOK), `name` (norske kroner) , `symbol` (kr) or `narrowSymbol` (for a shorter symbol variant). Defaults to `narrowSymbol`.
   */
  currency_display?: string;

  /**
   * Use either `before` or `after` to change/define the position of the currency. Defaults to `auto` (Browser API defaults, but with exception if locale is `nb-NO` or `no`, use after as the default position).
   */
  currency_position?: NumberCurrencyPosition;

  /**
   * "Bank Account Number": use `true` to use the default Norwegian style (2000 12 34567) formatting.
   */
  ban?: NumberBan;

  /**
   * "National Identification Number": use `true` to use the default Norwegian style (180892 12345) formatting.
   */
  nin?: NumberNin;

  /**
   * Use `true` to use the default Norwegian style (22 22 22 22) of phone number formatting, regulated by the <a href="https://lovdata.no/forskrift/2004-02-16-426/§16">Norwegian authority</a>. More info by <a href="https://www.sprakradet.no/sprakhjelp/Skriveregler/Dato/#tlf">Sprakradet</a> as well.
   */
  phone?: NumberPhone;

  /**
   * "Organization Number": use `true` to use the default Norwegian style (123 456 789) formatting. Screen readers get a digit by digit.
   */
  org?: NumberOrg;

  /**
   * Use `tel` or `sms` to enable a clickable / touchable anchor link.
   */
  link?: NumberLink;

  /**
   * Accepts all <a href="https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/toLocaleString">number.toLocaleString</a> options as an object - can also be a JSON given as the parameter e.g. `options=&#39;{"minimumFractionDigits":"2"}&#39;`.
   */
  options?: NumberOptions;

  /**
   * Set a number to define the amount of decimals. Like `decimals="0"` will ensure that decimals are simply not shown. The default decimals for currency usage, is `2` (Browser API default).
   */
  decimals?: NumberDecimals;

  /**
   * Use `false` to disable the auto select all on first click. Defaults to `true`.
   */
  selectall?: NumberSelectall;

  /**
   * Use `false` to disable the auto copy feature. Defaults to `true`.
   */
  copy_selection?: NumberCopySelection;

  /**
   * If set to `true`, the decimal will NOT be rounded. Normally, by using `toFixed` or by using `maximumFractionDigits`, decimals get rounded.
   */
  omit_rounding?: NumberOmitRounding;

  /**
   * If set to `true` a dirty string will be parsed to to extract the number (`prefix -123.45 suffix` would result in e.g. `kr -123,45`).
   */
  clean?: NumberClean;

  /**
   * Define what HTML element should be used. Defaults to `<span>`.
   */
  element?: NumberElement;

  /**
   * Has to be an object with either: `top`, `right`, `bottom` or `left`. Use spacing values like: `small`, `1rem`, `1` or , `16px`.
   */
  space?: NumberSpace;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. Will use `margin-top`.
   */
  top?: NumberTop;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-right`.
   */
  right?: NumberRight;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-bottom`.
   */
  bottom?: NumberBottom;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-left`.
   */
  left?: NumberLeft;
  class?: string;
  className?: string;

  /**
   * A number.
   */
  children?: NumberChildren;
}
export default class Number extends React.Component<NumberProps, any> {
  static defaultProps: object;
  render(): JSX.Element;
}
