"use strict";

require("core-js/modules/es.symbol.js");

require("core-js/modules/es.symbol.description.js");

require("core-js/modules/es.symbol.iterator.js");

require("core-js/modules/es.array.filter.js");

require("core-js/modules/es.array.find.js");

require("core-js/modules/es.array.find-index.js");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.array.join.js");

require("core-js/modules/es.array.map.js");

require("core-js/modules/es.array.reduce.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.regexp.exec.js");

require("core-js/modules/es.string.iterator.js");

require("core-js/modules/es.string.match.js");

require("core-js/modules/web.dom-collections.iterator.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.findClosest = exports.getCurrentDataTitle = exports.prepareDerivedState = exports.prepareStartupState = exports.getCurrentData = exports.getEventData = exports.parseCurrentValue = exports.getSelectedItemValue = exports.getCurrentIndex = exports.getData = exports.normalizeData = exports.preSelectData = exports.hasObjectKeyAsValue = exports.parseContentTitle = void 0;

var _react = _interopRequireDefault(require("react"));

var _componentHelper = require("../../shared/component-helper");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var parseContentTitle = function parseContentTitle(dataItem) {
  var _dataItem;

  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$separator = _ref.separator,
      separator = _ref$separator === void 0 ? '\n' : _ref$separator,
      _ref$removeNumericOnl = _ref.removeNumericOnlyValues,
      removeNumericOnlyValues = _ref$removeNumericOnl === void 0 ? false : _ref$removeNumericOnl,
      _ref$preferSelectedVa = _ref.preferSelectedValue,
      preferSelectedValue = _ref$preferSelectedVa === void 0 ? false : _ref$preferSelectedVa;

  var ret = '';
  var onlyNumericRegex = /[0-9.,-\s]+/;

  if (Array.isArray(dataItem) && dataItem.length > 0) {
    dataItem = {
      content: dataItem
    };
  }

  var hasValue = (_dataItem = dataItem) === null || _dataItem === void 0 ? void 0 : _dataItem.selected_value;

  if (!(preferSelectedValue && hasValue) && dataItem && Array.isArray(dataItem.content)) {
    ret = dataItem.content.reduce(function (acc, cur) {
      cur = (0, _componentHelper.convertJsxToString)(cur, ' ');

      if (cur === false) {
        return acc;
      }

      var found = removeNumericOnlyValues && cur && cur.match(onlyNumericRegex);

      if (!(found && found[0].length === cur.length)) {
        acc.push(cur);
      }

      return acc;
    }, []).join(separator);
  } else {
    ret = (0, _componentHelper.convertJsxToString)(dataItem && dataItem.content || dataItem, ' ');
  }

  if (hasValue) {
    if (preferSelectedValue) {
      ret = String((0, _componentHelper.convertJsxToString)(dataItem.selected_value));
    } else if (!onlyNumericRegex.test(dataItem.selected_value)) {
      ret = String((0, _componentHelper.convertJsxToString)(dataItem.selected_value)) + separator + ret;
    }
  }

  if (Array.isArray(dataItem) && dataItem.length === 0) {
    ret = null;
  }

  if (ret && ret.length === 1 && ret[0].ignore_events) {
    return null;
  }

  return ret;
};

exports.parseContentTitle = parseContentTitle;

var hasObjectKeyAsValue = function hasObjectKeyAsValue(data) {
  var _data;

  data = ((_data = data) === null || _data === void 0 ? void 0 : _data.raw_data) || data;
  return data && _typeof(data) === 'object' && !Array.isArray(data);
};

exports.hasObjectKeyAsValue = hasObjectKeyAsValue;

var preSelectData = function preSelectData(data) {
  if (typeof data === 'string') {
    data = data[0] === '{' || data[0] === '[' ? JSON.parse(data) : null;
  } else if (data && _react.default.isValidElement(data)) {
    data = [];
  } else if (typeof data === 'function') {
    data = data();
  }

  return data;
};

exports.preSelectData = preSelectData;

var normalizeData = function normalizeData(props) {
  var data = preSelectData(props.data || props.children || props);

  if (data && _typeof(data) === 'object' && !Array.isArray(data)) {
    var list = [];

    for (var i in data) {
      list.push({
        selected_key: i,
        value: i,
        content: data[i],
        type: 'object'
      });
    }

    data = list;
  }

  return (data || []).map(function (item, __id) {
    if (typeof item === 'string' || Array.isArray(item) || _react.default.isValidElement(item)) {
      item = {
        content: item,
        __isTransformed: true
      };
    }

    return typeof item.__id !== 'undefined' ? item : _extends({}, item, {
      __id: __id
    });
  });
};

exports.normalizeData = normalizeData;

var getData = function getData(props) {
  if (props.prepared_data && Array.isArray(props.prepared_data)) {
    return props.prepared_data;
  }

  return normalizeData(props);
};

exports.getData = getData;

var getCurrentIndex = function getCurrentIndex(value, data) {
  if (/[^0-9]/.test(String(value))) {
    return data === null || data === void 0 ? void 0 : data.findIndex(function (cur) {
      return parseCurrentValue(cur) === value;
    });
  } else if (parseFloat(value) > -1) {
      return value;
    }

  return null;
};

exports.getCurrentIndex = getCurrentIndex;

var getSelectedItemValue = function getSelectedItemValue(value, state) {
  if (hasObjectKeyAsValue(state)) {
    return parseCurrentValue(state.data.filter(function (_, i) {
      return i === parseFloat(value);
    })[0]);
  }

  return value;
};

exports.getSelectedItemValue = getSelectedItemValue;

var parseCurrentValue = function parseCurrentValue(current) {
  if (typeof (current === null || current === void 0 ? void 0 : current.selected_key) !== 'undefined') {
    return current === null || current === void 0 ? void 0 : current.selected_key;
  }

  if (typeof (current === null || current === void 0 ? void 0 : current.content) !== 'undefined') {
    return current === null || current === void 0 ? void 0 : current.content;
  }

  return current;
};

exports.parseCurrentValue = parseCurrentValue;

var getEventData = function getEventData(item_index, data) {
  data = getCurrentData(item_index, data);

  if (data && data.__id) {
    data = _extends({}, data);
    delete data.__id;
    delete data.__isTransformed;
  }

  return data;
};

exports.getEventData = getEventData;

var getCurrentData = function getCurrentData(item_index, data) {
  if (typeof data === 'function') {
    data = normalizeData(data);
  }

  data = data && data.find(function (_ref2) {
    var __id = _ref2.__id;
    return __id == item_index;
  }) || null;

  if (data && data.__isTransformed) {
    data = parseCurrentValue(data);
  }

  return data;
};

exports.getCurrentData = getCurrentData;

var prepareStartupState = function prepareStartupState(props) {
  var raw_data = preSelectData(props.raw_data || props.data || props.children);
  var data = getData(props);
  var opened = props.opened !== null ? (0, _componentHelper.isTrue)(props.opened) : null;
  var selected_item = null;

  if (props.value !== null && typeof props.value !== 'undefined' && props.value !== 'initval') {
    selected_item = getCurrentIndex(props.value, data);
  } else if (props.default_value !== null) {
    selected_item = getCurrentIndex(props.default_value, data);
  }

  return {
    opened: opened,
    data: data,
    init_data: props.data,
    original_data: data,
    raw_data: raw_data,
    direction: props.direction,
    max_height: props.max_height,
    selected_item: selected_item,
    active_item: selected_item,
    on_hide: props.on_hide,
    on_show: props.on_show,
    on_chnage: props.on_chnage,
    on_select: props.on_select,
    _listenForPropChanges: false
  };
};

exports.prepareStartupState = prepareStartupState;

var prepareDerivedState = function prepareDerivedState(props, state) {
  if (state.opened && !state.data && typeof props.data === 'function') {
    state.data = getData(props);
  }

  if (state._listenForPropChanges) {
    if (props.data && props.data !== state.init_data) {
      state.data = getData(props);
      state.init_data = props.data;
    }

    state.usePortal = props.skip_portal !== null ? !(0, _componentHelper.isTrue)(props.skip_portal) : true;

    if (typeof props.wrapper_element === 'string' && typeof document !== 'undefined') {
      var wrapper_element = document.querySelector(props.wrapper_element);

      if (wrapper_element) {
        state.wrapper_element = wrapper_element;
      }
    } else if (props.wrapper_element) {
      state.wrapper_element = props.wrapper_element;
    }

    if (typeof props.value !== 'undefined' && props.value !== 'initval' && state.selected_item !== props.value && (state._value !== props.value || (0, _componentHelper.isTrue)(props.prevent_selection))) {
      state.selected_item = getCurrentIndex(props.value, state.data);

      if (typeof props.on_state_update === 'function') {
        (0, _componentHelper.dispatchCustomElementEvent)({
          props: props
        }, 'on_state_update', {
          selected_item: state.selected_item,
          value: getSelectedItemValue(state.selected_item, state),
          data: getEventData(state.selected_item, state.data)
        });
      }
    }

    if (!(parseFloat(state.active_item) > -1)) {
      state.active_item = state.selected_item;
    }

    if (props.direction !== 'auto' && props.direction !== state.direction) {
      state.direction = props.direction;
    }

    if (parseFloat(state.selected_item) > -1) {
      state.current_title = getCurrentDataTitle(state.selected_item, state.data);
    }
  }

  state._value = props.value;
  state._listenForPropChanges = true;
  return state;
};

exports.prepareDerivedState = prepareDerivedState;

var getCurrentDataTitle = function getCurrentDataTitle(selected_item, data) {
  var currentData = getCurrentData(selected_item, data);
  return parseContentTitle(currentData, {
    separator: ' ',
    preferSelectedValue: true
  });
};

exports.getCurrentDataTitle = getCurrentDataTitle;

var findClosest = function findClosest(arr, val) {
  return Math.max.apply(null, arr.filter(function (v) {
    return v <= val;
  }));
};

exports.findClosest = findClosest;