"use strict";

require("core-js/modules/es.array.for-each.js");

require("core-js/modules/es.array.includes.js");

require("core-js/modules/es.object.assign.js");

require("core-js/modules/es.regexp.exec.js");

require("core-js/modules/es.string.includes.js");

require("core-js/modules/es.string.split.js");

require("core-js/modules/web.dom-collections.for-each.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.prepareContext = void 0;

var _react = _interopRequireDefault(require("react"));

var _defaults = require("./defaults");

var _locales = _interopRequireDefault(require("./locales"));

var _componentHelper = require("./component-helper");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var prepareContext = function prepareContext() {
  var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var locales = props.locales ? (0, _componentHelper.extend)(_locales.default, props.locales) : _locales.default;

  if (props.__newContext) {
    _extends(props, props.__newContext);

    delete props.__newContext;
  }

  var key = handleLocaleFallbacks(props.locale || _defaults.LOCALE, locales);
  var translation = locales[key] || _locales.default[_defaults.LOCALE] || {};

  if (locales[key]) {
    locales[key] = destruct(locales[key], translation);
  }

  var context = _extends({
    updateTranslation: function updateTranslation(locale, translation) {
      context.translation = context.locales[locale] = translation;
    },
    getTranslation: function getTranslation(props) {
      if (props) {
        var lang = props.lang || props.locale;

        if (lang && context.locales[lang] && lang !== key) {
          return context.locales[lang];
        }
      }

      return context.translation;
    },
    locales: locales
  }, props, {
    translation: translation
  });

  return context;
};

exports.prepareContext = prepareContext;

function handleLocaleFallbacks(locale, locales) {
  if (!locales[locale]) {
    if (locale === 'en' || locale.split('-')[0] === 'en') {
      return 'en-GB';
    }
  }

  return locale;
}

var Context = _react.default.createContext(prepareContext({
  locale: _defaults.LOCALE,
  currency: _defaults.CURRENCY,
  currency_display: _defaults.CURRENCY_DISPLAY
}));

var _default = Context;
exports.default = _default;

function destruct(source, validKeys) {
  for (var k in source) {
    if (String(k).includes('.')) {
      var list = k.split('.');

      if (validKeys[list[0]]) {
        (function () {
          var val = source[k];
          var last = list.length - 1;
          list.forEach(function (k, i) {
            source[k] = i === last ? val : source[k];
            source = source[k] || {};
          });
        })();
      }
    }
  }

  return source;
}