"use strict";

require("core-js/modules/es.array.find.js");

require("core-js/modules/es.array.for-each.js");

require("core-js/modules/es.array.index-of.js");

require("core-js/modules/es.array.join.js");

require("core-js/modules/es.array.splice.js");

require("core-js/modules/es.object.assign.js");

require("core-js/modules/es.regexp.exec.js");

require("core-js/modules/es.string.match.js");

require("core-js/modules/web.dom-collections.for-each.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.clearAllBodyScrollLocks = exports.enableBodyScroll = exports.disableBodyScroll = void 0;

var _componentHelper = require("../component-helper");

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var isServer = function isServer() {
  return typeof window === 'undefined';
};

var detectOS = function detectOS(ua) {
  ua = ua || navigator.userAgent;
  var ipad = /(iPad).*OS\s([\d_]+)/.test(ua);
  var iphone = !ipad && /(iPhone\sOS)\s([\d_]+)/.test(ua);
  var android = /(Android);?[\s/]+([\d.]+)?/.test(ua);
  var ios = iphone || ipad;
  return {
    ios: ios,
    android: android
  };
};

var detectiOSVersion = function detectiOSVersion() {
  var match = navigator.appVersion.match(/OS (\d+)_(\d+)_?(\d+)?/);
  var version;

  if (match !== undefined && match !== null) {
    version = [parseInt(match[1], 10), parseInt(match[2], 10), parseInt(match[3] || 0, 10)];
    return parseFloat(version.join('.'));
  }

  return false;
};

var lockedNum = 0;
var initialClientY = 0;
var initialClientX = 0;
var unLockCallback = null;
var documentListenerAdded = false;
var lockedElements = [];
var eventListenerOptions = getEventListenerOptions({
  passive: false
});

function getEventListenerOptions(options) {
  if (isServer()) {
    return;
  }

  var isSupportOptions = true;
  var capture = options.capture;
  return isSupportOptions ? options : typeof capture !== 'undefined' ? capture : false;
}

var setOverflowHiddenPc = function setOverflowHiddenPc() {
  try {
    var $html = document.documentElement;
    var $body = document.body;

    var htmlStyle = _extends({}, $html.style);

    var bodyStyle = _extends({}, $body.style);

    var scrollBarWidth = window.innerWidth - $body.clientWidth;
    $html.style.height = 'auto';
    $html.style.overflow = 'hidden';
    $body.style.overflow = 'hidden';
    $body.style.height = 'auto';
    $body.style.boxSizing = 'border-box';
    $body.style.paddingRight = "".concat(scrollBarWidth, "px");
    return function () {
      try {
        ;
        ['height', 'overflow'].forEach(function (x) {
          $html.style[x] = htmlStyle[x] || '';
        });
        ['overflow', 'height', 'boxSizing', 'paddingRight'].forEach(function (x) {
          $body.style[x] = bodyStyle[x] || '';
        });
      } catch (e) {}
    };
  } catch (e) {}
};

var setOverflowHiddenMobile = function setOverflowHiddenMobile() {
  try {
    var $html = document.documentElement;
    var $body = document.body;
    var scrollTop = $html.scrollTop || $body.scrollTop;

    var htmlStyle = _extends({}, $html.style);

    var bodyStyle = _extends({}, $body.style);

    $html.style.height = '100%';
    $html.style.overflow = 'hidden';
    $body.style.top = "-".concat(scrollTop, "px");
    $body.style.width = '100%';
    $body.style.height = 'auto';
    $body.style.position = 'fixed';
    $body.style.overflow = 'hidden';
    return function () {
      try {
        ;
        ['height', 'overflow'].forEach(function (x) {
          $html.style[x] = htmlStyle[x] || '';
        });
        ['top', 'width', 'height', 'overflow', 'position'].forEach(function (x) {
          $body.style[x] = bodyStyle[x] || '';
        });
        var scrollBehavior = window.getComputedStyle($html).scrollBehavior;
        $html.style.scrollBehavior = 'auto';
        $html.scrollTop = scrollTop;
        $html.style.scrollBehavior = scrollBehavior;
      } catch (e) {}
    };
  } catch (e) {}
};

var preventDefault = function preventDefault(event) {
  var found = lockedElements.find(function (targetElement) {
    return (0, _componentHelper.isChildOfElement)(event.target, targetElement);
  });

  if (found || !event.cancelable) {
    return;
  }

  event.preventDefault();
};

var handleScroll = function handleScroll(event, targetElement) {
  try {
    if (targetElement) {
      var scrollTop = targetElement.scrollTop,
          scrollLeft = targetElement.scrollLeft,
          scrollWidth = targetElement.scrollWidth,
          scrollHeight = targetElement.scrollHeight,
          clientWidth = targetElement.clientWidth,
          clientHeight = targetElement.clientHeight;
      var clientX = event.targetTouches[0].clientX - initialClientX;
      var clientY = event.targetTouches[0].clientY - initialClientY;
      var isVertical = Math.abs(clientY) > Math.abs(clientX);
      var isOnTop = clientY > 0 && scrollTop === 0;
      var isOnLeft = clientX > 0 && scrollLeft === 0;
      var isOnRight = clientX < 0 && scrollLeft + clientWidth + 1 >= scrollWidth;
      var isOnBottom = clientY < 0 && scrollTop + clientHeight + 1 >= scrollHeight;

      if (isVertical && (isOnTop || isOnBottom) || !isVertical && (isOnLeft || isOnRight)) {
        var hasScrollbar = (0, _componentHelper.isChildOfElement)(event.target, targetElement, _componentHelper.checkIfHasScrollbar);

        if (hasScrollbar && hasScrollbar !== targetElement) {
          return true;
        }

        return event.cancelable && event.preventDefault();
      }
    }

    event.stopPropagation();
    return true;
  } catch (e) {}
};

var checkTargetElement = function checkTargetElement(targetElement) {
  if (targetElement) return;
  if (targetElement === null) return;
  console.warn("If scrolling is also required in the floating layer, " + "the target element must be provided.");
};

var disableBodyScroll = function disableBodyScroll(targetElement) {
  if (isServer()) {
    return;
  }

  checkTargetElement(targetElement);

  try {
    if (detectOS().ios) {
      if (detectiOSVersion() >= 14) {
        if (lockedNum <= 0) {
          unLockCallback = setOverflowHiddenMobile();
        }
      } else {
        if (targetElement) {
          var elementArray = Array.isArray(targetElement) ? targetElement : [targetElement];
          elementArray.forEach(function (element) {
            if (element && lockedElements.indexOf(element) === -1) {
              element.ontouchstart = function (event) {
                initialClientY = event.targetTouches[0].clientY;
                initialClientX = event.targetTouches[0].clientX;
              };

              element.ontouchmove = function (event) {
                if (event.targetTouches.length !== 1) {
                  return;
                }

                handleScroll(event, element);
              };

              lockedElements.push(element);
            }
          });
        }

        if (!documentListenerAdded) {
          document.addEventListener('touchmove', preventDefault, eventListenerOptions);
          documentListenerAdded = true;
        }
      }
    } else if (lockedNum <= 0) {
      unLockCallback = detectOS().android ? setOverflowHiddenMobile() : setOverflowHiddenPc();
    }

    lockedNum += 1;
  } catch (e) {}
};

exports.disableBodyScroll = disableBodyScroll;

var enableBodyScroll = function enableBodyScroll(targetElement) {
  if (isServer()) {
    return;
  }

  checkTargetElement(targetElement);

  try {
    lockedNum -= 1;

    if (lockedNum > 0) {
      return;
    }

    if (typeof unLockCallback === 'function') {
      unLockCallback();
    }

    if (detectOS().ios && !(detectiOSVersion() >= 14)) {
      if (targetElement) {
        var elementArray = Array.isArray(targetElement) ? targetElement : [targetElement];
        elementArray.forEach(function (element) {
          var index = lockedElements.indexOf(element);

          if (index !== -1) {
            element.ontouchmove = null;
            element.ontouchstart = null;
            lockedElements.splice(index, 1);
          }
        });
      }

      if (documentListenerAdded) {
        document.removeEventListener('touchmove', preventDefault, eventListenerOptions);
        documentListenerAdded = false;
      }
    }
  } catch (e) {}
};

exports.enableBodyScroll = enableBodyScroll;

var clearAllBodyScrollLocks = function clearAllBodyScrollLocks() {
  if (isServer()) {
    return;
  }

  try {
    lockedNum = 0;

    if (!detectOS().ios && typeof unLockCallback === 'function') {
      unLockCallback();
    }

    if (detectOS().ios && !(detectiOSVersion() >= 14)) {
      if (lockedElements && lockedElements.length) {
        var element = lockedElements.pop();

        while (element) {
          element.ontouchmove = null;
          element.ontouchstart = null;
          element = lockedElements.pop();
        }
      }
    }

    if (documentListenerAdded) {
      document.removeEventListener('touchmove', preventDefault, eventListenerOptions);
      documentListenerAdded = false;
    }
  } catch (e) {}
};

exports.clearAllBodyScrollLocks = clearAllBodyScrollLocks;