import * as React from 'react';
import AccordionContent from './AccordionContent';
import AccordionHeader from './AccordionHeader';
import AccordionProvider from './AccordionProvider';
export type AccordionExpanded = string | boolean;
export type AccordionNoAnimation = string | boolean;
export type AccordionExpandedSsr = string | boolean;
export type AccordionPrerender = string | boolean;
export type AccordionPreventRerender = string | boolean;
export type AccordionPreventRerenderConditional = string | boolean;
export type AccordionRememberState = string | boolean;
export type AccordionFlushRememberedState = string | boolean;
export type AccordionSingleContainer = string | boolean;
export type AccordionVariant = 'default' | 'outlined' | 'filled';
export type AccordionAllowCloseAll = string | boolean;
export type AccordionDisabled = string | boolean;
export type AccordionSkeleton = string | boolean;
export type AccordionHeading = boolean | React.ReactNode;
export type AccordionHeadingLevel = string | number;
export type AccordionIconPosition = 'left' | 'right';
export type AccordionAttributes = string | Object;
export type AccordionSpace =
  | string
  | number
  | boolean
  | {
      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. Will use `margin-top`.
       */
      top?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-right`.
       */
      right?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-bottom`.
       */
      bottom?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-left`.
       */
      left?: string | number | boolean;
    };
export type AccordionTop = string | number | boolean;
export type AccordionRight = string | number | boolean;
export type AccordionBottom = string | number | boolean;
export type AccordionLeft = string | number | boolean;

/**
 * NB: Do not change the docs (comments) in here. The docs are updated during build time by "generateTypes.js" and "fetchPropertiesFromDocs.js".
 */
export interface AccordionProps extends React.HTMLProps<HTMLElement> {
  label?: React.ReactNode;
  title?: string;
  expanded?: AccordionExpanded;
  no_animation?: AccordionNoAnimation;
  expanded_ssr?: AccordionExpandedSsr;
  prerender?: AccordionPrerender;
  prevent_rerender?: AccordionPreventRerender;
  prevent_rerender_conditional?: AccordionPreventRerenderConditional;
  remember_state?: AccordionRememberState;
  flush_remembered_state?: AccordionFlushRememberedState;
  single_container?: AccordionSingleContainer;
  variant?: AccordionVariant;
  left_component?: React.ReactNode;
  allow_close_all?: AccordionAllowCloseAll;
  disabled?: AccordionDisabled;
  skeleton?: AccordionSkeleton;

  /**
   * Docs.
   */
  id?: string;
  group?: string;
  element?: React.ReactNode;
  heading?: AccordionHeading;
  heading_level?: AccordionHeadingLevel;
  icon?: React.ReactNode;
  icon_position?: AccordionIconPosition;
  icon_size?: string;
  attributes?: AccordionAttributes;
  class?: string;
  className?: string;
  children?: React.ReactNode;

  /**
   * Has to be an object with either: `top`, `right`, `bottom` or `left`. Use spacing values like: `small`, `1rem`, `1` or , `16px`.
   */
  space?: AccordionSpace;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. Will use `margin-top`.
   */
  top?: AccordionTop;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-right`.
   */
  right?: AccordionRight;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-bottom`.
   */
  bottom?: AccordionBottom;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-left`.
   */
  left?: AccordionLeft;
  custom_element?: Object;
  custom_method?: (...args: any[]) => any;

  /**
   * Will be called by user click interaction. Returns an object with a boolean state `expanded` inside `{ expanded, id, event, ...event }`.
   */
  on_change?: (...args: any[]) => any;
  on_state_update?: (...args: any[]) => any;
}
export default class Accordion extends React.Component<
  AccordionProps,
  any
> {
  static defaultProps: object;
  static Content = AccordionContent;
  static Header = AccordionHeader;
  static Provider = AccordionProvider;
  render(): JSX.Element;
}
