import * as React from 'react';
export type ModalContentMode = 'modal' | 'drawer';
export type ModalContentHideCloseButton = string | boolean;
export type ModalContentSpacing = string | boolean;
export type ModalContentPreventClose = string | boolean;
export type ModalContentPreventCoreStyle = string | boolean;
export type ModalContentAnimationDuration = string | number;
export type ModalContentNoAnimation = string | boolean;
export type ModalContentNoAnimationOnMobile = string | boolean;
export type ModalContentMinWidth = string | number;
export type ModalContentMaxWidth = string | number;
export type ModalContentFullscreen = string | boolean;
export type ModalContentChildren =
  | string
  | React.ReactNode
  | ((...args: any[]) => any);

/**
 * NB: Do not change the docs (comments) in here. The docs are updated during build time by "generateTypes.js" and "fetchPropertiesFromDocs.js".
 */
export interface ModalContentProps extends React.HTMLProps<HTMLElement> {
  /**
   * The content which will appear when triggering the modal.
   */
  modal_content: React.ReactNode;

  /**
   * The modal mode. Can be set to `drawer`. Defaults to `modal`.
   */
  mode?: ModalContentMode;
  hide?: boolean;

  /**
   * The id used internal in the modal root element. Defaults to `root`, so the element id will be `dnb-modal-root`.
   */
  id?: string;
  root_id?: string;

  /**
   * The ID of the trigger component, describing the modal content. Defaults to the internal `trigger`, so make sure You define the `trigger_title`.
   */
  labelled_by?: string;

  /**
   * The Modal handles the first focus – automatically. How ever, you can defined a custom focus selector the will be used instead `focus_selector=".css-selector"`.
   */
  focus_selector?: string;

  /**
   * Defines an unique identifier to a modal. Use it in case you have to refer in some way to the modal content wrapper.
   */
  content_id?: string;

  /**
   * The modal title. Displays on the very top of the content.
   */
  title?: React.ReactNode;

  /**
   * The title of the close button. Defaults to <em>Close Modal Window</em>.
   */
  close_title?: string;

  /**
   * If truthy, the close button will now be shown.
   */
  hide_close_button?: ModalContentHideCloseButton;

  /**
   * If set to `false` then the modal content will be shown without any spacing. Defaults to `true`.
   */
  spacing?: ModalContentSpacing;

  /**
   * If set to `true` (boolean or string), then the user can&#39;t close the modal.
   */
  prevent_close?: ModalContentPreventClose;

  /**
   * By default the modal content gets added the core style class `dnb-core-style`. Use `false` to disable this behavior.
   */
  prevent_core_style?: ModalContentPreventCoreStyle;
  animation_duration?: ModalContentAnimationDuration;

  /**
   * If set to `true`, no open/close animation will be shown. Defaults to false.
   */
  no_animation?: ModalContentNoAnimation;

  /**
   * Same as `no_animation`, but gets triggered only if the viewport width is less than `40em`. Defaults to false.
   */
  no_animation_on_mobile?: ModalContentNoAnimationOnMobile;

  /**
   * The minimum Modal content width, defined by a CSS width value like `50vw` (50% of the viewport). Be careful on using fixed `min_width` so you don&#39;t break responsiveness. Defaults to `30rem` (average width is set to `60vw`).
   */
  min_width?: ModalContentMinWidth;

  /**
   * The maximum Modal content width, defined by a CSS width value like `20rem`. Defaults to `60rem` (average width is set to `60vw`).
   */
  max_width?: ModalContentMaxWidth;

  /**
   * If set to `true` then the modal content will be shown as fullscreen, without showing the original content behind. Can be set to `false` to omit the auto fullscreen. Defaults to `auto`.
   */
  fullscreen?: ModalContentFullscreen;

  /**
   * Define the inner horizontal alignment of the content. Can be set to `left`, `center`, `right` and `centered`. If `centered`, then the content will also be centered vertically. Defaults to `left`.
   */
  align_content?: string;

  /**
   * For `drawer` mode only. Defines the placement on what side the Drawer should be opened. Can be set to `left`, `right`, `top` and `bottom`. Defaults to `right`.
   */
  container_placement?: string;

  /**
   * Give the inner content wrapper a class name (maps to `dnb-modal__content__inner`).
   */
  class?: string;

  /**
   * Give the content wrapper a custom class name (maps to `dnb-modal__content`).
   */
  content_class?: string;

  /**
   * Give the page overlay a custom class name (maps to `dnb-modal__overlay`).
   */
  overlay_class?: string;
  closeModal: (...args: any[]) => any;

  /**
   * Give the inner content wrapper a class name (maps to `dnb-modal__content__inner`).
   */
  className?: string;

  /**
   * The content which will appear when triggering the modal.
   */
  children?: ModalContentChildren;
}
export default class ModalContent extends React.Component<
  ModalContentProps,
  any
> {
  static defaultProps: object;
  render(): JSX.Element;
}
export type CloseButtonStyleType = 'button' | 'cross';

/**
 * NB: Do not change the docs (comments) in here. The docs are updated during build time by "generateTypes.js" and "fetchPropertiesFromDocs.js".
 */
export interface CloseButtonProps {
  style_type?: CloseButtonStyleType;
  on_click: (...args: any[]) => any;

  /**
   * The title of the close button. Defaults to <em>Close Modal Window</em>.
   */
  close_title?: string;

  /**
   * Give the inner content wrapper a class name (maps to `dnb-modal__content__inner`).
   */
  className?: string;
}
export class CloseButton extends React.Component<CloseButtonProps, any> {
  static defaultProps: object;
  render(): JSX.Element;
}
