import "core-js/modules/es.array.concat.js";
import "core-js/modules/es.array.find.js";
import "core-js/modules/es.array.for-each.js";
import "core-js/modules/web.dom-collections.for-each.js";
import _extends from "@babel/runtime/helpers/esm/extends";
import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import _assertThisInitialized from "@babel/runtime/helpers/esm/assertThisInitialized";
import _inherits from "@babel/runtime/helpers/esm/inherits";
import _possibleConstructorReturn from "@babel/runtime/helpers/esm/possibleConstructorReturn";
import _getPrototypeOf from "@babel/runtime/helpers/esm/getPrototypeOf";

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

import React from 'react';
import PropTypes from 'prop-types';
import Context from '../../shared/Context';
import { isTrue, dispatchCustomElementEvent } from '../../shared/component-helper';
import { ContentObject } from './PaginationHelpers';
import PaginationContext from './PaginationContext';

var PaginationProvider = function (_React$PureComponent) {
  _inherits(PaginationProvider, _React$PureComponent);

  var _super = _createSuper(PaginationProvider);

  function PaginationProvider(_props) {
    var _this;

    _classCallCheck(this, PaginationProvider);

    _this = _super.call(this, _props);

    _this.setContent = function (newContent) {
      var content = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
      var position = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

      if (!Array.isArray(newContent) && content) {
        newContent = [newContent, content];
      }

      var pageNo = parseFloat(newContent[0]);
      newContent = newContent[1];

      if (typeof newContent === 'function') {
        content = newContent();
      } else if (React.isValidElement(newContent)) {
        content = newContent;
      }

      if (content) {
        var itemToPrepare = _this.state.items.find(function (_ref) {
          var p = _ref.pageNo;
          return p === pageNo;
        });

        var items = null;

        if (!itemToPrepare) {
          items = _this.prefillItems(pageNo, {
            position: position,
            skipObserver: true
          });
          itemToPrepare = items.find(function (_ref2) {
            var p = _ref2.pageNo;
            return p === pageNo;
          });
        }

        if (itemToPrepare.content) {
          itemToPrepare.update(content);
        } else {
          itemToPrepare.insert(content);
        }

        _this.setState({
          items: _toConsumableArray(items || _this.state.items),
          currentPage: pageNo,
          _listenForPropChanges: false
        }, _this.callOnPageUpdate);
      }
    };

    _this.resetContent = function () {
      clearTimeout(_this.resetContentTimeout);
      _this.resetContentTimeout = setTimeout(function () {
        _this.setState({
          items: [],
          _listenForPropChanges: false
        });
      }, 10);
    };

    _this.resetInfinity = function () {
      var pageNo = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _this.state.startupPage;
      var lowerPage = pageNo;
      var upperPage = pageNo;
      var currentPage = pageNo;
      _this._hasEndedInfinity = true;

      _this.setState({
        hasEndedInfinity: true,
        lowerPage: lowerPage,
        upperPage: upperPage,
        currentPage: currentPage,
        _listenForPropChanges: false
      });

      clearTimeout(_this.resetInfinityTimeout);
      _this.resetInfinityTimeout = setTimeout(function () {
        _this.startInfinity();
      }, _this.state.minTime);
    };

    _this.startInfinity = function () {
      _this._hasEndedInfinity = false;

      _this.setState({
        hasEndedInfinity: false,
        _listenForPropChanges: false
      });
    };

    _this.endInfinity = function () {
      _this._hasEndedInfinity = true;

      _this.setState({
        hasEndedInfinity: true,
        _listenForPropChanges: false
      }, function () {
        var pageNo = _this.state.currentPage + 1;
        dispatchCustomElementEvent(_assertThisInitialized(_this), 'on_end', _extends({
          page: pageNo,
          pageNo: pageNo
        }, _assertThisInitialized(_this)));
      });
    };

    _this.setItems = function (items, cb) {
      _this.setState({
        items: items,
        _listenForPropChanges: false
      }, cb);
    };

    _this.prefillItems = function (pageNo) {
      var props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var items = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _this.state.items;
      var position = props.position || (pageNo < _this.state.currentPage ? 'before' : 'after');

      var obj = _extends({
        pageNo: pageNo,
        position: position,
        skipObserver: false
      }, props);

      switch (position) {
        case 'before':
          return [new ContentObject(obj)].concat(_toConsumableArray(items));

        case 'after':
          return [].concat(_toConsumableArray(items), [new ContentObject(obj)]);
      }
    };

    _this.setStateHandler = function (state, cb) {
      _this.setState(_extends({}, state, {
        _listenForPropChanges: false
      }), cb);
    };

    _this.callOnPageUpdate = function () {
      if (Array.isArray(_this._updateStack)) {
        _this._updateStack.forEach(function (cb) {
          if (typeof cb === 'function') {
            cb();
          }
        });

        _this._updateStack = [];
      }
    };

    _this.onPageUpdate = function (fn) {
      _this._updateStack = _this._updateStack || [];

      _this._updateStack.push(fn);
    };

    _this.state = {
      items: [],
      isLoading: false,
      _listenForPropChanges: true
    };

    if (_props.rerender) {
      _this.rerender = function (_ref3) {
        var store = _ref3.current;

        if (store && store.pageNo > 0) {
          clearTimeout(_this.rerenderTimeout);
          _this.rerenderTimeout = setTimeout(function () {
            return _this.setContent(store.pageNo, store.content);
          }, 1);
        }
      };

      _props.rerender.current = _this.rerender;
    }

    return _this;
  }

  _createClass(PaginationProvider, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      var _this$props = this.props,
          set_content_handler = _this$props.set_content_handler,
          reset_content_handler = _this$props.reset_content_handler,
          reset_pagination_handler = _this$props.reset_pagination_handler,
          end_infinity_handler = _this$props.end_infinity_handler;

      if (typeof set_content_handler === 'function') {
        set_content_handler(this.setContent);
      }

      if (typeof reset_content_handler === 'function') {
        reset_content_handler(this.resetContent);
      }

      if (typeof reset_pagination_handler === 'function') {
        reset_pagination_handler(this.resetInfinity);
      }

      if (typeof end_infinity_handler === 'function') {
        end_infinity_handler(this.endInfinity);
      }

      if (this.props.store && this.props.store.current) {
        var store = this.props.store.current;
        this.setContent(store.pageNo, store.content);
      }

      this._isMounted = true;
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      clearTimeout(this.rerenderTimeout);
      clearTimeout(this.resetContentTimeout);
      clearTimeout(this.resetInfinityTimeout);
      clearTimeout(this.callOnPageUpdateTimeout);
      this._isMounted = false;
    }
  }, {
    key: "render",
    value: function render() {
      var children = this.props.children;

      if (this.props.useMarkerOnly) {
        clearTimeout(this.callOnPageUpdateTimeout);
        this.callOnPageUpdateTimeout = setTimeout(this.callOnPageUpdate, 1);
      }

      return React.createElement(PaginationContext.Provider, {
        value: _extends({}, this.context, {
          pagination: _extends({
            setContent: this.setContent,
            resetContent: this.resetContent,
            resetInfinity: this.resetInfinity,
            resetPagination: this.resetInfinity,
            endInfinity: this.endInfinity,
            setItems: this.setItems,
            prefillItems: this.prefillItems,
            setState: this.setStateHandler,
            onPageUpdate: this.onPageUpdate,
            _hasEndedInfinity: this._hasEndedInfinity
          }, this.props, this.state)
        })
      }, children);
    }
  }], [{
    key: "getDerivedStateFromProps",
    value: function getDerivedStateFromProps(props, state) {
      if (state._listenForPropChanges) {
        if (props.page_count !== null) {
          state.pageCount = parseFloat(props.page_count) || 1;
        }

        if (props.current_page !== null) {
          state.currentPage = parseFloat(props.current_page) || 1;
        }

        if (typeof state.startupPage === 'undefined') {
          state.startupPage = parseFloat(props.startup_page) || parseFloat(props.current_page) || 1;
        }

        state.parallelLoadCount = parseFloat(props.parallel_load_count) || 1;
        state.minTime = parseFloat(props.min_wait_time) || 0;
        state.placeMakerBeforeContent = isTrue(props.place_maker_before_content);

        if (props.useMarkerOnly) {
          if (!state.lowerPage) {
            state.lowerPage = state.upperPage = state.startupPage;
          }
        }

        if (props.reset_pagination_handler !== null && isTrue(props.reset_pagination_handler)) {
          if (props.useMarkerOnly) {
            state.lowerPage = state.upperPage = state.startupPage;
          }
        }

        if (props.reset_content_handler !== null && isTrue(props.reset_content_handler)) {
          state.items = [];
          state.pageCount = parseFloat(props.page_count) || 1;
        }

        if (typeof props.items === 'string' && props.items[0] === '[') {
          state.items = JSON.parse(props.items);
        } else if (Array.isArray(props.items)) {
          state.items = props.items;
        }
      }

      state._listenForPropChanges = true;
      return state;
    }
  }]);

  return PaginationProvider;
}(React.PureComponent);

PaginationProvider.contextType = Context;
PaginationProvider.defaultProps = {
  startup_page: null,
  current_page: null,
  page_count: null,
  set_content_handler: null,
  reset_content_handler: null,
  reset_pagination_handler: null,
  end_infinity_handler: null,
  rerender: null,
  store: null,
  useMarkerOnly: null,
  children: null
};
export { PaginationProvider as default };
process.env.NODE_ENV !== "production" ? PaginationProvider.propTypes = {
  startup_page: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  current_page: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  page_count: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  set_content_handler: PropTypes.func,
  reset_content_handler: PropTypes.func,
  reset_pagination_handler: PropTypes.func,
  end_infinity_handler: PropTypes.func,
  rerender: PropTypes.shape({
    current: PropTypes.func
  }),
  store: PropTypes.shape({
    current: PropTypes.oneOfType([PropTypes.object, PropTypes.func])
  }),
  useMarkerOnly: PropTypes.bool,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node, PropTypes.object, PropTypes.array])
} : void 0;