import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { warn, isTrue, makeUniqueId, registerElement, extendPropsWithContext, validateDOMAttributes, dispatchCustomElementEvent } from '../../shared/component-helper';
import { createSpacingClasses } from '../space/SpacingHelper';
import AccordionProvider from './AccordionProvider';
import AccordionHeader from './AccordionHeader';
import AccordionContent from './AccordionContent';
import AccordionContext from './AccordionContext';
import AccordionProviderContext from './AccordionProviderContext';
import Context from '../../shared/Context';
import { accordionPropTypes, accordionDefaultProps } from './AccordionPropTypes';

class AccordionStore {
  constructor(id) {
    this._id = id;
    this._instances = [];
  }

  onChange({
    id
  }) {
    this._instances.forEach(inst => {
      if (inst._id !== id) {
        inst.close();
      }
    });
  }

  addInstance(instance) {
    this._instances.push(instance);
  }

  removeInstance(instance) {
    this._instances = this._instances.filter(inst => inst !== instance);
  }

}

var _ref = React.createElement(AccordionHeader, null);

export default class Accordion extends React.PureComponent {
  static enableWebComponent() {
    registerElement(Accordion.tagName, Accordion, Accordion.defaultProps);
  }

  static getDerivedStateFromProps(props, state) {
    if (state._listenForPropChanges) {
      if (props.expanded !== state._expanded) {
        state.expanded = isTrue(props.expanded);
        state._expanded = props.expanded;
      }

      if (props.group && !state.group) {
        state.group = props.group;
      }
    }

    state._listenForPropChanges = true;
    return state;
  }

  constructor(props, context) {
    super(props);

    this.handleDisabledClick = e => {
      e.preventDefault();
      return false;
    };

    this.callOnChangeHandler = (...params) => {
      var _this$context;

      this.callOnChange(...params);

      if ((_this$context = this.context) !== null && _this$context !== void 0 && _this$context.onChange) {
        var _this$context2;

        (_this$context2 = this.context) === null || _this$context2 === void 0 ? void 0 : _this$context2.onChange(...params);
      }

      if (this.state.group && typeof window !== 'undefined') {
        var _window, _window$__dnbAccordio;

        (_window = window) === null || _window === void 0 ? void 0 : (_window$__dnbAccordio = _window.__dnbAccordion[this.state.group]) === null || _window$__dnbAccordio === void 0 ? void 0 : _window$__dnbAccordio.onChange(...params);
      }
    };

    this.callOnChange = ({
      expanded,
      event
    }) => {
      this.changeOpened(expanded, event);
      dispatchCustomElementEvent(this, 'on_change', {
        expanded,
        event
      });
    };

    this._id = props.id || makeUniqueId();
    this.state = {
      _expanded: props.expanded,
      expanded: props.expanded !== null ? isTrue(props.expanded) : isTrue(context === null || context === void 0 ? void 0 : context.expanded),
      group: props.group || (context === null || context === void 0 ? void 0 : context.group),
      no_animation: isTrue(props.no_animation || (context === null || context === void 0 ? void 0 : context.no_animation)),
      _listenForPropChanges: false
    };

    if (typeof window === 'undefined' && isTrue(props.expanded_ssr || (context === null || context === void 0 ? void 0 : context.expanded_ssr))) {
      this.state.expanded = true;
    }

    if (this.state.group && typeof window !== 'undefined') {
      window.__dnbAccordion = window.__dnbAccordion || {};
      window.__dnbAccordion[this.state.group] = window.__dnbAccordion[this.state.group] || new AccordionStore(this.state.group);

      window.__dnbAccordion[this.state.group].addInstance(this);
    }

    this.store = new Store({
      id: props.id,
      group: this.state.group
    });

    if (isTrue(props.remember_state || context.remember_state) && isTrue(props.expanded)) {
      const expanded = this.store.getState();

      if (expanded === false) {
        this.state.expanded = false;
      }
    }

    if (context && typeof (context === null || context === void 0 ? void 0 : context.onInit) === 'function') {
      context.onInit(this);
    }

    if (typeof window !== 'undefined' && isTrue(props.expanded_ssr || (context === null || context === void 0 ? void 0 : context.expanded_ssr))) {
      this.state.expanded = false;
    }

    if (isTrue(this.props.remember_state || context.remember_state)) {
      const expanded = this.store.getState();

      if (expanded) {
        this.state.expanded = true;
      }
    }
  }

  componentDidMount() {
    this._isMounted = true;
  }

  componentWillUnmount() {
    this._isMounted = false;
    clearTimeout(this._animationState);
    clearTimeout(this._openTimeout);
    clearTimeout(this._changeOpenState);

    if (this.state.group && typeof window !== 'undefined') {
      var _window2, _window2$__dnbAccordi;

      (_window2 = window) === null || _window2 === void 0 ? void 0 : (_window2$__dnbAccordi = _window2.__dnbAccordion[this.state.group]) === null || _window2$__dnbAccordi === void 0 ? void 0 : _window2$__dnbAccordi.removeInstance(this);
    }
  }

  componentDidUpdate(props) {
    var _this$context3;

    if (isTrue(this.context.flush_remembered_state)) {
      this.store.flush();
      this.setState({
        expanded: isTrue(this.props.expanded)
      });
    }

    if ((_this$context3 = this.context) !== null && _this$context3 !== void 0 && _this$context3.expanded_id && this.context.expanded_id === props.id) {
      this.setState({
        expanded: true
      });
    }
  }

  setExpandedState(expanded) {
    this.setState({
      expanded,
      _listenForPropChanges: false
    });
  }

  close() {
    this.changeOpened(false);
  }

  changeOpened(expanded) {
    if (!this._isMounted) {
      return;
    }

    this.setState({
      expanded,
      _listenForPropChanges: false
    });

    if (isTrue(this.props.remember_state) || isTrue(this.context.remember_state)) {
      this.store.saveState(expanded);
    }
  }

  hasAccordionHeader(children) {
    if (!Array.isArray(children)) {
      children = [children];
    }

    return children.findIndex(cur => React.isValidElement(cur) && cur.type === AccordionHeader) !== -1;
  }

  hasAccordionContent(children) {
    if (!Array.isArray(children)) {
      children = [children];
    }

    return children.findIndex(cur => React.isValidElement(cur) && cur.type === AccordionContent) !== -1;
  }

  render() {
    return React.createElement(Context.Consumer, null, globalContext => React.createElement(AccordionContext.Consumer, null, nestedContext => {
      const {
        no_animation
      } = this.state;
      let {
        expanded
      } = this.state;
      const props = extendPropsWithContext(this.props, Accordion.defaultProps, this.context, nestedContext, {
        skeleton: globalContext === null || globalContext === void 0 ? void 0 : globalContext.skeleton
      }, globalContext.accordion, globalContext.translation.Accordion);

      if (expanded === null && globalContext.accordion) {
        if (globalContext.accordion.expanded) {
          expanded = isTrue(props.expanded);
        }
      }

      const {
        variant,
        className,
        class: _className,
        prerender,
        prevent_rerender,
        prevent_rerender_conditional,
        single_container,
        remember_state,
        disabled,
        skeleton,
        children
      } = props,
            rest = _objectWithoutPropertiesLoose(props, ["variant", "className", "class", "prerender", "prevent_rerender", "prevent_rerender_conditional", "single_container", "remember_state", "disabled", "skeleton", "no_animation", "expanded_ssr", "children", "id", "group", "expanded", "title", "description", "left_component", "icon", "icon_position", "icon_size", "attributes", "on_change", "on_state_update", "custom_method", "custom_element"]);

      const id = this._id;
      const mainParams = {
        id,
        className: classnames('dnb-accordion', createSpacingClasses(props), className, _className, expanded && 'dnb-accordion--expanded', no_animation && 'dnb-accordion--no-animation', variant && `dnb-accordion__variant--${variant}`, isTrue(prerender) && 'dnb-accordion--prerender')
      };

      if (this.state.open) {
        mainParams.open = true;
      }

      validateDOMAttributes(this.props, rest);
      const extendProps = extendPropsWithContext(this.props, Accordion.defaultProps, this.state, this.context);

      const context = _extends({}, extendProps, {
        id,
        expanded,
        prerender: isTrue(prerender),
        prevent_rerender: isTrue(prevent_rerender),
        prevent_rerender_conditional: isTrue(prevent_rerender_conditional),
        single_container: isTrue(single_container),
        remember_state: isTrue(remember_state),
        disabled: isTrue(disabled),
        skeleton: isTrue(skeleton),
        callOnChange: this.callOnChangeHandler
      });

      if (isTrue(disabled)) {
        mainParams.onClick = this.handleDisabledClick;
      }

      return React.createElement(AccordionContext.Provider, {
        value: context
      }, React.createElement("div", mainParams, this.hasAccordionHeader(children) ? null : _ref, this.hasAccordionContent(children) ? children : React.createElement(AccordionContent, null, children)));
    }));
  }

}
Accordion.tagName = 'dnb-accordion';
Accordion.contextType = AccordionProviderContext;
Accordion.Provider = AccordionProvider;
Accordion.Header = AccordionHeader;
Accordion.Content = AccordionContent;
Accordion.defaultProps = _extends({}, accordionDefaultProps);
process.env.NODE_ENV !== "production" ? Accordion.propTypes = _extends({}, accordionPropTypes) : void 0;

class Group extends React.PureComponent {
  constructor(props) {
    super(props);
    this.state = {};

    this.onInit = instance => {
      if (instance.props.id && !this._IDs.includes(instance.props.id)) {
        this._IDs.push(instance.props.id);
      }
    };

    let group;

    if (props.id) {
      group = props.id;
    } else if (!props.group) {
      group = '#' + makeUniqueId();
    }

    this.state.group = group;
    this.store = new Store({
      group
    });
    this._IDs = [];

    if (isTrue(props.remember_state) && !props.id) {
      rememberWarning('accordion group');
    }
  }

  componentDidMount() {
    const storedData = this.store.getData();

    if (storedData !== null && storedData !== void 0 && storedData.id) {
      if (!this._IDs.includes(storedData.id)) {
        const expanded_id = this._IDs[0];

        if (expanded_id) {
          this.setState({
            expanded_id
          }, () => {
            this.store.saveState(true, expanded_id);
            this.setState({
              expanded_id: null
            });
          });
        }
      }
    }
  }

  render() {
    return React.createElement(AccordionProvider, _extends({
      onInit: this.onInit
    }, this.props, this.state));
  }

}

Group.defaultProps = {
  id: null,
  group: null,
  remember_state: null
};
process.env.NODE_ENV !== "production" ? Group.propTypes = {
  id: PropTypes.string,
  group: PropTypes.string,
  remember_state: PropTypes.oneOfType([PropTypes.string, PropTypes.bool])
} : void 0;
Accordion.Group = Group;

Accordion.Group.Store = (group, id = null) => {
  return new Store({
    group,
    id
  });
};

Accordion.Store = id => {
  return new Store({
    id
  });
};

function rememberWarning(type = 'accordion') {
  warn(`Missing "id" prop the ${type}! "remember_state" is enabled.`);
}

class Store {
  constructor({
    id,
    group
  }) {
    this.id = id;
    this.group = group;
    return this;
  }

  storeId(id = this.id) {
    if (this.group) {
      if (this.group[0] === '#') {
        return null;
      }

      id = this.group;
    }

    return `dnb-accordion-${id}`;
  }

  saveState(expanded, id = this.id, opts = {}) {
    if (id) {
      try {
        const store = this.getData() || {};

        if (this.group) {
          if (expanded) {
            store.id = id;
          } else if (opts && opts.force) {
            store.id = null;
          }
        } else {
          store.expanded = expanded;
        }

        const storeId = this.storeId(id);

        if (storeId) {
          window.localStorage.setItem(storeId, JSON.stringify(store));
        }
      } catch (e) {}
    } else {
      rememberWarning();
    }
  }

  getData(id = this.id) {
    const storeId = this.storeId(id);

    if (storeId) {
      try {
        if (Object.prototype.hasOwnProperty.call(window.localStorage, storeId)) {
          return JSON.parse(window.localStorage.getItem(storeId));
        }
      } catch (e) {}
    }

    return null;
  }

  getState(id = this.id) {
    let state = null;
    const store = this.getData(id);

    if (store) {
      if (typeof store.id !== 'undefined') {
        state = id === store.id;
      } else if (store.expanded !== 'undefined') {
        state = isTrue(store.expanded);
      }
    }

    return state;
  }

  flush(id = this.id) {
    if (id) {
      try {
        const storeId = this.storeId(id);

        if (storeId) {
          window.localStorage.setItem(storeId, null);
        }
      } catch (e) {}
    }
  }

}