import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import { isTrue, validateDOMAttributes, processChildren, getPreviousSibling, AnimateHeight } from '../../shared/component-helper';
import classnames from 'classnames';
import AccordionContext from './AccordionContext';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
export default class AccordionContent extends React.PureComponent {
  static getContent(props) {
    return processChildren(props);
  }

  constructor(props, context) {
    super(props);
    this._ref = React.createRef();
    this.state = {
      isInitial: !context.expanded,
      isAnimating: false,
      keepContentInDom: null
    };
    this.anim = new AnimateHeight();
    this.anim.onStart(() => {
      this.setState({
        isAnimating: true
      });
    });
    this.anim.onEnd(() => {
      this.setState({
        isAnimating: false
      });
      this.setState({
        keepContentInDom: this.context.expanded
      });
    });

    if (props.instance && Object.prototype.hasOwnProperty.call(props.instance, 'current')) {
      props.instance.current = this;
    }
  }

  componentDidMount() {
    this.anim.setElem(this._ref.current, getPreviousSibling('dnb-accordion-group--single-container', this._ref.current));
  }

  componentWillUnmount() {
    this.anim.remove();
  }

  componentDidUpdate(prevProps) {
    const {
      expanded,
      single_container
    } = this.context;

    if (expanded !== this.state._expanded) {
      const isInitial = !expanded && this.state.isInitial;
      this.setState({
        _expanded: expanded,
        isInitial: false,
        keepContentInDom: true
      }, () => {
        if (expanded) {
          this.anim.open({
            animate: !isInitial
          });
        } else {
          this.anim.close({
            animate: !isInitial
          });
        }
      });
    }

    if (isTrue(single_container) && AccordionContent.getContent(prevProps) !== AccordionContent.getContent(this.props)) {
      this.anim.setContainerHeight();
    }
  }

  setContainerHeight() {
    var _this$anim;

    (_this$anim = this.anim) === null || _this$anim === void 0 ? void 0 : _this$anim.setContainerHeight();
  }

  renderContent() {
    const children = AccordionContent.getContent(this.props);
    const {
      expanded,
      prerender,
      prevent_rerender,
      prevent_rerender_conditional
    } = this.context;
    let content = children;

    if (typeof content === 'string') {
      content = React.createElement("p", {
        className: "dnb-p"
      }, content);
    }

    content = expanded || prerender || this.state.keepContentInDom || this.state.isAnimating ? children : null;

    if (isTrue(prevent_rerender)) {
      if (isTrue(prevent_rerender_conditional) && this._cache !== content) {
        this._cache = content;
      }

      if (this._cache) {
        content = this._cache;
      } else {
        this._cache = content;
      }
    }

    return content;
  }

  render() {
    const _this$props = this.props,
          {
      className
    } = _this$props,
          rest = _objectWithoutPropertiesLoose(_this$props, ["className", "instance"]);

    const {
      keepContentInDom,
      isAnimating
    } = this.state;
    const {
      id,
      expanded,
      disabled
    } = this.context;
    const content = this.renderContent();

    const wrapperParams = _extends({
      className: classnames('dnb-accordion__content', className, !expanded && !keepContentInDom && 'dnb-accordion__content--hidden', isAnimating && 'dnb-accordion__content--is-animating')
    }, rest);

    const innerParams = {
      id: `${id}-content`,
      role: 'region',
      'aria-labelledby': `${id}-header`,
      className: classnames('dnb-accordion__content__inner', createSpacingClasses(rest), keepContentInDom === false && 'dnb-accordion__content__inner--remove-content')
    };

    if (expanded) {
      innerParams['aria-expanded'] = true;
    }

    if (!expanded || disabled) {
      innerParams.disabled = true;
      innerParams['aria-hidden'] = true;
    }

    validateDOMAttributes(this.props, wrapperParams);
    validateDOMAttributes(null, innerParams);
    return React.createElement("div", _extends({}, wrapperParams, {
      ref: this._ref
    }), React.createElement("div", innerParams, content));
  }

}
AccordionContent.contextType = AccordionContext;
AccordionContent.defaultProps = {
  instance: null,
  className: null,
  children: null
};
process.env.NODE_ENV !== "production" ? AccordionContent.propTypes = _extends({
  instance: PropTypes.object
}, spacingPropTypes, {
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.func])
}) : void 0;