import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import { isTrue, validateDOMAttributes, extendPropsWithContext } from '../../shared/component-helper';
import IconPrimary from '../../components/icon-primary/IconPrimary';
import classnames from 'classnames';
import keycode from 'keycode';
import AccordionContext from './AccordionContext';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import { skeletonDOMAttributes, createSkeletonClass } from '../skeleton/SkeletonHelper';

function AccordionHeaderTitle(_ref) {
  let {
    children
  } = _ref,
      rest = _objectWithoutPropertiesLoose(_ref, ["children"]);

  return React.createElement("span", {
    className: classnames('dnb-accordion__header__title', createSpacingClasses(rest))
  }, children);
}

process.env.NODE_ENV !== "production" ? AccordionHeaderTitle.propTypes = _extends({}, spacingPropTypes, {
  children: PropTypes.node
}) : void 0;
AccordionHeaderTitle.defaultProps = {
  children: null
};

function AccordionHeaderDescription(_ref2) {
  let {
    children
  } = _ref2,
      rest = _objectWithoutPropertiesLoose(_ref2, ["children"]);

  return children ? React.createElement("span", {
    className: classnames('dnb-accordion__header__description', createSpacingClasses(rest))
  }, children) : null;
}

process.env.NODE_ENV !== "production" ? AccordionHeaderDescription.propTypes = _extends({}, spacingPropTypes, {
  children: PropTypes.node
}) : void 0;
AccordionHeaderDescription.defaultProps = {
  children: null
};

function AccordionHeaderContainer(_ref3) {
  let {
    children
  } = _ref3,
      rest = _objectWithoutPropertiesLoose(_ref3, ["children"]);

  return children ? React.createElement("span", {
    className: classnames('dnb-accordion__header__container', createSpacingClasses(rest))
  }, children) : null;
}

process.env.NODE_ENV !== "production" ? AccordionHeaderContainer.propTypes = _extends({}, spacingPropTypes, {
  children: PropTypes.node
}) : void 0;
AccordionHeaderContainer.defaultProps = {
  children: null
};

function AccordionHeaderIcon(_ref4) {
  let {
    icon
  } = _ref4,
      rest = _objectWithoutPropertiesLoose(_ref4, ["icon"]);

  return React.createElement("span", {
    className: "dnb-accordion__header__icon"
  }, React.createElement(IconPrimary, _extends({}, rest, {
    icon: icon || 'chevron-down',
    "aria-hidden": true
  })));
}

process.env.NODE_ENV !== "production" ? AccordionHeaderIcon.propTypes = {
  icon: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.func]),
  size: PropTypes.string
} : void 0;
AccordionHeaderIcon.defaultProps = {
  icon: null,
  size: 'medium'
};
export default class AccordionHeader extends React.PureComponent {
  constructor(props) {
    super(props);

    this.onKeyDownHandler = event => {
      switch (keycode(event)) {
        case 'enter':
        case 'space':
          event.preventDefault();
          this.onClickHandler(event);
          break;
      }
    };

    this.onClickHandler = event => {
      const {
        id,
        group
      } = this.context;

      if (this.canClick()) {
        const expanded = !this.context.expanded;
        this.context.callOnChange({
          id,
          group,
          expanded,
          event
        });
        this.setState({
          hadClick: true
        });
      }
    };

    this.onMouseOverHandler = () => {
      this.setState({
        hover: true
      });
    };

    this.onMouseOutHander = () => {
      this.setState({
        hover: false,
        hadClick: false
      });
    };

    this.state = {};
  }

  canClick() {
    const {
      expanded,
      allow_close_all,
      group
    } = this.context;
    return !group || group && !expanded || isTrue(allow_close_all);
  }

  render() {
    const props = extendPropsWithContext(this.props, AccordionHeader.defaultProps, this.context);
    const {
      id,
      left_component,
      expanded,
      title,
      description,
      element,
      heading,
      heading_level,
      icon,
      icon_size,
      disabled,
      skeleton
    } = props;

    const _this$props = this.props,
          {
      children,
      className
    } = _this$props,
          rest = _objectWithoutPropertiesLoose(_this$props, ["children", "className", "left_component", "expanded", "title", "description", "icon", "icon_size", "disabled"]);

    const {
      hover,
      hadClick
    } = this.state;
    let {
      icon_position
    } = props;
    const defaultParts = [React.createElement(AccordionHeaderIcon, {
      key: "icon",
      icon: icon,
      size: icon_size
    }), React.createElement(AccordionHeaderContainer, {
      key: "container"
    }, left_component), React.createElement(AccordionHeaderTitle, {
      key: "title"
    }, title || (Array.isArray(children) ? children.filter(cur => !React.isValidElement(cur)) : children)), React.createElement(AccordionHeaderDescription, {
      key: "description"
    }, description)];

    if (Array.isArray(children)) {
      const removeParts = [];
      children.forEach(cur => {
        if (React.isValidElement(cur)) {
          const part = defaultParts.find(c => c.type === cur.type);

          if (part) {
            removeParts.push(part);
          }

          defaultParts.push(cur);
        }
      });
      removeParts.forEach(part => {
        const index = defaultParts.findIndex(c => c === part);

        if (index > -1) {
          defaultParts.splice(index, 1);
        }
      });
    }

    const partsToRender = [];
    const wrapperParts = [];
    const wrapperComp = React.createElement("span", {
      className: "dnb-accordion__header__wrapper",
      key: "wrapper"
    }, wrapperParts);
    defaultParts.forEach(part => {
      if (React.isValidElement(part) && (part.type === AccordionHeaderTitle || part.type === AccordionHeaderDescription)) {
        wrapperParts.push(part);

        if (partsToRender.findIndex(c => c === wrapperComp) === -1) {
          partsToRender.push(wrapperComp);
        }
      } else {
        partsToRender.push(part);
      }
    });

    if (icon_position === null) {
      const iconIndex = partsToRender.findIndex(c => c.type === AccordionHeaderIcon);

      if (iconIndex > 1) {
        icon_position = 'right';
      }

      if (left_component) {
        icon_position = 'right';
      }
    }

    const headerParams = _extends({
      id: `${id}-header`,
      'aria-controls': `${id}-content`,
      'aria-expanded': this.context.expanded,
      role: 'button',
      tabIndex: '0',
      className: classnames('dnb-accordion__header', createSkeletonClass('font', skeleton, this.context), createSpacingClasses(rest), className, icon_position && `dnb-accordion__header__icon--${icon_position}`, hover && hadClick && 'dnb-accordion--hover', !this.canClick() && 'dnb-accordion__header--prevent-click', description && 'dnb-accordion__header--description'),
      disabled
    }, rest);

    if (disabled || isTrue(skeleton)) {
      headerParams.tabIndex = '-1';
      headerParams.disabled = true;
      headerParams['aria-disabled'] = true;
    } else {
      headerParams.onClick = this.onClickHandler;
      headerParams.onKeyDown = this.onKeyDownHandler;
      headerParams.onMouseOver = this.onMouseOverHandler;
      headerParams.onMouseOut = this.onMouseOutHander;
    }

    skeletonDOMAttributes(headerParams, skeleton, this.context);
    validateDOMAttributes(this.props, headerParams);
    let Element = 'div';

    if (isTrue(heading)) {
      headerParams.role = 'heading';
      headerParams['aria-level'] = heading_level ? String(heading_level) : 'h2';
    } else if (heading) {
      headerParams.role = null;
      Element = heading;
    } else if (element) {
      headerParams.role = null;
      Element = element;
    }

    return React.createElement(Element, headerParams, partsToRender);
  }

}
AccordionHeader.contextType = AccordionContext;
AccordionHeader.defaultProps = {
  id: null,
  title: null,
  description: null,
  left_component: null,
  element: null,
  heading: null,
  heading_level: null,
  icon: null,
  icon_position: null,
  icon_size: 'medium',
  disabled: null,
  skeleton: null,
  className: null,
  children: null
};
AccordionHeader.Container = AccordionHeaderContainer;
AccordionHeader.Icon = AccordionHeaderIcon;
AccordionHeader.Title = AccordionHeaderTitle;
AccordionHeader.Description = AccordionHeaderDescription;
process.env.NODE_ENV !== "production" ? AccordionHeader.propTypes = _extends({
  title: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.func]),
  description: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.func]),
  left_component: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.func]),
  element: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.func]),
  heading: PropTypes.oneOfType([PropTypes.bool, PropTypes.string, PropTypes.node, PropTypes.func]),
  heading_level: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  icon: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.func]),
  icon_position: PropTypes.oneOf(['left', 'right']),
  icon_size: PropTypes.string,
  disabled: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool])
}, spacingPropTypes, {
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.func])
}) : void 0;