import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { isTrue, makeUniqueId, extendPropsWithContext, registerElement, validateDOMAttributes, dispatchCustomElementEvent } from '../../shared/component-helper';
import { createSpacingClasses } from '../space/SpacingHelper';
import { accordionPropTypes, accordionDefaultProps } from './AccordionPropTypes';
import Context from '../../shared/Context';
import AccordionGroupContext from './AccordionProviderContext';
export default class AccordionGroup extends React.PureComponent {
  static enableWebComponent() {
    registerElement(AccordionGroup.tagName, AccordionGroup, AccordionGroup.defaultProps);
  }

  constructor(props) {
    super(props);

    this.onChangeHandler = event => {
      dispatchCustomElementEvent(this, 'on_change', {
        id: event.id,
        expanded: event.expanded,
        event
      });
    };

    this._id = props.id || makeUniqueId();
    this.state = {
      _listenForPropChanges: true
    };
  }

  render() {
    const props = extendPropsWithContext(this.props, AccordionGroup.defaultProps, this.context.FormRow, this.context.Accordion, this.context.getTranslation(this.props).Accordion);

    const {
      single_container,
      className,
      class: _className,
      children
    } = props,
          rest = _objectWithoutPropertiesLoose(props, ["expanded", "expanded_id", "prerender", "prevent_rerender", "single_container", "allow_close_all", "remember_state", "flush_remembered_state", "disabled", "group", "onInit", "className", "class", "id", "children"]);

    const id = this._id;
    const classes = classnames('dnb-accordion-group', createSpacingClasses(props), className, _className, isTrue(single_container) && 'dnb-accordion-group--single-container');

    const params = _extends({}, rest);

    validateDOMAttributes(this.props, params);

    if (!props.group && isTrue(props.single_container)) {
      props.group = makeUniqueId();
    }

    const context = _extends({}, props, {
      id,
      onChange: this.onChangeHandler
    });

    return React.createElement(AccordionGroupContext.Provider, {
      value: context
    }, React.createElement("div", {
      className: classes
    }, React.createElement("span", _extends({
      id: id,
      className: "dnb-accordion-group__shell",
      role: "group"
    }, params), React.createElement("span", {
      className: "dnb-accordion-group__children"
    }, children))));
  }

}
AccordionGroup.tagName = 'dnb-accordion-group';
AccordionGroup.contextType = Context;
AccordionGroup.defaultProps = _extends({}, accordionDefaultProps, {
  expanded_id: null,
  onInit: null
});
process.env.NODE_ENV !== "production" ? AccordionGroup.propTypes = _extends({}, accordionPropTypes, {
  expanded_id: PropTypes.string,
  onInit: PropTypes.func
}) : void 0;