import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import keycode from 'keycode';
import { warn, isTrue, isTouchDevice, makeUniqueId, extendPropsWithContext, registerElement, validateDOMAttributes, dispatchCustomElementEvent, getStatusState, combineDescribedBy, convertJsxToString } from '../../shared/component-helper';
import { IS_MAC, IS_WIN, IS_IE11, IS_EDGE, debounce } from '../../shared/helpers';
import AlignmentHelper from '../../shared/AlignmentHelper';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import Suffix from '../../shared/helpers/Suffix';
import FormLabel from '../form-label/FormLabel';
import FormStatus from '../form-status/FormStatus';
import IconPrimary from '../icon-primary/IconPrimary';
import Input, { SubmitButton } from '../input/Input';
import ProgressIndicator from '../progress-indicator/ProgressIndicator';
import DrawerList from '../../fragments/drawer-list/DrawerList';
import DrawerListContext from '../../fragments/drawer-list/DrawerListContext';
import DrawerListProvider from '../../fragments/drawer-list/DrawerListProvider';
import { parseContentTitle, getCurrentData } from '../../fragments/drawer-list/DrawerListHelpers';
export default class Autocomplete extends React.PureComponent {
  static enableWebComponent() {
    registerElement(Autocomplete.tagName, Autocomplete, Autocomplete.defaultProps);
  }

  render() {
    return React.createElement(DrawerListProvider, _extends({}, this.props, {
      data: this.props.data || this.props.children,
      opened: null,
      tagName: "dnb-autocomplete",
      ignore_events: false,
      prevent_focus: true,
      skip_keysearch: true
    }), React.createElement(AutocompleteInstance, this.props));
  }

}
Autocomplete.tagName = 'dnb-autocomplete';
Autocomplete.defaultProps = {
  id: null,
  mode: 'sync',
  title: 'Option Menu',
  placeholder: null,
  no_options: null,
  show_all: null,
  aria_live_options: null,
  indicator_label: null,
  submit_button_title: null,
  submit_button_icon: 'chevron_down',
  input_ref: null,
  icon_size: null,
  icon_position: 'left',
  triangle_position: null,
  input_icon: 'loupe',
  label: null,
  label_direction: null,
  label_sr_only: null,
  keep_value: null,
  keep_value_and_selection: null,
  status: null,
  status_state: 'error',
  status_animation: null,
  global_status_id: null,
  suffix: null,
  disable_filter: false,
  disable_reorder: false,
  scrollable: true,
  focusable: false,
  disable_highlighting: false,
  max_height: null,
  direction: 'auto',
  skip_portal: null,
  no_animation: false,
  no_scroll_animation: false,
  show_submit_button: false,
  submit_element: null,
  prevent_selection: false,
  size: 'default',
  align_autocomplete: null,
  options_render: null,
  data: null,
  search_in_word_index: 3,
  default_value: null,
  value: 'initval',
  input_value: 'initval',
  icon: null,
  open_on_focus: false,
  prevent_close: false,
  keep_open: false,
  opened: null,
  disabled: null,
  stretch: null,
  skeleton: null,
  drawer_class: null,
  class: null,
  className: null,
  children: null,
  custom_element: null,
  custom_method: null,
  on_show: null,
  on_hide: null,
  on_type: null,
  on_focus: null,
  on_blur: null,
  on_change: null,
  on_select: null,
  on_state_update: null,
  input_component: null
};
process.env.NODE_ENV !== "production" ? Autocomplete.propTypes = _extends({
  id: PropTypes.string,
  mode: PropTypes.oneOf(['sync', 'async']),
  title: PropTypes.oneOfType([PropTypes.string, PropTypes.node]),
  placeholder: PropTypes.oneOfType([PropTypes.string, PropTypes.node]),
  no_options: PropTypes.oneOfType([PropTypes.string, PropTypes.node]),
  show_all: PropTypes.oneOfType([PropTypes.string, PropTypes.node]),
  aria_live_options: PropTypes.oneOfType([PropTypes.string, PropTypes.node]),
  indicator_label: PropTypes.oneOfType([PropTypes.string, PropTypes.node]),
  submit_button_title: PropTypes.string,
  submit_button_icon: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.func]),
  input_ref: PropTypes.oneOfType([PropTypes.func, PropTypes.object]),
  icon_size: PropTypes.string,
  icon_position: PropTypes.oneOf(['left', 'right']),
  triangle_position: PropTypes.oneOf(['left', 'right']),
  input_icon: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.func]),
  label: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  label_direction: PropTypes.oneOf(['horizontal', 'vertical']),
  label_sr_only: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  keep_value: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  keep_value_and_selection: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  status: PropTypes.oneOfType([PropTypes.string, PropTypes.bool, PropTypes.func, PropTypes.node]),
  status_state: PropTypes.string,
  status_animation: PropTypes.string,
  global_status_id: PropTypes.string,
  suffix: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  disable_filter: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  disable_reorder: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  scrollable: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  focusable: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  disable_highlighting: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  direction: PropTypes.oneOf(['auto', 'top', 'bottom']),
  max_height: PropTypes.number,
  skip_portal: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  no_animation: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  no_scroll_animation: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  show_submit_button: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  submit_element: PropTypes.node,
  prevent_selection: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  size: PropTypes.oneOf(['default', 'small', 'medium', 'large']),
  align_autocomplete: PropTypes.oneOf(['left', 'right']),
  options_render: PropTypes.oneOfType([PropTypes.object, PropTypes.func, PropTypes.node]),
  input_component: PropTypes.oneOfType([PropTypes.func, PropTypes.node]),
  data: PropTypes.oneOfType([PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node, PropTypes.object]), PropTypes.arrayOf(PropTypes.oneOfType([PropTypes.oneOfType([PropTypes.string, PropTypes.node]), PropTypes.shape({
    selected_value: PropTypes.oneOfType([PropTypes.string, PropTypes.node]),
    content: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.arrayOf(PropTypes.string)])
  })]))]),
  search_in_word_index: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  default_value: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  value: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  input_value: PropTypes.string,
  icon: PropTypes.oneOfType([PropTypes.string, PropTypes.node]),
  open_on_focus: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  prevent_close: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  keep_open: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  opened: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  disabled: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  stretch: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  drawer_class: PropTypes.string
}, spacingPropTypes, {
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node, PropTypes.object, PropTypes.array]),
  custom_element: PropTypes.object,
  custom_method: PropTypes.func,
  on_show: PropTypes.func,
  on_type: PropTypes.func,
  on_focus: PropTypes.func,
  on_blur: PropTypes.func,
  on_hide: PropTypes.func,
  on_change: PropTypes.func,
  on_select: PropTypes.func,
  on_state_update: PropTypes.func
}) : void 0;

var _ref2 = React.createElement(IconPrimary, {
  icon: "arrow_down"
});

var _ref3 = React.createElement(AlignmentHelper, null);

class AutocompleteInstance extends React.PureComponent {
  static parseDataItem(dataItem) {
    const toParse = parseContentTitle(dataItem, {
      separator: ' '
    });

    if (typeof toParse !== 'string' && Array.isArray(toParse)) {
      return AutocompleteInstance.parseDataItem(toParse);
    }

    return toParse;
  }

  static createSearchIndex(data) {
    return data.map(dataItem => {
      const searchChunk = AutocompleteInstance.parseDataItem(dataItem);
      return {
        dataItem,
        searchChunk
      };
    });
  }

  static getCurrentDataTitle(selected_item, data) {
    const currentData = getCurrentData(selected_item, data);
    return parseContentTitle(currentData, {
      separator: ' ',
      preferSelectedValue: true
    });
  }

  static getDerivedStateFromProps(props, state) {
    if (state._listenForPropChanges) {
      state.skipHighlight = isTrue(props.disable_highlighting);

      if (props.input_value !== 'initval' && typeof state.inputValue === 'undefined' && props.input_value && props.input_value.length > 0) {
        state.inputValue = props.input_value;
      }

      if (props.data !== state.init_data) {
        state.updateData(props.data);
        state.init_data = props.data;
      }
    }

    state._listenForPropChanges = true;
    return state;
  }

  constructor(_props, context) {
    super(_props);

    this.setVisible = () => {
      this.context.drawerList.setWrapperElement(this._refShell.current).setVisible();
    };

    this.setHidden = () => {
      this.context.drawerList.setHidden();
    };

    this.toggleVisible = ({
      hasFilter = false
    } = {}) => {
      if (!hasFilter && !isTrue(this.props.prevent_close) && !this.context.drawerList.hidden && this.context.drawerList.opened) {
        this.setHidden();
      } else {
        this.setVisibleByContext();
      }
    };

    this.setVisibleByContext = (options = {}) => {
      const skipFilter = this.state.showAllNextTime;

      if (skipFilter) {
        this.setState({
          showAllNextTime: false,
          _listenForPropChanges: false
        });
      }

      this.runFilterToHighlight(_extends({
        fillDataIfEmpty: true,
        skipFilter
      }, options));
      this.setVisible();
    };

    this.scrollToActiveItem = active_item => {
      this.context.drawerList.scrollToItem(active_item, {
        scrollTo: false
      });
    };

    this.scrollToSelectedItem = selected_item => {
      this.context.drawerList.scrollToAndSetActiveItem(selected_item, {
        scrollTo: false
      });
    };

    this.onInputChangeHandler = ({
      value,
      event
    }, options = {}) => {
      this.setState({
        typedInputValue: value,
        inputValue: value,
        _listenForPropChanges: false
      });
      dispatchCustomElementEvent(this, 'on_type', _extends({
        value,
        event
      }, this.getEventObjects('on_type')));
      value = String(value).trim();

      if (value !== this.state.inputValue) {
        this.runFilterWithSideEffects(value, options);
      }
    };

    this.runFilterWithSideEffects = (value, options = {}) => {
      const data = this.runFilter(value, options);
      const count = this.countData(data);

      if (value && value.length > 0) {
        if (count === 0) {
          if (this.state.mode !== 'async') {
            this.showNoOptionsItem();
          }
        } else if (count > 0) {
          this.context.drawerList.setData(this.wrapWithShowAll(data));
          this.context.drawerList.setState({
            cache_hash: value + count
          });

          if (count === 1) {
            this.context.drawerList.setState({
              active_item: data[0].__id
            });
          }
        }
      } else {
        this.totalReset();
        this.showAllItems();
      }

      this.setVisible();
      this.setAriaLiveUpdate();
      return data;
    };

    this.runFilterToHighlight = (_ref = {}, value = this.state.inputValue) => {
      let {
        fillDataIfEmpty = false
      } = _ref,
          options = _objectWithoutPropertiesLoose(_ref, ["fillDataIfEmpty"]);

      const possibleTitle = AutocompleteInstance.getCurrentDataTitle(this.context.drawerList.selected_item, this.context.drawerList.original_data);

      if (value === possibleTitle) {
        return;
      }

      value = String(value || '').trim();
      this.setState({
        skipHighlight: false,
        _listenForPropChanges: false
      });
      let data = this.runFilter(value, options);

      if (fillDataIfEmpty && data.length === 0 && value === '') {
        data = this.context.drawerList.original_data;
      }

      this.context.drawerList.setData(this.wrapWithShowAll(data));
      this.context.drawerList.setState({
        cache_hash: value + this.countData(data)
      });
      this.setAriaLiveUpdate();
      return data;
    };

    this.wrapWithShowAll = data => {
      if (!this.hasFilterActive(data)) {
        return data;
      }

      const lastItem = data.slice(-1)[0];

      if (lastItem && !lastItem.show_all) {
        const {
          show_all
        } = this._props;
        data.push({
          __id: lastItem.__id + 1,
          class_name: 'dnb-autocomplete__show-all',
          show_all: true,
          active_item: false,
          selected_item: false,
          content: React.createElement(React.Fragment, null, _ref2, show_all)
        });
      }

      return data;
    };

    this.setInputValue = inputValue => {
      this.setState({
        inputValue,
        _listenForPropChanges: false
      });
    };

    this.emptyData = () => {
      this._rC = {};
      this.setState({
        inputValue: '',
        typedInputValue: null,
        _listenForPropChanges: false
      });
      this.context.drawerList.setData(() => [], () => {
        this.setSearchIndex({
          overwriteSearchIndex: true
        });
        this.resetSelections();
        this.totalReset();
      }, {
        overwriteOriginalData: true
      });
    };

    this.showNoOptionsItem = () => {
      this.resetSelections();
      this.ignoreEvents();
      this.context.drawerList.setData([{
        class_name: 'dnb-autocomplete__no-options',
        content: this._props.no_options,
        ignore_events: true,
        __id: 'no_options'
      }]);
      this.context.drawerList.setState({
        cache_hash: 'no_options'
      });
      this.setVisible();
    };

    this.showIndicatorItem = () => {
      this.resetSelections();
      this.ignoreEvents();
      this.context.drawerList.setData([{
        class_name: 'dnb-autocomplete__indicator',
        content: React.createElement(ProgressIndicator, {
          label: this._props.indicator_label
        }),
        ignore_events: true,
        __id: 'indicator'
      }]);
      this.context.drawerList.setState({
        cache_hash: 'indicator'
      });
      this.setVisible();
    };

    this.showIndicator = () => {
      if (!this.state.visibleIndicator) {
        this.setState({
          visibleIndicator: true,
          _listenForPropChanges: false
        });
      }
    };

    this.hideIndicator = () => {
      this.setState({
        visibleIndicator: false,
        _listenForPropChanges: false
      });
    };

    this.setMode = mode => {
      this.setState({
        mode,
        _listenForPropChanges: false
      });
    };

    this.updateData = rawData => {
      this.context.drawerList.setState({
        cache_hash: 'updateData'
      });
      this.context.drawerList.setData(() => rawData, newData => {
        this.setSearchIndex({
          overwriteSearchIndex: true,
          data: newData
        }, () => {
          const {
            typedInputValue
          } = this.state;

          if (typedInputValue && typedInputValue.length > 0) {
            const filteredData = this.runFilterWithSideEffects(typedInputValue);

            if (this.countData(filteredData) === 0) {
              this.showNoOptionsItem();
            }
          } else {
            this.resetSelections();

            if (this.context.drawerList.opened) {
              this.showAllItems();
            }
          }
        });
      }, {
        overwriteOriginalData: true
      });
      return this;
    };

    this.onInputKeyDownHandler = ({
      event: e
    }) => {
      const key = keycode(e);

      switch (key) {
        case 'page up':
        case 'page down':
        case 'home':
        case 'end':
        case 'down':
        case 'up':
          e.preventDefault();
          break;
      }
    };

    this.onInputClickHandler = e => {
      const {
        value
      } = e.target;
      this.setVisibleByContext({
        value
      });
    };

    this.onInputFocusHandler = event => {
      if (this.state.skipFocusDuringChange) {
        return;
      }

      const {
        open_on_focus,
        keep_value_and_selection
      } = this.props;

      if (!this.state.hasFocus) {
        if (isTrue(open_on_focus)) {
          const {
            value
          } = event.target;
          this.setVisibleByContext({
            value
          });
        } else {
          this.setSearchIndex();
        }

        if (isTrue(keep_value_and_selection)) {
          this.showAll();
        }

        dispatchCustomElementEvent(this, 'on_focus', _extends({
          event
        }, this.getEventObjects('on_focus')));
        this.setState({
          hasFocus: true,
          hasBlur: false
        });
      }
    };

    this.onBlurHandler = event => {
      const {
        input_value,
        open_on_focus,
        keep_value,
        keep_value_and_selection,
        prevent_selection
      } = this.props;

      if (!isTrue(keep_value_and_selection)) {
        this.setState({
          typedInputValue: null,
          _listenForPropChanges: false
        });
      }

      if (isTrue(open_on_focus)) {
        this.setHidden();
      }

      if (!isTrue(prevent_selection) && !isTrue(keep_value_and_selection)) {
        const inputValue = AutocompleteInstance.getCurrentDataTitle(this.context.drawerList.selected_item, this.context.drawerList.original_data);
        clearTimeout(this._selectTimeout);
        this._selectTimeout = setTimeout(() => {
          if (parseFloat(this.context.drawerList.selected_item) > -1) {
            this.setState({
              inputValue,
              _listenForPropChanges: false
            });
          } else if (!(input_value !== 'initval' && input_value.length > 0) && !isTrue(keep_value)) {
            this.setState({
              inputValue: '',
              _listenForPropChanges: false
            });
          }
        }, 1);
      }

      if (!this.state.hasBlur) {
        dispatchCustomElementEvent(this, 'on_blur', _extends({
          event
        }, this.getEventObjects('on_blur')));
        this.setState({
          hasBlur: true,
          hasFocus: false
        });
      }
    };

    this.onTriggerKeyDownHandler = e => {
      const key = keycode(e);

      switch (key) {
        case 'space':
          {
            this.setVisible();
          }
          break;
      }

      switch (key) {
        case 'space':
        case 'enter':
        case 'page up':
        case 'page down':
        case 'down':
        case 'up':
          {
            e.preventDefault();

            try {
              this._refInput.current._ref.current.focus();
            } catch (e) {
              warn(e);
            }
          }
          break;
      }
    };

    this.onSubmit = () => {
      this.toggleVisible();
    };

    this.onShellKeyDownHandler = e => {
      const key = keycode(e);

      switch (key) {
        case 'up':
        case 'down':
          if (!this.context.drawerList.opened) {
            e.preventDefault();
            this.setVisible();
          }

          break;

        case 'esc':
          this.setState({
            showAllNextTime: true,
            _listenForPropChanges: false
          });
          break;

        case 'enter':
          e.preventDefault();

          if (!this.context.drawerList.opened && this.hasFilterActive()) {
            this.ignoreEvents();
            this.showAll();
          }

          if ((!this.hasValidData() || !this.hasSelectedItem()) && !this.hasActiveItem()) {
            clearTimeout(this._toggleVisibleTimeout);
            this._toggleVisibleTimeout = setTimeout(this.toggleVisible, 1);
          } else {
            this.setVisible();
          }

          break;
      }
    };

    this.getEventObjects = key => {
      const attributes = this.attributes;
      return {
        attributes,
        dataList: this.context.drawerList.data,
        updateData: this.updateData,
        showAllItems: this.showAllItems,
        setVisible: this.setVisible,
        setHidden: this.setHidden,
        emptyData: this.emptyData,
        setInputValue: this.setInputValue,
        showNoOptionsItem: this.showNoOptionsItem,
        showIndicatorItem: this.showIndicatorItem,
        showIndicator: this.showIndicator,
        hideIndicator: this.hideIndicator,
        setMode: this.setMode,
        debounce: (func, props = {}, wait = 250) => {
          this.dbf = this.dbf || {};
          return (this.dbf[key] || (this.dbf[key] = debounce(func, wait, {
            context: this
          })))(props);
        }
      };
    };

    this.hasShowMore = (data = this.context.drawerList.data) => {
      const lastItem = data.slice(-1)[0];
      return lastItem && lastItem.show_all === true;
    };

    this.countData = (data = this.context.drawerList.data) => {
      const count = data.length;
      return count > 0 && this.hasShowMore(data) ? count - 1 : count;
    };

    this.hasValidData = (data = this.context.drawerList.data) => {
      if (this.countData(data) > 0) {
        const first = data[0];

        if (!first.show_all && !['no_options', 'indicator'].includes(first.__id)) {
          return true;
        }
      }

      return false;
    };

    this.hasSelectedItem = () => {
      return parseFloat(this.context.drawerList.selected_item) > -1;
    };

    this.hasActiveItem = () => {
      return parseFloat(this.context.drawerList.active_item) > -1;
    };

    this.hasFilterActive = (data = this.context.drawerList.data) => {
      return !(this.context.drawerList.original_data && this.context.drawerList.original_data.length === this.countData(data));
    };

    this.ignoreEvents = () => {
      clearTimeout(this.showAllTimeout);
      this.context.drawerList.setState({
        ignore_events: true
      }, () => {
        this.showAllTimeout = setTimeout(() => {
          this.context && this.context.drawerList && this.context.drawerList.setState({
            ignore_events: false
          });
        }, 10);
      });
    };

    this.showAll = () => {
      this.resetFilter();
      this.context.drawerList.setState({
        cache_hash: 'all'
      });
      this.runFilterToHighlight({
        skipFilter: true,
        fillDataIfEmpty: true
      });
    };

    this.showAllItems = () => {
      this.resetFilter();
      this.context.drawerList.setState({
        cache_hash: 'all'
      });
      this.scrollToSelectedItem(this.context.drawerList.selected_item);
    };

    this.totalReset = () => {
      if (!isTrue(this.props.keep_value) && !isTrue(this.props.keep_value_and_selection)) {
        this.setState({
          inputValue: null
        });
      }

      this.setState({
        typedInputValue: null,
        _listenForPropChanges: false
      });
      this.context.drawerList.setState({
        selected_item: null
      });
    };

    this.resetSelections = () => {
      this.context.drawerList.setState({
        active_item: null
      });
    };

    this.resetFilter = () => {
      this.context.drawerList.setData(this.context.drawerList.original_data);
    };

    this.runFilter = (value, {
      data = null,
      searchIndex = this.state.searchIndex,
      skipHighlight = false,
      skipFilter = false,
      skipReorder = false
    } = {}) => {
      const words = value.split(/\s+/g).filter(Boolean);
      const wordsCount = words.length;
      const wordCond = '^|\\s';

      const findWords = item => words.map((word, wordIndex) => ({
        word,
        wordIndex,
        score: wordsCount - wordIndex
      })).filter(({
        word,
        wordIndex
      }) => typeof item === 'string' && new RegExp(wordIndex > this.inWordIndex ? `${word}` : `(${wordCond})${word}`, 'i').test(item));

      if (data) {
        searchIndex = this.setSearchIndex({
          data
        });
      } else if (!searchIndex) {
          searchIndex = this.setSearchIndex();
        }

      if (typeof searchIndex === 'undefined') {
        return [];
      }

      const S = '\uFFFE';
      const E = '\uFFFF';
      const tagS = '<span class="dnb-drawer-list__option__item--highlight">';
      const tagE = '</span>';
      searchIndex = searchIndex.map(item => {
        const listOfFoundWords = findWords(item.searchChunk);

        if (typeof item.dataItem === 'string') {
          item.dataItem = {
            content: item.dataItem
          };
        }

        if (!item.dataItem.render) {
          item.dataItem = _extends({}, item.dataItem);
        }

        item.dataItem.render = (children, id) => {
          const cacheHash = id + value;
          this._rC = this._rC || {};

          if (this._rC[cacheHash]) {
            return this._rC[cacheHash];
          }

          let Component = null;

          if (typeof children !== 'string') {
            if (!Array.isArray(children)) {
              children = [children];
            }

            Component = children;
            children = children.map(child => convertJsxToString(child));
          }

          if (typeof children === 'string') {
            children = [children];
          }

          children = children.map((segment, sIndex) => {
            if (skipHighlight || this.state.skipHighlight) {
              return segment;
            }

            listOfFoundWords.forEach(({
              word,
              wordIndex
            }) => {
              if (wordIndex > this.inWordIndex) {
                segment = segment.replace(new RegExp(`(${word})`, 'gi'), `${S}$1${E}`);
              } else {
                segment = segment.replace(new RegExp(`(${wordCond})(${word})`, 'gi'), `$1${S}$2${E}`);
              }
            });

            if (segment.includes(S)) {
              const __html = segment.replace(new RegExp(`(${S})+`, 'g'), S).replace(new RegExp(`(${E})+`, 'g'), E).replace(new RegExp(`(${E}${S})`, 'g'), '').replace(new RegExp(S, 'g'), tagS).replace(new RegExp(E, 'g'), tagE);

              return React.createElement("span", {
                key: cacheHash + sIndex,
                dangerouslySetInnerHTML: {
                  __html
                }
              });
            }

            return segment;
          }).map((c, i, a) => i < a.length - 1 ? [c, ' '] : c);

          if (Component) {
            children = Array.isArray(Component) ? Component.map((Comp, i) => React.cloneElement(Comp, {
              key: 'clone' + cacheHash + i
            }, children[i])) : React.cloneElement(Component, {
              key: 'clone' + cacheHash
            }, children);
          }

          return this._rC[cacheHash] = children;
        };

        const totalScore = listOfFoundWords.reduce((acc, {
          score
        }) => acc += score, 0);

        if (this.skipFilter || skipFilter) {
          return item.dataItem;
        }

        return {
          countFindings: listOfFoundWords.length + totalScore,
          item
        };
      });

      if (!this.skipFilter && !skipFilter) {
        searchIndex = searchIndex.filter(({
          countFindings
        }) => countFindings);

        if (!this.skipReorder && !skipReorder) {
          searchIndex = searchIndex.sort(({
            countFindings: a
          }, {
            countFindings: b
          }) => b - a);
        }

        searchIndex = searchIndex.map(({
          item
        }) => item.dataItem);
      }

      return searchIndex;
    };

    this.onHideHandler = (args = {}) => {
      const res = dispatchCustomElementEvent(this, 'on_hide', _extends({}, args, this.getEventObjects('on_hide')));

      if (res !== false) {
        this.setFocusOnInput();
      }

      return res;
    };

    this.onSelectHandler = args => {
      if (parseFloat(args.active_item) > -1) {
        dispatchCustomElementEvent(this, 'on_select', _extends({}, args, this.getEventObjects('on_select')));
      }
    };

    this.onPreChangeHandler = ({
      data,
      selected_item
    }) => {
      if (data && data.show_all) {
        this.showAll();

        if (parseFloat(selected_item) > -1) {
          const active_item = selected_item - 1;
          this.context.drawerList.setState({
            active_item
          });
          this.scrollToActiveItem(active_item);
        }

        return false;
      }
    };

    this.onChangeHandler = args => {
      const selected_item = args.selected_item;
      const {
        prevent_selection,
        keep_open
      } = this.props;

      if (!isTrue(prevent_selection)) {
        if (!isTrue(keep_open)) {
          this.setState({
            skipFocusDuringChange: true,
            skipHighlight: true,
            _listenForPropChanges: false
          });
          this.setHidden();

          try {
            this.context.drawerList._refUl.current.focus({
              preventScroll: true
            });
          } catch (e) {}

          clearTimeout(this._selectTimeout);
          this._selectTimeout = setTimeout(() => {
            this.setState({
              inputValue: AutocompleteInstance.getCurrentDataTitle(selected_item, this.context.drawerList.data),
              skipFocusDuringChange: false,
              _listenForPropChanges: false
            });
            this.setFocusOnInput();
          }, 200);
        } else {
          this.setState({
            inputValue: AutocompleteInstance.getCurrentDataTitle(selected_item, this.context.drawerList.data),
            _listenForPropChanges: false
          });
        }
      }

      dispatchCustomElementEvent(this, 'on_change', _extends({}, args, this.getEventObjects('on_change')));
    };

    this._id = _props.id || makeUniqueId();
    this.attributes = {};
    this.state = this.state || {};
    this.state._listenForPropChanges = true;
    this.state.mode = _props.mode;
    this.state.init_data = _props.data;
    this.state.updateData = this.updateData;

    if (context.drawerList && context.drawerList.current_title) {
      this.state.inputValue = context.drawerList.current_title;
    }

    this._ref = React.createRef();
    this._refShell = React.createRef();
    this._refInput = React.createRef();
    this.isTouchDevice = isTouchDevice();
    this.skipFilter = isTrue(_props.disable_filter);
    this.skipReorder = isTrue(_props.disable_reorder);
    this.inWordIndex = (parseFloat(_props.search_in_word_index) || 3) - 2;
  }

  componentDidMount() {
    if (isTrue(this.props.opened)) {
      this.runFilterToHighlight({
        fillDataIfEmpty: true
      });
      this.setVisible();
    }
  }

  componentDidUpdate(prevProps) {
    if (prevProps.value !== this.props.value) {
      const inputValue = AutocompleteInstance.getCurrentDataTitle(this.context.drawerList.selected_item, this.context.drawerList.original_data);
      this.setState({
        inputValue
      });
    }
  }

  componentWillUnmount() {
    this.setHidden();
    clearTimeout(this._hideTimeout);
    clearTimeout(this._selectTimeout);
    clearTimeout(this._ariaLiveUpdateTiemout);
    clearTimeout(this._focusTimeout);
    clearTimeout(this._toggleVisibleTimeout);
  }

  setSearchIndex({
    overwriteSearchIndex = false,
    data = this.context.drawerList.original_data
  } = {}, cb) {
    this._rC = {};

    if (!overwriteSearchIndex && this.state.searchIndex) {
      return this.state.searchIndex;
    }

    const searchIndex = AutocompleteInstance.createSearchIndex(data);
    this.setState({
      searchIndex,
      _listenForPropChanges: false
    }, cb);
    return searchIndex;
  }

  setFocusOnInput() {
    this.setState({
      hasFocus: true
    }, () => {
      try {
        this._refInput.current._ref.current.focus({
          preventScroll: true
        });
      } catch (e) {}

      clearTimeout(this._focusTimeout);
      this._focusTimeout = setTimeout(() => {
        this.setState({
          hasFocus: false
        });
      }, 1);
    });
  }

  setAriaLiveUpdate() {
    const {
      opened
    } = this.context.drawerList;
    const {
      aria_live_options,
      no_options
    } = this._props;
    clearTimeout(this._ariaLiveUpdateTiemout);
    this._ariaLiveUpdateTiemout = setTimeout(() => {
      if (opened) {
        let newString = null;
        const count = this.countData();

        if (count > 0) {
          newString = String(aria_live_options).replace('%s', count);
        } else {
          newString = no_options;
        }

        if (newString) {
          this.setState({
            ariaLiveUpdate: newString,
            _listenForPropChanges: false
          });
          this._ariaLiveUpdateTiemout = setTimeout(() => {
            this.setState({
              ariaLiveUpdate: null,
              _listenForPropChanges: false
            });
          }, 1e3);
        }
      }
    }, 1e3);
  }

  render() {
    const props = this._props = extendPropsWithContext(this.props, Autocomplete.defaultProps, this.context.getTranslation(this.props).Autocomplete, this.context.FormRow, this.context.Autocomplete);

    const {
      title,
      placeholder,
      label,
      label_direction,
      label_sr_only,
      icon_size,
      input_icon,
      size,
      align_autocomplete,
      fixed_position,
      status,
      status_state,
      status_animation,
      global_status_id,
      suffix,
      scrollable,
      focusable,
      keep_open,
      prevent_close,
      no_animation,
      no_scroll_animation,
      show_submit_button,
      submit_element,
      input_component: CustomInput,
      options_render,
      prevent_selection,
      max_height,
      default_value,
      submit_button_title,
      submit_button_icon,
      drawer_class,
      input_ref,
      className,
      class: _className,
      disabled,
      stretch,
      skeleton,
      triangle_position,
      icon_position,
      skip_portal
    } = props,
          attributes = _objectWithoutPropertiesLoose(props, ["title", "placeholder", "label", "label_direction", "label_sr_only", "icon_size", "input_icon", "size", "align_autocomplete", "fixed_position", "status", "status_state", "status_animation", "global_status_id", "suffix", "scrollable", "focusable", "keep_open", "prevent_close", "no_animation", "no_scroll_animation", "show_submit_button", "submit_element", "input_component", "options_render", "prevent_selection", "max_height", "default_value", "submit_button_title", "submit_button_icon", "drawer_class", "input_ref", "className", "class", "disabled", "stretch", "skeleton", "triangle_position", "icon_position", "skip_portal", "mode", "data", "children", "direction", "id", "opened", "value", "input_value", "indicator_label", "no_options", "show_all", "aria_live_options", "disable_highlighting"]);

    const id = this._id;
    const showStatus = getStatusState(status);
    const {
      inputValue,
      visibleIndicator,
      ariaLiveUpdate
    } = this.state;
    const {
      hidden,
      selected_item,
      active_item,
      direction,
      opened
    } = this.context.drawerList;
    const isExpanded = Boolean(opened) && this.hasValidData();

    _extends(this.context.drawerList.attributes, validateDOMAttributes(null, attributes));

    const mainParams = {
      className: classnames(`dnb-autocomplete dnb-autocomplete--${direction} dnb-form-component`, createSpacingClasses(props), _className, className, opened && 'dnb-autocomplete--opened', label_direction && `dnb-autocomplete--${label_direction}`, icon_position && `dnb-autocomplete--icon-position-${icon_position}`, align_autocomplete && `dnb-autocomplete--${align_autocomplete}`, visibleIndicator && 'dnb-autocomplete--show-indicator', size && `dnb-autocomplete--${size}`, isTrue(stretch) && `dnb-autocomplete--stretch`, status && `dnb-autocomplete__status--${status_state}`, showStatus && 'dnb-autocomplete__form-status')
    };
    const shellParams = {
      className: 'dnb-autocomplete__shell dnb-no-focus',
      ref: this._refShell,
      onKeyDown: this.onShellKeyDownHandler
    };

    const inputParams = _extends({
      className: 'dnb-autocomplete__input' + (opened ? " dnb-button--active" : ""),
      id,
      value: inputValue,
      autoCapitalize: 'none',
      spellCheck: 'false',
      autoCorrect: 'off',
      autoComplete: 'off',
      role: 'combobox',
      'aria-autocomplete': 'both',
      'aria-controls': `${id}-ul`,
      'aria-haspopup': 'listbox',
      'aria-expanded': isExpanded,
      onMouseDown: this.onInputClickHandler,
      onTouchStart: this.onInputClickHandler,
      onKeyDown: this.onInputKeyDownHandler,
      onChange: this.onInputChangeHandler,
      onFocus: this.onInputFocusHandler,
      onBlur: this.onBlurHandler,
      icon_position,
      inner_ref: input_ref,
      disabled,
      skeleton
    }, attributes);

    if (!(parseFloat(selected_item) > -1)) {
      inputParams.placeholder = placeholder || title;

      if (!(IS_WIN && (IS_IE11 || IS_EDGE))) {
        inputParams['aria-placeholder'] = undefined;
      }
    }

    if (!hidden) {
      if (parseFloat(active_item) > -1) {
        inputParams['aria-activedescendant'] = `option-${id}-${active_item}`;
      } else if (!isTrue(prevent_selection) && parseFloat(selected_item) > -1) {
        inputParams['aria-activedescendant'] = `option-${id}-${selected_item}`;
      }
    }

    if (showStatus || suffix) {
      inputParams['aria-describedby'] = combineDescribedBy(inputParams, showStatus ? id + '-status' : null, suffix ? id + '-suffix' : null);
    }

    let submitButton = false;

    if (submit_element && React.isValidElement(submit_element) || isTrue(show_submit_button)) {
      const triggerParams = {
        id: id + '-submit-button',
        disabled,
        status: !opened && status ? status_state : null,
        onKeyDown: this.onTriggerKeyDownHandler,
        onSubmit: this.onSubmit,
        'aria-haspopup': 'listbox',
        'aria-expanded': isExpanded
      };

      if (submit_element && React.isValidElement(submit_element)) {
        submitButton = React.cloneElement(submit_element, triggerParams);
      } else if (isTrue(show_submit_button)) {
        submitButton = React.createElement(SubmitButton, _extends({
          icon: submit_button_icon,
          icon_size: icon_size || (size === 'large' ? 'medium' : 'default'),
          title: submit_button_title,
          variant: "secondary",
          size: size === 'default' ? 'medium' : size
        }, triggerParams));
      }
    }

    validateDOMAttributes(null, mainParams);
    validateDOMAttributes(null, shellParams);
    this.attributes = validateDOMAttributes(null, attributes);
    return React.createElement("span", mainParams, label && React.createElement(FormLabel, {
      id: id + '-label',
      for_id: id,
      text: label,
      label_direction: label_direction,
      sr_only: label_sr_only,
      disabled: disabled,
      skeleton: skeleton,
      onMouseDown: this.toggleVisible
    }), React.createElement("span", {
      className: "dnb-autocomplete__inner",
      ref: this._ref
    }, _ref3, showStatus && React.createElement(FormStatus, {
      id: id + '-form-status',
      global_status_id: global_status_id,
      label: label,
      text_id: id + '-status',
      text: status,
      status: status_state,
      animation: status_animation,
      skeleton: skeleton
    }), React.createElement("span", {
      className: "dnb-autocomplete__row"
    }, React.createElement("span", shellParams, CustomInput ? React.createElement(CustomInput, inputParams) : React.createElement(Input, _extends({
      icon: visibleIndicator ? React.createElement(ProgressIndicator, {
        size: size === 'large' ? 'medium' : 'small'
      }) : input_icon,
      icon_size: icon_size || (size === 'large' ? 'medium' : 'default'),
      size: size,
      status: !opened && status ? status_state : null,
      status_state: status_state,
      type: null,
      submit_element: submitButton,
      input_state: this.state.skipFocusDuringChange ? 'focus' : undefined,
      ref: this._refInput
    }, inputParams)), React.createElement(DrawerList, {
      id: id,
      className: classnames('dnb-autocomplete__root', drawer_class),
      inner_class: "dnb-autocomplete__list",
      value: selected_item,
      default_value: default_value,
      scrollable: scrollable,
      focusable: focusable,
      no_animation: no_animation,
      no_scroll_animation: no_scroll_animation,
      skip_portal: skip_portal,
      prevent_selection: prevent_selection,
      triangle_position: triangle_position || icon_position,
      keep_open: keep_open,
      prevent_close: prevent_close,
      align_drawer: align_autocomplete,
      fixed_position: fixed_position,
      disabled: disabled,
      max_height: max_height,
      direction: direction,
      size: size,
      options_render: options_render,
      on_change: this.onChangeHandler,
      on_select: this.onSelectHandler,
      on_hide: this.onHideHandler,
      on_pre_change: this.onPreChangeHandler
    })), suffix && React.createElement(Suffix, {
      className: "dnb-autocomplete__suffix",
      id: id + '-suffix',
      context: props
    }, suffix))), IS_MAC && React.createElement("span", {
      className: "dnb-sr-only",
      "aria-live": "assertive"
    }, AutocompleteInstance.getCurrentDataTitle(active_item, this.context.drawerList.original_data)), React.createElement("span", {
      className: "dnb-sr-only",
      "aria-live": "assertive"
    }, ariaLiveUpdate));
  }

}

AutocompleteInstance.defaultProps = Autocomplete.defaultProps;
AutocompleteInstance.contextType = DrawerListContext;
process.env.NODE_ENV !== "production" ? AutocompleteInstance.propTypes = Autocomplete.propTypes : void 0;
Autocomplete.HorizontalItem = DrawerList.HorizontalItem;