import * as React from 'react';
export type ButtonText = string | React.ReactNode;
export type ButtonVariant =
  | 'primary'
  | 'secondary'
  | 'tertiary'
  | 'signal';
export type ButtonSize = 'default' | 'small' | 'medium' | 'large';
export type ButtonIcon =
  | string
  | React.ReactNode
  | ((...args: any[]) => any);
export type ButtonIconPosition = 'left' | 'right';
export type ButtonIconSize = string | number;
export type ButtonTooltip =
  | string
  | ((...args: any[]) => any)
  | React.ReactNode;
export type ButtonStatus =
  | string
  | boolean
  | ((...args: any[]) => any)
  | React.ReactNode;
export type ButtonWrap = string | boolean;
export type ButtonBounding = string | boolean;
export type ButtonSkeleton = string | boolean;
export type ButtonDisabled = string | boolean;
export type ButtonChildren =
  | string
  | ((...args: any[]) => any)
  | React.ReactNode;
export type ButtonSpace =
  | string
  | number
  | boolean
  | {
      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. Will use `margin-top`.
       */
      top?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-right`.
       */
      right?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-bottom`.
       */
      bottom?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-left`.
       */
      left?: string | number | boolean;
    };
export type ButtonTop = string | number | boolean;
export type ButtonRight = string | number | boolean;
export type ButtonBottom = string | number | boolean;
export type ButtonLeft = string | number | boolean;
export type ButtonOnClick = string | ((...args: any[]) => any);

/**
 * NB: Do not change the docs (comments) in here. The docs are updated during build time by "generateTypes.js" and "fetchPropertiesFromDocs.js".
 */
export interface ButtonProps extends React.HTMLProps<HTMLElement> {
  /**
   * The content of the button. `children` are treated as `text` as long as the typeof is a string.
   */
  text?: ButtonText;

  /**
   * `button`, `reset` or `submit` for the `type` HTML attribute (default to `button`) .
   */
  type?: string;

  /**
   * Title of the button. Optional, but should always be included because of accessibility.
   */
  title?: string;

  /**
   * Defines the kind of button. Possible values are `primary`, `secondary`, `tertiary` and `signal`.
   */
  variant?: ButtonVariant;

  /**
   * The size of the button. For now there is "medium", "default" and "large".
   */
  size?: ButtonSize;

  /**
   * To be included in the button.
   */
  icon?: ButtonIcon;

  /**
   * Position of icon inside the button. Set to `left` or `right`. Defaults to `right` if not set.
   */
  icon_position?: ButtonIconPosition;

  /**
   * Define icon width and height. Defaults to 16px
   */
  icon_size?: ButtonIconSize;
  tooltip?: ButtonTooltip;

  /**
   * Set it to either `status="error"` or a text with a status message. The style defaults to an error message. You can use `true` to only get the status color, without a message.
   */
  status?: ButtonStatus;

  /**
   * Defines the state of the status. Currently there are two statuses `[error, info]`. Defaults to `error`.
   */
  status_state?: string;
  status_animation?: string;

  /**
   * The `status_id` used for the target <a href="/uilib/components/global-status">GlobalStatus</a>.
   */
  global_status_id?: string;
  id?: string;

  /**
   * Any extra modifying class.
   */
  class?: string;

  /**
   * If you want the button to behave as a link. Use with caution! A link should normally visually be a link and not a button.
   */
  href?: string;

  /**
   * If set to `true` the button text will wrap in to new lines if the overflow point is reached. Defaults to `false`.
   */
  wrap?: ButtonWrap;

  /**
   * Set it to `true` in order to extend the bounding box (above the visual button background). You may also look into the HTML class `dnb-button__bounding` if it needs some CSS customization in order to get the particular button right for your use-case.
   */
  bounding?: ButtonBounding;
  skeleton?: ButtonSkeleton;
  disabled?: ButtonDisabled;
  inner_ref?: Object;
  className?: string;
  innerRef?: Object;

  /**
   * The content of the button. `children` are treated as `text` as long as the typeof is a string.
   */
  children?: ButtonChildren;

  /**
   * Only meant to be used for special use cases. Defaults to `button` or `a` depending if href is set or not.
   */
  element?: React.ReactNode;

  /**
   * Has to be an object with either: `top`, `right`, `bottom` or `left`. Use spacing values like: `small`, `1rem`, `1` or , `16px`.
   */
  space?: ButtonSpace;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. Will use `margin-top`.
   */
  top?: ButtonTop;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-right`.
   */
  right?: ButtonRight;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-bottom`.
   */
  bottom?: ButtonBottom;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-left`.
   */
  left?: ButtonLeft;
  custom_element?: Object;
  custom_method?: (...args: any[]) => any;
  onClick?: (...args: any[]) => any;

  /**
   * Will be called on a click event. Returns an object with the native event: `{ event }`.
   */
  on_click?: ButtonOnClick;
}
export default class Button extends React.Component<ButtonProps, any> {
  static defaultProps: object;
  render(): JSX.Element;
}
