import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import { convertStringToDate } from './DatePickerCalc';
import ToggleButton from '../toggle-button/ToggleButton';
import DatePickerContext from './DatePickerContext';
export default class DatePickerAddon extends React.PureComponent {
  constructor(...args) {
    super(...args);
    this.state = {
      currentShortcut: null,
      _listenForPropChanges: true
    };
  }

  setDate({
    value,
    event
  }) {
    this.setState({
      currentShortcut: value,
      _listenForPropChanges: false
    });
    const start_date = value.date || value.start_date;
    const end_date = value.end_date;
    const startDate = typeof start_date === 'function' ? start_date(this.getCurrentDates()) : start_date ? convertStringToDate(start_date) : null;
    const endDate = typeof end_date === 'function' ? end_date(this.getCurrentDates()) : end_date ? convertStringToDate(end_date) : null;
    this.callOnChange({
      startDate,
      endDate: endDate || startDate,
      event
    });

    if (value.close_on_select) {
      this.context.hidePicker(event);
    }
  }

  getCurrentDates() {
    const {
      startDate,
      endDate
    } = this.context;
    return {
      date: startDate,
      start_date: startDate,
      end_date: endDate
    };
  }

  callOnChange({
    startDate,
    endDate,
    event = null
  } = {}) {
    this.context.setDate({
      startDate,
      endDate,
      changeMonthViews: true
    });
    this.context.callOnChangeHandler({
      startDate,
      endDate,
      event
    });
  }

  render() {
    const {
      shortcuts,
      renderElement
    } = this.props;
    const shortcutsArray = shortcuts ? typeof shortcuts === 'string' ? JSON.parse(shortcuts) : shortcuts : [];
    const hasShortcuts = shortcutsArray && shortcutsArray.length > 0;

    if (!hasShortcuts && !renderElement) {
      return React.createElement(React.Fragment, null);
    }

    const shortcutElements = hasShortcuts && React.createElement(ToggleButton.Group, {
      value: this.state.currentShortcut,
      on_change: ({
        value,
        event
      }) => this.setDate({
        value,
        event
      })
    }, shortcutsArray.map((_ref, i) => {
      let {
        title
      } = _ref,
          rest = _objectWithoutPropertiesLoose(_ref, ["title"]);

      return React.createElement(ToggleButton, {
        key: i,
        text: title,
        value: rest
      });
    }));
    return React.createElement("div", {
      className: "dnb-date-picker__addon"
    }, renderElement, shortcutElements);
  }

}
DatePickerAddon.contextType = DatePickerContext;
DatePickerAddon.defaultProps = {
  shortcuts: null,
  renderElement: null
};
process.env.NODE_ENV !== "production" ? DatePickerAddon.propTypes = {
  shortcuts: PropTypes.oneOfType([PropTypes.array, PropTypes.func]),
  renderElement: PropTypes.oneOfType([PropTypes.node, PropTypes.func])
} : void 0;