import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import format from 'date-fns/format';
import subMonths from 'date-fns/subMonths';
import addMonths from 'date-fns/addMonths';
import addWeeks from 'date-fns/addWeeks';
import addDays from 'date-fns/addDays';
import isSameDay from 'date-fns/isSameDay';
import isSameMonth from 'date-fns/isSameMonth';
import startOfDay from 'date-fns/startOfDay';
import endOfDay from 'date-fns/endOfDay';
import differenceInCalendarDays from 'date-fns/differenceInCalendarDays';
import differenceInMonths from 'date-fns/differenceInMonths';
import lastDayOfMonth from 'date-fns/lastDayOfMonth';
import setDate from 'date-fns/setDate';
import keycode from 'keycode';
import nbLocale from 'date-fns/locale/nb';
import { isDisabled, makeDayObject, toRange, getWeek, dayOffset, getCalendar } from './DatePickerCalc';
import Button from '../button/Button';
import DatePickerContext from './DatePickerContext';
export default class DatePickerCalendar extends React.PureComponent {
  constructor(props) {
    super(props);

    this.onKeyDownHandler = event => {
      var _event$target;

      if (String(event === null || event === void 0 ? void 0 : (_event$target = event.target) === null || _event$target === void 0 ? void 0 : _event$target.nodeName).toLowerCase() === 'td') {
        try {
          this._listRef.current.focus({
            preventScroll: true
          });
        } catch (e) {}
      }

      const {
        nr,
        isRange,
        onlyMonth,
        hideNav,
        onKeyDown
      } = this.props;

      if (typeof onKeyDown === 'function') {
        return onKeyDown(event, this._listRef, nr);
      }

      const keyCode = keycode(event);

      switch (keyCode) {
        case 'enter':
        case 'space':
        case 'left':
        case 'right':
        case 'up':
        case 'down':
          event.preventDefault();
          event.persist();
          break;

        default:
          return;
      }

      let type = nr === 0 ? 'start' : 'end';

      if (!isRange) {
        type = 'start';
      }

      let newDate = this.context[`${type}Date`];

      if (newDate) {
        newDate = this.keyNavCalc(newDate, keyCode);
      } else {
        newDate = this.context[`${type}Month`] || (isRange && nr === 1 ? addMonths(new Date(), 1) : new Date());
      }

      if (newDate === this.context[`${type}Date`]) {
        switch (keyCode) {
          case 'enter':
          case 'space':
            this.callOnSelect({
              event,
              nr,
              hidePicker: true
            });
            break;
        }
      } else {
        const state = {};
        const currentMonth = this.context[`${type}Month`];

        if (currentMonth && !this.context[`${type}Date`] || currentMonth && Math.abs(differenceInMonths(newDate, currentMonth)) > 1) {
          if (!isRange) {
            newDate = currentMonth;
          } else {
            newDate = nr === 0 ? setDate(currentMonth, 1) : lastDayOfMonth(currentMonth);
          }
        } else if (currentMonth && !isSameMonth(this.context[`${type}Date`], currentMonth)) {
          state[`${type}Month`] = newDate;
        }

        newDate = this.findValid(newDate, keyCode);

        if (this.hasReachedEnd(newDate)) {
          return;
        }

        state[`${type}Date`] = newDate;

        if (!isRange) {
          state.endDate = newDate;
        } else {
          if (!this.context.startDate) {
            state.startDate = newDate;
          }

          if (!this.context.endDate) {
            state.endDate = newDate;
          }
        }

        if (onlyMonth || hideNav) {
          if (!isSameMonth(state.startDate, this.context.startDate) || !isSameMonth(state.endDate, this.context.startDate)) {
              return;
            }
        }

        state.changeMonthViews = true;
        this.context.setDate(state, () => {
          this.callOnSelect({
            event,
            nr,
            hidePicker: false
          });

          if (this._listRef && this._listRef.current) {
            this._listRef.current.focus({
              preventScroll: true
            });
          }
        });
      }
    };

    this.hasReachedEnd = date => isDisabled(date, this.context.minDate, this.context.maxDate);

    this.buildClassNames = day => classnames(day.className, day.isStartDate && 'dnb-date-picker__day--start-date', day.isEndDate && 'dnb-date-picker__day--end-date', day.isPreview && 'dnb-date-picker__day--preview', day.isWithinSelection && 'dnb-date-picker__day--within-selection', day.isSelectable && 'dnb-date-picker__day--selectable', day.isInactive && 'dnb-date-picker__day--inactive', day.isDisabled && 'dnb-date-picker__day--disabled', day.isToday && 'dnb-date-picker__day--today');

    this._listRef = React.createRef();
    this._labelRef = React.createRef();
    this._days = {};
    this._cache = {};
  }

  componentDidMount() {
    if (!this.props.noAutofocus && this.props.nr === 0) {
      if (this._listRef.current) {
        this._listRef.current.focus({
          preventScroll: true
        });
      }
    }
  }

  callOnSelect(args) {
    this.props.onSelect && this.props.onSelect(args);
  }

  keyNavCalc(date, keyCode) {
    switch (keyCode) {
      case 'left':
        date = addDays(date, -1);
        break;

      case 'right':
        date = addDays(date, 1);
        break;

      case 'up':
        date = addWeeks(date, -1);
        break;

      case 'down':
        date = addWeeks(date, 1);
        break;
    }

    return date;
  }

  findValid(date, keyCode) {
    if (this.context.props.on_days_render) {
      const month = format(date, 'yyyy-MM');

      if (!this._days) {
        return date;
      }

      if (!this._days[month]) {
        this.getDays(date);
      }

      if (Array.isArray(this._days[month])) {
        const foundDate = this._days[month].find(cur => isSameDay(cur.date, date));

        if (foundDate && foundDate.date && (foundDate.isDisabled || foundDate.isSelectable === false || foundDate.isInactive)) {
          const nextDate = this.keyNavCalc(foundDate.date, keyCode);
          foundDate.date = this.findValid(nextDate, keyCode);
        }

        if (foundDate && foundDate.date) {
          return foundDate.date;
        }
      }
    }

    return date;
  }

  getCacheKey() {
    const {
      nr,
      month,
      firstDayOfWeek,
      onlyMonth,
      hideNextMonthWeek
    } = this.props;
    const {
      startDate,
      endDate,
      hoverDate,
      maxDate,
      minDate
    } = this.context;
    return [nr, month, firstDayOfWeek, onlyMonth, hideNextMonthWeek, startDate, endDate, hoverDate, maxDate, minDate].join('|');
  }

  getMemorizedDays(month) {
    const key = this.getCacheKey();

    if (this._cache[key]) {
      return this._cache[key];
    } else {
      let count = 0;
      return this._cache[key] = Object.values(this.getDays(month).reduce((acc, cur, i) => {
        acc[count] = acc[count] || [];
        acc[count].push(cur);

        if (i % 7 === 6) {
          count++;
        }

        return acc;
      }, {}));
    }
  }

  getDays(month) {
    const {
      nr,
      firstDayOfWeek,
      onlyMonth,
      hideNextMonthWeek
    } = this.props;
    const {
      startDate,
      endDate,
      hoverDate,
      maxDate,
      minDate
    } = this.context;
    let days = getCalendar(month || new Date(), dayOffset(firstDayOfWeek), {
      onlyMonth,
      hideNextMonthWeek
    }).map(date => makeDayObject(date, {
      startDate,
      endDate,
      hoverDate,
      minDate,
      maxDate,
      month
    }));

    if (this.context.props.on_days_render) {
      const changedDays = this.context.props.on_days_render(days, nr);

      if (Array.isArray(changedDays)) {
        days = changedDays;
      }
    }

    this._days[format(month, 'yyyy-MM')] = days;
    return days;
  }

  render() {
    const {
      id,
      nr,
      rtl,
      month,
      isRange,
      titleFormat,
      locale,
      firstDayOfWeek,
      dayOfWeekFormat,
      hideNav,
      hideDays,
      onPrev,
      onNext,
      resetDate,
      onHover,
      prevBtn,
      nextBtn
    } = this.props;
    const {
      startDate,
      endDate,
      hoverDate,
      maxDate,
      minDate,
      translation: {
        DatePicker: {
          selected_month
        }
      }
    } = this.context;
    const weekDays = this.getMemorizedDays(month);
    return React.createElement("div", {
      className: 'dnb-date-picker__calendar' + (rtl ? " rtl" : ""),
      lang: locale === null || locale === void 0 ? void 0 : locale.code
    }, !hideNav && React.createElement("div", {
      className: "dnb-date-picker__header"
    }, React.createElement("div", {
      className: "dnb-date-picker__header__nav"
    }, React.createElement(PrevButton, {
      nr: nr,
      minDate: minDate,
      month: month,
      locale: locale,
      context: this.context,
      prevBtn: prevBtn,
      onPrev: onPrev
    })), React.createElement("label", {
      id: `${id}--title`,
      className: "dnb-date-picker__header__title dnb-no-focus",
      title: selected_month.replace(/%s/, format(month, titleFormat, {
        locale
      })),
      tabIndex: "-1",
      ref: this._labelRef
    }, format(month, titleFormat, {
      locale
    })), React.createElement("div", {
      className: "dnb-date-picker__header__nav"
    }, React.createElement(NextButton, {
      nr: nr,
      maxDate: maxDate,
      month: month,
      locale: locale,
      context: this.context,
      nextBtn: nextBtn,
      onNext: onNext
    }))), React.createElement("table", {
      role: "grid",
      className: "dnb-no-focus",
      tabIndex: "0",
      "aria-labelledby": `${id}--title`,
      onKeyDown: this.onKeyDownHandler,
      ref: this._listRef
    }, !hideDays && React.createElement("thead", {
      "aria-hidden": true
    }, React.createElement("tr", {
      role: "row",
      className: "dnb-date-picker__labels"
    }, getWeek(dayOffset(firstDayOfWeek)).map((day, i) => React.createElement("th", {
      key: i,
      role: "columnheader",
      scope: "col",
      className: `dnb-date-picker__labels__day dnb-date-picker__labels__day--${format(day, 'i', {
        locale
      })}`,
      "aria-label": format(day, 'EEEE', {
        locale
      })
    }, format(day, dayOfWeekFormat, {
      locale
    }))))), React.createElement("tbody", null, weekDays.map((week, i) => {
      return React.createElement("tr", {
        key: 'week' + i,
        role: "row",
        className: "dnb-date-picker__days"
      }, week.map((day, i) => {
        const title = format(day.date, 'PPPP', {
          locale
        });
        const handleAsDisabled = day.isLastMonth || day.isNextMonth || day.isDisabled || day.isInactive;
        const paramsCell = {};
        paramsCell.tabIndex = '-1';

        if (day.isStartDate) {
          paramsCell.id = id + '--button-start';
        } else if (day.isEndDate) {
          paramsCell.id = id + '--button-end';
        }

        const paramsButton = {};

        if (nr === 0 ? day.isStartDate : day.isEndDate) {
          paramsButton['aria-current'] = 'date';
          paramsCell['aria-selected'] = true;
        }

        return React.createElement("td", _extends({
          key: 'day' + i,
          role: "gridcell",
          className: classnames("dnb-date-picker__day dnb-no-focus", this.buildClassNames(day)),
          onFocus: this.onKeyDownHandler
        }, paramsCell), React.createElement(Button, _extends({
          size: "medium",
          variant: "secondary",
          text: format(day.date, 'd', {
            locale
          }),
          bounding: true,
          disabled: handleAsDisabled,
          tabIndex: handleAsDisabled ? '0' : '-1',
          "aria-disabled": handleAsDisabled,
          "aria-label": title
        }, paramsButton, {
          onClick: handleAsDisabled ? undefined : ({
            event
          }) => onSelectRange({
            day,
            isRange,
            startDate,
            endDate,
            resetDate,
            event,
            onSelect: state => this.context.setDate(state, () => this.callOnSelect({
              event,
              nr,
              hidePicker: !isRange
            }))
          }),
          onMouseOver: handleAsDisabled ? undefined : () => onHoverDay({
            day,
            hoverDate,
            onHover
          }),
          onFocus: handleAsDisabled ? undefined : () => onHoverDay({
            day,
            hoverDate,
            onHover
          })
        })));
      }));
    }))));
  }

}
DatePickerCalendar.contextType = DatePickerContext;
DatePickerCalendar.defaultProps = {
  id: null,
  nr: null,
  month: null,
  prevBtn: true,
  nextBtn: true,
  titleFormat: 'MMMM yyyy',
  dayOfWeekFormat: 'EEEEEE',
  firstDayOfWeek: 'monday',
  hideNav: false,
  hideDays: false,
  onlyMonth: false,
  hideNextMonthWeek: false,
  noAutofocus: false,
  locale: nbLocale,
  rtl: false,
  onHover: null,
  onSelect: null,
  onPrev: null,
  onNext: null,
  isRange: null,
  resetDate: true,
  onKeyDown: null
};
process.env.NODE_ENV !== "production" ? DatePickerCalendar.propTypes = {
  id: PropTypes.string,
  nr: PropTypes.number,
  month: PropTypes.instanceOf(Date),
  prevBtn: PropTypes.bool,
  nextBtn: PropTypes.bool,
  titleFormat: PropTypes.string,
  dayOfWeekFormat: PropTypes.string,
  firstDayOfWeek: PropTypes.string,
  hideNav: PropTypes.bool,
  hideDays: PropTypes.bool,
  onlyMonth: PropTypes.bool,
  hideNextMonthWeek: PropTypes.bool,
  noAutofocus: PropTypes.bool,
  onHover: PropTypes.func,
  onSelect: PropTypes.func,
  onPrev: PropTypes.func,
  onNext: PropTypes.func,
  locale: PropTypes.object,
  rtl: PropTypes.bool,
  isRange: PropTypes.bool,
  resetDate: PropTypes.bool,
  onKeyDown: PropTypes.func
} : void 0;

const PrevButton = ({
  nr,
  minDate,
  month,
  locale,
  context,
  prevBtn,
  onPrev,
  onKeyDown
}) => {
  if (!prevBtn) {
    return React.createElement(React.Fragment, null);
  }

  const disabled = minDate && isSameMonth(month, minDate);

  const onClick = () => onPrev && !disabled && onPrev({
    nr
  });

  const {
    translation: {
      DatePicker: {
        prev_month
      }
    }
  } = context;
  const title = prev_month.replace(/%s/, format(subMonths(month, 1), 'MMMM yyyy', {
    locale
  }));
  return React.createElement(Button, {
    className: 'dnb-date-picker__prev' + (disabled ? " disabled" : ""),
    icon: "chevron_left",
    size: "small",
    "aria-label": title,
    onClick: onClick,
    onKeyDown: onKeyDown
  });
};

process.env.NODE_ENV !== "production" ? PrevButton.propTypes = {
  nr: PropTypes.number.isRequired,
  minDate: PropTypes.instanceOf(Date),
  month: PropTypes.object.isRequired,
  locale: PropTypes.object.isRequired,
  context: PropTypes.object.isRequired,
  prevBtn: PropTypes.bool.isRequired,
  onPrev: PropTypes.func.isRequired,
  onKeyDown: PropTypes.func
} : void 0;
PrevButton.defaultProps = {
  minDate: null,
  onKeyDown: null
};

const NextButton = ({
  nr,
  maxDate,
  month,
  locale,
  context,
  nextBtn,
  onNext,
  onKeyDown
}) => {
  if (!nextBtn) {
    return React.createElement(React.Fragment, null);
  }

  const disabled = maxDate && isSameMonth(month, maxDate);

  const onClick = () => onNext && !disabled && onNext({
    nr
  });

  const {
    translation: {
      DatePicker: {
        next_month
      }
    }
  } = context;
  const title = next_month.replace(/%s/, format(addMonths(month, 1), 'MMMM yyyy', {
    locale
  }));
  return nextBtn && React.createElement(Button, {
    className: 'dnb-date-picker__next' + (disabled ? " disabled" : ""),
    icon: "chevron_right",
    size: "small",
    "aria-label": title,
    onClick: onClick,
    onKeyDown: onKeyDown
  });
};

process.env.NODE_ENV !== "production" ? NextButton.propTypes = {
  nr: PropTypes.number.isRequired,
  maxDate: PropTypes.instanceOf(Date),
  month: PropTypes.object.isRequired,
  locale: PropTypes.object.isRequired,
  context: PropTypes.object.isRequired,
  nextBtn: PropTypes.bool.isRequired,
  onNext: PropTypes.func.isRequired,
  onKeyDown: PropTypes.func
} : void 0;
NextButton.defaultProps = {
  maxDate: null,
  onKeyDown: null
};

const onSelectRange = ({
  day,
  isRange,
  startDate,
  endDate,
  onSelect,
  resetDate,
  event
}) => {
  event.persist();

  if (!isRange) {
    onSelect({
      startDate: startOfDay(day.date),
      endDate: endOfDay(day.date),
      event
    });
  } else if (!startDate || resetDate && startDate && endDate) {
    onSelect({
      startDate: startOfDay(day.date),
      endDate: undefined,
      event
    });
  } else {
    const hasEndDate = endDate;
    const daysToStartDate = Math.abs(differenceInCalendarDays(startDate, day.date));
    const daysToEndDate = Math.abs(differenceInCalendarDays(endDate, day.date));
    let range = toRange(startDate, day.date);

    if (hasEndDate && !resetDate && daysToStartDate < daysToEndDate) {
      range = toRange(endDate, day.date);
    }

    onSelect({
      startDate: startOfDay(range.startDate),
      endDate: endOfDay(range.endDate),
      event
    });
  }
};

const onHoverDay = ({
  day,
  hoverDate,
  onHover
}) => {
  if (!isSameDay(day.date, hoverDate)) {
    onHover && onHover(day.date);
  }
};