import React from 'react';
import PropTypes from 'prop-types';
import Button from '../button/Button';
import DatePickerContext from './DatePickerContext';
import { isTrue } from '../../shared/component-helper';
import { convertStringToDate } from './DatePickerCalc';

var _ref = React.createElement("span", null);

var _ref2 = React.createElement("span", null);

var _ref3 = React.createElement("span", null);

export default class DatePickerFooter extends React.PureComponent {
  constructor(..._args) {
    super(..._args);

    this.onSubmitHandler = args => {
      const {
        onSubmit
      } = this.props;

      if (onSubmit) {
        onSubmit(args);
      }
    };

    this.onCancelHandler = args => {
      const {
        date_format
      } = this.context.props;
      const startDate = this.context._startDate ? convertStringToDate(this.context._startDate, {
        date_format
      }) : null;
      const endDate = this.context._endDate ? convertStringToDate(this.context._endDate, {
        date_format
      }) : startDate;

      if (args && args.event) {
        args.event.persist();
      }

      this.context.setDate({
        startDate,
        endDate
      }, () => {
        const {
          onCancel
        } = this.props;

        if (onCancel) {
          onCancel(args);
        }
      });
    };

    this.onResetHandler = args => {
      if (args && args.event) {
        args.event.persist();
      }

      this.context.setDate({
        date: undefined,
        startDate: undefined,
        endDate: undefined
      }, () => {
        const {
          onReset
        } = this.props;

        if (onReset) {
          onReset(args);
        }
      });
    };
  }

  render() {
    const {
      isRange
    } = this.props;
    const {
      show_reset_button,
      show_cancel_button,
      show_submit_button
    } = this.context.props;

    if (!isRange && !isTrue(show_submit_button) && !isTrue(show_cancel_button)) {
      return React.createElement(React.Fragment, null);
    }

    const {
      submit_button_text,
      cancel_button_text,
      reset_button_text
    } = this.context.translation.DatePicker;
    return React.createElement("div", {
      className: "dnb-date-picker__footer"
    }, (isRange || isTrue(show_submit_button)) && React.createElement(Button, {
      text: submit_button_text,
      onClick: this.onSubmitHandler
    }) || _ref, React.createElement("span", null, isTrue(show_reset_button) && React.createElement(Button, {
      text: reset_button_text,
      icon: "reset",
      icon_position: "left",
      variant: "tertiary",
      onClick: this.onResetHandler
    }) || _ref2, (isRange || isTrue(show_cancel_button)) && React.createElement(Button, {
      text: cancel_button_text,
      icon: "close",
      icon_position: "left",
      variant: "tertiary",
      onClick: this.onCancelHandler
    }) || _ref3));
  }

}
DatePickerFooter.contextType = DatePickerContext;
DatePickerFooter.defaultProps = {
  onSubmit: null,
  onCancel: null,
  onReset: null
};
process.env.NODE_ENV !== "production" ? DatePickerFooter.propTypes = {
  isRange: PropTypes.bool.isRequired,
  onSubmit: PropTypes.func,
  onCancel: PropTypes.func,
  onReset: PropTypes.func
} : void 0;