import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import PropTypes from 'prop-types';
import addDays from 'date-fns/addDays';
import addMonths from 'date-fns/addMonths';
import addYears from 'date-fns/addYears';
import isValid from 'date-fns/isValid';
import parseISO from 'date-fns/parseISO';
import classnames from 'classnames';
import MaskedInput from 'react-text-mask';
import Input, { SubmitButton } from '../input/Input';
import keycode from 'keycode';
import { warn, validateDOMAttributes, dispatchCustomElementEvent } from '../../shared/component-helper';
import { convertStringToDate } from './DatePickerCalc';
import DatePickerContext from './DatePickerContext';

var _ref = React.createElement("span", {
  className: "dnb-date-picker--separator",
  "aria-hidden": true
}, ' – ');

export default class DatePickerInput extends React.PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      _listenForPropChanges: true,
      focusState: 'virgin'
    };

    this.shortcutHandler = async e => {
      if (this.focusMode) {
        var _window;

        const success = (e.clipboardData || ((_window = window) === null || _window === void 0 ? void 0 : _window.clipboardData)).getData('text');

        if (success) {
          e.preventDefault();

          try {
            const separators = ['.', '/'];
            const possibleFormats = ['yyyy-MM-dd'];
            possibleFormats.forEach(date => {
              separators.forEach(sep => {
                possibleFormats.push(date.replace(/-/g, sep));
              });
            });
            possibleFormats.forEach(date => {
              possibleFormats.push(date.split('').reverse().join(''));
            });
            let date;

            for (let i = 0, l = possibleFormats.length; i < l; ++i) {
              date = convertStringToDate(success, {
                date_format: possibleFormats[i]
              });

              if (date) {
                break;
              }
            }

            const mode = this.focusMode === 'start' ? 'startDate' : 'endDate';

            if (date && !this.state[mode]) {
              this.context.setState({
                [mode]: date
              });
            }
          } catch (e) {
            warn(e);
          }
        }
      }
    };

    this.callOnChangeAsInvalid = state => {
      const {
        startDate,
        endDate,
        event
      } = _extends({}, this.context, state);

      this.context.updateState({
        hoverDate: null
      });

      if (this.context.hasHadValidDate) {
        this.context.callOnChangeHandler({
          startDate,
          endDate,
          event
        });
        this.context.updateState({
          hasHadValidDate: false
        });
      }
    };

    this.callOnChange = ({
      startDate,
      endDate,
      event
    }) => {
      const state = {
        changeMonthViews: true,
        hasHadValidDate: false
      };

      if (typeof startDate !== 'undefined' && isValid(startDate)) {
        state.startDate = startDate;
      }

      if (!this.props.isRange) {
        endDate = startDate;
      }

      if (typeof endDate !== 'undefined' && isValid(endDate)) {
        state.endDate = endDate;
      }

      this.context.setDate(state, () => {
        if (typeof startDate !== 'undefined' && isValid(startDate) || typeof endDate !== 'undefined' && isValid(endDate)) {
          this.context.callOnChangeHandler({
            event
          });
        }
      });
    };

    this.callOnType = ({
      event
    }) => {
      const getDates = () => ['start', 'end'].reduce((acc, mode) => {
        acc[`${mode}Date`] = [this[`_${mode}Year`] || this.context[`__${mode}Year`] || 'yyyy', this[`_${mode}Month`] || this.context[`__${mode}Month`] || 'mm', this[`_${mode}Day`] || this.context[`__${mode}Day`] || 'dd'].join('-');
        return acc;
      }, {});

      let {
        startDate,
        endDate
      } = getDates();
      startDate = parseISO(startDate);
      endDate = parseISO(endDate);

      if (!isValid(startDate)) {
        startDate = null;
      }

      if (!isValid(endDate)) {
        endDate = null;
      }

      let returnObject = this.context.getReturnObject({
        startDate,
        endDate,
        event
      });

      if (returnObject.is_valid === false || returnObject.is_valid_start_date === false || returnObject.is_valid_end_date === false) {
        const {
          startDate,
          endDate
        } = getDates();
        const typedDates = this.props.isRange ? {
          start_date: startDate,
          end_date: endDate
        } : {
          date: startDate
        };
        returnObject = _extends({}, returnObject, typedDates);
      }

      dispatchCustomElementEvent(this.context, 'on_type', returnObject);
    };

    this.prepareCounting = async ({
      keyCode,
      target,
      event
    }) => {
      const isDate = target.getAttribute('class').match(/__input--([day|month|year]+)($|\s)/)[1];
      const isInRange = target.getAttribute('id').match(/[0-9]-([start|end]+)-/)[1];
      let date = isInRange === 'start' ? this.context.startDate : this.context.endDate;

      if (!date) {
        return;
      }

      const count = keyCode === 'up' ? 1 : -1;

      if (keyCode === 'up' || keyCode === 'down') {
        switch (isDate) {
          case 'day':
            date = addDays(date, count);
            break;

          case 'month':
            date = addMonths(date, count);
            break;

          case 'year':
            date = addYears(date, count);
            break;
        }
      }

      this.callOnChange({
        [isInRange === 'start' ? 'startDate' : 'endDate']: date,
        event
      });
      await wait(1);
      const endPos = target.value.length;
      target.focus();
      target.setSelectionRange(0, endPos);
    };

    this.onFocusHandler = event => {
      try {
        const target = event.target;
        const endPos = target.value.length;
        target.focus();
        target.setSelectionRange(0, endPos);
      } catch (e) {
        warn(e);
      }

      this.setState({
        focusState: 'focus',
        _listenForPropChanges: false
      });
    };

    this.onBlurHandler = () => {
      this.focusMode = null;
      this.setState({
        focusState: 'blur',
        _listenForPropChanges: false
      });
    };

    this.onKeyDownHandler = async event => {
      const keyCode = keycode(event);
      const target = event.target;

      switch (keyCode) {
        case 'up':
        case 'down':
          event.persist();
          event.preventDefault();
          this.prepareCounting({
            event,
            keyCode,
            target
          });
          return false;

        case 'tab':
          return false;
      }

      const size = parseFloat(target.getAttribute('size'));
      const firstSelectionStart = target.selectionStart;
      await wait(1);
      const secondSelectionStart = target.selectionStart;
      const isValid = /[0-9]/.test(keyCode);
      const index = this.refList.findIndex(({
        current
      }) => current.inputElement === target);

      if (index < this.refList.length - 1 && (secondSelectionStart === size && isValid && keyCode !== 'left' && keyCode !== 'backspace' || firstSelectionStart === size && keyCode === 'right')) {
        try {
          if (!this.refList[index + 1].current) {
            return;
          }

          const nextSibling = this.refList[index + 1].current.inputElement;

          if (nextSibling) {
            nextSibling.focus();
            nextSibling.setSelectionRange(0, 0);
          }
        } catch (e) {
          warn(e);
        }
      } else if (firstSelectionStart === 0 && index > 0) {
        switch (keyCode) {
          case 'left':
          case 'backspace':
            try {
              const prevSibling = this.refList[index - 1].current.inputElement;

              if (prevSibling) {
                const endPos = prevSibling.value.length;
                prevSibling.focus();
                prevSibling.setSelectionRange(endPos, endPos);
              }
            } catch (e) {
              warn(e);
            }

            break;
        }
      }
    };

    this.set_startDay = event => {
      this.setDate(event, 'start', 'Day');
    };

    this.set_startMonth = event => {
      this.setDate(event, 'start', 'Month');
    };

    this.set_startYear = event => {
      this.setDate(event, 'start', 'Year');
    };

    this.set_endDay = event => {
      this.setDate(event, 'end', 'Day');
    };

    this.set_endMonth = event => {
      this.setDate(event, 'end', 'Month');
    };

    this.set_endYear = event => {
      this.setDate(event, 'end', 'Year');
    };

    this.setDate = (event, mode, type) => {
      event.persist();
      const value = event.target.value;
      this[`_${mode}${type}`] = value;

      if (this.context[`${mode}Date`]) {
        this[`tmp_${mode}Date`] = this.context[`${mode}Date`];
      }

      const fallback = this[`tmp_${mode}Date`];
      const year = this[`_${mode}Year`] || fallback && fallback.getFullYear();
      const month = this[`_${mode}Month`] || fallback && fallback.getMonth() + 1;
      const day = this[`_${mode}Day`] || fallback && fallback.getDate();
      const date = new Date(parseFloat(year), parseFloat(month) - 1, parseFloat(day));
      const isValidDate = !/[^0-9]/.test(day) && !/[^0-9]/.test(month) && !/[^0-9]/.test(year) && isValid(date) && date.getDate() == parseFloat(day) && date.getMonth() + 1 == parseFloat(month) && date.getFullYear() == parseFloat(year);

      if (isValidDate) {
        this.callOnChange({
          [`${mode}Date`]: date,
          event
        });
      } else {
        this.context.setDate({
          [`${mode}Date`]: null
        });
        this.context.updateState({
          [`__${mode}${type}`]: value
        });
        this.callOnChangeAsInvalid({
          [`${mode}Date`]: null,
          event
        });
      }

      this.callOnType({
        event
      });
    };

    this.renderInputElement = params => {
      const {
        id,
        isRange
      } = this.props;
      this.refList = [];
      const startDateList = this.generateDateList(params, 'start');
      const endDateList = this.generateDateList(params, 'end');
      return React.createElement("span", {
        id: `${id}-input`,
        className: "dnb-date-picker__input__wrapper"
      }, startDateList, isRange && _ref, isRange && endDateList);
    };

    const _separators = props.maskOrder.match(props.separatorRexExp);

    this.maskList = props.maskOrder.split(props.separatorRexExp).reduce((acc, cur) => {
      if (cur) {
        acc.push(cur);

        if (_separators.length > 0) {
          acc.push(_separators.shift());
        }
      }

      return acc;
    }, []);
    this._startDayRef = React.createRef();
    this._startMonthRef = React.createRef();
    this._startYearRef = React.createRef();
    this._endDayRef = React.createRef();
    this._endMonthRef = React.createRef();
    this._endYearRef = React.createRef();
  }

  componentWillUnmount() {
    if (this._shortcuts) {
      this._shortcuts.remove(this.osShortcut);
    }
  }

  getPlaceholderChar(value) {
    const index = this.props.maskOrder.indexOf(value);
    return this.props.maskPlaceholder[index];
  }

  generateDateList(params, mode) {
    return this.maskList.map((value, i) => {
      const state = value.slice(0, 1);
      const placeholderChar = this.getPlaceholderChar(value);
      const {
        input_element,
        separatorRexExp,
        isRange
      } = this.props;
      const {
        day,
        month,
        year
      } = this.context.translation.DatePicker;
      const isRangeLabe = isRange ? `${this.context.translation.DatePicker[mode]} ` : '';

      if (!separatorRexExp.test(value)) {
        if (!input_element) {
          params = _extends({}, params, {
            onKeyDown: this.onKeyDownHandler,
            onMouseUp: selectInput,
            onPaste: this.shortcutHandler,
            onFocus: e => {
              this.focusMode = mode;
              this.onFocusHandler(e);
            },
            onBlur: this.onBlurHandler,
            placeholderChar
          });
        }

        const Input = typeof input_element === 'string' ? input_element : InputElement;

        switch (state) {
          case 'd':
            this.refList.push(this[`_${mode}DayRef`]);
            return React.createElement(React.Fragment, {
              key: 'dd' + i
            }, React.createElement(Input, _extends({}, params, {
              id: `${this.props.id}-${mode}-day`,
              key: 'di' + i,
              className: classnames("dnb-date-picker__input dnb-date-picker__input--day", params.className),
              size: "2",
              mask: [/[0-3]/, /[0-9]/],
              ref: this[`_${mode}DayRef`],
              onChange: this[`set_${mode}Day`],
              value: this.context[`__${mode}Day`],
              "aria-labelledby": `${this.props.id}-${mode}-day-label`
            })), React.createElement("label", {
              key: 'dl' + i,
              hidden: true,
              id: `${this.props.id}-${mode}-day-label`
            }, isRangeLabe + day));

          case 'm':
            this.refList.push(this[`_${mode}MonthRef`]);
            return React.createElement(React.Fragment, {
              key: 'mm' + i
            }, React.createElement(Input, _extends({}, params, {
              id: `${this.props.id}-${mode}-month`,
              key: 'mi' + i,
              className: classnames("dnb-date-picker__input dnb-date-picker__input--month", params.className),
              size: "2",
              mask: [/[0-1]/, /[0-9]/],
              ref: this[`_${mode}MonthRef`],
              onChange: this[`set_${mode}Month`],
              value: this.context[`__${mode}Month`],
              "aria-labelledby": `${this.props.id}-${mode}-month-label`
            })), React.createElement("label", {
              key: 'ml' + i,
              hidden: true,
              id: `${this.props.id}-${mode}-month-label`
            }, isRangeLabe + month));

          case 'y':
            this.refList.push(this[`_${mode}YearRef`]);
            return React.createElement(React.Fragment, {
              key: 'yy' + i
            }, React.createElement(Input, _extends({}, params, {
              id: `${this.props.id}-${mode}-year`,
              key: 'yi' + i,
              className: classnames("dnb-date-picker__input dnb-date-picker__input--year", params.className),
              size: "4",
              mask: [/[1-2]/, /[0-9]/, /[0-9]/, /[0-9]/],
              ref: this[`_${mode}YearRef`],
              onChange: this[`set_${mode}Year`],
              value: this.context[`__${mode}Year`],
              "aria-labelledby": `${this.props.id}-${mode}-year-label`
            })), React.createElement("label", {
              key: 'yl' + i,
              hidden: true,
              id: `${this.props.id}-${mode}-year-label`
            }, isRangeLabe + year));
        }
      }

      return React.createElement("span", {
        key: 's' + i,
        className: "dnb-date-picker--separator",
        "aria-hidden": true
      }, placeholderChar);
    });
  }

  formatDate() {
    const {
      open_picker_text
    } = this.context.translation.DatePicker;
    const {
      selectedDateTitle
    } = this.props;
    return selectedDateTitle ? `${selectedDateTitle}, ${open_picker_text}` : open_picker_text;
  }

  render() {
    const _this$props = this.props,
          {
      id,
      title,
      submitAttributes,
      onSubmit,
      input_element,
      locale,
      disabled,
      skeleton,
      opened,
      status,
      status_state
    } = _this$props,
          attributes = _objectWithoutPropertiesLoose(_this$props, ["id", "title", "submitAttributes", "isRange", "maskOrder", "maskPlaceholder", "separatorRexExp", "onChange", "onFocus", "onSubmit", "selectedDateTitle", "showInput", "input_element", "locale", "disabled", "skeleton", "opened", "status", "status_state"]);

    const {
      focusState
    } = this.state;
    validateDOMAttributes(this.props, attributes);
    validateDOMAttributes(null, submitAttributes);
    return React.createElement("fieldset", {
      className: "dnb-date-picker__fieldset",
      lang: locale === null || locale === void 0 ? void 0 : locale.code
    }, this.context.props.label && React.createElement("legend", {
      className: "dnb-sr-only"
    }, this.context.props.label), React.createElement(Input, _extends({
      id: `${id}__input`,
      input_state: disabled ? 'disabled' : focusState,
      input_element: input_element && typeof input_element !== 'string' ? typeof input_element === 'function' ? input_element(this.props) : input_element : this.renderInputElement,
      disabled: disabled || skeleton,
      skeleton: skeleton,
      status: !opened ? status : null,
      status_state: status_state,
      submit_element: React.createElement(SubmitButton, _extends({
        id: id,
        disabled: disabled,
        skeleton: skeleton,
        className: opened ? 'dnb-button--active' : null,
        "aria-label": this.formatDate(),
        title: title,
        type: "button",
        icon: "calendar",
        variant: "secondary",
        on_submit: onSubmit
      }, submitAttributes)),
      lang: locale === null || locale === void 0 ? void 0 : locale.code
    }, attributes)), ' ');
  }

}
DatePickerInput.contextType = DatePickerContext;
DatePickerInput.defaultProps = {
  id: null,
  title: null,
  selectedDateTitle: null,
  maskOrder: 'dd/mm/yyyy',
  maskPlaceholder: 'dd/mm/åååå',
  separatorRexExp: /[-/ ]/g,
  submitAttributes: null,
  isRange: null,
  status: null,
  status_state: 'error',
  input_element: null,
  disabled: null,
  locale: null,
  skeleton: null,
  opened: false,
  showInput: null,
  onChange: null,
  onSubmit: null,
  onFocus: null
};
process.env.NODE_ENV !== "production" ? DatePickerInput.propTypes = {
  id: PropTypes.string,
  title: PropTypes.string,
  selectedDateTitle: PropTypes.string,
  maskOrder: PropTypes.string,
  maskPlaceholder: PropTypes.string,
  separatorRexExp: PropTypes.instanceOf(RegExp),
  submitAttributes: PropTypes.object,
  isRange: PropTypes.bool,
  status: PropTypes.oneOfType([PropTypes.string, PropTypes.bool, PropTypes.func, PropTypes.node]),
  status_state: PropTypes.string,
  input_element: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  locale: PropTypes.object,
  disabled: PropTypes.bool,
  skeleton: PropTypes.bool,
  opened: PropTypes.bool,
  showInput: PropTypes.bool,
  onChange: PropTypes.func,
  onSubmit: PropTypes.func,
  onFocus: PropTypes.func
} : void 0;

const selectInput = e => {
  e.target.focus();
  e.target.select();
};

const InputElement = React.forwardRef((props, innerRef) => {
  return React.createElement(MaskedInput, _extends({
    guide: true,
    showMask: true,
    keepCharPositions: false,
    autoComplete: "off",
    autoCapitalize: "none",
    spellCheck: false,
    autoCorrect: "off",
    ref: innerRef
  }, props));
});

const wait = t => new Promise(r => setTimeout(r, t));