import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import PropTypes from 'prop-types';
import addMonths from 'date-fns/addMonths';
import subMonths from 'date-fns/subMonths';
import DatePickerCalendar from './DatePickerCalendar';
import DatePickerContext from './DatePickerContext';
export default class DatePickerRange extends React.PureComponent {
  constructor(..._args) {
    super(..._args);

    this.onSelectHandler = args => {
      this.context.callOnChangeHandler(args);
      this.props.onChange && this.props.onChange(_extends({
        hidePicker: !this.props.isRange
      }, args));
    };

    this.callOnNav = () => {
      this.props.onNav && this.props.onNav(this.context.views);
    };

    this.setNavState = state => {
      this.context.updateState(state);
    };

    this.onNext = ({
      nr
    }) => {
      const views = this.context.views.map(c => {
        if (c.nr === nr) {
          const month = addMonths(c.month, 1);
          this.setNavState({
            nr,
            [`${nr === 0 ? 'start' : 'end'}Month`]: month
          });
          return _extends({}, c, {
            month
          });
        }

        if (this.props.isLink && c.nr === 1) {
          const month = addMonths(c.month, 1);
          this.setNavState({
            nr,
            [`${nr !== 0 ? 'start' : 'end'}Month`]: month
          });
          return _extends({}, c, {
            month
          });
        }

        return c;
      });
      this.context.setViews(views, this.callOnNav);
    };

    this.onPrev = ({
      nr
    }) => {
      const views = this.context.views.map(c => {
        if (c.nr === nr) {
          const month = subMonths(c.month, 1);
          this.setNavState({
            nr,
            [`${nr === 0 ? 'start' : 'end'}Month`]: month
          });
          return _extends({}, c, {
            month
          });
        }

        if (this.props.isLink && c.nr === 1) {
          const month = subMonths(c.month, 1);
          this.setNavState({
            nr,
            [`${nr !== 0 ? 'start' : 'end'}Month`]: month
          });
          return _extends({}, c, {
            month
          });
        }

        return c;
      });
      this.context.setViews(views, this.callOnNav);
    };

    this.onHover = hoverDate => {
      this.context.updateState({
        hoverDate
      });
    };
  }

  render() {
    const _this$props = this.props,
          {
      id
    } = _this$props,
          props = _objectWithoutPropertiesLoose(_this$props, ["id"]);

    return React.createElement("div", {
      className: "dnb-date-picker__views"
    }, this.context.views.map((calendar, i) => React.createElement(DatePickerCalendar, _extends({
      key: calendar.nr,
      id: `${id}-${i}-`
    }, props, calendar, {
      onSelect: this.onSelectHandler,
      onHover: this.onHover,
      onPrev: this.onPrev,
      onNext: this.onNext
    }))));
  }

}
DatePickerRange.contextType = DatePickerContext;
DatePickerRange.defaultProps = {
  id: null,
  isRange: null,
  isLink: null,
  isSync: null,
  onlyMonth: null,
  hideNav: null,
  views: null,
  onChange: null,
  onNav: null
};
process.env.NODE_ENV !== "production" ? DatePickerRange.propTypes = {
  id: PropTypes.string,
  isRange: PropTypes.bool,
  isLink: PropTypes.bool,
  isSync: PropTypes.bool,
  onlyMonth: PropTypes.bool,
  hideNav: PropTypes.bool,
  views: PropTypes.oneOfType([PropTypes.number, PropTypes.arrayOf(PropTypes.object)]),
  onChange: PropTypes.func,
  onNav: PropTypes.func
} : void 0;
export const getViews = (state, isRange) => {
  return (Array.isArray(state.views) ? state.views : Array(isRange ? 2 : state.views).fill(1)).map((view, nr) => _extends({}, view, {
    month: getMonthView(state, nr),
    nr
  }));
};

const getMonthView = (state, nr) => {
  if ((state.startMonth || state.startDate) && nr === 0) {
    return state.startMonth || state.startDate;
  }

  if ((state.endMonth || state.endDate) && nr === 1) {
    return state.endMonth || state.endDate;
  }

  return addMonths(getFallbackMonth(state), nr);
};

const getFallbackMonth = state => {
  return state.startMonth || state.startDate || new Date();
};