import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import keycode from 'keycode';
import { warn, isTrue, makeUniqueId, extendPropsWithContext, registerElement, validateDOMAttributes, getStatusState, combineDescribedBy, dispatchCustomElementEvent } from '../../shared/component-helper';
import AlignmentHelper from '../../shared/AlignmentHelper';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import Suffix from '../../shared/helpers/Suffix';
import Icon from '../icon-primary/IconPrimary';
import FormLabel from '../form-label/FormLabel';
import FormStatus from '../form-status/FormStatus';
import Button from '../button/Button';
import DrawerList from '../../fragments/drawer-list/DrawerList';
import DrawerListContext from '../../fragments/drawer-list/DrawerListContext';
import DrawerListProvider from '../../fragments/drawer-list/DrawerListProvider';
import { parseContentTitle, getCurrentData } from '../../fragments/drawer-list/DrawerListHelpers';
export default class Dropdown extends React.PureComponent {
  static enableWebComponent() {
    registerElement(Dropdown.tagName, Dropdown, Dropdown.defaultProps);
  }

  render() {
    const {
      more_menu,
      action_menu,
      prevent_selection,
      children,
      data
    } = this.props;
    return React.createElement(DrawerListProvider, _extends({}, this.props, {
      data: data || children,
      opened: null,
      tagName: "dnb-dropdown",
      ignore_events: false,
      prevent_selection: isTrue(more_menu) || isTrue(action_menu) || isTrue(prevent_selection)
    }), React.createElement(DropdownInstance, this.props));
  }

}
Dropdown.tagName = 'dnb-dropdown';
Dropdown.defaultProps = {
  id: null,
  title: 'Option Menu',
  icon: null,
  icon_size: null,
  icon_position: null,
  triangle_position: null,
  label: null,
  label_direction: null,
  label_sr_only: null,
  status: null,
  status_state: 'error',
  status_animation: null,
  global_status_id: null,
  suffix: null,
  scrollable: true,
  focusable: false,
  max_height: null,
  direction: 'auto',
  skip_portal: null,
  no_animation: false,
  no_scroll_animation: false,
  prevent_selection: false,
  more_menu: false,
  action_menu: false,
  independent_width: false,
  size: 'default',
  align_dropdown: null,
  trigger_component: null,
  data: null,
  default_value: null,
  value: 'initval',
  open_on_focus: false,
  prevent_close: false,
  keep_open: false,
  opened: false,
  disabled: null,
  stretch: null,
  skeleton: null,
  class: null,
  className: null,
  children: null,
  custom_element: null,
  custom_method: null,
  on_show: null,
  on_hide: null,
  on_change: null,
  on_select: null,
  on_state_update: null
};
process.env.NODE_ENV !== "production" ? Dropdown.propTypes = _extends({
  id: PropTypes.string,
  title: PropTypes.oneOfType([PropTypes.string, PropTypes.node]),
  icon: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.func]),
  icon_size: PropTypes.string,
  icon_position: PropTypes.oneOf(['left', 'right']),
  triangle_position: PropTypes.oneOf(['left', 'right']),
  label: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  label_direction: PropTypes.oneOf(['horizontal', 'vertical']),
  label_sr_only: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  status: PropTypes.oneOfType([PropTypes.string, PropTypes.bool, PropTypes.func, PropTypes.node]),
  status_state: PropTypes.string,
  status_animation: PropTypes.string,
  global_status_id: PropTypes.string,
  suffix: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  scrollable: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  focusable: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  direction: PropTypes.oneOf(['auto', 'top', 'bottom']),
  max_height: PropTypes.number,
  skip_portal: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  no_animation: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  no_scroll_animation: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  prevent_selection: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  more_menu: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  action_menu: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  independent_width: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  size: PropTypes.oneOf(['default', 'small', 'medium', 'large']),
  align_dropdown: PropTypes.oneOf(['left', 'right']),
  trigger_component: PropTypes.oneOfType([PropTypes.func, PropTypes.node]),
  data: PropTypes.oneOfType([PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node, PropTypes.object]), PropTypes.arrayOf(PropTypes.oneOfType([PropTypes.oneOfType([PropTypes.string, PropTypes.node]), PropTypes.shape({
    selected_value: PropTypes.oneOfType([PropTypes.string, PropTypes.node]),
    content: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.arrayOf(PropTypes.string)])
  })]))]),
  default_value: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  value: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  open_on_focus: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  prevent_close: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  keep_open: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  opened: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  disabled: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  stretch: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  class: PropTypes.string
}, spacingPropTypes, {
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node, PropTypes.object, PropTypes.array]),
  custom_element: PropTypes.object,
  custom_method: PropTypes.func,
  on_show: PropTypes.func,
  on_hide: PropTypes.func,
  on_change: PropTypes.func,
  on_select: PropTypes.func,
  on_state_update: PropTypes.func
}) : void 0;

var _ref = React.createElement(AlignmentHelper, null);

class DropdownInstance extends React.PureComponent {
  constructor(props) {
    super(props);

    this.setVisible = () => {
      this.context.drawerList.setWrapperElement(this._refShell.current).setVisible();
    };

    this.setHidden = (...args) => {
      this.context.drawerList.setHidden(...args);
    };

    this.onFocusHandler = () => {
      if (isTrue(this.props.open_on_focus)) {
        this.setVisible();
      }
    };

    this.onBlurHandler = () => {
      if (isTrue(this.props.open_on_focus)) {
        this.setHidden();
      }
    };

    this.toggleVisible = () => {
      if (!this.context.drawerList.hidden && this.context.drawerList.opened) {
        this.setHidden();
      } else {
        this.setVisible();
      }
    };

    this.onMouseDownHandler = () => {
      if (!this.context.drawerList.hidden && this.context.drawerList.opened) {
        this.setHidden();
      } else {
        this.setVisible();
      }
    };

    this.onTriggerKeyDownHandler = e => {
      switch (keycode(e)) {
        case 'enter':
        case 'space':
        case 'up':
        case 'down':
          e.preventDefault();
          this.setVisible();
          break;

        case 'esc':
          this.setHidden();
          break;

        case 'home':
        case 'end':
        case 'page down':
        case 'page up':
          e.preventDefault();
          break;
      }
    };

    this.onHideHandler = (args = {}) => {
      const attributes = this.attributes || {};
      const res = dispatchCustomElementEvent(this, 'on_hide', _extends({}, args, {
        attributes
      }));

      if (res !== false) {
        clearTimeout(this._focusTimeout);
        this._focusTimeout = setTimeout(() => {
          try {
            const element = this._refButton.current._ref.current;

            if (element && typeof element.focus === 'function') {
              element.focus({
                preventScroll: true
              });
              dispatchCustomElementEvent(this, 'on_hide_focus', {
                element
              });
            }
          } catch (e) {}
        }, 1);
      }

      return res;
    };

    this.onSelectHandler = args => {
      if (parseFloat(args.active_item) > -1) {
        const attributes = this.attributes || {};
        dispatchCustomElementEvent(this, 'on_select', _extends({}, args, {
          attributes
        }));
      }
    };

    this.onChangeHandler = args => {
      const attributes = this.attributes || {};
      dispatchCustomElementEvent(this, 'on_change', _extends({}, args, {
        attributes
      }));
    };

    this._id = props.id || makeUniqueId();
    this.attributes = {};
    this.state = this.state || {};
    this._ref = React.createRef();
    this._refShell = React.createRef();
    this._refButton = React.createRef();
    const dep = 'selected_item';

    if (typeof props[dep] !== 'undefined') {
      warn(`Dropdown: Please use "value" instead of "${dep}".`);
    }
  }

  componentDidMount() {
    if (isTrue(this.props.opened)) {
      this.setVisible();
    }
  }

  componentWillUnmount() {
    this.setHidden();
    clearTimeout(this._hideTimeout);
    clearTimeout(this._focusTimeout);
  }

  getTitle(title = null) {
    const {
      data
    } = this.context.drawerList;

    if (data && data.length > 0) {
      const currentOptionData = getCurrentData(this.context.drawerList.selected_item, data);

      if (currentOptionData) {
        title = currentOptionData.selected_value || parseContentTitle(currentOptionData);
      }
    }

    return title;
  }

  render() {
    var _this$context;

    const props = extendPropsWithContext(this.props, Dropdown.defaultProps, {
      skeleton: (_this$context = this.context) === null || _this$context === void 0 ? void 0 : _this$context.skeleton
    }, this.context.getTranslation(this.props).Dropdown, this.context.FormRow, this.context.Dropdown);

    const {
      label,
      label_direction,
      label_sr_only,
      icon_size,
      size,
      fixed_position,
      use_drawer_on_mobile,
      enable_body_lock,
      status,
      status_state,
      status_animation,
      global_status_id,
      suffix,
      scrollable,
      focusable,
      keep_open,
      prevent_close,
      no_animation,
      no_scroll_animation,
      triangle_position,
      skip_portal,
      trigger_component: CustomTrigger,
      more_menu,
      action_menu,
      independent_width,
      prevent_selection,
      max_height,
      default_value,
      className,
      class: _className,
      disabled,
      stretch,
      skeleton,
      title: _title
    } = props,
          attributes = _objectWithoutPropertiesLoose(props, ["label", "label_direction", "label_sr_only", "icon_size", "size", "fixed_position", "use_drawer_on_mobile", "enable_body_lock", "status", "status_state", "status_animation", "global_status_id", "suffix", "scrollable", "focusable", "keep_open", "prevent_close", "no_animation", "no_scroll_animation", "triangle_position", "skip_portal", "trigger_component", "more_menu", "action_menu", "independent_width", "prevent_selection", "max_height", "default_value", "className", "class", "disabled", "stretch", "skeleton", "title", "icon", "align_dropdown", "icon_position", "data", "children", "direction", "id", "opened", "value"]);

    let {
      icon,
      icon_position,
      align_dropdown
    } = props;
    const id = this._id;
    const handleAsMenu = isTrue(action_menu) || isTrue(more_menu) || isTrue(prevent_selection);
    const isPopupMenu = isTrue(more_menu) || !(_title && _title.length > 0);

    if (isPopupMenu) {
      icon = icon || (isTrue(more_menu) ? 'more' : 'chevron_down');
    }

    if (isPopupMenu || isTrue(action_menu)) {
      if (icon_position !== 'right' && align_dropdown !== 'right') {
        icon_position = 'left';
        align_dropdown = 'left';
      }
    }

    const {
      selected_item,
      direction,
      opened
    } = this.context.drawerList;
    const showStatus = getStatusState(status);
    const title = this.getTitle(_title);

    _extends(this.context.drawerList.attributes, validateDOMAttributes(null, attributes));

    const mainParams = {
      className: classnames(`dnb-dropdown dnb-dropdown--${direction} dnb-dropdown--icon-position-${icon_position || 'right'} dnb-dropdown--${align_dropdown || 'right'} dnb-form-component`, (isTrue(independent_width) || isTrue(action_menu)) && 'dnb-dropdown--independent-width', createSpacingClasses(props), _className, className, opened && 'dnb-dropdown--opened', label_direction && `dnb-dropdown--${label_direction}`, isPopupMenu && 'dnb-dropdown--is-popup', isTrue(action_menu) && `dnb-dropdown--action-menu`, size && `dnb-dropdown--${size}`, isTrue(stretch) && `dnb-dropdown--stretch`, status && `dnb-dropdown__status--${status_state}`, showStatus && 'dnb-dropdown__form-status')
    };

    const triggerParams = _extends({
      className: 'dnb-dropdown__trigger' + (opened ? " dnb-button--active" : ""),
      id,
      disabled,
      'aria-haspopup': handleAsMenu ? true : 'listbox',
      'aria-expanded': opened
    }, attributes, {
      onFocus: this.onFocusHandler,
      onBlur: this.onBlurHandler,
      onMouseDown: this.onMouseDownHandler,
      onKeyDown: this.onTriggerKeyDownHandler
    });

    if (opened) {
      triggerParams['aria-controls'] = `${id}-drawer-list`;
    }

    if (showStatus || suffix) {
      triggerParams['aria-describedby'] = combineDescribedBy(triggerParams, showStatus ? id + '-status' : null, suffix ? id + '-suffix' : null);
    }

    if (label) {
      triggerParams['aria-labelledby'] = [triggerParams['aria-labelledby'], id + '-label', id].filter(Boolean).join(' ');
    }

    validateDOMAttributes(null, mainParams);
    validateDOMAttributes(this.props, triggerParams);
    this.attributes = validateDOMAttributes(null, attributes);
    return React.createElement("span", mainParams, label && React.createElement(FormLabel, {
      id: id + '-label',
      for_id: id,
      text: label,
      label_direction: label_direction,
      sr_only: label_sr_only,
      disabled: disabled,
      skeleton: skeleton,
      onMouseDown: this.toggleVisible
    }), React.createElement("span", {
      className: "dnb-dropdown__inner",
      ref: this._ref
    }, _ref, showStatus && React.createElement(FormStatus, {
      id: id + '-form-status',
      global_status_id: global_status_id,
      label: label,
      text_id: id + '-status',
      text: status,
      status: status_state,
      animation: status_animation,
      skeleton: skeleton
    }), React.createElement("span", {
      className: "dnb-dropdown__row"
    }, React.createElement("span", {
      className: "dnb-dropdown__shell",
      ref: this._refShell
    }, CustomTrigger ? React.createElement(CustomTrigger, triggerParams) : React.createElement(Button, _extends({
      variant: "secondary",
      icon: false,
      size: size === 'default' ? 'medium' : size,
      ref: this._refButton
    }, triggerParams), !isPopupMenu && React.createElement("span", {
      className: "dnb-dropdown__text dnb-button__text"
    }, React.createElement("span", {
      className: "dnb-dropdown__text__inner"
    }, title)), React.createElement("span", {
      "aria-hidden": true,
      className: 'dnb-dropdown__icon' + (parseFloat(selected_item) === 0 ? " dnb-dropdown__icon--first" : "")
    }, icon !== false && React.createElement(Icon, {
      icon: icon || 'chevron_down',
      size: icon_size || (size === 'large' ? 'medium' : 'default')
    }))), React.createElement(DrawerList, {
      id: id,
      role: handleAsMenu ? 'menu' : 'listbox',
      inner_class: "dnb-dropdown__list",
      value: selected_item,
      default_value: default_value,
      scrollable: scrollable,
      focusable: focusable,
      no_animation: no_animation,
      no_scroll_animation: no_scroll_animation,
      skip_portal: skip_portal,
      prevent_selection: handleAsMenu,
      action_menu: action_menu,
      triangle_position: triangle_position || icon_position || 'right',
      keep_open: keep_open,
      prevent_close: prevent_close,
      independent_width: isTrue(independent_width) || isPopupMenu || action_menu,
      is_popup: isPopupMenu || action_menu,
      align_drawer: align_dropdown || 'right',
      fixed_position: fixed_position,
      use_drawer_on_mobile: use_drawer_on_mobile || action_menu,
      enable_body_lock: enable_body_lock,
      disabled: disabled,
      max_height: max_height,
      direction: direction,
      size: size,
      on_change: this.onChangeHandler,
      on_select: this.onSelectHandler,
      on_hide: this.onHideHandler
    })), suffix && React.createElement(Suffix, {
      className: "dnb-dropdown__suffix",
      id: id + '-suffix',
      context: props
    }, suffix))));
  }

}

DropdownInstance.defaultProps = Dropdown.defaultProps;
DropdownInstance.contextType = DrawerListContext;
process.env.NODE_ENV !== "production" ? DropdownInstance.propTypes = Dropdown.propTypes : void 0;
Dropdown.HorizontalItem = DrawerList.HorizontalItem;