import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { extendPropsWithContext, isTrue, registerElement, validateDOMAttributes, processChildren } from '../../shared/component-helper';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import { createSkeletonClass, skeletonDOMAttributes } from '../skeleton/SkeletonHelper';
import Context from '../../shared/Context';
export default class FormLabel extends React.PureComponent {
  static enableWebComponent() {
    registerElement(FormLabel.tagName, FormLabel, FormLabel.defaultProps);
  }

  static getContent(props) {
    if (props.text) return props.text;
    return processChildren(props);
  }

  render() {
    var _this$context;

    const props = extendPropsWithContext(this.props, FormLabel.defaultProps, {
      skeleton: (_this$context = this.context) === null || _this$context === void 0 ? void 0 : _this$context.skeleton
    }, this.context.FormRow, this.context.FormLabel);

    const {
      for_id,
      element,
      title,
      className,
      id,
      disabled,
      skeleton,
      label_direction,
      vertical,
      sr_only,
      class: _className
    } = props,
          attributes = _objectWithoutPropertiesLoose(props, ["for_id", "element", "title", "className", "id", "disabled", "skeleton", "label_direction", "vertical", "sr_only", "class", "text"]);

    const content = FormLabel.getContent(this.props);

    const params = _extends({
      className: classnames('dnb-form-label', (isTrue(vertical) || label_direction === 'vertical') && `dnb-form-label--vertical`, createSkeletonClass('font', skeleton, this.context), createSpacingClasses(props), className, _className, isTrue(sr_only) && 'dnb-form-label--sr-only'),
      htmlFor: for_id,
      id,
      title,
      disabled: isTrue(disabled)
    }, attributes);

    if (disabled) {
      params.disabled = true;
    }

    skeletonDOMAttributes(params, skeleton, this.context);
    validateDOMAttributes(this.props, params);
    params.children = content;
    const Element = element;
    return React.createElement(Element, params);
  }

}
FormLabel.tagName = 'dnb-form-label';
FormLabel.contextType = Context;
FormLabel.defaultProps = {
  for_id: null,
  element: 'label',
  title: null,
  text: null,
  id: null,
  class: null,
  disabled: null,
  skeleton: null,
  label_direction: null,
  vertical: null,
  sr_only: null,
  className: null,
  children: null
};
process.env.NODE_ENV !== "production" ? FormLabel.propTypes = _extends({
  for_id: PropTypes.string,
  element: PropTypes.string,
  title: PropTypes.string,
  text: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  id: PropTypes.string,
  class: PropTypes.string,
  disabled: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  label_direction: PropTypes.oneOf(['vertical', 'horizontal']),
  vertical: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  sr_only: PropTypes.oneOfType([PropTypes.string, PropTypes.bool])
}, spacingPropTypes, {
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node])
}) : void 0;