import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { isTrue, makeUniqueId, extend, registerElement, validateDOMAttributes, processChildren, dispatchCustomElementEvent } from '../../shared/component-helper';
import { createSpacingClasses } from '../space/SpacingHelper';
import Context from '../../shared/Context';
import hashSum from '../../shared/libs/HashSum';
import { formRowDefaultProps, formRowPropTypes } from '../form-row/FormRow';
export default class FormSet extends React.PureComponent {
  static enableWebComponent() {
    registerElement(FormSet.tagName, FormSet, FormSet.defaultProps);
  }

  static getContent(props) {
    return processChildren(props);
  }

  constructor(props) {
    super(props);

    this.onSubmitHandler = event => {
      const {
        prevent_submit
      } = this.props;

      if (isTrue(prevent_submit)) {
        event.preventDefault();
      }

      dispatchCustomElementEvent(this, 'on_submit', {
        event
      });
    };

    this._id = props.id || makeUniqueId();
  }

  render() {
    const _this$props = this.props,
          {
      element,
      no_form,
      disabled,
      className,
      class: _className
    } = _this$props,
          rest = _objectWithoutPropertiesLoose(_this$props, ["element", "no_form", "prevent_submit", "disabled", "id", "className", "class"]);

    const formRowProps = Object.entries(rest).reduce((acc, [k, v]) => {
      if (typeof formRowDefaultProps[k] !== 'undefined' && k !== 'id' && k !== 'children' && k !== 'label') {
        acc[k] = v;
      }

      return acc;
    }, {});
    const attributes = Object.entries(rest).reduce((acc, [k, v]) => {
      if (typeof formRowProps[k] === 'undefined') {
        acc[k] = v;
      }

      return acc;
    }, {});

    const params = _extends({
      className: classnames('dnb-form-set', createSpacingClasses(this.props), className, _className)
    }, attributes);

    if (!isTrue(no_form)) {
      params.onSubmit = this.onSubmitHandler;
    }

    validateDOMAttributes(this.props, params);
    const content = FormSet.getContent(this.props);

    if (hashSum(this._cachedContext) !== hashSum(this.context)) {
      this._cachedContext = this.context;

      const FormRow = _extends({}, formRowProps, {
        disabled
      });

      this._contextWeUse = extend(this.context, {
        FormRow
      });
    }

    const Element = isTrue(no_form) ? 'div' : element;
    return React.createElement(Context.Provider, {
      value: this._contextWeUse
    }, React.createElement(Element, params, content));
  }

}
FormSet.tagName = 'dnb-form-set';
FormSet.contextType = Context;
FormSet.defaultProps = {
  element: 'form',
  no_form: false,
  prevent_submit: false,
  on_submit: null,
  custom_element: null,
  custom_method: null
};
process.env.NODE_ENV !== "production" ? FormSet.propTypes = _extends({
  element: PropTypes.string,
  no_form: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  prevent_submit: PropTypes.oneOfType([PropTypes.string, PropTypes.bool])
}, formRowPropTypes, {
  on_submit: PropTypes.func,
  custom_element: PropTypes.object,
  custom_method: PropTypes.func
}) : void 0;