import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { isTrue, validateDOMAttributes, registerElement } from '../../shared/component-helper';
import '../../shared/helpers';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import HeadingContext from './HeadingContext';
import Context from '../../shared/Context';
import HeadingProvider from './HeadingProvider';
import { createSkeletonClass } from '../skeleton/SkeletonHelper';
import { correctHeadingLevel, resetLevels, resetAllLevels, setNextLevel, globalSyncCounter, globalHeadingCounter, windupHeadings, teardownHeadings, debugCounter } from './HeadingHelpers';
import { initCounter } from './HeadingCounter';
export const levelResolution = {
  1: 'xx-large',
  2: 'large',
  3: 'medium',
  4: 'basis',
  5: 'small',
  6: 'x-small'
};
export default class Heading extends React.PureComponent {
  static enableWebComponent() {
    registerElement(Heading.tagName, Heading, Heading.defaultProps);
  }

  static getDerivedStateFromProps(props, state) {
    if (state._listenForPropChanges) {
      const level = parseFloat(props.level);

      if (state.prevLevel !== props.level && level > 0 && level !== state.level || props.relevel) {
        var _state$context$headin, _state$context$headin2;

        state.counter.skipMakeMeReady();
        const {
          level: newLevel
        } = correctHeadingLevel({
          counter: state.counter,
          level,
          bypassChecks: isTrue(props.skip_correction) || isTrue((_state$context$headin = state.context.heading) === null || _state$context$headin === void 0 ? void 0 : _state$context$headin.skip_correction),
          source: props.text || props.children,
          debug: props.debug || ((_state$context$headin2 = state.context.heading) === null || _state$context$headin2 === void 0 ? void 0 : _state$context$headin2.debug)
        });
        state.level = state.prevLevel = newLevel;
      }
    }

    state._listenForPropChanges = true;
    return state;
  }

  constructor(props, context) {
    var _context$heading, _state$context$headin3, _state$context$headin4;

    super(props);
    this._ref = React.createRef();
    const state = {
      context,
      _listenForPropChanges: true
    };

    if ((_context$heading = context.heading) !== null && _context$heading !== void 0 && _context$heading.counter) {
      state.counter = initCounter(props);
      state.counter.setContextCounter(context.heading.counter);
      state.counter.isHeading = true;
    } else {
      state.counter = initCounter(props);
      state.counter.setContextCounter(globalHeadingCounter.current);
      state.counter.isHeading = true;
    }

    state.counter = correctHeadingLevel({
      ref: props,
      counter: state.counter,
      level: parseFloat(props.level),
      inherit: isTrue(props.inherit),
      reset: props.reset,
      increase: isTrue(props.increase) || isTrue(props.up),
      decrease: isTrue(props.decrease) || isTrue(props.down),
      bypassChecks: isTrue(props.skip_correction) || isTrue((_state$context$headin3 = state.context.heading) === null || _state$context$headin3 === void 0 ? void 0 : _state$context$headin3.skip_correction),
      source: props.text || props.children,
      debug: props.debug || ((_state$context$headin4 = state.context.heading) === null || _state$context$headin4 === void 0 ? void 0 : _state$context$headin4.debug)
    });
    globalSyncCounter.current = state.counter;
    state.level = state.counter.level;
    state.prevLevel = props.level;
    this.state = state;
  }

  componentDidMount() {
    windupHeadings();
    this.state.counter.windup();
  }

  componentWillUnmount() {
    teardownHeadings();
    this.state.counter.teardown();
  }

  render() {
    var _this$context$heading, _this$context$heading2;

    const _this$props = this.props,
          {
      text,
      debug: _debug,
      debug_counter: _debug_counter,
      size: _size,
      class: _className,
      className,
      children
    } = _this$props,
          rest = _objectWithoutPropertiesLoose(_this$props, ["text", "relevel", "group", "debug", "debug_counter", "reset", "skip_correction", "increase", "decrease", "up", "down", "inherit", "level", "size", "skeleton", "element", "class", "className", "children"]);

    let {
      size,
      element,
      skeleton
    } = this.props;
    const {
      level
    } = this.state;
    const debug = _debug || ((_this$context$heading = this.context.heading) === null || _this$context$heading === void 0 ? void 0 : _this$context$heading.debug);
    const debug_counter = _debug_counter || ((_this$context$heading2 = this.context.heading) === null || _this$context$heading2 === void 0 ? void 0 : _this$context$heading2.debug_counter);

    const attributes = _extends({}, rest);

    if (element === 'auto' || element === null) {
      element = `h${level || '6'}`;

      if (_size === 'auto' || _size === null) {
        size = levelResolution[level || '6'];
      }
    } else {
      if (!attributes.role) {
        attributes.role = 'heading';
      }

      if (!attributes['aria-level']) {
        attributes['aria-level'] = String(level);
      }
    }

    validateDOMAttributes(this.props, attributes);
    const Element = element;
    return React.createElement(Context.Consumer, null, context => {
      if (typeof (context === null || context === void 0 ? void 0 : context.skeleton) !== 'undefined') {
        skeleton = context.skeleton;
      }

      attributes.className = classnames(`dnb-heading dnb-h--${size}`, createSkeletonClass('font', skeleton, this.context), className, _className, createSpacingClasses(this.props));
      attributes.children = React.createElement(React.Fragment, null, debug && React.createElement("span", {
        className: "dnb-heading__debug"
      }, `[h${level || '6'}] `, debug_counter && React.createElement(React.Fragment, null, ' ', React.createElement("span", {
        className: "dnb-code"
      }, debugCounter(this.state.counter)))), text || children);
      return React.createElement(Element, _extends({
        ref: this._ref
      }, attributes));
    });
  }

}
Heading.tagName = 'dnb-heading';
Heading.contextType = HeadingContext;
Heading.defaultProps = {
  id: null,
  group: null,
  text: null,
  size: 'auto',
  level: null,
  increase: null,
  decrease: null,
  up: null,
  down: null,
  skip_correction: null,
  debug: null,
  debug_counter: null,
  counter: null,
  reset: null,
  inherit: null,
  skeleton: null,
  element: 'auto',
  class: null,
  className: null,
  children: null
};
process.env.NODE_ENV !== "production" ? Heading.propTypes = _extends({
  id: PropTypes.string,
  group: PropTypes.string,
  text: PropTypes.oneOfType([PropTypes.node, PropTypes.func]),
  size: PropTypes.oneOf(['auto', 'xx-large', 'x-large', 'large', 'medium', 'basis', 'small', 'x-small']),
  level: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
  increase: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  decrease: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  up: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  down: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  skip_correction: PropTypes.bool,
  debug: PropTypes.oneOfType([PropTypes.bool, PropTypes.func]),
  debug_counter: PropTypes.oneOfType([PropTypes.bool, PropTypes.func]),
  counter: PropTypes.any,
  inherit: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  reset: PropTypes.oneOfType([PropTypes.number, PropTypes.string, PropTypes.bool]),
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  element: PropTypes.string
}, spacingPropTypes, {
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.func])
}) : void 0;
Heading.Level = HeadingProvider;

Heading.Increase = props => React.createElement(HeadingProvider, _extends({
  increase: true
}, props));

Heading.Decrease = props => React.createElement(HeadingProvider, _extends({
  decrease: true
}, props));

Heading.Up = props => React.createElement(HeadingProvider, _extends({
  increase: true
}, props));

Heading.Down = props => React.createElement(HeadingProvider, _extends({
  decrease: true
}, props));

Heading.Reset = props => {
  var _globalHeadingCounter;

  (_globalHeadingCounter = globalHeadingCounter.current) === null || _globalHeadingCounter === void 0 ? void 0 : _globalHeadingCounter.reset();
  return React.createElement(HeadingProvider, props);
};

Heading.resetLevels = resetLevels;
Heading.setNextLevel = setNextLevel;
export { resetAllLevels, resetLevels, setNextLevel };