import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import PropTypes from 'prop-types';
import { isTrue } from '../../shared/component-helper';
import HeadingContext from './HeadingContext';
import { globalSyncCounter, globalHeadingCounter, correctHeadingLevel, windupHeadings, teardownHeadings, debugCounter } from './HeadingHelpers';
import { initCounter } from './HeadingCounter';
export default class HeadingProvider extends React.PureComponent {
  static getDerivedStateFromProps(props, state) {
    if (state._listenForPropChanges) {
      const level = parseFloat(props.level);

      if (state.prevLevel !== props.level && level > 0 && level !== state.level || props.relevel) {
        var _state$context$headin, _state$context$headin2;

        const {
          level: newLevel
        } = correctHeadingLevel({
          counter: state.counter,
          level,
          bypassChecks: isTrue(props.skip_correction) || isTrue((_state$context$headin = state.context.heading) === null || _state$context$headin === void 0 ? void 0 : _state$context$headin.skip_correction),
          source: props.text || props.children,
          debug: props.debug || ((_state$context$headin2 = state.context.heading) === null || _state$context$headin2 === void 0 ? void 0 : _state$context$headin2.debug)
        });
        state.level = state.prevLevel = newLevel;
      }

      state._listenForPropChanges = true;
      return state;
    }
  }

  constructor(props, context) {
    super(props);
    const state = {
      context,
      _listenForPropChanges: true
    };
    const existingContext = context.heading;
    state.counter = initCounter(props);
    state.newProps = props;

    if (existingContext) {
      state.newProps = _extends({}, existingContext, props);
    }

    if (existingContext) {
      state.counter.setContextCounter(existingContext.counter);
    } else {
      state.counter.setContextCounter(globalHeadingCounter.current);
    }

    state.counter = correctHeadingLevel({
      ref: props,
      counter: state.counter,
      level: parseFloat(props.level),
      inherit: isTrue(props.inherit),
      reset: props.reset,
      increase: isTrue(props.increase) || isTrue(props.up),
      decrease: isTrue(props.decrease) || isTrue(props.down),
      bypassChecks: isTrue(state.newProps.skip_correction),
      source: props.text || props.children,
      debug: state.newProps.debug
    });
    globalSyncCounter.current = state.counter;
    state.level = state.counter.level;
    state.prevLevel = state.newProps.level || state.counter.level;
    this.state = state;
  }

  componentDidMount() {
    windupHeadings();
  }

  componentWillUnmount() {
    teardownHeadings();
  }

  render() {
    return React.createElement(HeadingContext.Provider, {
      value: {
        heading: _extends({}, this.state.newProps, this.state)
      }
    }, this.state.newProps.debug_counter && React.createElement("span", {
      className: "dnb-heading__context"
    }, React.createElement("span", {
      className: "dnb-heading__debug"
    }, "Context:", ' ', React.createElement("span", {
      className: "dnb-code"
    }, debugCounter(this.state.counter))), this.props.children) || this.props.children);
  }

}
HeadingProvider.contextType = HeadingContext;
HeadingProvider.defaultProps = {
  id: null,
  group: null,
  level: null,
  increase: null,
  decrease: null,
  up: null,
  down: null,
  inherit: null,
  counter: null,
  reset: null,
  text: null,
  children: null
};
process.env.NODE_ENV !== "production" ? HeadingProvider.propTypes = {
  id: PropTypes.string,
  group: PropTypes.string,
  level: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
  increase: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  decrease: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  up: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  down: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  inherit: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  counter: PropTypes.any,
  reset: PropTypes.oneOfType([PropTypes.number, PropTypes.string, PropTypes.bool]),
  text: PropTypes.any,
  children: PropTypes.any
} : void 0;