import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import { registerElement } from '../../shared/component-helper';
import Context from '../../shared/Context';
import Modal from '../modal/Modal';
import HelpButtonInstance from './HelpButtonInstance';
import { buttonPropTypes } from '../button/Button';
export default class HelpButton extends React.PureComponent {
  static enableWebComponent() {
    registerElement(HelpButton.tagName, HelpButton, HelpButton.defaultProps);
  }

  static getContent(props) {
    if (props.modal_content) {
      return props.modal_content;
    }

    return typeof props.children === 'function' ? props.children(props) : props.children;
  }

  render() {
    const _this$props = this.props,
          {
      modal_props
    } = _this$props,
          params = _objectWithoutPropertiesLoose(_this$props, ["modal_content", "children", "modal_props"]);

    const content = HelpButton.getContent(this.props);

    if (params.icon === null) {
      params.icon = 'question';
    }

    if (content) {
      if (!params.title) {
        params.title = this.context.getTranslation(this.props).HelpButton.title;
      }

      return React.createElement(Modal, _extends({
        trigger_attributes: params
      }, modal_props), content);
    }

    return React.createElement(HelpButtonInstance, params);
  }

}
HelpButton.contextType = Context;
HelpButton.tagName = 'dnb-help-button';
HelpButton.defaultProps = {
  variant: 'secondary',
  icon: null,
  icon_position: 'left',
  modal_content: null,
  modal_props: null
};
process.env.NODE_ENV !== "production" ? HelpButton.propTypes = _extends({}, buttonPropTypes, {
  icon: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.func]),
  icon_position: PropTypes.oneOf(['left', 'right']),
  modal_content: PropTypes.node,
  modal_props: PropTypes.object
}) : void 0;