import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { ErrorHandler } from '../../shared/error-helper';
import { isTrue, registerElement, validateDOMAttributes, processChildren, extendPropsWithContext } from '../../shared/component-helper';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import { createSkeletonClass } from '../skeleton/SkeletonHelper';
import Context from '../../shared/Context';
export const DefaultIconSize = 16;
export const DefaultIconSizes = {
  default: 16,
  medium: 24
};
export const ListDefaultIconSizes = [['default', 16], ['medium', 24]];
export const ValidIconSizes = ['small', 'default', 'medium', 'large', 'x-large', 'xx-large'];
export const iconPropTypes = _extends({
  icon: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.func]),
  modifier: PropTypes.string,
  size: PropTypes.oneOfType([PropTypes.number, PropTypes.string, PropTypes.oneOf(['default', 'medium', 'large'])])
}, spacingPropTypes, {
  width: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  height: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  border: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  color: PropTypes.string,
  inherit_color: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  alt: PropTypes.string,
  title: PropTypes.string,
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  attributes: PropTypes.oneOfType([PropTypes.string, PropTypes.object]),
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.func])
});
export default class Icon extends React.PureComponent {
  static enableWebComponent(tag = Icon.tagName, inst = Icon, props = Icon.defaultProps) {
    registerElement(tag, inst, props);
  }

  static getIcon(props) {
    if (props.icon) {
      return props.icon;
    }

    return processChildren(props);
  }

  render() {
    var _this$context;

    const props = extendPropsWithContext(this.props, Icon.defaultProps, {
      skeleton: (_this$context = this.context) === null || _this$context === void 0 ? void 0 : _this$context.skeleton
    }, this.context.FormRow, this.context.Icon);
    const {
      icon,
      size,
      wrapperParams,
      iconParams,
      alt
    } = prepareIcon(props, this.context);
    const IconContainer = prerenderIcon({
      icon,
      size,
      alt
    });
    if (!IconContainer) return React.createElement(React.Fragment, null);
    return React.createElement("span", wrapperParams, React.createElement(IconContainer, iconParams));
  }

}
Icon.tagName = 'dnb-icon';
Icon.contextType = Context;
Icon.defaultProps = {
  icon: null,
  modifier: null,
  size: null,
  width: null,
  height: null,
  border: null,
  color: null,
  inherit_color: true,
  alt: null,
  title: null,
  skeleton: null,
  attributes: null,
  className: null,
  children: null
};
process.env.NODE_ENV !== "production" ? Icon.propTypes = _extends({}, iconPropTypes) : void 0;
export const getIconNameFromComponent = icon => {
  const name = typeof icon === 'string' ? icon : icon && (icon.displayName || icon.name);

  if (/^data:image\//.test(name)) {
    return null;
  }

  return name;
};
export const calcSize = props => {
  const {
    icon,
    size,
    width,
    height
  } = props;
  let sizeAsInt = -1;
  let sizeAsString = null;

  if (!size || size === DefaultIconSize) {
    const name = getIconNameFromComponent(icon);
    const nameParts = String(name || '').split('_');

    if (nameParts.length > 1) {
      const lastPartOfIconName = nameParts.reverse()[0];
      const potentialSize = ListDefaultIconSizes.filter(([key]) => key === lastPartOfIconName).reduce((acc, [key, value]) => {
        return key && value;
      }, null);

      if (potentialSize) {
        sizeAsInt = potentialSize;
      }

      if (ValidIconSizes.includes(lastPartOfIconName)) {
        sizeAsString = lastPartOfIconName;
      }
    } else {
      if (typeof icon === 'function') {
        const elem = icon();

        if (elem.props) {
          let potentialSize = -1;

          if (elem.props.width) {
            potentialSize = elem.props.width;
          }

          if (!potentialSize && elem.props.viewBox) {
            potentialSize = /[0-9]+ [0-9]+ ([0-9]+)/.exec(elem.props.viewBox)[1];
          }

          if (potentialSize) {
            sizeAsInt = potentialSize;
          }
        }
      }
    }
  } else if (typeof size === 'string' && !(parseFloat(size) > 0)) {
      sizeAsInt = ListDefaultIconSizes.filter(([key]) => key === size).reduce((acc, [key, value]) => {
        return key && value;
      }, -1);

      if (ValidIconSizes.includes(size)) {
        sizeAsString = size;
      }
    } else if (parseFloat(size) > 0) {
        sizeAsInt = ListDefaultIconSizes.filter(([key, value]) => key && value === parseFloat(size)).reduce((acc, [key, value]) => {
          if (key && value) return value;
          return acc;
        }, -1);

        if (sizeAsInt === -1) {
          sizeAsInt = parseFloat(size);
          sizeAsString = 'custom-size';
        }
      }

  if (!sizeAsString && sizeAsInt > 0) {
    const potentialSizeAsString = ListDefaultIconSizes.reduce((acc, [key, value]) => {
      if (key && value === sizeAsInt) {
        return key;
      }

      return acc;
    }, null);

    if (potentialSizeAsString) {
      sizeAsString = potentialSizeAsString;
    }
  }

  const {
    sizeAsString: isCustomSize,
    params: iconParams
  } = prepareIconParams({
    sizeAsString,
    sizeAsInt,
    size,
    width,
    height
  });

  if (isCustomSize) {
    sizeAsString = isCustomSize;
  }

  if (!(sizeAsInt > 0)) {
    sizeAsInt = DefaultIconSize;
  }

  if (size === 'auto') {
    iconParams.width = '100%';
    iconParams.height = '100%';
    sizeAsString = 'auto';
  }

  return {
    iconParams,
    sizeAsInt,
    sizeAsString
  };
};

const prepareIconParams = (_ref) => {
  let {
    sizeAsString
  } = _ref,
      rest = _objectWithoutPropertiesLoose(_ref, ["sizeAsString"]);

  const {
    size,
    width,
    height,
    sizeAsInt
  } = rest;
  const params = {};

  if (!sizeAsString && !(sizeAsInt > 0) && parseFloat(size) > -1) {
    params.width = params.height = parseFloat(size);
  } else if (sizeAsString === 'custom-size') {
    params.width = params.height = parseFloat(sizeAsInt);
  }

  if (parseFloat(width) > -1) {
    sizeAsString = 'custom-size';
    params.width = parseFloat(width);
  }

  if (parseFloat(height) > -1) {
    sizeAsString = 'custom-size';
    params.height = parseFloat(height);
  }

  validateDOMAttributes({}, params);
  return {
    params,
    sizeAsString
  };
};

export const prepareIcon = (props, context) => {
  const {
    icon,
    size,
    width,
    height,
    border,
    color,
    inherit_color,
    modifier,
    alt,
    title,
    skeleton,
    class: _className,
    className
  } = props,
        attributes = _objectWithoutPropertiesLoose(props, ["icon", "size", "width", "height", "border", "color", "inherit_color", "modifier", "alt", "title", "skeleton", "class", "className"]);

  const {
    sizeAsString,
    iconParams
  } = calcSize({
    icon,
    size,
    width,
    height
  });

  if (color) {
    iconParams.color = color;
  }

  const label = icon ? getIconNameFromComponent(icon) : null;
  const wrapperParams = validateDOMAttributes(props, _extends({
    role: alt ? 'img' : 'presentation',
    alt,
    'aria-label': label ? label.replace(/_/g, ' ') + ' icon' : null,
    title
  }, attributes));

  if (!alt && typeof wrapperParams['aria-hidden'] === 'undefined') {
    wrapperParams['aria-hidden'] = true;
  }

  wrapperParams.className = classnames('dnb-icon', sizeAsString ? `dnb-icon--${sizeAsString}` : `dnb-icon--default`, createSkeletonClass(null, skeleton, context), createSpacingClasses(props), _className, className, modifier && `dnb-icon--${modifier}`, isTrue(border) && 'dnb-icon--border', isTrue(inherit_color) && 'dnb-icon--inherit-color');
  let iconToRender = Icon.getIcon(props);

  if (typeof iconToRender.defaultProps !== 'undefined') {
    iconToRender = React.createElement(iconToRender, validateDOMAttributes({}, {
      color,
      icon,
      size,
      width,
      height
    }));
  }

  return _extends({}, props, {
    icon: iconToRender,
    alt,
    iconParams,
    wrapperParams
  });
};
export const prerenderIcon = ({
  icon,
  size = null,
  listOfIcons = null,
  alt = null
} = {}) => {
  if (typeof icon === 'string' && /^data:image\//.test(icon)) {
    return () => React.createElement("img", {
      src: icon,
      alt: alt || 'no-alt'
    });
  }

  if (typeof icon === 'function') {
    const elem = icon();

    if (React.isValidElement(elem)) {
      return icon;
    }

    return elem;
  }

  if (React.isValidElement(icon) || Array.isArray(icon)) {
    return () => icon;
  }

  try {
    icon = iconCase(icon);

    if (size && DefaultIconSizes[size] && size !== 'default' && !(parseFloat(size) > 0) && !icon.includes(size)) {
      icon = `${icon}_${size}`;
    }

    const mod = (listOfIcons.dnbIcons ? listOfIcons.dnbIcons : listOfIcons)[icon];
    return mod && mod.default ? mod.default : mod;
  } catch (e) {
    new ErrorHandler(`Icon '${icon}' did not exist!`);
    return null;
  }
};
export const iconCase = name => name.replace(/((?!^)[A-Z])/g, '_$1').toLowerCase().replace(/^[0-9]/g, '$1').replace(/[^a-z0-9_]/gi, '_');