import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import Context from '../../shared/Context';
import Input, { SubmitButton } from './Input';
import { makeUniqueId, extendPropsWithContext, convertStatusToStateOnly, combineDescribedBy, dispatchCustomElementEvent } from '../../shared/component-helper';
import IconView from '../../icons/view';
import IconViewOff from '../../icons/hide';
import IconViewMedium from '../../icons/view_medium';
import IconViewOffMedium from '../../icons/hide_medium';
export default class InputPassword extends React.PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      hidden: true
    };

    this.toggleVisibility = event => {
      const hidden = this.state.hidden;
      this.setState({
        hidden: !hidden
      }, () => {
        if (this._ref.current) {
          this._ref.current.focus();
        }
      });
      dispatchCustomElementEvent(this, hidden ? 'on_show_password' : 'on_hide_password', {
        event
      });
    };

    this._id = props.id || makeUniqueId();
    this._ref = props.inner_ref || React.createRef();
  }

  render() {
    var _this$context;

    const props = extendPropsWithContext(this.props, InputPassword.defaultProps, {
      skeleton: (_this$context = this.context) === null || _this$context === void 0 ? void 0 : _this$context.skeleton
    }, this.context.getTranslation(this.props).Input, this.context.FormRow, this.context.Input, this.context.InputPassword);
    const id = this._id;
    const params = {};
    params['aria-describedby'] = combineDescribedBy(this.props, id + '-submit-button');
    return React.createElement(Input, _extends({
      id: id
    }, this.props, params, {
      className: classnames('dnb-input--password', this.props.className),
      type: this.state.hidden ? 'password' : 'text',
      inner_ref: this._ref,
      submit_element: React.createElement(SubmitButton, {
        id: id + '-submit-button',
        type: "button",
        variant: "secondary",
        "aria-controls": id,
        "aria-label": this.state.hidden ? props.show_password : props.hide_password,
        icon: props.size === 'large' ? this.state.hidden ? IconViewMedium : IconViewOffMedium : this.state.hidden ? IconView : IconViewOff,
        skeleton: props.skeleton,
        status: convertStatusToStateOnly(props.status, props.status_state),
        status_state: props.status_state,
        onClick: this.toggleVisibility
      })
    }));
  }

}
InputPassword.contextType = Context;
InputPassword.defaultProps = _extends({
  show_password: null,
  hide_password: null,
  on_show_password: null,
  on_hide_password: null
}, Input.defaultProps);
process.env.NODE_ENV !== "production" ? InputPassword.propTypes = _extends({
  show_password: PropTypes.string,
  hide_password: PropTypes.string,
  on_show_password: PropTypes.func,
  on_hide_password: PropTypes.func
}, Input.propTypes) : void 0;