import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import Context from '../../shared/Context';
import { LOCALE, CURRENCY, CURRENCY_DISPLAY } from '../../shared/defaults';
import { warn, isTrue, makeUniqueId, validateDOMAttributes, registerElement, convertJsxToString, extend } from '../../shared/component-helper';
import { getSelectedText, copyToClipboard, insertElementBeforeSelection, hasSelectedText, IS_IOS, IS_MAC, IS_WIN, IS_IE11 } from '../../shared/helpers';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
const NUMBER_CHARS = '-0-9,.';
export default class Number extends React.PureComponent {
  static enableWebComponent() {
    registerElement(Number.tagName, Number, Number.defaultProps);
  }

  constructor(props) {
    super(props);

    this.shortcutHandler = () => {
      const fx = createSelectionFX(this.cleanedValue);
      fx.run();
    };

    this.onBlurHandler = () => {
      this.setState({
        selected: false
      });
    };

    this.onContextMenuHandler = () => {
      if (!hasSelectedText()) {
        clearTimeout(this._selectAllTimeout);
        this._selectAllTimeout = setTimeout(() => {
          this.setFocus();
        }, 1);
      }
    };

    this.onClickHandler = () => {
      if (!hasSelectedText()) {
        this.setFocus();
      }
    };

    this._ref = React.createRef();
    this._selectionRef = React.createRef();
    this._id = props.id || makeUniqueId();
    this.state = {
      selected: false
    };
  }

  componentDidMount() {
    clearTimeout(this._selectAllTimeout);

    if (IS_IOS && !hasiOSFix) {
      hasiOSFix = true;
      runIOSSelectionFix();
    }
  }

  setFocus() {
    this.setState({
      selected: true
    }, () => {
      if (this._selectionRef.current) {
        this._selectionRef.current.focus();
      }

      this.selectAll();
    });
  }

  selectAll() {
    try {
      const elem = this._selectionRef.current || this._ref.current;

      if (elem) {
        const selection = window.getSelection();
        const range = document.createRange();
        range.selectNodeContents(elem);
        selection.removeAllRanges();
        selection.addRange(range);
      }
    } catch (e) {
      warn(e);
    }
  }

  runFix(comp, className) {
    if (typeof comp === 'function') {
      comp = comp();
    }

    if (React.isValidElement(comp)) {
      return React.cloneElement(comp, {
        className: classnames(comp.props.className, className)
      });
    }

    return React.createElement("span", {
      className: className
    }, comp);
  }

  render() {
    const _this$props = this.props,
          {
      value: _value,
      prefix,
      suffix,
      children,
      currency,
      currency_display,
      currency_position,
      ban,
      nin,
      phone,
      org,
      link: _link,
      options,
      locale,
      decimals,
      omit_rounding,
      clean,
      selectall,
      copy_selection,
      element,
      class: _className,
      className
    } = _this$props,
          rest = _objectWithoutPropertiesLoose(_this$props, ["id", "value", "prefix", "suffix", "children", "currency", "currency_display", "currency_position", "ban", "nin", "phone", "org", "link", "options", "locale", "decimals", "omit_rounding", "clean", "selectall", "copy_selection", "element", "class", "className"]);

    let link = _link;
    let value = _value;

    if (children !== null) {
      value = children;
    }

    const formatOptions = {
      locale,
      currency,
      currency_display,
      currency_position,
      ban,
      nin,
      phone,
      org,
      decimals,
      omit_rounding: isTrue(omit_rounding),
      options,
      clean: isTrue(clean),
      returnAria: true
    };

    if (this.context) {
      const useContext = extend(true, {
        locale: null,
        currency: null
      }, this.context, this.context.getTranslation(this.props).Number);

      if (useContext) {
        if (useContext.locale && !locale) {
          formatOptions.locale = useContext.locale;
        }

        if (useContext.currency && isTrue(currency)) {
          formatOptions.options = formatOptions.options ? _extends({}, formatOptions.options) : {};
          formatOptions.options.currency = useContext.currency;
        }
      }
    }

    let {
      cleanedValue,
      number: display,
      aria,
      locale: lang
    } = format(value, formatOptions);
    this.cleanedValue = cleanedValue;

    const attributes = _extends({
      ref: this._ref,
      className: classnames('dnb-number', className, _className, (isTrue(currency) || typeof currency === 'string') && 'dnb-number--currency', createSpacingClasses(this.props), isTrue(selectall) && 'dnb-number--selectall', this.state.selected && 'dnb-number--selected', link && 'dnb-anchor')
    }, rest);

    attributes['role'] = 'text';
    const displayParams = {};

    if (isTrue(selectall) || isTrue(copy_selection)) {
      displayParams.onClick = this.onClickHandler;
      displayParams.onContextMenu = this.onContextMenuHandler;
    }

    validateDOMAttributes(this.props, attributes);

    if (prefix) {
      display = React.createElement(React.Fragment, null, this.runFix(prefix, 'dnb-number__prefix'), " ", display);
      aria = String(`${convertJsxToString(this.runFix(prefix, 'dnb-number__prefix'))} ${aria}`);
    }

    if (suffix) {
      display = React.createElement(React.Fragment, null, display, " ", this.runFix(suffix, 'dnb-number__suffix'));
      aria = `${aria} ${convertJsxToString(this.runFix(suffix, 'dnb-number__suffix'))}`;
    }

    if (link) {
      if (isTrue(link)) {
        link = 'tel';
      }

      return React.createElement("a", _extends({
        href: `${link}:${display}`
      }, attributes), display);
    }

    const Element = element;
    return React.createElement(Element, _extends({
      lang: lang
    }, attributes), React.createElement("span", _extends({
      className: "dnb-number__visible",
      "aria-describedby": this._id,
      "aria-hidden": true
    }, displayParams), display), React.createElement("span", {
      id: this._id,
      className: "dnb-number__sr-only dnb-sr-only--inline"
    }, aria), isTrue(copy_selection) && React.createElement("span", {
      className: "dnb-number__selection dnb-no-focus",
      ref: this._selectionRef,
      tabIndex: -1,
      onBlur: this.onBlurHandler,
      onCopy: this.shortcutHandler,
      "aria-hidden": true
    }, cleanedValue));
  }

}
Number.tagName = 'dnb-number';
Number.contextType = Context;
Number.defaultProps = {
  id: null,
  value: null,
  locale: null,
  prefix: null,
  suffix: null,
  currency: null,
  currency_display: null,
  currency_position: null,
  ban: null,
  nin: null,
  phone: null,
  org: null,
  link: null,
  options: null,
  decimals: null,
  selectall: true,
  copy_selection: true,
  omit_rounding: null,
  clean: null,
  element: 'span',
  class: null,
  className: null,
  children: null
};
process.env.NODE_ENV !== "production" ? Number.propTypes = _extends({
  id: PropTypes.string,
  value: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
  locale: PropTypes.string,
  prefix: PropTypes.oneOfType([PropTypes.node, PropTypes.func]),
  suffix: PropTypes.oneOfType([PropTypes.node, PropTypes.func]),
  currency: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  currency_display: PropTypes.string,
  currency_position: PropTypes.oneOf(['auto', 'before', 'after']),
  ban: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  nin: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  phone: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  org: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  link: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  options: PropTypes.oneOfType([PropTypes.object, PropTypes.string]),
  decimals: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
  selectall: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  copy_selection: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  omit_rounding: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  clean: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  element: PropTypes.oneOfType([PropTypes.string, PropTypes.bool])
}, spacingPropTypes, {
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.func])
}) : void 0;
export const format = (value, {
  locale = null,
  clean = false,
  phone = null,
  org = null,
  ban = null,
  nin = null,
  currency = null,
  currency_display = CURRENCY_DISPLAY,
  currency_position = null,
  decimals = null,
  omit_rounding = null,
  options = null,
  returnAria = false
} = {}) => {
  let display = value;
  let aria = null;
  let type = 'number';
  const isCurrency = isTrue(currency) || typeof currency === 'string';

  if (!locale) {
    locale = LOCALE;
  } else if (locale === 'auto') {
    try {
      locale = window.navigator.language;
    } catch (e) {
      warn(e);
    }
  }

  const opts = (typeof options === 'string' && options[0] === '{' ? JSON.parse(options) : options) || {};
  let deci = parseFloat(decimals);

  if (isCurrency && isNaN(deci)) {
    deci = 2;
  }

  if (deci >= 0) {
    const isNumber = typeof value === 'number';
    opts.minimumFractionDigits = deci;
    opts.maximumFractionDigits = deci;
    value = String(clean ? cleanNumber(value) : value);
    const pos = value.indexOf('.');

    if (pos > 0 && omit_rounding === true) {
      value = String(value).substr(0, pos + 1 + deci);
    }

    if (isNumber) {
      value = parseFloat(value);
    }
  } else {
    opts.maximumFractionDigits = 20;
  }

  if (isTrue(phone)) {
    type = 'phone';
    const {
      number: _number,
      aria: _aria
    } = formatPhone(value, locale);
    value = cleanNumber(value);
    display = _number;
    aria = _aria;
  } else if (isTrue(ban)) {
    type = 'ban';
    const {
      number: _number,
      aria: _aria
    } = formatBAN(value, locale);
    display = _number;
    aria = _aria;
  } else if (isTrue(nin)) {
    type = 'nin';
    const {
      number: _number,
      aria: _aria
    } = formatNIN(value, locale);
    display = _number;
    aria = _aria;
  } else if (isTrue(org)) {
    type = 'org';
    const {
      number: _number,
      aria: _aria
    } = formatORG(value, locale);
    display = _number;
    aria = _aria;
  } else if (isCurrency) {
    type = 'currency';
    let cleanedNumber = deci >= 0 ? value : clean ? cleanNumber(value) : value;
    opts.currency = opts.currency || (isTrue(currency) ? CURRENCY : currency);
    opts.style = 'currency';
    opts.currencyDisplay = opts.currencyDisplay || currency_display || CURRENCY_DISPLAY;

    if (typeof opts.minimumFractionDigits === 'undefined' && String(value).indexOf('.') === -1 && cleanedNumber % 1 === 0) {
      opts.minimumFractionDigits = 0;
    }

    display = formatNumber(cleanedNumber, locale, opts);
    display = cleanupMinus(display);
    display = prepareCurrencyPosition(display, currency_position, locale);
    aria = formatNumber(cleanedNumber, locale, _extends({
      minimumFractionDigits: 0,
      maximumFractionDigits: 2
    }, opts, {
      currencyDisplay: 'name'
    }));
    aria = enhanceSR(cleanedNumber, aria, locale);

    if (IS_IE11) {
      display = display.replace(/^\((.*)\)$/, '-$1');
      aria = aria.replace(/^\((.*)\)$/, '-$1');
      display = cleanupMinus(display);
      aria = cleanupMinus(aria);
    }
  } else {
    display = formatNumber(value, locale, opts);
    display = cleanupMinus(display);
    aria = formatNumber(value, locale, _extends({}, opts, {
      minimumFractionDigits: 1,
      maximumFractionDigits: 20
    }));
    aria = enhanceSR(value, aria, locale);
  }

  if (aria === null) {
    aria = display;
  }

  const cleanedValue = formatNumber(value, locale, opts, ({
    type,
    value
  }) => {
    switch (type) {
      case 'currency':
      case 'group':
      case 'literal':
        return '';

      default:
        return value;
    }
  });
  return returnAria ? {
    value,
    cleanedValue,
    number: display,
    aria,
    locale,
    type
  } : display;
};

const prepareCurrencyPosition = (display, position = null, locale = null) => {
  if (!position && locale && /no$/i.test(locale)) {
    position = 'after';
  }

  if (position) {
    const sign = String(display).replace(new RegExp(`([${NUMBER_CHARS}])`, 'g'), '').trim();
    const signPos = String(display).indexOf(sign);
    let start = 0;
    let end = 0;

    if (signPos === 0) {
      start = sign.length;
      end = display.length;
    } else {
        end = signPos;
      }

    if (!(end > 0)) {
      return display;
    }

    const num = String(display).substr(start, end).trim();

    switch (position) {
      case 'before':
        display = `${sign} ${num}`;
        break;

      case 'after':
        display = `${num} ${sign}`;
        break;
    }
  }

  return display;
};

const cleanupMinus = display => {
  const reg = '^(-|−|‐|‒|–|—|―)';
  const first = display[0];
  const second = display[1];

  if (new RegExp(reg).test(first)) {
    if (parseFloat(second) > 0) {
      display = display.replace(new RegExp(reg + '(.*)'), '-$2');
    } else {
      display = display.replace(new RegExp(reg + '([^0-9]+)(.*)'), '$2-$3');
    }
  }

  return display;
};

const enhanceSR = (value, aria) => {
  if (IS_MAC && Math.abs(parseFloat(value)) <= 99999) {
    aria = String(aria).replace(/\s([0-9])/g, '$1');
  }

  aria = cleanupMinus(aria);
  return aria;
};

export const formatNumber = (number, locale, options = {}, formatter = null) => {
  try {
    if (typeof Intl !== 'undefined' && typeof Intl.NumberFormat === 'function') {
      const inst = Intl.NumberFormat(locale, options);

      if (formatter) {
        return inst.formatToParts(number).map(val => formatter(val)).reduce((str, part) => str + part);
      } else {
        return inst.format(number);
      }
    } else if (typeof Number !== 'undefined' && typeof Number.toLocaleString === 'function') {
      return parseFloat(number).toLocaleString(locale, options);
    }
  } catch (e) {
    warn(`Number could not be formatted: ${JSON.stringify([number, locale, options])}`, e);
  }

  return number;
};
export const formatPhone = (number, locale = null) => {
  let display = number;
  let aria = null;

  switch (locale) {
    default:
      {
        number = String(number).replace(/[^+0-9]/g, '');
        let code = '';

        if (number.length > 8 && number.substr(0, 2) !== '00') {
          number = '+' + number;
          number = number.replace(/^\+{2,}/, '+');
        }

        if (number[0] === '+') {
          code = number.substr(0, 3) + ' ';
          number = number.substr(3);
        } else if (number.substr(0, 2) === '00') {
          code = number.substr(0, 4) + ' ';
          number = number.substr(4);
        }

        code = code.replace('+', '00');
        const length = number.length;

        if (length === 8 && number[0] === '8') {
          display = code + number.split(/([0-9]{3})([0-9]{2})/).filter(s => s).join(' ');
        } else {
          if (length < 6) {
            display = code + number;
          } else {
            display = code + number.split(length === 6 ? /^(\+[0-9]{2})|([0-9]{3})/ : /^(\+[0-9]{2})|([0-9]{2})/).filter(s => s).join(' ');
          }
        }

        aria = code + number.split(/([0-9]{2})/).filter(s => s).join(' ');
      }
  }

  if (aria === null) {
    aria = display;
  }

  return {
    number: display,
    aria
  };
};
export const formatBAN = (number, locale = null) => {
  number = String(number).replace(/[^0-9]/g, '');
  let display = number;
  let aria = null;

  switch (locale) {
    default:
      {
        display = number.split(/([0-9]{4})([0-9]{2})([0-9]{1,})/).filter(s => s).join(' ');
        aria = number.split(/([0-9]{2})/).filter(s => s).join(' ');
      }
  }

  if (aria === null) {
    aria = display;
  }

  return {
    number: display,
    aria
  };
};
export const formatORG = (number, locale = null) => {
  number = String(number).replace(/[^0-9]/g, '');
  let display = number;
  let aria = null;

  switch (locale) {
    default:
      {
        display = number.split(/([0-9]{3})/).filter(s => s).join(' ');
        aria = number.split(/([0-9]{1})/).filter(s => s).join(' ');
      }
  }

  if (aria === null) {
    aria = display;
  }

  return {
    number: display,
    aria
  };
};
export const formatNIN = (number, locale = null) => {
  number = String(number).replace(/[^0-9]/g, '');
  let display = number;
  let aria = null;

  switch (locale) {
    default:
      {
        display = number.split(/([0-9]{6})/).filter(s => s).join(' ');
        aria = display.split(/([0-9]{2})([0-9]{2})([0-9]{2}) ([0-9]{1})([0-9]{1})([0-9]{1})([0-9]{1})([0-9]{1})/).filter(s => s).join(IS_WIN ? '. ' : ' ');
      }
  }

  if (aria === null) {
    aria = display;
  }

  return {
    number: display,
    aria
  };
};
export function cleanNumber(num) {
  if (typeof num === 'number') {
    return num;
  }

  num = String(num).trim();

  if (/^[^0-9-]/.test(num)) {
    num.replace(/^(^[^0-9-]+)/, '');
  }

  let usesThousand = ',';
  let usesDecimal = '\\.';

  if (/(\s)([0-9]{3})/.test(num)) {
    usesThousand = '\\s';
    usesDecimal = ',';
  } else if (/(\.)([0-9]{3})/.test(num) && !/([,'][0-9]{3})(\.)([0-9]{3})/.test(num)) {
        usesThousand = '\\.';
        usesDecimal = ",|·|'";
      } else if (/(,)([0-9]{3})/.test(num)) {
        usesThousand = ',';
        usesDecimal = '\\.|·';
      } else if (/(')([0-9]{3})/.test(num)) {
          usesThousand = "'";
          usesDecimal = '\\.|,';
        }

  const thousandReg = new RegExp(`([0-9]|)(${usesThousand})([0-9]{3})`, 'g');

  if (thousandReg.test(num)) {
    num = num.replace(thousandReg, '$1$3');
  }

  const decimalReg = new RegExp(`(${usesDecimal})([0-9]{1,2})`, 'g');

  if (decimalReg.test(num)) {
    num = num.replace(decimalReg, '.$2');
  }

  const decimalBackup = new RegExp(`(${usesDecimal})([0-9]{3,})`, 'g');

  if (decimalBackup.test(num)) {
    num = num.replace(decimalBackup, '.$2');
  }

  return num.replace(new RegExp(`([^${NUMBER_CHARS}])`, 'g'), '');
}
let hasiOSFix = false;
export function runIOSSelectionFix() {
  try {
    const selection = window.getSelection();
    const range = document.createRange();
    selection.removeAllRanges();
    selection.addRange(range);
  } catch (e) {}
}
export async function copyWithEffect(string) {
  let success = null;

  if (string) {
    try {
      const fx = createSelectionFX(string);
      success = await copyToClipboard(string);

      if (success === true) {
        fx.run();
      }
    } catch (e) {
      warn(e);
      success = e;
    }
  }

  return success;
}
export function createSelectionFX(string) {
  let height = 32;
  let left = 0;
  let top = 0;
  let elem;

  try {
    var _cR$, _cR$2, _cR$3;

    const cR = window.getSelection().getRangeAt(0).getClientRects();
    height = (_cR$ = cR[0]) === null || _cR$ === void 0 ? void 0 : _cR$.height;
    left = (_cR$2 = cR[0]) === null || _cR$2 === void 0 ? void 0 : _cR$2.left;
    top = (_cR$3 = cR[0]) === null || _cR$3 === void 0 ? void 0 : _cR$3.top;
  } catch (e) {}

  try {
    if (!(top > 0) && !(left > 0)) {
      const posElem = document.createElement('span');
      posElem.setAttribute('class', 'dnb-number__fx__selection');
      insertElementBeforeSelection(posElem);
      ({
        top,
        left
      } = posElem.getBoundingClientRect());
      top -= height / 1.333;
      posElem.parentElement.removeChild(posElem);
    }

    elem = document.createElement('span');
    elem.textContent = String(string);
    elem.setAttribute('class', 'dnb-number__fx dnb-core-style');
    elem.style.top = `${top}px`;
    elem.style.left = `${left + getSelectedText().length / 2}px`;
  } catch (e) {
    warn(e);
  }

  return new class SelectionFx {
    remove() {
      try {
        document.body.removeChild(elem);
      } catch (e) {}
    }

    run() {
      try {
        document.body.appendChild(elem);
        setTimeout(this.remove, 800);
      } catch (e) {
        warn(e);
      }
    }

  }();
}