import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import PaginationContext from './PaginationContext';
import PaginationProvider from './PaginationProvider';
import { registerElement, validateDOMAttributes, extendPropsWithContext } from '../../shared/component-helper';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import { PaginationIndicator } from './PaginationHelpers';
import InfinityScroller from './PaginationInfinity';
import PaginationBar from './PaginationBar';
const paginationPropTypes = process.env.NODE_ENV !== "production" ? _extends({
  startup_page: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  current_page: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  page_count: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  startup_count: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  parallel_load_count: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  place_maker_before_content: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  min_wait_time: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  mode: PropTypes.oneOf(['pagination', 'infinity']),
  use_load_button: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  items: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
  hide_progress_indicator: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  set_content_handler: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  reset_content_handler: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  reset_pagination_handler: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  end_infinity_handler: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  page_element: PropTypes.oneOfType([PropTypes.object, PropTypes.node, PropTypes.func, PropTypes.string]),
  fallback_element: PropTypes.oneOfType([PropTypes.object, PropTypes.node, PropTypes.func, PropTypes.string]),
  marker_element: PropTypes.oneOfType([PropTypes.object, PropTypes.node, PropTypes.func, PropTypes.string]),
  indicator_element: PropTypes.oneOfType([PropTypes.node, PropTypes.func, PropTypes.string]),
  align: PropTypes.string,
  button_title: PropTypes.string,
  prev_title: PropTypes.string,
  next_title: PropTypes.string,
  more_pages: PropTypes.string,
  is_loading_text: PropTypes.string,
  load_button_text: PropTypes.string
}, spacingPropTypes, {
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.func]),
  on_change: PropTypes.func,
  on_startup: PropTypes.func,
  on_load: PropTypes.func,
  on_end: PropTypes.func
}) : {};
const paginationDefaultProps = {
  startup_page: null,
  current_page: null,
  page_count: null,
  mode: 'pagination',
  use_load_button: false,
  items: null,
  hide_progress_indicator: false,
  set_content_handler: null,
  reset_content_handler: null,
  page_element: undefined,
  fallback_element: undefined,
  marker_element: undefined,
  indicator_element: undefined,
  align: 'left',
  button_title: null,
  prev_title: null,
  next_title: null,
  more_pages: null,
  is_loading_text: null,
  load_button_text: null,
  startup_count: 1,
  parallel_load_count: 1,
  place_maker_before_content: false,
  min_wait_time: 400,
  class: null,
  className: null,
  children: null,
  on_change: null,
  on_startup: null,
  on_load: null,
  on_end: null
};
export default class Pagination extends React.PureComponent {
  static enableWebComponent() {
    registerElement(Pagination.tagName, Pagination, paginationDefaultProps);
  }

  render() {
    return React.createElement(PaginationProvider, _extends({
      tagName: Pagination.tagName
    }, this.props), React.createElement(PaginationInstance, this.props));
  }

}
Pagination.tagName = 'dnb-pagination';
Pagination.defaultProps = paginationDefaultProps;
process.env.NODE_ENV !== "production" ? Pagination.propTypes = _extends({}, paginationPropTypes) : void 0;

var _ref = React.createElement(InfinityScroller, null);

class PaginationInstance extends React.PureComponent {
  constructor(props) {
    super(props);
    this._contentRef = React.createRef();
  }

  render() {
    const props = extendPropsWithContext(this.props, paginationDefaultProps, this.context.getTranslation(this.props).Pagination, this.context.FormRow, this.context.Pagination);

    const {
      align,
      children,
      className,
      class: _className
    } = props,
          attributes = _objectWithoutPropertiesLoose(props, ["align", "children", "className", "class", "tagName", "page_count", "current_page", "startup_page", "mode", "hide_progress_indicator", "use_load_button"]);

    const {
      currentPage,
      items,
      fallback_element,
      indicator_element
    } = this.context.pagination;

    if (this.context.pagination.mode === 'pagination') {
      var _items$find;

      const mainParams = _extends({
        className: classnames('dnb-pagination', createSpacingClasses(props), className, _className, align && `dnb-pagination--${align}`)
      }, attributes);

      validateDOMAttributes(props, mainParams);
      const content = (_items$find = items.find(({
        pageNo
      }) => pageNo === currentPage)) === null || _items$find === void 0 ? void 0 : _items$find.content;
      return React.createElement(React.Fragment, null, this.context.pagination.mode === 'infinity' && typeof children !== 'function' && children, React.createElement("div", mainParams, React.createElement(PaginationBar, {
        contentRef: this._contentRef
      }, children), items.length > 0 && React.createElement(PaginationContent, {
        ref: this._contentRef
      }, content || React.createElement(PaginationIndicator, {
        indicator_element: indicator_element || fallback_element
      }))));
    }

    return _ref;
  }

}

PaginationInstance.defaultProps = paginationDefaultProps;
PaginationInstance.contextType = PaginationContext;
process.env.NODE_ENV !== "production" ? PaginationInstance.propTypes = _extends({}, paginationPropTypes) : void 0;
export class InfinityMarker extends React.PureComponent {
  static enableWebComponent() {
    registerElement(InfinityMarker.tagName, InfinityMarker, paginationDefaultProps);
  }

  render() {
    const _this$props = this.props,
          {
      children
    } = _this$props,
          props = _objectWithoutPropertiesLoose(_this$props, ["children"]);

    return React.createElement(PaginationProvider, _extends({
      useMarkerOnly: true,
      tagName: InfinityMarker.tagName
    }, props), React.createElement(InfinityScroller, props, children));
  }

}
InfinityMarker.tagName = 'dnb-infinity-marker';
InfinityMarker.defaultProps = paginationDefaultProps;
process.env.NODE_ENV !== "production" ? InfinityMarker.propTypes = _extends({}, paginationPropTypes) : void 0;
const PaginationContent = React.forwardRef((_ref2, ref) => {
  let {
    children
  } = _ref2,
      props = _objectWithoutPropertiesLoose(_ref2, ["children"]);

  return React.createElement("div", _extends({
    className: "dnb-pagination__content dnb-no-focus",
    tabIndex: "-1"
  }, props, {
    ref: ref
  }), children);
});
process.env.NODE_ENV !== "production" ? PaginationContent.propTypes = {
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.func]).isRequired
} : void 0;
Pagination.Bar = PaginationBar;
Pagination.Content = PaginationContent;
const PaginationWrapper = Pagination;
const InfinityMarkerWrapper = InfinityMarker;
export const createPagination = (initProps = {}) => {
  const store = React.createRef({});
  const rerender = React.createRef(null);

  const _setContent = React.createRef(null);

  const _resetContent = React.createRef(null);

  const _resetInfinity = React.createRef(null);

  const _endInfinity = React.createRef(null);

  const setContent = (pageNo, content) => {
    if (pageNo > 0) {
      store.current = _extends({}, store.current, {
        pageNo,
        content
      });
      rerender.current && rerender.current(store);
    }
  };

  const resetContent = () => {
    _resetContent.current && _resetContent.current();
  };

  const resetInfinity = () => {
    _resetInfinity.current && _resetInfinity.current();
  };

  const endInfinity = () => {
    _endInfinity.current && _endInfinity.current();
  };

  const args = props => _extends({}, _extends({}, initProps, props), {
    store,
    rerender,
    set_content_handler: fn => _setContent.current = fn,
    reset_content_handler: fn => _resetContent.current = fn,
    reset_pagination_handler: fn => _resetInfinity.current = fn,
    end_infinity_handler: fn => _endInfinity.current = fn
  });

  const Pagination = props => React.createElement(PaginationWrapper, _extends({
    tagName: PaginationWrapper.tagName
  }, args(props)));

  const InfinityMarker = props => React.createElement(InfinityMarkerWrapper, _extends({
    tagName: InfinityMarkerWrapper.tagName
  }, args(props)));

  return {
    Pagination,
    InfinityMarker,
    setContent,
    resetContent,
    resetInfinity,
    endInfinity
  };
};