import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import PropTypes from 'prop-types';
import { dispatchCustomElementEvent, extendPropsWithContext } from '../../shared/component-helper';
import { calculatePagination, getDotsAriaLabel } from './PaginationCalculation';
import PaginationContext from './PaginationContext';
import Button from '../button/Button';

var _ref = React.createElement("div", {
  key: "dot-1"
});

var _ref2 = React.createElement("div", {
  key: "dot-2"
});

var _ref3 = React.createElement("div", {
  key: "dot-3"
});

export default class PaginationBar extends React.PureComponent {
  constructor(...args) {
    super(...args);

    this.setPage = (currentPage, event = null) => {
      this.keepPageHeight();
      this.focusPage();
      this.context.pagination.setState({
        currentPage
      });
      this.preparePageContent(currentPage);
      dispatchCustomElementEvent(this.context.pagination, 'on_change', _extends({
        page: currentPage,
        pageNo: currentPage
      }, this.context.pagination, {
        event
      }));
    };

    this.setPrevPage = () => {
      this.setPage(this.context.pagination.currentPage - 1);
    };

    this.setNextPage = () => {
      this.setPage(this.context.pagination.currentPage + 1);
    };

    this.clickHandler = ({
      pageNo,
      event
    }) => {
      this.setPage(pageNo, event);
    };
  }

  componentDidMount() {
    const pgn = this.context.pagination;
    const currentPage = pgn.startupPage || pgn.currentPage;
    pgn.setState({
      currentPage
    }, () => this.preparePageContent(currentPage));
  }

  componentDidUpdate({
    children
  }) {
    if (this.props.children !== children) {
      this.updatePageContent();
    }
  }

  hasChildrenCallabck() {
    return typeof this.props.children === 'function';
  }

  preparePageContent(pageNo) {
    let potentialElement = this.props.children;
    const items = this.context.pagination.prefillItems(this.context.pagination.currentPage, {
      skipObserver: true
    });
    this.context.pagination.setState({
      items
    });

    if (this.hasChildrenCallabck()) {
      potentialElement = this.props.children(_extends({
        pageNo,
        page: pageNo
      }, this.context.pagination));
    }

    if (potentialElement && React.isValidElement(potentialElement)) {
      this.context.pagination.setContent([pageNo, potentialElement]);
    }
  }

  updatePageContent(pageNo = this.context.pagination.currentPage) {
    let potentialElement = this.props.children;

    if (this.hasChildrenCallabck()) {
      potentialElement = this.props.children(_extends({
        pageNo,
        page: pageNo
      }, this.context.pagination));
    }

    if (potentialElement && React.isValidElement(potentialElement)) {
      this.context.pagination.setContent([pageNo, potentialElement]);
    }
  }

  focusPage() {
    this.context.pagination.onPageUpdate(() => {
      try {
        const elem = this.props.contentRef.current;
        elem.focus();
      } catch (e) {}
    });
  }

  keepPageHeight() {
    try {
      const elem = this.props.contentRef.current;
      const pageHeight = elem.offsetHeight;
      elem.style.height = `${pageHeight / 16}rem`;
      elem.style.minHeight = elem.style.height;
    } catch (e) {}

    this.context.pagination.onPageUpdate(() => {
      try {
        const elem = this.props.contentRef.current;
        elem.style.height = 'auto';
        elem.style.minHeight = elem.style.height;
      } catch (e) {}
    });
  }

  render() {
    const props = extendPropsWithContext(this.props, PaginationBar.defaultProps, this.context.getTranslation(this.props).Pagination);
    const {
      button_title,
      prev_title,
      next_title,
      more_pages
    } = props;
    const {
      pageCount,
      currentPage
    } = this.context.pagination;
    const prevIsDisabled = currentPage === 1;
    const nextIsDisabled = currentPage === pageCount || pageCount === 0;
    const pages = calculatePagination(pageCount, currentPage);
    return React.createElement("div", {
      className: 'dnb-pagination__bar' + (pageCount >= 8 ? " dnb-pagination--many-pages" : "")
    }, React.createElement(Button, {
      key: "left-arrow",
      className: "dnb-pagination__button",
      disabled: prevIsDisabled,
      size: "small",
      icon: "chevron_left",
      on_click: this.setPrevPage,
      title: prevIsDisabled ? null : prev_title
    }), React.createElement("div", {
      className: "dnb-pagination__bar__inner"
    }, pages[0].map(pageNo => React.createElement(Button, {
      key: pageNo,
      className: "dnb-pagination__button",
      size: "medium",
      text: String(pageNo),
      "aria-label": button_title.replace('%s', pageNo),
      variant: pageNo === currentPage ? 'primary' : 'secondary',
      "aria-current": pageNo === currentPage ? 'page' : null,
      on_click: event => this.clickHandler({
        pageNo,
        event
      })
    })), pages.slice(1).map((list, idx) => React.createElement(React.Fragment, {
      key: idx
    }, React.createElement("div", {
      key: `dots-${idx}`,
      className: "dnb-pagination__dots",
      "aria-label": getDotsAriaLabel({
        more_pages,
        list,
        pages
      })
    }, _ref, _ref2, _ref3), list.map(pageNo => React.createElement(Button, {
      key: pageNo,
      className: "dnb-pagination__button",
      size: "medium",
      text: String(pageNo),
      "aria-label": button_title.replace('%s', pageNo),
      variant: pageNo === currentPage ? 'primary' : 'secondary',
      "aria-current": pageNo === currentPage ? 'page' : null,
      on_click: event => this.clickHandler({
        pageNo,
        event
      })
    }))))), React.createElement(Button, {
      key: "right-arrow",
      className: "dnb-pagination__button",
      disabled: nextIsDisabled,
      size: "small",
      icon: "chevron_right",
      on_click: this.setNextPage,
      title: nextIsDisabled ? null : next_title
    }));
  }

}
PaginationBar.contextType = PaginationContext;
PaginationBar.defaultProps = {
  button_title: null,
  prev_title: null,
  next_title: null,
  more_pages: null,
  contentRef: null,
  children: null
};
process.env.NODE_ENV !== "production" ? PaginationBar.propTypes = {
  button_title: PropTypes.string,
  prev_title: PropTypes.string,
  next_title: PropTypes.string,
  more_pages: PropTypes.string,
  contentRef: PropTypes.object,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.func])
} : void 0;