import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import PropTypes from 'prop-types';
import ReactDOM from 'react-dom';
import { warn, isTrue, dispatchCustomElementEvent } from '../../shared/component-helper';
import Context from '../../shared/Context';
import Button from '../button/Button';
import { preparePageElement, isTrElement, PaginationIndicator } from './PaginationHelpers';
import PaginationContext from './PaginationContext';
export default class InfinityScroller extends React.PureComponent {
  constructor(_props, context) {
    super(_props);

    this.startup = () => {
      const {
        startupPage,
        startup_count
      } = this.context.pagination;
      const startupCount = parseFloat(startup_count);

      for (let i = 0, newPageNo; i < startupCount; ++i) {
        newPageNo = startupPage + i;
        this.getNewContent(newPageNo, {
          position: 'after',
          skipObserver: newPageNo < startupCount
        }, {
          isStartup: i === 0
        });
      }
    };

    this.getNewContent = (newPageNo, props = {}, {
      isStartup = false
    } = {}) => {
      const {
        pageCount,
        endInfinity
      } = this.context.pagination;

      if (newPageNo > pageCount) {
        return endInfinity();
      }

      const exists = this.context.pagination.items.findIndex(obj => {
        return obj.pageNo === newPageNo;
      }) > -1;

      if (exists) {
        return;
      }

      const items = this.context.pagination.prefillItems(newPageNo, props);
      this.context.pagination.setItems(items, () => this.callEventHandler(newPageNo, {
        isStartup
      }));
    };

    this.hideIndicator = isTrue(context.pagination.hide_progress_indicator);
    this.useLoadButton = isTrue(context.pagination.use_load_button);
    this.lastElement = React.createRef();
    this.callOnUnmount = [];
  }

  componentWillUnmount() {
    clearTimeout(this._startupTimeout);
    clearTimeout(this._bufferTimeout);
    this.callOnUnmount.forEach(f => typeof f === 'function' && f());
  }

  waitForReachedTime(fn, ...props) {
    this.callbackBuffer = this.callbackBuffer || [];
    this.callbackBuffer.push({
      fn,
      props
    });
    this.callBuffer();
  }

  callBuffer() {
    const {
      minTime
    } = this.context.pagination;
    const diff = this._lastCall > 0 ? new Date().getTime() - this._lastCall : 0;
    const waitTime = diff < minTime ? minTime : 0;
    clearTimeout(this._bufferTimeout);
    this._bufferTimeout = setTimeout(() => {
      if (this.callbackBuffer.length > 0) {
        this._lastCall = new Date().getTime();
        const {
          fn,
          props
        } = this.callbackBuffer.shift();
        fn(...props);
        this.callBuffer();
      }
    }, waitTime);
  }

  callEventHandler(pageNo, {
    isStartup = false,
    callOnEnd = false,
    onDispatch = null
  } = {}) {
    this.waitForReachedTime(({
      pageNo,
      isStartup
    }) => {
      const context = this.context.pagination;

      const createEvent = eventName => {
        const ret = dispatchCustomElementEvent(context, eventName, _extends({
          page: pageNo,
          pageNo
        }, context));

        if (typeof onDispatch === 'function') {
          onDispatch();
        }

        if (typeof ret === 'function') {
          this.callOnUnmount.push(ret);
        }
      };

      if (callOnEnd) {
        createEvent('on_end');
      } else {
        if (isStartup) {
          createEvent('on_startup');
        } else {
          createEvent('on_change');
        }

        createEvent('on_load');
      }
    }, {
      pageNo,
      isStartup
    });
  }

  handleInfinityMarker() {
    const {
      children
    } = this.props;
    const {
      lowerPage,
      upperPage,
      pageCount,
      hasEndedInfinity,
      parallelLoadCount,
      fallback_element,
      marker_element,
      indicator_element
    } = this.context.pagination;

    const Marker = () => React.createElement(InteractionMarker, {
      pageNo: upperPage,
      markerElement: marker_element || fallback_element,
      onVisible: pageNo => {
        for (let i = 0, newPageNo; i < parallelLoadCount; ++i) {
          newPageNo = pageNo + 1 + i;
          this.context.pagination.onPageUpdate(() => {
            this.context.pagination.setState({
              upperPage: newPageNo,
              skipObserver: i + 1 < parallelLoadCount
            });
          });
          this.callEventHandler(newPageNo);
        }
      }
    });

    const LoadButton = () => React.createElement(InfinityLoadButton, {
      icon: "arrow_up",
      element: fallback_element,
      pressed_element: React.createElement(PaginationIndicator, {
        indicator_element: indicator_element || fallback_element
      }),
      on_click: () => {
        const newPageNo = lowerPage - 1;
        this.context.pagination.onPageUpdate(() => {
          this.context.pagination.setState({
            lowerPage: newPageNo
          });
        });
        this.callEventHandler(newPageNo);
      }
    });

    return React.createElement(React.Fragment, null, lowerPage > 1 && React.createElement(LoadButton, null), children, !hasEndedInfinity && (typeof pageCount === 'undefined' || upperPage < pageCount) && React.createElement(Marker, null), !hasEndedInfinity && !this.hideIndicator && (typeof pageCount === 'undefined' || upperPage < pageCount) && React.createElement(PaginationIndicator, {
      indicator_element: indicator_element || fallback_element
    }));
  }

  render() {
    const {
      items,
      pageCount,
      startupPage,
      hasEndedInfinity,
      parallelLoadCount,
      placeMakerBeforeContent,
      page_element,
      fallback_element,
      marker_element,
      indicator_element
    } = this.context.pagination;

    if (this.context.pagination.useMarkerOnly) {
      return this.handleInfinityMarker();
    }

    if (!(items && items.length > 0)) {
      clearTimeout(this._startupTimeout);
      this._startupTimeout = setTimeout(this.startup, 1);
      return null;
    }

    const Element = preparePageElement(page_element || React.Fragment);
    return items.map(({
      pageNo,
      hasContent,
      content,
      ref,
      skipObserver,
      ScrollElement
    }, idx) => {
      const isLastItem = idx === items.length - 1;
      const Elem = hasContent && ScrollElement || Element;
      const marker = hasContent && !this.useLoadButton && !skipObserver && !hasEndedInfinity && (typeof pageCount === 'undefined' || pageNo <= pageCount) && React.createElement(InteractionMarker, {
        pageNo: pageNo,
        markerElement: marker_element || fallback_element,
        onVisible: pageNo => {
          for (let i = 0, newPageNo; i < parallelLoadCount; ++i) {
            newPageNo = pageNo + 1 + i;
            this.getNewContent(newPageNo, {
              position: 'after',
              skipObserver: i + 1 < parallelLoadCount
            });
          }
        }
      });
      return React.createElement(Elem, {
        key: pageNo,
        ref: ref
      }, hasContent && startupPage > 1 && pageNo > 1 && pageNo <= startupPage && React.createElement(InfinityLoadButton, {
        element: fallback_element,
        icon: "arrow_up",
        on_click: event => this.getNewContent(pageNo - 1, {
          position: 'before',
          skipObserver: true,
          event
        })
      }), placeMakerBeforeContent && marker, content, !placeMakerBeforeContent && marker, (parallelLoadCount > 1 && idx > 0 ? isLastItem : true) && !hasContent && !this.hideIndicator && React.createElement(PaginationIndicator, {
        indicator_element: indicator_element || fallback_element
      }), hasContent && this.useLoadButton && isLastItem && (typeof pageCount === 'undefined' || pageNo < pageCount) && React.createElement(InfinityLoadButton, {
        element: fallback_element,
        icon: "arrow_down",
        on_click: event => this.getNewContent(pageNo + 1, {
          position: 'after',
          skipObserver: true,
          ScrollElement: props => hasContent && React.createElement(ScrollToElement, _extends({
            page_element: page_element
          }, props)),
          event
        })
      }));
    });
  }

}
InfinityScroller.contextType = PaginationContext;
InfinityScroller.defaultProps = {
  children: null
};
process.env.NODE_ENV !== "production" ? InfinityScroller.propTypes = {
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.func])
} : void 0;

class InteractionMarker extends React.PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      isConnected: false
    };

    this.callReady = () => {
      var _this$intersectionObs;

      (_this$intersectionObs = this.intersectionObserver) === null || _this$intersectionObs === void 0 ? void 0 : _this$intersectionObs.disconnect();
      this.intersectionObserver = null;
      clearTimeout(this._readyTimeout);
      this._readyTimeout = setTimeout(() => {
        if (this._isMounted) {
          this.setState({
            isConnected: true
          });
        }

        this.props.onVisible(this.props.pageNo);
      }, 1);
    };

    if (typeof props.markerElement === 'function') {
      warn('Pagination: Please use a string or React element e.g. marker_element="tr"');
    }

    this._ref = React.createRef();

    if (typeof IntersectionObserver !== 'undefined') {
      this.intersectionObserver = new IntersectionObserver(entries => {
        const [{
          isIntersecting
        }] = entries;

        if (isIntersecting) {
          this.callReady();
        }
      });
    } else {
      warn('Pagination is missing IntersectionObserver supported!');
    }
  }

  componentDidMount() {
    if (this._ref.current) {
      var _this$intersectionObs2;

      this._isMounted = true;
      (_this$intersectionObs2 = this.intersectionObserver) === null || _this$intersectionObs2 === void 0 ? void 0 : _this$intersectionObs2.observe(this._ref.current);
    }
  }

  componentWillUnmount() {
    var _this$intersectionObs3;

    this._isMounted = false;
    clearTimeout(this._readyTimeout);
    (_this$intersectionObs3 = this.intersectionObserver) === null || _this$intersectionObs3 === void 0 ? void 0 : _this$intersectionObs3.disconnect();
  }

  render() {
    const {
      markerElement
    } = this.props;

    if (this.state.isConnected || !this.intersectionObserver) {
      return null;
    }

    const Element = markerElement && isTrElement(markerElement) ? 'tr' : 'div';
    const ElementChild = markerElement && isTrElement(markerElement) ? 'td' : 'div';
    return React.createElement(Element, {
      className: "dnb-pagination__marker dnb-table--ignore"
    }, React.createElement(ElementChild, {
      className: "dnb-pagination__marker__inner",
      ref: this._ref
    }));
  }

}

InteractionMarker.defaultProps = {
  markerElement: null
};
process.env.NODE_ENV !== "production" ? InteractionMarker.propTypes = {
  pageNo: PropTypes.number.isRequired,
  onVisible: PropTypes.func.isRequired,
  markerElement: PropTypes.oneOfType([PropTypes.object, PropTypes.node, PropTypes.func, PropTypes.string])
} : void 0;
export class InfinityLoadButton extends React.PureComponent {
  constructor(...args) {
    super(...args);
    this.state = {
      isPressed: false
    };

    this.onClickHandler = e => {
      this.setState({
        isPressed: true
      });

      if (typeof this.props.on_click === 'function') {
        this.props.on_click(e);
      }
    };
  }

  render() {
    const {
      element,
      icon
    } = this.props;
    const Element = element;
    const ElementChild = isTrElement(Element) ? 'td' : 'div';
    return this.state.isPressed ? this.props.pressed_element : React.createElement(Element, null, React.createElement(ElementChild, {
      className: "dnb-pagination__loadbar"
    }, React.createElement(Button, {
      size: "medium",
      icon: icon,
      icon_position: "left",
      text: this.context.translation.Pagination.load_button_text,
      variant: "secondary",
      on_click: this.onClickHandler
    })));
  }

}
InfinityLoadButton.contextType = Context;
InfinityLoadButton.defaultProps = {
  element: 'div',
  pressed_element: null,
  icon: 'arrow_down'
};
process.env.NODE_ENV !== "production" ? InfinityLoadButton.propTypes = {
  element: PropTypes.oneOfType([PropTypes.object, PropTypes.node, PropTypes.func, PropTypes.string]),
  pressed_element: PropTypes.oneOfType([PropTypes.object, PropTypes.node, PropTypes.func]),
  icon: PropTypes.string.isRequired,
  on_click: PropTypes.func.isRequired
} : void 0;

class ScrollToElement extends React.PureComponent {
  componentDidMount() {
    const elem = ReactDOM.findDOMNode(this);
    this.scrollToPage(elem);
  }

  scrollToPage(element) {
    if (element && typeof element.scrollIntoView === 'function') {
      element.scrollIntoView({
        block: 'nearest',
        behavior: 'smooth'
      });
    }
  }

  render() {
    const _this$props = this.props,
          {
      page_element
    } = _this$props,
          props = _objectWithoutPropertiesLoose(_this$props, ["page_element"]);

    const Element = preparePageElement(page_element || React.Fragment);
    return React.createElement(Element, props);
  }

}

ScrollToElement.defaultProps = {
  page_element: null
};
process.env.NODE_ENV !== "production" ? ScrollToElement.propTypes = {
  page_element: PropTypes.oneOfType([PropTypes.object, PropTypes.node, PropTypes.func, PropTypes.string])
} : void 0;