import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import Context from '../../shared/Context';
import { isTrue, registerElement, validateDOMAttributes, dispatchCustomElementEvent, extendPropsWithContext } from '../../shared/component-helper';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import ProgressIndicatorCircular from './ProgressIndicatorCircular';
export default class ProgressIndicator extends React.PureComponent {
  constructor(...args) {
    super(...args);
    this.state = {
      _listenForPropChanges: true
    };

    this.callOnCompleteHandler = () => {
      this.completeTimeout = setTimeout(() => {
        this.setState({
          complete: true
        });

        if (typeof this.props.on_complete === 'function') {
          this.fadeOutTimeout = setTimeout(() => {
            dispatchCustomElementEvent(this, 'on_complete');
          }, 600);
        }
      }, 200);
    };
  }

  static enableWebComponent() {
    registerElement(ProgressIndicator.tagName, ProgressIndicator, ProgressIndicator.defaultProps);
  }

  static getDerivedStateFromProps(props, state) {
    if (state._listenForPropChanges) {
      state.visible = isTrue(props.visible);

      if (state.visible) {
        state.complete = false;
      }

      if (parseFloat(props.progress) > -1) {
        state.progress = props.progress;
      }
    }

    state._listenForPropChanges = true;
    return state;
  }

  componentWillUnmount() {
    clearTimeout(this.completeTimeout);
    clearTimeout(this.fadeOutTimeout);
  }

  render() {
    const props = extendPropsWithContext(this.props, ProgressIndicator.defaultProps, this.context.getTranslation(this.props).ProgressIndicator, this.context.FormRow, this.context.ProgressIndicator);

    const {
      type,
      size,
      no_animation,
      on_complete,
      label,
      indicator_label,
      label_direction,
      show_label,
      className,
      class: _className,
      children
    } = props,
          attributes = _objectWithoutPropertiesLoose(props, ["type", "size", "no_animation", "on_complete", "label", "indicator_label", "label_direction", "show_label", "className", "class", "children", "progress", "visible", "complete"]);

    const {
      progress,
      visible,
      complete
    } = this.state;

    const params = _extends({}, attributes);

    const hasProgressIndicator = parseFloat(progress) > -1;

    if (visible && !hasProgressIndicator) {
      params.role = 'alert';
      params['aria-busy'] = true;
    }

    const indicatorLabel = label || children || isTrue(show_label) && indicator_label;
    validateDOMAttributes(this.props, params);
    return React.createElement("div", _extends({
      className: classnames('dnb-progress-indicator', createSpacingClasses(props), className, _className, visible && 'dnb-progress-indicator--visible', complete && 'dnb-progress-indicator--complete', label_direction && `dnb-progress-indicator--${label_direction}`, isTrue(no_animation) && 'dnb-progress-indicator--no-animation')
    }, params), type === 'circular' && React.createElement(ProgressIndicatorCircular, {
      size: size,
      progress: progress,
      visible: visible,
      complete: complete,
      onComplete: on_complete,
      callOnCompleteHandler: this.callOnCompleteHandler
    }), indicatorLabel && React.createElement("div", {
      className: "dnb-progress-indicator__label"
    }, React.createElement("p", {
      className: "dnb-p"
    }, indicatorLabel)));
  }

}
ProgressIndicator.tagName = 'dnb-progress-indicator';
ProgressIndicator.contextType = Context;
ProgressIndicator.defaultProps = {
  visible: true,
  type: 'circular',
  no_animation: false,
  size: 'default',
  progress: null,
  label: null,
  label_direction: 'horizontal',
  show_label: false,
  indicator_label: null,
  section_style: null,
  section_spacing: null,
  class: null,
  className: null,
  children: null,
  on_complete: null
};
process.env.NODE_ENV !== "production" ? ProgressIndicator.propTypes = _extends({
  visible: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  type: PropTypes.oneOf(['circular']),
  no_animation: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  size: PropTypes.oneOf(['default', 'small', 'medium', 'large', 'huge']),
  progress: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  label: PropTypes.node,
  label_direction: PropTypes.string,
  show_label: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  indicator_label: PropTypes.string,
  section_style: PropTypes.string,
  section_spacing: PropTypes.string
}, spacingPropTypes, {
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.func]),
  on_complete: PropTypes.func
}) : void 0;