import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { validateDOMAttributes } from '../../shared/component-helper';
import { IS_IE11, IS_EDGE } from '../../shared/helpers';
export default class ProgressIndicatorCircular extends React.PureComponent {
  static getDerivedStateFromProps(props, state) {
    state.progress = parseFloat(props.progress);
    state.visible = props.visible;
    state.complete = props.complete;
    return state;
  }

  constructor(props) {
    super(props);
    this.useAnimationFrame = typeof props.onComplete === 'function' || IS_IE11 || IS_EDGE;
    this._refDark = React.createRef();
    this._refLight = React.createRef();
    this.state = {
      animate: false
    };
  }

  componentDidMount() {
    if (this.useAnimationFrame) {
      this.startAnimationFirstTime();
    }
  }

  componentWillUnmount() {
    this.stopAnimation();
  }

  stopAnimation() {
    this.setState({
      animate: false
    });

    if (this.startupTimeout) {
      clearTimeout(this.startupTimeout);
    }
  }

  startAnimationFirstTime() {
    this.setState({
      animate: false
    });
    this.startupTimeout = setTimeout(() => this.startAnimation(), 300);
  }

  startAnimation() {
    this.setState({
      animate: true
    }, () => {
      if (this._refDark.current) {
        this.animate(this._refDark.current, true, this.props.callOnCompleteHandler);
      }

      if (this._refLight.current) {
        this.animate(this._refLight.current, false);
      }
    });
  }

  animate(element, animateOnStart = true, callback = null) {
    const min = 1;
    const max = 88;
    let start = 0,
        ms = 0,
        prog = max,
        setProg = animateOnStart,
        animate = true,
        completeCalled = false,
        stopNextRound = false;

    const step = timestamp => {
      if (!start) {
        start = timestamp;
      }

      ms = timestamp - start;

      if (animate) {
        if (!this.state.visible && prog < 5) {
          prog = min;
        }

        if (setProg) {
          element.style['stroke-dashoffset'] = prog;
        } else if (!animateOnStart) {
          element.style['stroke-dashoffset'] = max;
        }
      }

      if (stopNextRound) {
        animate = false;

        if (!completeCalled) {
          completeCalled = true;

          if (animateOnStart && typeof callback === 'function') {
            callback();
          }
        } else if (this.state.visible && ms % 1e3 > 950) {
          stopNextRound = false;
        }
      } else {
        stopNextRound = !this.state.visible && prog === min;
        animate = true;
        completeCalled = false;
      }

      prog = Math.round(max - max / 1e3 * (ms % 1e3));
      setProg = animateOnStart ? Math.ceil(ms / 1e3) % 2 === 1 || ms === 0 : Math.ceil(ms / 1e3) % 2 === 0 && ms !== 0;

      if (this.state.animate) {
        window.requestAnimationFrame(step);
      }
    };

    if (typeof window !== 'undefined' && window.requestAnimationFrame) {
      window.requestAnimationFrame(step);
    }
  }

  render() {
    const _this$props = this.props,
          {
      size,
      maxOffset
    } = _this$props,
          rest = _objectWithoutPropertiesLoose(_this$props, ["size", "maxOffset", "progress", "visible", "complete", "onComplete", "callOnCompleteHandler"]);

    const {
      progress
    } = this.state;
    const strokeDashoffset = maxOffset - maxOffset / 100 * progress;
    const hasProgressIndicator = parseFloat(progress) > -1;

    const params = _extends({}, rest);

    if (hasProgressIndicator) {
      params['title'] = `${progress}%`;
      params['aria-label'] = `${progress}%`;
    } else {
      params['aria-hidden'] = true;
    }

    validateDOMAttributes(this.props, params);
    return React.createElement("div", _extends({
      className: classnames('dnb-progress-indicator__circular', size && `dnb-progress-indicator__circular--${size}`, hasProgressIndicator && 'dnb-progress-indicator__circular--has-progress-indicator')
    }, params), _ref, React.createElement(Circle, {
      className: "dnb-progress-indicator__circular__line dark dark" + (hasProgressIndicator || this.useAnimationFrame ? " paused" : ""),
      style: hasProgressIndicator ? {
        strokeDashoffset
      } : {},
      ref: this._refDark
    }), !hasProgressIndicator && React.createElement(Circle, {
      className: "dnb-progress-indicator__circular__line light" + (this.useAnimationFrame ? " paused" : ""),
      ref: this._refLight
    }));
  }

}
ProgressIndicatorCircular.defaultProps = {
  size: null,
  visible: true,
  complete: false,
  progress: null,
  maxOffset: 88,
  onComplete: null,
  callOnCompleteHandler: null
};
process.env.NODE_ENV !== "production" ? ProgressIndicatorCircular.propTypes = {
  size: PropTypes.string,
  visible: PropTypes.bool,
  complete: PropTypes.bool,
  progress: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  maxOffset: PropTypes.number,
  onComplete: PropTypes.func,
  callOnCompleteHandler: PropTypes.func
} : void 0;

var _ref3 = React.createElement("circle", {
  className: "dnb-progress-indicator__circular__circle",
  fill: "none",
  strokeWidth: "4",
  cx: "16",
  cy: "16",
  r: "14"
});

const Circle = React.forwardRef((_ref2, ref) => {
  let {
    className
  } = _ref2,
      rest = _objectWithoutPropertiesLoose(_ref2, ["className"]);

  return React.createElement("svg", _extends({
    className: className,
    viewBox: "0 0 32 32",
    shapeRendering: "geometricPrecision",
    ref: ref
  }, rest), _ref3);
}),
      _ref = React.createElement(Circle, {
  className: "dnb-progress-indicator__circular__line light paused"
});

process.env.NODE_ENV !== "production" ? Circle.propTypes = {
  className: PropTypes.string.isRequired
} : void 0;