import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import keycode from 'keycode';
import { isTrue, makeUniqueId, extendPropsWithContext, registerElement, validateDOMAttributes, getStatusState, combineDescribedBy, dispatchCustomElementEvent } from '../../shared/component-helper';
import AlignmentHelper from '../../shared/AlignmentHelper';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import { skeletonDOMAttributes, createSkeletonClass } from '../skeleton/SkeletonHelper';
import FormLabel from '../form-label/FormLabel';
import FormStatus from '../form-status/FormStatus';
import RadioGroup from './RadioGroup';
import RadioGroupContext from './RadioGroupContext';
import Context from '../../shared/Context';
import Suffix from '../../shared/helpers/Suffix';

var _ref = React.createElement(AlignmentHelper, null);

var _ref2 = React.createElement("span", {
  className: "dnb-radio__focus",
  "aria-hidden": true
});

var _ref3 = React.createElement("span", {
  className: "dnb-radio__dot",
  "aria-hidden": true
});

export default class Radio extends React.PureComponent {
  static enableWebComponent() {
    registerElement(Radio.tagName, Radio, Radio.defaultProps);
  }

  static getDerivedStateFromProps(props, state) {
    if (state._listenForPropChanges) {
      if (props.checked !== state._checked) {
        state.checked = Radio.parseChecked(props.checked);
      }
    }

    state._listenForPropChanges = true;

    if (state.checked !== state.__checked) {
      dispatchCustomElementEvent({
        props
      }, 'on_state_update', {
        checked: state.checked
      });
    }

    state._checked = props.checked;
    state.__checked = state.checked;
    return state;
  }

  constructor(props) {
    super(props);

    this.onKeyDownHandler = event => {
      const key = keycode(event);

      if (this.isInNoGroup()) {
        switch (key) {
          case 'enter':
            this.onChangeHandler(event);
            break;
        }
      } else if (this.isContextGroupOrSingle()) {
        switch (key) {
          case 'space':
          case 'enter':
            {
              const {
                value
              } = this.context;

              if (value !== null && typeof value !== 'undefined') {
                event.preventDefault();
              }

              if (key === 'enter') {
                const checked = !this.state.checked;
                this.setState({
                  checked,
                  _listenForPropChanges: false
                });
              }

              break;
            }
        }
      } else {
        switch (key) {
          case 'space':
            {
              event.preventDefault();
              break;
            }
        }
      }

      dispatchCustomElementEvent(this, 'on_key_down', {
        event
      });
    };

    this.onChangeHandler = _event => {
      const event = _event;

      if (isTrue(this.props.readOnly)) {
        return event.preventDefault();
      }

      const value = event.target.value;
      const checked = !this.state.checked;

      if (this.isPlainGroup()) {
        setTimeout(() => {
          this.setState({
            checked,
            _listenForPropChanges: false
          }, () => this.callOnChange({
            value,
            checked,
            event
          }));
        }, 1);
      } else {
        this.setState({
          checked,
          _listenForPropChanges: false
        });
        this.callOnChange({
          value,
          checked,
          event
        });
      }
    };

    this.isContextGroupOrSingle = () => typeof this.context.value !== 'undefined' && !this.props.group;

    this.isPlainGroup = () => typeof this.context.value === 'undefined' && this.props.group;

    this.isInNoGroup = () => typeof this.context.value === 'undefined' && !this.props.group;

    this.onClickHandler = event => {
      if (isTrue(this.props.readOnly)) {
        return event.preventDefault();
      }

      if (!this.isPlainGroup()) {
        return;
      }

      const value = event.target.value;
      const checked = event.target.checked;
      this.callOnChange({
        value,
        checked,
        event
      });
    };

    this.callOnChange = ({
      value,
      checked,
      event
    }) => {
      const {
        group
      } = this.props;

      if (this.context.onChange) {
        this.context.onChange({
          value
        });
      }

      dispatchCustomElementEvent(this, 'on_change', {
        group,
        checked,
        value,
        event
      });

      if (this._refInput.current) {
        this._refInput.current.focus();
      }
    };

    this._refInput = React.createRef();
    this._id = props.id || makeUniqueId();
    this.state = {
      _listenForPropChanges: true
    };
  }

  render() {
    return React.createElement(Context.Consumer, null, context => {
      const props = extendPropsWithContext(this.props, Radio.defaultProps, this.context, {
        skeleton: context === null || context === void 0 ? void 0 : context.skeleton
      }, context.FormRow, context.Radio);

      const {
        status,
        status_state,
        status_animation,
        global_status_id,
        suffix,
        label,
        label_sr_only,
        label_position,
        size,
        readOnly,
        skeleton,
        className,
        class: _className
      } = props,
            rest = _objectWithoutPropertiesLoose(props, ["status", "status_state", "status_animation", "global_status_id", "suffix", "label", "label_sr_only", "label_position", "size", "readOnly", "skeleton", "className", "class", "id", "group", "value", "checked", "disabled", "attributes", "children", "on_change", "on_state_update", "custom_method", "custom_element"]);

      let {
        checked
      } = this.state;
      let {
        value,
        group,
        disabled
      } = props;
      const hasContext = typeof this.context.name !== 'undefined';

      if (hasContext) {
        if (typeof this.context.value !== 'undefined') {
          checked = this.context.value === value;
        }

        group = this.context.name;
        disabled = isTrue(this.context.disabled);
      } else if (typeof rest.name !== 'undefined') {
        group = rest.name;
      }

      const id = this._id;
      const showStatus = getStatusState(status);
      const mainParams = {
        className: classnames('dnb-radio', createSpacingClasses(props), className, _className, status && `dnb-radio__status--${status_state}`, size && `dnb-radio--${size}`, label && `dnb-radio--label-position-${label_position || 'right'}`)
      };

      const inputParams = _extends({
        role: hasContext || group ? 'radio' : null,
        type: hasContext || group ? 'radio' : 'checkbox'
      }, rest);

      if (showStatus || suffix) {
        inputParams['aria-describedby'] = combineDescribedBy(inputParams, showStatus ? id + '-status' : null, suffix ? id + '-suffix' : null);
      }

      if (readOnly) {
        inputParams['aria-readonly'] = inputParams.readOnly = true;
      }

      if (!group) {
        inputParams.type = 'checkbox';
        inputParams.role = 'radio';
      }

      skeletonDOMAttributes(inputParams, skeleton, this.context);
      validateDOMAttributes(this.props, inputParams);
      const labelComp = label && React.createElement(FormLabel, {
        id: id + '-label',
        for_id: id,
        text: label,
        disabled: disabled,
        skeleton: skeleton,
        sr_only: label_sr_only
      });
      return React.createElement("span", mainParams, React.createElement("span", {
        className: "dnb-radio__order"
      }, label_position === 'left' && labelComp, React.createElement("span", {
        className: "dnb-radio__inner"
      }, _ref, showStatus && React.createElement(FormStatus, {
        id: id + '-form-status',
        global_status_id: global_status_id,
        label: label,
        text_id: id + '-status',
        width_selector: id + ', ' + id + '-label',
        text: status,
        status: status_state,
        animation: status_animation,
        skeleton: skeleton
      }), React.createElement("span", {
        className: "dnb-radio__row"
      }, React.createElement("span", {
        className: "dnb-radio__shell"
      }, React.createElement("input", _extends({
        type: "radio",
        value: value,
        id: id,
        name: group,
        className: "dnb-radio__input",
        checked: checked,
        "aria-checked": checked,
        disabled: isTrue(disabled),
        ref: this._refInput
      }, inputParams, {
        onChange: this.onChangeHandler,
        onClick: this.onClickHandler,
        onKeyDown: this.onKeyDownHandler
      })), React.createElement("span", {
        className: classnames('dnb-radio__button', createSkeletonClass('shape', skeleton, this.context)),
        "aria-hidden": true
      }), _ref2, _ref3), label_position !== 'left' && labelComp, suffix && React.createElement(Suffix, {
        className: "dnb-radio__suffix",
        id: id + '-suffix',
        context: props
      }, suffix)))));
    });
  }

}
Radio.tagName = 'dnb-radio';
Radio.contextType = RadioGroupContext;
Radio.defaultProps = {
  label: null,
  label_sr_only: null,
  label_position: null,
  checked: null,
  disabled: false,
  id: null,
  size: null,
  group: null,
  status: null,
  status_state: 'error',
  status_animation: null,
  global_status_id: null,
  suffix: null,
  value: '',
  attributes: null,
  readOnly: false,
  skeleton: null,
  class: null,
  className: null,
  children: null,
  custom_element: null,
  custom_method: null,
  on_change: null,
  on_state_update: null
};
Radio.Group = RadioGroup;

Radio.parseChecked = state => /true|on/.test(String(state));

process.env.NODE_ENV !== "production" ? Radio.propTypes = _extends({
  label: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  label_sr_only: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  label_position: PropTypes.oneOf(['left', 'right']),
  checked: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  disabled: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  id: PropTypes.string,
  group: PropTypes.string,
  size: PropTypes.oneOf(['default', 'medium', 'large']),
  status: PropTypes.oneOfType([PropTypes.string, PropTypes.bool, PropTypes.func, PropTypes.node]),
  status_state: PropTypes.string,
  status_animation: PropTypes.string,
  global_status_id: PropTypes.string,
  suffix: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  value: PropTypes.string,
  attributes: PropTypes.oneOfType([PropTypes.string, PropTypes.object]),
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  readOnly: PropTypes.oneOfType([PropTypes.string, PropTypes.bool])
}, spacingPropTypes, {
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  custom_element: PropTypes.object,
  custom_method: PropTypes.func,
  on_change: PropTypes.func,
  on_state_update: PropTypes.func
}) : void 0;