import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import Context from '../../shared/Context';
import { isTrue, registerElement, validateDOMAttributes, processChildren, extendPropsWithContext } from '../../shared/component-helper';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
export default class Section extends React.PureComponent {
  static enableWebComponent() {
    registerElement(Section.tagName, Section, Section.defaultProps);
  }

  static getContent(props) {
    if (props.text) return props.text;
    return processChildren(props);
  }

  constructor(props) {
    super(props);
    this._ref = React.createRef();
  }

  render() {
    const props = extendPropsWithContext(this.props, Section.defaultProps, this.context.FormRow, this.context.Section);

    const {
      element,
      style_type,
      spacing,
      className,
      class: _className
    } = props,
          attributes = _objectWithoutPropertiesLoose(props, ["element", "style_type", "spacing", "className", "class"]);

    const content = Section.getContent(this.props);

    const params = _extends({
      className: classnames(`dnb-section dnb-section--${style_type || 'mint-green-12'}`, (isTrue(spacing) || spacing) && `dnb-section--spacing${!/true|false/.test(String(spacing)) ? '-' + spacing : ''}`, createSpacingClasses(props), className, _className)
    }, attributes);

    validateDOMAttributes(this.props, params);
    const Element = element || 'section';
    return React.createElement(Element, _extends({}, params, {
      ref: this._ref
    }), content);
  }

}
Section.tagName = 'dnb-section';
Section.contextType = Context;
Section.defaultProps = {
  style_type: null,
  spacing: null,
  element: 'section',
  class: null,
  className: null,
  children: null
};
process.env.NODE_ENV !== "production" ? Section.propTypes = _extends({
  style_type: PropTypes.string,
  spacing: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  element: PropTypes.string
}, spacingPropTypes, {
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node])
}) : void 0;