import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { extendPropsWithContext, validateDOMAttributes, isTrue } from '../../shared/component-helper';
import { LOCALE } from '../../shared/defaults';
import Space from '../space/Space';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import Context from '../../shared/Context';
import Provider from '../../shared/Provider';
export default class Skeleton extends React.PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      ariaLiveUpdate: null
    };
  }

  componentWillUnmount() {
    clearTimeout(this._ariaLiveUpdateTiemout);
  }

  componentDidUpdate(prevProps) {
    if (prevProps.show !== this.props.show) {
      this.setAriaLiveUpdate();
    }
  }

  getProps(props = this.props, context = this.context) {
    return extendPropsWithContext(props, Skeleton.defaultProps, {
      skeleton: context.Skeleton || context.skeleton,
      style_type: context.skeleton_style_type,
      no_animation: context.skeleton_no_animation
    }, context.getTranslation(props).Skeleton);
  }

  setAriaLiveUpdate() {
    clearTimeout(this._ariaLiveUpdateTiemout);
    this._ariaLiveUpdateTiemout = setTimeout(() => {
      const {
        aria_bussy,
        aria_ready
      } = this.getProps();
      let newString = null;

      if (isTrue(this.props.show)) {
        newString = aria_bussy;
      } else {
        newString = aria_ready;
      }

      if (newString) {
        this.setState({
          ariaLiveUpdate: newString
        });
        this._ariaLiveUpdateTiemout = setTimeout(() => {
          this.setState({
            ariaLiveUpdate: null
          });
        }, 1e3);
      }
    }, 1e3);
  }

  render() {
    const props = this.getProps();

    const {
      show,
      style_type,
      no_animation,
      figure,
      skeleton,
      aria_bussy,
      className,
      class: _className,
      children
    } = props,
          attributes = _objectWithoutPropertiesLoose(props, ["show", "style_type", "no_animation", "figure", "skeleton", "aria_bussy", "aria_ready", "className", "class", "children"]);

    const {
      ariaLiveUpdate
    } = this.state;
    const showSkeleton = typeof show === 'boolean' || typeof show === 'string' ? isTrue(show) : skeleton;

    const params = _extends({
      className: classnames(figure ? 'dnb-skeleton__figure' : 'dnb-skeleton__root', createSpacingClasses(props), className, _className, isTrue(showSkeleton) && 'dnb-skeleton', showSkeleton && style_type && `dnb-skeleton--${style_type}`, isTrue(no_animation) && 'dnb-skeleton--no-animation'),
      'aria-busy': showSkeleton,
      'aria-label': showSkeleton ? aria_bussy : undefined,
      lang: this.context.locale || LOCALE
    }, attributes);

    validateDOMAttributes(props, params);
    return React.createElement(Space, params, figure ? showSkeleton ? typeof figure === 'function' ? figure() : figure : children : React.createElement(Provider, {
      skeleton: showSkeleton,
      skeleton_style_type: style_type,
      skeleton_no_animation: no_animation
    }, children), React.createElement("span", {
      className: "dnb-sr-only",
      "aria-live": "assertive"
    }, ariaLiveUpdate));
  }

}
Skeleton.contextType = Context;
Skeleton.defaultProps = {
  show: null,
  skeleton: null,
  style_type: null,
  no_animation: null,
  figure: null,
  aria_bussy: null,
  aria_ready: null,
  element: null,
  class: null,
  className: null,
  children: null
};
process.env.NODE_ENV !== "production" ? Skeleton.propTypes = _extends({
  show: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  no_animation: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  style_type: PropTypes.oneOfType([PropTypes.oneOf(['lines']), PropTypes.string]),
  figure: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  element: PropTypes.node
}, spacingPropTypes, {
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node])
}) : void 0;

function Exclude(props) {
  return React.createElement(Provider, _extends({
    skeleton: false
  }, props));
}

Skeleton.Exclude = Exclude;