import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { isTrue, extendPropsWithContext, registerElement, processChildren, validateDOMAttributes } from '../../shared/component-helper';
import Context from '../../shared/Context';
import { createSpacingClasses, isInline, spacingPropTypes, spacingDefaultProps } from './SpacingHelper';
import { skeletonDOMAttributes, createSkeletonClass } from '../skeleton/SkeletonHelper';
export { spacingPropTypes };
export default class Space extends React.PureComponent {
  static enableWebComponent() {
    registerElement(Space.tagName, Space, Space.defaultProps);
  }

  static getContent(props) {
    return processChildren(props);
  }

  render() {
    var _this$context;

    const props = this.context.space ? extendPropsWithContext(this.props, Space.defaultProps, {
      skeleton: (_this$context = this.context) === null || _this$context === void 0 ? void 0 : _this$context.skeleton
    }, this.context.space) : this.props;

    const {
      element,
      inline,
      no_collapse,
      top,
      right,
      bottom,
      left,
      space,
      skeleton,
      className,
      class: _className
    } = props,
          attributes = _objectWithoutPropertiesLoose(props, ["element", "inline", "no_collapse", "top", "right", "bottom", "left", "space", "skeleton", "id", "className", "class"]);

    const children = Space.getContent(this.props);

    const params = _extends({
      className: classnames('dnb-space', createSkeletonClass(null, skeleton), createSpacingClasses({
        top,
        right,
        bottom,
        left,
        space
      }), className, _className, isTrue(inline) && 'dnb-space--inline')
    }, attributes);

    skeletonDOMAttributes(params, skeleton);
    validateDOMAttributes(this.props, params);
    return React.createElement(Element, _extends({
      element: element,
      no_collapse: no_collapse
    }, params), children);
  }

}
Space.tagName = 'dnb-space';
Space.contextType = Context;
Space.defaultProps = _extends({
  id: null,
  element: 'div',
  inline: null,
  no_collapse: null
}, spacingDefaultProps, {
  skeleton: null,
  class: null,
  className: null,
  children: null
});
process.env.NODE_ENV !== "production" ? Space.propTypes = _extends({
  id: PropTypes.string,
  element: PropTypes.string,
  inline: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  no_collapse: PropTypes.oneOfType([PropTypes.string, PropTypes.bool])
}, spacingPropTypes, {
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node])
}) : void 0;

const Element = (_ref) => {
  let {
    element: E,
    no_collapse,
    children
  } = _ref,
      props = _objectWithoutPropertiesLoose(_ref, ["element", "no_collapse", "children"]);

  if (isTrue(no_collapse)) {
    const R = E === 'span' || isInline(Element) ? 'span' : 'div';
    return React.createElement(R, {
      className: 'dnb-space--no-collapse' + (isInline(Element) ? " dnb-space--inline" : "")
    }, React.createElement(E, props, children));
  }

  return React.createElement(E, props, children);
};

process.env.NODE_ENV !== "production" ? Element.propTypes = {
  children: PropTypes.node,
  element: PropTypes.string,
  no_collapse: PropTypes.bool
} : void 0;
Element.defaultProps = {
  children: null,
  element: 'div',
  no_collapse: true
};