import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import Context from '../../shared/Context';
import { warn, registerElement, validateDOMAttributes, processChildren, extendPropsWithContext } from '../../shared/component-helper';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import { skeletonDOMAttributes, createSkeletonClass } from '../skeleton/SkeletonHelper';
import StepItem from './StepIndicatorItem';
export default class StepIndicator extends React.PureComponent {
  static enableWebComponent() {
    registerElement(StepIndicator.tagName, StepIndicator, StepIndicator.defaultProps);
  }

  static getData(props) {
    let res = [];
    if (props.data) res = props.data;else res = processChildren(props);
    if (typeof res === 'string') return res[0] === '[' ? JSON.parse(res) : [];
    return res || [];
  }

  static getDerivedStateFromProps(props, state) {
    if (state._listenForPropChanges) {
      if (props.data) {
        if (state._data !== props.data) {
          state._data = props.data;
          state.data = StepIndicator.getData(props);
        }
      }

      if (state.activeItem !== props.active_item) {
        state.activeItem = parseFloat(props.active_item);
      }

      if (props.active_url && state.activeUrl !== props.active_url) {
        state.activeUrl = props.active_url;
      }

      if ((state.activeUrl || !(parseFloat(state.activeItem) > 0)) && state.data.length > 0) {
        state.activeItem = state.data.reduce((acc, {
          url
        }, i) => url && (url === state.activeItem || url === state.activeUrl) ? i : acc, parseFloat(props.active_item));

        if (!(state.activeItem > -1)) {
          state.activeItem = null;
        }
      }
    }

    state._listenForPropChanges = true;
    return state;
  }

  constructor(props) {
    super(props);

    this.setActimeItem = activeItem => {
      this.setState({
        activeItem,
        _listenForPropChanges: false
      });
    };

    this.state = {
      hasReached: [],
      _listenForPropChanges: true
    };
    const sn = 'show_numbers';

    if (typeof props[sn] !== 'undefined') {
      warn('StepIndicator: "show_numbers" is deprecated. Use "hide_numbers" instead.');
    }
  }

  render() {
    var _this$context;

    const props = extendPropsWithContext(this.props, StepIndicator.defaultProps, {
      skeleton: (_this$context = this.context) === null || _this$context === void 0 ? void 0 : _this$context.skeleton
    }, this.context.getTranslation(this.props).StepIndicator, this.context.FormRow, this.context.StepIndicator);

    const {
      hide_numbers,
      use_navigation,
      on_item_render,
      step_title,
      on_change,
      skeleton,
      className,
      class: _className
    } = props,
          attributes = _objectWithoutPropertiesLoose(props, ["active_item", "active_url", "hide_numbers", "use_navigation", "on_item_render", "step_title", "on_change", "skeleton", "className", "class", "data", "children"]);

    const data = StepIndicator.getData(this.props);
    const {
      activeItem
    } = this.state;

    const params = _extends({
      'aria-label': 'progress',
      className: classnames('dnb-step-indicator', createSkeletonClass('font', skeleton, this.context), createSpacingClasses(props), className, _className)
    }, attributes);

    skeletonDOMAttributes(params, skeleton, this.context);
    validateDOMAttributes(this.props, params);

    if (!this.state.hasReached.includes(activeItem)) {
      this.state.hasReached.push(activeItem);
    }

    const countSteps = data.length;
    return React.createElement("nav", params, countSteps > 0 && React.createElement("ol", {
      className: "dnb-step-indicator__list"
    }, data.map((props, i) => {
      if (typeof props === 'string') {
        props = {
          title: props
        };
      }

      const params = _extends({
        countSteps,
        currentItem: i,
        activeItem,
        hide_numbers,
        use_navigation,
        on_item_render,
        step_title,
        on_change
      }, props);

      return React.createElement(StepItem, _extends({
        key: i
      }, params, {
        setActimeItem: this.setActimeItem,
        hasReached: this.state.hasReached
      }));
    })));
  }

}
StepIndicator.tagName = 'dnb-step-indicator';
StepIndicator.contextType = Context;
StepIndicator.defaultProps = {
  data: [],
  step_title: '%step',
  active_item: null,
  active_url: null,
  hide_numbers: false,
  use_navigation: false,
  on_item_render: null,
  skeleton: false,
  class: null,
  className: null,
  children: null,
  on_change: null
};
process.env.NODE_ENV !== "production" ? StepIndicator.propTypes = _extends({
  data: PropTypes.oneOfType([PropTypes.string, PropTypes.arrayOf(PropTypes.string), PropTypes.arrayOf(PropTypes.shape({
    title: PropTypes.string.isRequired,
    url: PropTypes.string,
    is_active: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
    is_current: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
    url_future: PropTypes.string,
    url_passed: PropTypes.string,
    on_click: PropTypes.func,
    on_render: PropTypes.func
  }))]).isRequired,
  step_title: PropTypes.string,
  active_item: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  active_url: PropTypes.string,
  hide_numbers: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  use_navigation: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  on_item_render: PropTypes.func,
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool])
}, spacingPropTypes, {
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.func]),
  on_change: PropTypes.func
}) : void 0;