import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { isTrue, dispatchCustomElementEvent } from '../../shared/component-helper';
import { IS_MAC } from '../../shared/helpers';
export default class StepItem extends React.PureComponent {
  constructor(props) {
    super(props);

    this.onClickHandler = ({
      event,
      item,
      currentItem
    }) => {
      const {
        use_navigation,
        on_click,
        setActimeItem
      } = this.props;
      const params = {
        event,
        item,
        currentItem
      };

      if (isTrue(use_navigation) && typeof setActimeItem === 'function') {
        setActimeItem(currentItem);
      }

      const res = dispatchCustomElementEvent(this, 'on_click', params);

      if (typeof on_click === 'function' && res === false) {
        return;
      }

      dispatchCustomElementEvent(this, 'on_change', params);

      if (this._ref.current) {
        this._ref.current.blur();
      }
    };

    this._ref = React.createRef();
  }

  render() {
    const _this$props = this.props,
          {
      activeItem,
      currentItem,
      countSteps,
      is_active,
      is_current,
      url: _url,
      url_future,
      url_passed,
      hide_numbers,
      title,
      step_title,
      use_navigation,
      on_item_render,
      on_render,
      hasReached
    } = _this$props,
          rest = _objectWithoutPropertiesLoose(_this$props, ["activeItem", "currentItem", "countSteps", "is_active", "is_current", "url", "url_future", "url_passed", "hide_numbers", "title", "step_title", "use_navigation", "on_item_render", "on_render", "on_click", "on_change", "setActimeItem", "hasReached"]);

    let hasPassedAndIsCurrent = isTrue(is_active);
    let url = _url;

    if (_url) {
      if (currentItem > activeItem) {
        url = url_future;
        hasPassedAndIsCurrent = true;
      }

      if (url_passed && currentItem < activeItem) {
        url = url_passed;
        hasPassedAndIsCurrent = true;
      }
    }

    if (currentItem <= activeItem) {
      hasPassedAndIsCurrent = true;
    }

    if (hasReached.includes(currentItem)) {
      hasPassedAndIsCurrent = true;
    }

    const interactiveParams = _extends({}, rest);

    if (!hasPassedAndIsCurrent && currentItem > activeItem && !isTrue(is_active)) {
      interactiveParams['disabled'] = true;
      interactiveParams['aria-disabled'] = true;
    }

    interactiveParams.className = "dnb-anchor dnb-step-indicator__item-content dnb-step-indicator__item-content--link" + (interactiveParams['disabled'] ? " dnb-anchor--no-style" : "");
    const aria = step_title.replace('%step', currentItem + 1).replace('%count', countSteps);

    const StepItemWrapper = props => React.createElement(React.Fragment, null, !isTrue(hide_numbers) && React.createElement("span", _extends({
      className: "dnb-step-indicator__item-content--number",
      "aria-label": aria
    }, props), `${currentItem + 1}. `), React.createElement("span", _extends({
      className: "dnb-step-indicator__item-content--text"
    }, props), title));

    let itemComponent = React.createElement(StepItemWrapper, null);
    const props = {
      StepItem: StepItemWrapper,
      itemComponent,
      params: rest,
      props: this.props
    };

    if (typeof on_render === 'function') {
      itemComponent = on_render(props);
    } else if (typeof on_item_render === 'function') {
      itemComponent = on_item_render(props);
    }

    let child = null;

    if (isTrue(use_navigation)) {
      child = React.createElement("button", _extends({
        type: "button",
        onClick: event => this.onClickHandler({
          event,
          item: this.props,
          currentItem
        })
      }, interactiveParams, {
        ref: this._ref
      }), itemComponent);
    } else if (url) {
      child = React.createElement("a", _extends({
        href: url,
        onClick: event => this.onClickHandler({
          event,
          item: this.props,
          currentItem
        })
      }, interactiveParams), itemComponent);
    } else {
      const contentParams = {};

      if (IS_MAC) {
        contentParams.role = 'text';
      }

      child = React.createElement("span", _extends({
        className: "dnb-step-indicator__item-content dnb-step-indicator__item-content--static"
      }, contentParams), itemComponent);
    }

    const itemParams = {};

    if (currentItem === activeItem || isTrue(is_current)) {
      itemParams['aria-current'] = 'step';
    }

    return React.createElement("li", _extends({
      className: classnames('dnb-step-indicator__item', (currentItem === activeItem || isTrue(is_current)) && 'dnb-step-indicator--active', currentItem < activeItem && 'dnb-step-indicator--visited')
    }, itemParams), child);
  }

}
StepItem.defaultProps = {
  step_title: '%step',
  on_item_render: null,
  on_render: null,
  on_click: null,
  on_change: null,
  setActimeItem: null,
  hasReached: [],
  countSteps: null,
  hide_numbers: false,
  use_navigation: false,
  is_active: null,
  is_current: null,
  url: null,
  url_future: null,
  url_passed: null,
  activeItem: null
};
process.env.NODE_ENV !== "production" ? StepItem.propTypes = {
  title: PropTypes.string.isRequired,
  step_title: PropTypes.string,
  activeItem: PropTypes.number,
  currentItem: PropTypes.number.isRequired,
  hide_numbers: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  use_navigation: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  on_item_render: PropTypes.func,
  on_render: PropTypes.func,
  on_click: PropTypes.func,
  on_change: PropTypes.func,
  setActimeItem: PropTypes.func,
  hasReached: PropTypes.array,
  countSteps: PropTypes.number,
  is_active: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  is_current: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  url: PropTypes.string,
  url_future: PropTypes.string,
  url_passed: PropTypes.string
} : void 0;