import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import keycode from 'keycode';
import { isTrue, makeUniqueId, extendPropsWithContext, registerElement, validateDOMAttributes, getStatusState, combineDescribedBy, dispatchCustomElementEvent } from '../../shared/component-helper';
import AlignmentHelper from '../../shared/AlignmentHelper';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import { skeletonDOMAttributes, createSkeletonClass } from '../skeleton/SkeletonHelper';
import Context from '../../shared/Context';
import Suffix from '../../shared/helpers/Suffix';
import FormLabel from '../form-label/FormLabel';
import FormStatus from '../form-status/FormStatus';

var _ref = React.createElement(AlignmentHelper, null);

var _ref2 = React.createElement("span", {
  className: "dnb-switch__focus"
}, React.createElement("span", {
  className: "dnb-switch__focus__inner"
}));

export default class Switch extends React.PureComponent {
  static enableWebComponent() {
    registerElement(Switch.tagName, Switch, Switch.defaultProps);
  }

  static getDerivedStateFromProps(props, state) {
    if (state._listenForPropChanges) {
      if (props.checked !== state._checked) {
        if (props.default_state !== null && typeof state.checked === 'undefined') {
          state.checked = Switch.parseChecked(props.default_state);
        } else {
          state.checked = Switch.parseChecked(props.checked);
        }
      }
    }

    state._listenForPropChanges = true;

    if (state.checked !== state.__checked) {
      dispatchCustomElementEvent({
        props
      }, 'on_state_update', {
        checked: state.checked
      });
    }

    state._checked = props.checked;
    state.__checked = state.checked;
    return state;
  }

  constructor(props) {
    super(props);

    this.onKeyDownHandler = event => {
      switch (keycode(event)) {
        case 'enter':
          this.onChangeHandler(event);
          break;
      }
    };

    this.onChangeHandler = event => {
      if (isTrue(this.props.readOnly)) {
        return event.preventDefault();
      }

      const checked = !this.state.checked;
      this.setState({
        checked,
        _listenForPropChanges: false
      });
      dispatchCustomElementEvent(this, 'on_change', {
        checked,
        event
      });

      if (this.props.on_change_end) {
        clearTimeout(this._onChangeEndId);

        if (event && event.persist) {
          event.persist();
        }

        this._onChangeEndId = setTimeout(() => dispatchCustomElementEvent(this, 'on_change_end', {
          checked,
          event
        }), 500);
      }

      if (this._refInput.current) {
        this._refInput.current.focus();
      }
    };

    this._refInput = React.createRef();
    this._id = props.id || makeUniqueId();
    this.state = {
      _listenForPropChanges: true
    };
    this.helperParams = {
      onMouseDown: e => e.preventDefault()
    };
  }

  componentWillUnmount() {
    clearTimeout(this._onChangeEndId);
  }

  render() {
    var _this$context;

    const props = extendPropsWithContext(this.props, Switch.defaultProps, {
      skeleton: (_this$context = this.context) === null || _this$context === void 0 ? void 0 : _this$context.skeleton
    }, this.context.getTranslation(this.props).Switch, this.context.FormRow, this.context.Switch);

    const {
      value,
      size,
      status,
      status_state,
      status_animation,
      global_status_id,
      suffix,
      label,
      label_position,
      label_sr_only,
      title,
      disabled,
      readOnly,
      skeleton,
      className,
      class: _className
    } = props,
          rest = _objectWithoutPropertiesLoose(props, ["value", "size", "status", "status_state", "status_animation", "global_status_id", "suffix", "label", "label_position", "label_sr_only", "title", "disabled", "readOnly", "skeleton", "className", "class", "id", "checked", "attributes", "children", "on_change", "on_state_update", "custom_method", "custom_element"]);

    const {
      checked
    } = this.state;
    const id = this._id;
    const showStatus = getStatusState(status);
    const mainParams = {
      className: classnames(`dnb-switch dnb-switch--label-position-${label_position || 'right'} dnb-form-component`, createSkeletonClass(null, skeleton), createSpacingClasses(props), className, _className, size && `dnb-switch--${size}`, status && `dnb-switch__status--${status_state}`)
    };

    const inputParams = _extends({
      disabled: isTrue(disabled),
      checked
    }, rest);

    skeletonDOMAttributes(inputParams, skeleton, this.context);

    if (showStatus || suffix) {
      inputParams['aria-describedby'] = combineDescribedBy(inputParams, showStatus ? id + '-status' : null, suffix ? id + '-suffix' : null);
    }

    if (readOnly) {
      inputParams['aria-readonly'] = inputParams.readOnly = true;
    }

    validateDOMAttributes(this.props, inputParams);
    const labelComp = label && React.createElement(FormLabel, {
      id: id + '-label',
      for_id: id,
      text: label,
      disabled: disabled,
      skeleton: skeleton,
      sr_only: label_sr_only
    });
    return React.createElement("span", mainParams, React.createElement("span", {
      className: "dnb-switch__order"
    }, label_position === 'left' && labelComp, React.createElement("span", {
      className: "dnb-switch__inner"
    }, _ref, showStatus && React.createElement(FormStatus, {
      id: id + '-form-status',
      global_status_id: global_status_id,
      label: label,
      text_id: id + '-status',
      width_selector: id + ', ' + id + '-label',
      text: status,
      status: status_state,
      animation: status_animation,
      skeleton: skeleton
    }), React.createElement("span", {
      className: "dnb-switch__shell"
    }, (label_position === 'right' || !label_position) && labelComp, React.createElement("span", {
      className: "dnb-switch__row"
    }, React.createElement("input", _extends({
      id: id,
      name: id,
      type: "checkbox",
      role: "switch",
      title: title,
      "aria-checked": checked,
      className: "dnb-switch__input",
      value: checked ? value || '' : '',
      ref: this._refInput
    }, inputParams, {
      onChange: this.onChangeHandler,
      onKeyDown: this.onKeyDownHandler
    })), React.createElement("span", _extends({
      draggable: true,
      "aria-hidden": true,
      className: "dnb-switch__background",
      onDragStart: this.onChangeHandler
    }, this.helperParams)), React.createElement("span", {
      className: classnames('dnb-switch__button', createSkeletonClass('shape', skeleton, this.context)),
      "aria-hidden": true
    }, _ref2)), suffix && React.createElement(Suffix, {
      className: "dnb-switch__suffix",
      id: id + '-suffix',
      context: props
    }, suffix)))));
  }

}
Switch.tagName = 'dnb-switch';
Switch.contextType = Context;
Switch.defaultProps = {
  label: null,
  label_position: null,
  title: null,
  default_state: null,
  checked: null,
  disabled: null,
  id: null,
  size: null,
  status: null,
  status_state: 'error',
  status_animation: null,
  global_status_id: null,
  suffix: null,
  value: null,
  attributes: null,
  readOnly: false,
  skeleton: null,
  class: null,
  className: null,
  children: null,
  custom_element: null,
  custom_method: null,
  on_change: null,
  on_change_end: null,
  on_state_update: null
};

Switch.parseChecked = state => /true|on/.test(String(state));

process.env.NODE_ENV !== "production" ? Switch.propTypes = _extends({
  label: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  label_position: PropTypes.oneOf(['left', 'right']),
  title: PropTypes.string,
  default_state: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  checked: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  disabled: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  id: PropTypes.string,
  size: PropTypes.oneOf(['default', 'medium', 'large']),
  status: PropTypes.oneOfType([PropTypes.string, PropTypes.bool, PropTypes.func, PropTypes.node]),
  status_state: PropTypes.string,
  status_animation: PropTypes.string,
  global_status_id: PropTypes.string,
  suffix: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  value: PropTypes.string,
  attributes: PropTypes.oneOfType([PropTypes.string, PropTypes.object]),
  readOnly: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool])
}, spacingPropTypes, {
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  custom_element: PropTypes.object,
  custom_method: PropTypes.func,
  on_change: PropTypes.func,
  on_change_end: PropTypes.func,
  on_state_update: PropTypes.func
}) : void 0;