import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import keycode from 'keycode';
import Context from '../../shared/Context';
import { warn, slugify, isTrue, makeUniqueId, registerElement, extendPropsWithContext, validateDOMAttributes, dispatchCustomElementEvent, getPreviousSibling, filterProps } from '../../shared/component-helper';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import { createSkeletonClass, skeletonDOMAttributes } from '../skeleton/SkeletonHelper';
import Button from '../button/Button';
import whatInput from 'what-input';
import CustomContent from './TabsCustomContent';
import ContentWrapper from './TabsContentWrapper';

var _ref2 = React.createElement("span", null, "Tab content not found");

export default class Tabs extends React.PureComponent {
  static enableWebComponent() {
    registerElement(Tabs.tagName, Tabs, Tabs.defaultProps);
  }

  static getSelectedKeyOrFallback(selected_key, data) {
    let useKey = selected_key;

    if (!useKey) {
      useKey = data.reduce((acc, {
        selected,
        key
      }) => selected ? key : acc, null) || data[0] && data[0].key;
    } else {
      const keyExists = data.findIndex(({
        key
      }) => key == selected_key);

      if (keyExists === -1) {
        useKey = data[0] && data[0].key;
      }
    }

    return useKey;
  }

  static getDerivedStateFromProps(props, state) {
    if (state._listenForPropChanges) {
      if (props.data) {
        if (state._data !== props.data) {
          state._data = props.data;
          state.data = Tabs.getData(props);
        }
      } else if (props.children) {
        if (state._data !== props.children) {
          state._data = props.children;
          state.data = Tabs.getData(props);
        }
      }

      if (props.selected_key && state._selected_key !== props.selected_key) {
        state.selected_key = state._selected_key = Tabs.getSelectedKeyOrFallback(props.selected_key, state.data);
      }
    }

    state._listenForPropChanges = true;
    return state;
  }

  static getData(props) {
    let res = [];
    const data = !props.data && props.children ? props.children : props.data;

    if (Array.isArray(props.children) && (typeof props.children[0] === 'function' || React.isValidElement(props.children[0]))) {
      res = props.children.reduce((acc, content, i) => {
        if (content.props && content.props.displayName === 'CustomContent') {
            const dataProps = props.tabs && Array.isArray(props.tabs) && props.tabs[i] || {};

            const componentProps = _extends({}, content.props);

            if (componentProps.title === null) {
              delete componentProps.title;
            }

            const _dataProps$componentP = _extends({}, dataProps, componentProps, {
              children: null
            }),
                  {
              title,
              key: _key,
              hash
            } = _dataProps$componentP,
                  rest = _objectWithoutPropertiesLoose(_dataProps$componentP, ["title", "key", "hash"]);

            acc.push(_extends({
              title,
              key: (!_key && hash ? hash : _key) || slugify(title),
              content
            }, rest));
          }

        return acc;
      }, []);
    }

    if (!(res && res.length > 0)) {
      if (props.data && Array.isArray(data)) {
        res = data;
      } else if (typeof data === 'string') {
        res = data[0] === '[' ? JSON.parse(data) : [];
      } else if (data && typeof data === 'object') {
        res = Object.entries(data).reduce((acc, [key, obj]) => {
          if (obj) {
            acc.push(_extends({
              key
            }, obj));
          }

          return acc;
        }, []);
      }
    }

    return res || [];
  }

  constructor(props) {
    super(props);

    this.init = () => {
      if (this._isMounted) {
        this.addScrollBehaviour();
        this.scrollToLastPosition();
        this.scrollToTab('selected');

        if (this.getLastUsedTab() !== null) {
          this.setState(null, this.setFocusOnTab);
        }
      }
    };

    this.onResizeHandler = () => {
      const hasScrollbar = this._hasScrollbar = this.hasScrollbar();

      if (hasScrollbar !== this.state.hasScrollbar) {
        this.setState({
          hasScrollbar
        });
      }

      if (this._hasScrollbar) {
        try {
          if (Math.ceil(window.innerWidth / 16) <= 40 || this.isAtEdge()) {
            if (!this._tabsRef.current.style.marginLeft) {
              const style = window.getComputedStyle(this._tabsRef.current);

              if (!(Math.abs(parseFloat(style.marginLeft)) > 0)) {
                const diff = window.innerWidth - this._tabsRef.current.offsetWidth;
                let val = Math.round(diff / 16) / 2 * 16;
                this._tabsRef.current.style.marginLeft = `-${val}px`;
                this._tabsRef.current.style.marginRight = `-${val}px`;

                if (val < 32) {
                  val = 32;
                }

                this._tablistRef.current.style.paddingLeft = `${val}px`;
                this._tablistRef.current.style.paddingRight = `${val}px`;
              }
            }
          } else {
            this._tabsRef.current.style.marginLeft = '';
            this._tabsRef.current.style.marginRight = '';
            this._tablistRef.current.style.paddingLeft = '';
            this._tablistRef.current.style.paddingRight = '';
          }
        } catch (e) {}
      }

      this.setState({
        atEdge: this.isAtEdge()
      });

      if (hasScrollbar) {
        this.scrollToTab('selected');
      }
    };

    this.onTablistKeyDownHandler = e => {
      switch (keycode(e)) {
        case 'up':
        case 'page up':
        case 'left':
          e.preventDefault();
          this.focusPrevTab(e);
          break;

        case 'down':
        case 'page down':
        case 'right':
          e.preventDefault();
          this.focusNextTab(e);
          break;

        case 'home':
          e.preventDefault();
          this.focusFirstTab(e);
          break;

        case 'end':
          e.preventDefault();
          this.focusLastTab(e);
          break;
      }
    };

    this.focusFirstTab = e => {
      const key = this.state.data[0].key;
      this.focusTab(key, e, 'step');
      this.scrollToTab('focus');
    };

    this.focusLastTab = e => {
      const key = this.state.data[this.state.data.length - 1].key;
      this.focusTab(key, e, 'step');
      this.scrollToTab('focus');
    };

    this.focusPrevTab = e => {
      this.focusTab(-1, e, 'step');
      this.scrollToTab('focus');
    };

    this.focusNextTab = e => {
      this.focusTab(+1, e, 'step');
      this.scrollToTab('focus');
    };

    this.openPrevTab = e => {
      this.openTab(-1, e, 'step');
      this.scrollToTab('selected');
    };

    this.openNextTab = e => {
      this.openTab(+1, e, 'step');
      this.scrollToTab('selected');
    };

    this.handleVerticalScroll = () => {
      if (isTrue(this.props.scroll) && this._tablistRef.current && typeof this._tablistRef.current.scrollIntoView === 'function') {
        this._tablistRef.current.scrollIntoView({
          block: 'start',
          behavior: 'smooth'
        });
      }
    };

    this.onClickHandler = event => {
      let selected_key;

      try {
        selected_key = function (elem) {
          return getPreviousSibling('dnb-tabs__button', elem) || {
            dataset: {}
          };
        }(event.target).dataset.tabKey;
      } catch (e) {
        warn('Tabs Error:', e);
      }

      if (selected_key) {
        const ret = dispatchCustomElementEvent(this, 'on_click', this.getEventArgs({
          event,
          selected_key
        }));

        if (ret !== false) {
          this.openTab(selected_key, event);
          this.scrollToTab('selected');
        }
      }
    };

    this.getCurrentTitle = () => {
      const {
        selected_key
      } = this.state;
      const current = this.state.data.filter(({
        key
      }) => key == selected_key)[0];
      return current && current.title || null;
    };

    this.focusTab = (focus_key, event = null, mode = null) => {
      if (mode === 'step' && parseFloat(focus_key)) {
        focus_key = this.getStepKey(focus_key, this.state.focus_key);
      }

      this.setState({
        focus_key,
        _listenForPropChanges: false
      }, this.setFocusOnTab);
      dispatchCustomElementEvent(this, 'on_focus', this.getEventArgs({
        event,
        focus_key
      }));
      this.setWhatInput();
    };

    this.setFocusOnTab = () => {
      try {
        const elem = this._tablistRef.current.querySelector('.dnb-tabs__button.focus');

        elem.focus();

        if (!document.getElementById(`${this._id}-content`) && process.env.NODE_ENV !== 'test') {
          warn(`Could not find the required <Tabs.Content id="${this._id}-content" ... /> that provides role="tabpanel"`);
        }
      } catch (e) {
        warn(e);
      }
    };

    this.openTab = (selected_key, event = null, mode = null) => {
      this.saveLastPosition();
      this.saveLastUsedTab();
      this.resetWhatInput();

      if (mode === 'step' && parseFloat(selected_key)) {
        selected_key = this.getStepKey(selected_key, this.state.selected_key);
      }

      if (typeof selected_key !== 'undefined') {
        this.setState({
          selected_key,
          focus_key: selected_key,
          _listenForPropChanges: false
        }, this.handleVerticalScroll);
      }

      dispatchCustomElementEvent(this, 'on_change', this.getEventArgs({
        event,
        selected_key
      }));

      if (this.props.use_hash && typeof window !== 'undefined') {
        try {
          window.history.replaceState(undefined, undefined, `#${selected_key}`);
        } catch (e) {
          warn('Tabs Error:', e);
        }
      }
    };

    this.getContent = selected_key => {
      const {
        children,
        content: _content
      } = this.props;
      const contentToRender = children || _content;
      let content = null;

      if (contentToRender) {
        if (typeof contentToRender === 'object' && contentToRender[selected_key]) {
          content = contentToRender[selected_key];
        } else if (typeof contentToRender === 'function') {
          content = contentToRender.apply(this, [selected_key]);
        } else if (React.isValidElement(contentToRender)) {
          content = contentToRender;
        }
      }

      if (!content) {
        let items = [];

        if (Array.isArray(this.state.data)) {
          items = this.state.data;
        } else if (Array.isArray(contentToRender)) {
          items = contentToRender;
        }

        if (items) {
          content = items.filter(({
            key
          }) => key && selected_key && key == selected_key).reduce((acc, {
            content
          }) => content || acc, null);
        }
      }

      if (typeof content === 'function') {
        const Component = content;
        content = React.createElement(Component, null);
      }

      return content;
    };

    this.TabsWrapperHandler = (_ref) => {
      let {
        children
      } = _ref,
          rest = _objectWithoutPropertiesLoose(_ref, ["children"]);

      const {
        className,
        class: _className
      } = this.props;

      const attributes = _extends({}, filterProps(this.props, Tabs.defaultProps));

      const params = _extends({}, attributes, {
        className: classnames('dnb-tabs', createSpacingClasses(this.props), className, _className)
      });

      validateDOMAttributes(this.props, params);
      return React.createElement("div", _extends({}, params, rest), children);
    };

    this.TabsListHandler = ({
      children,
      className
    }) => {
      const {
        align,
        section_style,
        section_spacing
      } = this.props;
      const {
        hasScrollbar,
        atEdge
      } = this.state;
      return React.createElement("div", {
        className: classnames('dnb-tabs__tabs', className, align && `dnb-tabs__tabs--${align}`, section_style && `dnb-section dnb-section--${section_style}`, section_spacing && `dnb-section--spacing-${isTrue(section_spacing) ? 'default' : section_spacing}`, hasScrollbar && 'dnb-tabs--has-scrollbar', atEdge && 'dnb-tabs--at-edge'),
        ref: this._tabsRef
      }, React.createElement(ScrollNavButton, {
        onMouseDown: this.openPrevTab,
        icon: "chevron_left",
        className: classnames(hasScrollbar && 'dnb-tabs__scroll-nav-button--visible', this.state.isFirst && 'dnb-tabs__scroll-nav-button--hide')
      }), children, React.createElement(ScrollNavButton, {
        onMouseDown: this.openNextTab,
        icon: "chevron_right",
        className: classnames(hasScrollbar && 'dnb-tabs__scroll-nav-button--visible', this.state.isLast && 'dnb-tabs__scroll-nav-button--hide')
      }));
    };

    this.TabContentHandler = ({
      showEmptyMessage = false
    } = {}) => {
      const {
        selected_key
      } = this.state;
      const content = this.renderContent();
      return React.createElement(ContentWrapper, {
        id: this._id,
        selected_key: selected_key
      }, content || showEmptyMessage && _ref2);
    };

    this.TabsHandler = () => {
      const {
        label,
        skeleton
      } = this._props;
      const {
        selected_key
      } = this.state;
      const tabs = this.state.data.map(({
        title,
        key,
        disabled = false
      }) => {
        const itemParams = {};
        const isFocus = this.isFocus(key);
        const isSelected = this.isSelected(key);

        if (isSelected) {
          itemParams['aria-controls'] = `${this._id}-content`;
        }

        if (isTrue(disabled)) {
          itemParams.disabled = true;
          itemParams['aria-disabled'] = true;
        }

        skeletonDOMAttributes(itemParams, skeleton, this.context);
        return React.createElement("div", {
          className: classnames('dnb-tabs__button__snap', isFocus && 'focus', isSelected && 'selected'),
          key: `tab-${key}`
        }, React.createElement("button", _extends({
          type: "button",
          role: "tab",
          tabIndex: "-1",
          id: `${this._id}-tab-${key}`,
          "aria-selected": isSelected,
          className: classnames('dnb-tabs__button', isFocus && 'focus', isSelected && 'selected'),
          onClick: this.onClickHandler,
          "data-tab-key": key
        }, itemParams), React.createElement("span", {
          className: classnames('dnb-tabs__button__title', createSkeletonClass('font', skeleton, this.context))
        }, title), React.createElement(Dummy, null, title)));
      });
      const params = {};

      if (label) {
        params['aria-label'] = label;
      }

      if (selected_key) {
        params['aria-labelledby'] = `${this._id}-tab-${selected_key}`;
      }

      return React.createElement("div", _extends({
        role: "tablist",
        className: "dnb-tabs__tabs__tablist",
        tabIndex: "0",
        onKeyDown: this.onTablistKeyDownHandler,
        ref: this._tablistRef
      }, params), tabs);
    };

    this._id = props.id || makeUniqueId();
    const data = Tabs.getData(props);

    let _selected_key = Tabs.getSelectedKeyOrFallback(props.selected_key, data);

    if (props.use_hash && typeof window !== 'undefined') {
      try {
        const useHashKey = String(window.location.hash).replace('#', '');

        if (useHashKey && String(useHashKey).length > 0) {
          _selected_key = Tabs.getSelectedKeyOrFallback(useHashKey, data);
        }
      } catch (e) {}
    }

    const lastPosition = this.getLastPosition();
    this.state = {
      data,
      lastPosition,
      selected_key: _selected_key,
      focus_key: _selected_key,
      atEdge: false,
      hasScrollbar: lastPosition > -1,
      _selected_key: _selected_key,
      _data: props.data || props.children,
      _listenForPropChanges: true
    };
    this._tabsRef = React.createRef();
    this._tablistRef = React.createRef();
  }

  componentDidMount() {
    this._isMounted = true;

    if (document.readyState === 'complete') {
      this.init();
    } else if (typeof window !== 'undefined') {
      window.addEventListener('load', this.init);
    }
  }

  componentWillUnmount() {
    this._isMounted = false;
    this.resetWhatInput();
    clearTimeout(this._scrollToTabTimeout);

    if (typeof window !== 'undefined') {
      window.removeEventListener('resize', this.onResizeHandler);
      window.removeEventListener('load', this.init);
    }
  }

  componentDidUpdate() {
    this.onResizeHandler();
  }

  getLastPosition() {
    if (typeof window !== 'undefined') {
      try {
        const pos = parseFloat(window.localStorage.getItem('tmp-tabs-pos')) || -1;
        window.localStorage.removeItem('tmp-tabs-pos');
        return pos;
      } catch (e) {
        warn(e);
      }
    }

    return -1;
  }

  getLastUsedTab() {
    if (typeof window !== 'undefined') {
      try {
        const key = window.localStorage.getItem('tmp-tabs-last') || null;
        window.localStorage.removeItem('tmp-tabs-last');
        return key;
      } catch (e) {
        warn(e);
      }
    }

    return -1;
  }

  saveLastUsedTab() {
    if (typeof window !== 'undefined') {
      try {
        window.localStorage.setItem('tmp-tabs-last', this.state.selected_key);
      } catch (e) {
        warn(e);
      }
    }
  }

  saveLastPosition(pos = this._tablistRef.current.scrollLeft) {
    if (typeof window !== 'undefined') {
      try {
        window.localStorage.setItem('tmp-tabs-pos', pos);
      } catch (e) {
        warn(e);
      }
    }
  }

  hasScrollbar() {
    return this._tablistRef.current.scrollWidth - 1 > this._tablistRef.current.offsetWidth;
  }

  isAtEdge() {
    if (!this._hasScrollbar || typeof window === 'undefined') {
      return false;
    }

    try {
      const padding = parseFloat(window.getComputedStyle(this._tablistRef.current).paddingLeft);
      const width = this._tablistRef.current.offsetWidth + 2 + 64 - padding;
      const screenWidth = window.innerWidth;
      return width >= screenWidth;
    } catch (e) {}

    return false;
  }

  addScrollBehaviour() {
    this.onResizeHandler();

    if (typeof window !== 'undefined') {
      window.addEventListener('resize', this.onResizeHandler);
    }
  }

  scrollToLastPosition() {
    try {
      this._tablistRef.current.style.scrollBehavior = 'auto';
      this._tablistRef.current.scrollLeft = this.state.lastPosition;
      this._tablistRef.current.style.scrollBehavior = 'smooth';
    } catch (e) {}
  }

  scrollToTab(type) {
    if (typeof window === 'undefined') {
      return;
    }

    clearTimeout(this._scrollToTabTimeout);
    this._scrollToTabTimeout = setTimeout(() => {
      try {
        if ((this._hasScrollbar || this.state.hasScrollbar) && this._tablistRef.current) {
          const first = this._tablistRef.current.querySelector('.dnb-tabs__button__snap:first-of-type');

          const isFirst = first.classList.contains(type);

          const last = this._tablistRef.current.querySelector('.dnb-tabs__button__snap:last-of-type');

          const isLast = last.classList.contains(type);

          const elem = this._tablistRef.current.querySelector(`.dnb-tabs__button.${type}`);

          const style = window.getComputedStyle(this._tabsRef.current);
          const margin = parseFloat(style.marginLeft);
          let padding = margin < 0 ? parseFloat(style.paddingLeft) : 0;

          if (!isFirst && this.state.atEdge && parseFloat(style.paddingLeft) < 16) {
            const navButton = this._tabsRef.current.querySelector('.dnb-tabs__scroll-nav-button:first-of-type');

            padding = parseFloat(window.getComputedStyle(navButton).width);
          }

          const leftPadding = (margin < 0 ? Math.abs(margin) : 0) + padding + parseFloat(window.getComputedStyle(first).paddingLeft);
          const left = elem && !isFirst ? elem.offsetLeft - leftPadding : 0;

          this._tablistRef.current.scrollTo({
            left,
            behavior: window.IS_TEST ? 'auto' : 'smooth'
          });

          this.setState({
            isFirst,
            isLast
          });
        }
      } catch (e) {
        warn(e);
      }
    }, window.IS_TEST ? 0 : 100);
  }

  getStepKey(useKey, stateKey) {
    const currentData = this.state.data.filter(({
      disabled
    }) => !disabled);
    const currentIndex = currentData.reduce((acc, {
      key
    }, i) => key == stateKey ? i : acc, -1);
    let nextIndex = currentIndex + useKey;

    if (nextIndex < 0) {
      nextIndex = currentData.length - 1;
    }

    if (nextIndex >= currentData.length) {
      nextIndex = 0;
    }

    return currentData.reduce((acc, {
      key
    }, i) => i === nextIndex ? key : acc, null);
  }

  setWhatInput() {
    whatInput.specificKeys([9, 37, 39, 33, 34, 35, 36]);
  }

  resetWhatInput() {
    whatInput.specificKeys([9]);
  }

  getEventArgs(args) {
    const {
      selected_key,
      focus_key
    } = this.state;
    return _extends({
      key: typeof args.selected_key !== 'undefined' ? args.selected_key : selected_key,
      selected_key,
      focus_key
    }, args);
  }

  isFocus(tabKey) {
    return this.state.focus_key == tabKey;
  }

  isSelected(tabKey) {
    return this.state.selected_key == tabKey;
  }

  renderCachedContent() {
    const {
      selected_key,
      data
    } = this.state;
    const {
      prevent_rerender,
      prerender
    } = this.props;

    if (isTrue(prerender)) {
      this._cache = Object.entries(data).reduce((acc, [idx, cur]) => {
        acc[cur.key] = _extends({}, cur, {
          content: this.getContent(cur.key)
        });
        return acc;
      }, {});
    } else if (isTrue(prevent_rerender)) {
      this._cache = _extends({}, this._cache || {}, {
        [selected_key]: {
          content: this.getContent(selected_key)
        }
      });
    }

    const cachedContent = Object.entries(this._cache).map(([key, {
      content
    }]) => {
      const params = {};

      if (key !== String(selected_key)) {
        params.hidden = true;
        params['aria-hidden'] = true;
      }

      return React.createElement("div", _extends({
        key: key,
        className: "dnb-tabs__cached"
      }, params), content);
    });
    return cachedContent;
  }

  renderContent() {
    const {
      prevent_rerender,
      prerender
    } = this.props;

    if (isTrue(prevent_rerender) || isTrue(prerender)) {
      return this.renderCachedContent();
    }

    return this.getContent(this.state.selected_key);
  }

  render() {
    var _this$context;

    const props = this._props = extendPropsWithContext(this.props, Tabs.defaultProps, {
      skeleton: (_this$context = this.context) === null || _this$context === void 0 ? void 0 : _this$context.skeleton
    });
    const {
      render: customRenderer
    } = props;
    const TabItems = this.TabsHandler;
    TabItems.displayName = 'Tabs';
    const TabsList = this.TabsListHandler;
    TabsList.displayName = 'TabsList';
    const Wrapper = this.TabsWrapperHandler;
    Wrapper.displayName = 'TabsWrapper';
    const Content = this.TabContentHandler;
    Content.displayName = 'TabContent';

    if (typeof customRenderer === 'function') {
      return customRenderer({
        Wrapper,
        Content,
        TabsList,
        Tabs: TabItems
      });
    }

    return React.createElement(Wrapper, null, React.createElement(TabsList, null, React.createElement(TabItems, null)), React.createElement(Content, {
      showEmptyMessage: true
    }));
  }

}
Tabs.tagName = 'dnb-tabs';
Tabs.contextType = Context;
Tabs.defaultProps = {
  data: null,
  content: null,
  label: null,
  selected_key: null,
  align: 'left',
  section_style: null,
  section_spacing: null,
  use_hash: false,
  prerender: false,
  prevent_rerender: false,
  scroll: null,
  skeleton: null,
  id: null,
  class: null,
  className: null,
  children: null,
  render: null,
  on_change: null,
  on_click: null,
  on_focus: null
};
Tabs.Content = CustomContent;
Tabs.ContentWrapper = ContentWrapper;
process.env.NODE_ENV !== "production" ? Tabs.propTypes = _extends({
  data: PropTypes.oneOfType([PropTypes.string, PropTypes.arrayOf(PropTypes.shape({
    title: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.func]).isRequired,
    key: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired,
    selected: PropTypes.bool,
    disabled: PropTypes.bool
  })), PropTypes.objectOf(PropTypes.shape({
    title: PropTypes.string.isRequired,
    selected: PropTypes.bool,
    disabled: PropTypes.bool
  }))]),
  content: PropTypes.oneOfType([PropTypes.object, PropTypes.node, PropTypes.func]),
  label: PropTypes.string,
  selected_key: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  align: PropTypes.oneOf(['left', 'center', 'right']),
  section_style: PropTypes.string,
  section_spacing: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  use_hash: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  prerender: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  prevent_rerender: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  scroll: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  id: PropTypes.string
}, spacingPropTypes, {
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.object, PropTypes.node, PropTypes.func]),
  render: PropTypes.func,
  on_change: PropTypes.func,
  on_click: PropTypes.func,
  on_focus: PropTypes.func
}) : void 0;
export const Dummy = ({
  children
}) => {
  return React.createElement("span", {
    "aria-hidden": true,
    hidden: true,
    className: "dnb-dummy"
  }, children);
};
process.env.NODE_ENV !== "production" ? Dummy.propTypes = {
  children: PropTypes.node.isRequired
} : void 0;

const ScrollNavButton = props => {
  return React.createElement(Button, _extends({
    size: "medium",
    variant: "primary",
    tabIndex: "-1",
    bounding: true,
    "aria-hidden": true,
    icon_size: "medium"
  }, props, {
    className: classnames('dnb-tabs__scroll-nav-button', props.className)
  }));
};

process.env.NODE_ENV !== "production" ? ScrollNavButton.propTypes = {
  className: PropTypes.node.isRequired
} : void 0;