import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import FormLabel from '../form-label/FormLabel';
import FormStatus from '../form-status/FormStatus';
import { isTrue, makeUniqueId, extendPropsWithContext, registerElement, validateDOMAttributes, processChildren, getStatusState, combineDescribedBy, warn, dispatchCustomElementEvent } from '../../shared/component-helper';
import AlignmentHelper from '../../shared/AlignmentHelper';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import { skeletonDOMAttributes, createSkeletonClass } from '../skeleton/SkeletonHelper';
import Context from '../../shared/Context';
import Suffix from '../../shared/helpers/Suffix';

var _ref = React.createElement(AlignmentHelper, null);

export default class Textarea extends React.PureComponent {
  static enableWebComponent() {
    registerElement(Textarea.tagName, Textarea, Textarea.defaultProps);
  }

  static getDerivedStateFromProps(props, state) {
    const value = Textarea.getValue(props);

    if (state._listenForPropChanges && value !== 'initval' && value !== state.value) {
      if (value !== state.value && value !== state._value && typeof props.on_state_update === 'function') {
        dispatchCustomElementEvent({
          props
        }, 'on_state_update', {
          value
        });
      }

      state.value = value;
    }

    if (props.textarea_state) {
      state.textareaState = props.textarea_state;
    }

    state._listenForPropChanges = true;
    return state;
  }

  static hasValue(value) {
    return (typeof value === 'string' || typeof value === 'number') && String(value).length > 0 || false;
  }

  static getValue(props) {
    const value = processChildren(props);

    if (value === '' || Textarea.hasValue(value)) {
      return value;
    }

    return props.value;
  }

  constructor(props) {
    super(props);
    this.state = {
      textareaState: 'virgin',
      value: null,
      _value: null
    };

    this.onFocusHandler = event => {
      const {
        value
      } = this._ref.current;
      this.setState({
        value,
        _listenForPropChanges: false,
        textareaState: 'focus'
      });
      dispatchCustomElementEvent(this, 'on_focus', {
        value,
        event
      });
    };

    this.onBlurHandler = event => {
      const {
        value
      } = event.target;
      this.setState({
        value,
        _listenForPropChanges: false,
        textareaState: Textarea.hasValue(value) ? 'dirty' : 'initial'
      });
      dispatchCustomElementEvent(this, 'on_blur', {
        value,
        event
      });
    };

    this.onChangeHandler = event => {
      const {
        value
      } = event.target;
      const rows = this.getRows(value);
      const ret = dispatchCustomElementEvent(this, 'on_change', {
        value,
        rows,
        event
      });

      if (ret !== false) {
        this.setState({
          value,
          _listenForPropChanges: false
        });

        if (isTrue(this.props.autoresize)) {
          this.setAutosize(rows);
        }
      }
    };

    this.onKeyDownHandler = event => {
      const rows = this.getRows();
      const {
        value
      } = event.target;
      dispatchCustomElementEvent(this, 'on_key_down', {
        value,
        rows,
        event
      });
    };

    this.setAutosize = (rows = null) => {
      const elem = this._ref.current;

      if (!elem) {
        return;
      }

      rows = typeof rows === 'number' ? rows : this.getRows();

      try {
        if (typeof this._heightOffset === 'undefined') {
          this._heightOffset = elem.offsetHeight - elem.clientHeight;
        }

        elem.style.height = 'auto';
        let newHeight = elem.scrollHeight + this._heightOffset;
        const maxRows = parseFloat(this.props.autoresize_max_rows);

        if (maxRows > 0) {
          const maxHeight = maxRows * this.getLineHeight();

          if (rows > maxRows || newHeight > maxHeight) {
            newHeight = maxHeight;
          }
        }

        elem.style.height = newHeight + 'px';
      } catch (e) {
        warn(e);
      }
    };

    this._ref = props.inner_ref || React.createRef();
    this._id = props.id || makeUniqueId();
    this.state._listenForPropChanges = true;

    if (props.textarea_state) {
      this.state.textareaState = props.textarea_state;
    }

    this.state._value = props.value;
  }

  componentDidMount() {
    if (isTrue(this.props.autoresize) && typeof window !== 'undefined') {
      this.setAutosize();
      window.addEventListener('resize', this.setAutosize);
    }
  }

  componentWillUnmount() {
    if (isTrue(this.props.autoresize) && typeof window !== 'undefined') {
      window.removeEventListener('resize', this.setAutosize);
    }
  }

  getRows() {
    return Math.floor(this._ref.current.scrollHeight / this.getLineHeight()) || 1;
  }

  getLineHeight() {
    return parseFloat(getComputedStyle(this._ref.current).lineHeight) || 0;
  }

  render() {
    var _this$context;

    const props = extendPropsWithContext(this.props, Textarea.defaultProps, {
      skeleton: (_this$context = this.context) === null || _this$context === void 0 ? void 0 : _this$context.skeleton
    }, this.context.getTranslation(this.props).Textarea, this.context.FormRow, this.context.Textarea);

    const {
      label,
      label_direction,
      label_sr_only,
      status,
      status_state,
      status_animation,
      global_status_id,
      suffix,
      disabled,
      skeleton,
      stretch,
      placeholder,
      align,
      textarea_class,
      readOnly,
      textarea_attributes,
      class: _className,
      className,
      autoresize,
      textarea_element: _textarea_element
    } = props,
          attributes = _objectWithoutPropertiesLoose(props, ["label", "label_direction", "label_sr_only", "status", "status_state", "status_animation", "global_status_id", "suffix", "disabled", "skeleton", "stretch", "placeholder", "align", "textarea_class", "readOnly", "textarea_attributes", "class", "className", "autoresize", "autoresize_max_rows", "id", "children", "value", "textarea_element"]);

    const {
      value,
      textareaState
    } = this.state;
    const id = this._id;
    const showStatus = getStatusState(status);
    const hasValue = Textarea.hasValue(value);
    let {
      textarea_element: TextareaElement
    } = props;
    const textareaAttributes = textarea_attributes ? typeof textarea_attributes === 'string' ? JSON.parse(textarea_attributes) : textarea_attributes : {};

    const textareaParams = _extends({
      className: classnames('dnb-textarea__textarea', textarea_class),
      role: 'textbox',
      value: hasValue ? value : '',
      id,
      disabled: isTrue(disabled) || isTrue(skeleton),
      name: id,
      'aria-placeholder': placeholder
    }, attributes, textareaAttributes, {
      onChange: this.onChangeHandler,
      onFocus: this.onFocusHandler,
      onBlur: this.onBlurHandler,
      onKeyDown: this.onKeyDownHandler
    });

    if (showStatus || suffix) {
      textareaParams['aria-describedby'] = combineDescribedBy(textareaParams, showStatus ? id + '-status' : null, suffix ? id + '-suffix' : null);
    }

    if (readOnly) {
      textareaParams['aria-readonly'] = textareaParams.readOnly = true;
    }

    const mainParams = {
      className: classnames(`dnb-textarea dnb-textarea--${textareaState} dnb-form-component`, createSkeletonClass(null, skeleton), createSpacingClasses(props), _className, className, hasValue && 'dnb-textarea--has-content', align && `dnb-textarea__align--${align}`, status && `dnb-textarea__status--${status_state}`, autoresize && 'dnb-textarea__autoresize', label_direction && `dnb-textarea--${label_direction}`, isTrue(stretch) && `dnb-textarea--stretch`)
    };
    const innerParams = {
      className: classnames('dnb-textarea__inner', createSkeletonClass('shape', skeleton, this.context))
    };
    const shellParams = {
      className: 'dnb-textarea__shell'
    };

    if (isTrue(disabled) || isTrue(skeleton)) {
      shellParams['aria-disabled'] = true;
    }

    const placeholderStyle = parseFloat(this.props.rows) > 0 ? {
      '--textarea-rows': parseFloat(this.props.rows)
    } : null;
    skeletonDOMAttributes(innerParams, skeleton, this.context);
    validateDOMAttributes(this.props, textareaParams);
    validateDOMAttributes(null, innerParams);
    validateDOMAttributes(null, shellParams);

    if (TextareaElement && typeof TextareaElement === 'function') {
      TextareaElement = TextareaElement(textareaParams, this._ref);
    } else if (!TextareaElement && _textarea_element) {
      TextareaElement = _textarea_element;
    }

    return React.createElement("span", mainParams, label && React.createElement(FormLabel, {
      id: id + '-label',
      for_id: id,
      text: label,
      label_direction: label_direction,
      sr_only: label_sr_only,
      disabled: disabled,
      skeleton: skeleton
    }), React.createElement("span", innerParams, _ref, showStatus && React.createElement(FormStatus, {
      id: id + '-form-status',
      global_status_id: global_status_id,
      label: label,
      text_id: id + '-status',
      text: status,
      status: status_state,
      animation: status_animation,
      skeleton: skeleton
    }), React.createElement("span", {
      className: "dnb-textarea__row"
    }, React.createElement("span", shellParams, TextareaElement || React.createElement("textarea", _extends({
      ref: this._ref
    }, textareaParams)), placeholder && React.createElement("span", {
      "aria-hidden": true,
      className: 'dnb-textarea__placeholder' + (align ? ` dnb-textarea__align--${align}` : ""),
      style: placeholderStyle
    }, placeholder)), suffix && React.createElement(Suffix, {
      className: "dnb-textarea__suffix",
      id: id + '-suffix',
      context: props
    }, suffix))));
  }

}
Textarea.tagName = 'dnb-textarea';
Textarea.contextType = Context;
Textarea.defaultProps = {
  value: 'initval',
  id: null,
  label: null,
  label_direction: null,
  label_sr_only: null,
  status: null,
  textarea_state: null,
  status_state: 'error',
  status_animation: null,
  global_status_id: null,
  suffix: null,
  placeholder: null,
  align: null,
  stretch: null,
  disabled: null,
  skeleton: null,
  autoresize: null,
  autoresize_max_rows: null,
  textarea_class: null,
  class: null,
  textarea_attributes: null,
  readOnly: false,
  rows: null,
  cols: null,
  inner_ref: null,
  className: null,
  textarea_element: null,
  children: null,
  custom_element: null,
  custom_method: null,
  on_change: null,
  on_focus: null,
  on_blur: null,
  on_key_down: null,
  on_state_update: null
};
process.env.NODE_ENV !== "production" ? Textarea.propTypes = _extends({
  value: PropTypes.string,
  id: PropTypes.string,
  label: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  label_direction: PropTypes.oneOf(['horizontal', 'vertical']),
  label_sr_only: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  status: PropTypes.oneOfType([PropTypes.string, PropTypes.bool, PropTypes.func, PropTypes.node]),
  textarea_state: PropTypes.string,
  status_state: PropTypes.string,
  status_animation: PropTypes.string,
  global_status_id: PropTypes.string,
  suffix: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  placeholder: PropTypes.string,
  align: PropTypes.oneOf(['left', 'right']),
  stretch: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  disabled: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  autoresize: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  autoresize_max_rows: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  class: PropTypes.string,
  textarea_class: PropTypes.string,
  textarea_attributes: PropTypes.oneOfType([PropTypes.string, PropTypes.object]),
  readOnly: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  rows: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
  cols: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
  inner_ref: PropTypes.oneOfType([PropTypes.func, PropTypes.object])
}, spacingPropTypes, {
  className: PropTypes.string,
  textarea_element: PropTypes.oneOfType([PropTypes.func, PropTypes.node]),
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.func]),
  custom_element: PropTypes.object,
  custom_method: PropTypes.func,
  on_change: PropTypes.func,
  on_focus: PropTypes.func,
  on_blur: PropTypes.func,
  on_key_down: PropTypes.func,
  on_state_update: PropTypes.func
}) : void 0;