import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import keycode from 'keycode';
import { warn, isTrue, makeUniqueId, registerElement, extendPropsWithContext, validateDOMAttributes, getStatusState, combineDescribedBy, dispatchCustomElementEvent } from '../../shared/component-helper';
import AlignmentHelper from '../../shared/AlignmentHelper';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import Radio from '../radio/Radio';
import Checkbox from '../checkbox/Checkbox';
import Button from '../button/Button';
import FormLabel from '../form-label/FormLabel';
import FormStatus from '../form-status/FormStatus';
import ToggleButtonGroup from './ToggleButtonGroup';
import ToggleButtonGroupContext from './ToggleButtonGroupContext';
import Context from '../../shared/Context';
import Suffix from '../../shared/helpers/Suffix';

var _ref = React.createElement(AlignmentHelper, null);

export default class ToggleButton extends React.PureComponent {
  static enableWebComponent() {
    registerElement(ToggleButton.tagName, ToggleButton, ToggleButton.defaultProps);
  }

  static getDerivedStateFromProps(props, state) {
    if (state._listenForPropChanges) {
      if (props.checked !== state._checked) {
        state.checked = ToggleButton.parseChecked(props.checked);
      }
    }

    state._listenForPropChanges = true;

    if (state.checked !== state.__checked) {
      dispatchCustomElementEvent({
        props
      }, 'on_state_update', {
        checked: state.checked
      });
    }

    state._checked = props.checked;
    state.__checked = state.checked;
    return state;
  }

  constructor(props, context) {
    super(props);

    this.onKeyDownHandler = event => {
      switch (keycode(event)) {
        case 'enter':
          this.onClickHandler(event);
          break;
      }
    };

    this.onKeyUpHandler = event => {
      switch (keycode(event)) {
        case 'enter':
          this.onClickHandler(event);
          break;
      }
    };

    this.onClickHandler = ({
      event
    }) => {
      if (isTrue(this.props.readOnly)) {
        return event.preventDefault();
      }

      event.persist();

      if (!isTrue(this.context.multiselect) && this.props.value === this.context.value) {
        return;
      }

      const checked = !this.state.checked;
      this.setState({
        checked,
        _listenForPropChanges: false
      });
      this.callOnChange({
        checked,
        event
      });

      if (this._refButton.current && checked) {
        try {
          this._refButton.current._ref.current.focus();
        } catch (e) {
          warn(e);
        }
      }
    };

    this.callOnChange = ({
      checked,
      event
    }) => {
      const {
        value
      } = this.props;

      if (this.context.onChange) {
        this.context.onChange({
          value,
          event
        });
      }

      dispatchCustomElementEvent(this, 'on_change', {
        checked,
        value,
        event
      });
    };

    this._id = props.id || makeUniqueId();
    this._refButton = React.createRef();
    this.state = {
      _listenForPropChanges: true
    };

    if (context.name && typeof props.value !== 'undefined') {
      if (typeof context.value !== 'undefined') {
        this.state.checked = context.value === props.value;
        this.state._listenForPropChanges = false;
      } else if (context.values && Array.isArray(context.values)) {
        this.state.checked = context.values.includes(props.value);
        this.state._listenForPropChanges = false;
      } else if (ToggleButton.parseChecked(props.checked)) {
        if (context.setContext) {
          if (context.multiselect) {
            context.setContext(tmp => {
              return {
                values: tmp && Array.isArray(tmp.values) ? [...tmp.values, props.value] : [props.value]
              };
            });
          } else {
            context.setContext({
              value: props.value
            });
          }
        }
      }
    }
  }

  render() {
    return React.createElement(Context.Consumer, null, context => {
      const props = extendPropsWithContext(this.props, ToggleButton.defaultProps, this.context, context.translation.ToggleButton, context.FormRow, context.ToggleButton);

      const {
        status,
        status_state,
        status_animation,
        global_status_id,
        suffix,
        label,
        label_direction,
        label_sr_only,
        text,
        title,
        readOnly,
        className,
        class: _className,
        disabled,
        skeleton,
        variant,
        left_component,
        icon,
        icon_size,
        icon_position,
        value: propValue,
        children
      } = props,
            rest = _objectWithoutPropertiesLoose(props, ["status", "status_state", "status_animation", "global_status_id", "suffix", "label", "label_direction", "label_sr_only", "text", "title", "readOnly", "className", "class", "disabled", "skeleton", "variant", "left_component", "icon", "icon_size", "icon_position", "value", "id", "checked", "attributes", "children", "on_change", "on_state_update", "custom_method", "custom_element"]);

      let {
        checked
      } = this.state;

      if (!isTrue(this.context.multiselect) && typeof this.context.value !== 'undefined') {
        const contextValue = this.context.value;

        if (typeof propValue === 'string' || typeof propValue === 'number') {
          checked = propValue === contextValue;
        } else if (typeof JSON !== 'undefined') {
          checked = JSON.stringify(propValue) === JSON.stringify(contextValue);
        }
      }

      const id = this._id;
      const showStatus = getStatusState(status);
      const mainParams = {
        className: classnames('dnb-toggle-button', createSpacingClasses(props), className, _className, status && `dnb-toggle-button__status--${status_state}`, checked && `dnb-toggle-button--checked`, label_direction && `dnb-toggle-button--${label_direction}`)
      };
      validateDOMAttributes(this.props, rest);

      const buttonParams = _extends({
        id,
        disabled,
        skeleton,
        text: text || children,
        title,
        icon,
        icon_size,
        icon_position,
        ['aria-pressed']: String(checked)
      }, rest);

      const componentParams = {
        checked,
        disabled,
        ['aria-hidden']: true,
        tabIndex: '-1'
      };

      if (status) {
        if (status_state === 'info') {
          componentParams.status_state = 'info';
        } else {
          componentParams.status = 'error';
        }
      }

      if (showStatus || suffix) {
        buttonParams['aria-describedby'] = combineDescribedBy(buttonParams, showStatus ? id + '-status' : null, suffix ? id + '-suffix' : null);
      }

      if (readOnly) {
        buttonParams['aria-readonly'] = buttonParams.readOnly = true;
      }

      let leftComponent = null;

      switch (variant) {
        case 'radio':
          leftComponent = React.createElement(Radio, _extends({
            id: `${id}-radio`
          }, componentParams));
          break;

        case 'checkbox':
          leftComponent = React.createElement(Checkbox, _extends({
            id: `${id}-checkbox`
          }, componentParams));
          break;

        case 'default':
        default:
          leftComponent = left_component;
          break;
      }

      return React.createElement("span", mainParams, label && React.createElement(FormLabel, {
        id: id + '-label',
        for_id: id,
        text: label,
        disabled: disabled,
        skeleton: skeleton,
        label_direction: label_direction,
        sr_only: label_sr_only
      }), React.createElement("span", {
        className: "dnb-toggle-button__inner"
      }, showStatus && React.createElement(FormStatus, {
        id: id + '-form-status',
        global_status_id: global_status_id,
        label: label,
        text_id: id + '-status',
        text: status,
        status: status_state,
        animation: status_animation,
        skeleton: skeleton
      }), React.createElement("span", {
        className: "dnb-toggle-button__shell"
      }, _ref, React.createElement(Button, _extends({
        variant: "secondary",
        className: "dnb-toggle-button__button"
      }, buttonParams, {
        ref: this._refButton,
        onClick: this.onClickHandler,
        onKeyDown: this.onKeyDownHandler,
        onKeyUp: this.onKeyUpHandler
      }), leftComponent && React.createElement("span", {
        className: "dnb-toggle-button__component"
      }, leftComponent)), suffix && React.createElement(Suffix, {
        className: "dnb-toggle-button__suffix",
        id: id + '-suffix',
        context: props
      }, suffix))));
    });
  }

}
ToggleButton.Group = ToggleButtonGroup;
ToggleButton.tagName = 'dnb-toggle-button';
ToggleButton.contextType = ToggleButtonGroupContext;
ToggleButton.defaultProps = {
  text: null,
  label: null,
  label_direction: null,
  label_sr_only: null,
  title: null,
  checked: undefined,
  variant: null,
  left_component: null,
  disabled: null,
  skeleton: null,
  id: null,
  status: null,
  status_state: 'error',
  status_animation: null,
  global_status_id: null,
  suffix: null,
  value: '',
  icon: null,
  icon_position: 'right',
  icon_size: null,
  attributes: null,
  readOnly: false,
  class: null,
  className: null,
  children: null,
  custom_element: null,
  custom_method: null,
  on_change: null,
  on_state_update: null
};

ToggleButton.parseChecked = state => /true|on/.test(String(state));

process.env.NODE_ENV !== "production" ? ToggleButton.propTypes = _extends({
  text: PropTypes.string,
  label: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  label_direction: PropTypes.oneOf(['horizontal', 'vertical']),
  label_sr_only: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  title: PropTypes.string,
  checked: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  variant: PropTypes.oneOf(['default', 'checkbox', 'radio']),
  left_component: PropTypes.node,
  disabled: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  id: PropTypes.string,
  status: PropTypes.oneOfType([PropTypes.string, PropTypes.bool, PropTypes.func, PropTypes.node]),
  status_state: PropTypes.string,
  status_animation: PropTypes.string,
  global_status_id: PropTypes.string,
  suffix: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  value: PropTypes.oneOfType([PropTypes.string, PropTypes.number, PropTypes.object, PropTypes.array]),
  icon: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.func]),
  icon_position: PropTypes.oneOf(['left', 'right']),
  icon_size: PropTypes.string,
  attributes: PropTypes.oneOfType([PropTypes.string, PropTypes.object]),
  readOnly: PropTypes.oneOfType([PropTypes.string, PropTypes.bool])
}, spacingPropTypes, {
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  custom_element: PropTypes.object,
  custom_method: PropTypes.func,
  on_change: PropTypes.func,
  on_state_update: PropTypes.func
}) : void 0;