import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { isTrue, makeUniqueId, extendPropsWithContext, registerElement, validateDOMAttributes, getStatusState, combineDescribedBy, dispatchCustomElementEvent } from '../../shared/component-helper';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import FormRow from '../form-row/FormRow';
import FormStatus from '../form-status/FormStatus';
import Context from '../../shared/Context';
import Suffix from '../../shared/helpers/Suffix';
import ToggleButtonGroupContext from './ToggleButtonGroupContext';
export default class ToggleButtonGroup extends React.PureComponent {
  static enableWebComponent() {
    registerElement(ToggleButtonGroup.tagName, ToggleButtonGroup, ToggleButtonGroup.defaultProps);
  }

  static getDerivedStateFromProps(props, state) {
    if (state._listenForPropChanges) {
      if (typeof props.value !== 'undefined' && props.value !== state.value) {
        state.value = props.value;
      }

      if (typeof props.values !== 'undefined' && props.values !== state.values) {
        state.values = ToggleButtonGroup.getValues(props);
      }
    }

    state._listenForPropChanges = true;
    return state;
  }

  static getValues(props) {
    if (typeof props.values === 'string' && props.values[0] === '[') {
      return JSON.parse(props.values);
    }

    return props.values;
  }

  constructor(props) {
    super(props);

    this.onChangeHandler = ({
      value,
      event
    }) => {
      const {
        multiselect
      } = this.props;
      const values = this.state.values || [];

      if (isTrue(multiselect)) {
        if (!values.includes(value)) {
          values.push(value);
        } else {
          values.splice(values.indexOf(value), 1);
        }
      }

      this.setState({
        value,
        values,
        _listenForPropChanges: false
      });
      dispatchCustomElementEvent(this, 'on_change', {
        value,
        values,
        event
      });
    };

    this._refInput = React.createRef();
    this._id = props.id || makeUniqueId();
    this._name = props.name || makeUniqueId();
    this.state = {
      _listenForPropChanges: true
    };
  }

  render() {
    const props = extendPropsWithContext(this.props, ToggleButtonGroup.defaultProps, this.context.getTranslation(this.props).ToggleButton, this.context.FormRow, this.context.ToggleButtonGroup);

    const {
      status,
      status_state,
      status_animation,
      global_status_id,
      suffix,
      label_direction,
      label_sr_only,
      vertical,
      layout_direction,
      label,
      variant,
      left_component,
      no_fieldset,
      disabled,
      skeleton,
      className,
      class: _className,
      multiselect,
      children
    } = props,
          rest = _objectWithoutPropertiesLoose(props, ["status", "status_state", "status_animation", "global_status_id", "suffix", "label_direction", "label_sr_only", "vertical", "layout_direction", "label", "variant", "left_component", "no_fieldset", "disabled", "skeleton", "className", "class", "multiselect", "id", "name", "value", "values", "attributes", "children", "on_change", "custom_method", "custom_element"]);

    const {
      value,
      values
    } = this.state;
    const id = this._id;
    const showStatus = getStatusState(status);
    const classes = classnames(`dnb-toggle-button-group dnb-toggle-button-group--${layout_direction} dnb-form-component`, createSpacingClasses(props), className, _className, status && `dnb-toggle-button-group__status--${status_state}`, !label && 'dnb-toggle-button-group--no-label');

    const params = _extends({}, rest);

    if (showStatus || suffix) {
      params['aria-describedby'] = combineDescribedBy(params, showStatus ? id + '-status' : null, suffix ? id + '-suffix' : null);
    }

    if (label) {
      params['aria-labelledby'] = id + '-label';
    }

    validateDOMAttributes(this.props, params);
    const context = {
      name: this._name,
      value,
      values,
      multiselect: isTrue(multiselect),
      variant,
      left_component,
      disabled,
      skeleton,
      setContext: context => {
        if (typeof context === 'function') {
          context = context(this._tmp);
        }

        this._tmp = _extends({}, this._tmp, context);
        this.setState(_extends({}, context, {
          _listenForPropChanges: false
        }));
      },
      onChange: this.onChangeHandler
    };
    const formRowParams = {
      id,
      label,
      label_id: id + '-label',
      label_direction,
      label_sr_only,
      direction: label_direction,
      vertical,
      disabled,
      skeleton,
      no_fieldset,
      skipContentWrapperIfNested: true
    };
    return React.createElement(ToggleButtonGroupContext.Provider, {
      value: context
    }, React.createElement("div", {
      className: classes
    }, React.createElement(FormRow, formRowParams, React.createElement("span", _extends({
      id: id,
      className: "dnb-toggle-button-group__shell",
      role: "group"
    }, params), showStatus && React.createElement(FormStatus, {
      id: id + '-form-status',
      global_status_id: global_status_id,
      label: label,
      text_id: id + '-status',
      text: status,
      status: status_state,
      animation: status_animation,
      skeleton: skeleton
    }), React.createElement("span", {
      className: "dnb-toggle-button-group__children"
    }, children, suffix && React.createElement(Suffix, {
      className: "dnb-toggle-button-group__suffix",
      id: id + '-suffix',
      context: props
    }, suffix))))));
  }

}
ToggleButtonGroup.tagName = 'dnb-toggle-button-group';
ToggleButtonGroup.contextType = Context;
ToggleButtonGroup.defaultProps = {
  label: null,
  label_direction: null,
  label_sr_only: null,
  title: null,
  multiselect: null,
  variant: null,
  left_component: null,
  no_fieldset: null,
  disabled: null,
  skeleton: null,
  id: null,
  name: null,
  status: null,
  status_state: 'error',
  status_animation: null,
  global_status_id: null,
  suffix: null,
  vertical: null,
  layout_direction: 'row',
  value: undefined,
  values: undefined,
  attributes: null,
  class: null,
  className: null,
  children: null,
  custom_element: null,
  custom_method: null,
  on_change: null
};
process.env.NODE_ENV !== "production" ? ToggleButtonGroup.propTypes = _extends({
  label: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  label_direction: PropTypes.oneOf(['horizontal', 'vertical']),
  label_sr_only: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  title: PropTypes.string,
  multiselect: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  variant: PropTypes.oneOf(['default', 'checkbox', 'radio']),
  left_component: PropTypes.node,
  no_fieldset: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  disabled: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  id: PropTypes.string,
  name: PropTypes.string,
  status: PropTypes.oneOfType([PropTypes.string, PropTypes.bool, PropTypes.func, PropTypes.node]),
  status_state: PropTypes.string,
  status_animation: PropTypes.string,
  global_status_id: PropTypes.string,
  suffix: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  vertical: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  layout_direction: PropTypes.oneOf(['column', 'row']),
  value: PropTypes.oneOfType([PropTypes.string, PropTypes.number, PropTypes.object, PropTypes.array]),
  values: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
  attributes: PropTypes.oneOfType([PropTypes.string, PropTypes.object])
}, spacingPropTypes, {
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  custom_element: PropTypes.object,
  custom_method: PropTypes.func,
  on_change: PropTypes.func
}) : void 0;