import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import Context from '../../shared/Context';
import { makeUniqueId, extendPropsWithContext, registerElement, validateDOMAttributes, processChildren } from '../../shared/component-helper';
import { spacingPropTypes, createSpacingClasses } from '../space/SpacingHelper';
import TooltipWithEvents from './TooltipWithEvents';
import TooltipPortal from './TooltipPortal';
export default class Tooltip extends React.PureComponent {
  static enableWebComponent() {
    registerElement(Tooltip.tagName, Tooltip, Tooltip.defaultProps);
  }

  static getContent(props) {
    return processChildren(props);
  }

  constructor(props) {
    super(props);
    this._id = props.id || makeUniqueId();
  }

  render() {
    const props = extendPropsWithContext(this.props, Tooltip.defaultProps, this.context.getTranslation(this.props).Tooltip, this.context.FormRow, this.context.Tooltip);

    const {
      component,
      target,
      class: class_name,
      className,
      group
    } = props,
          params = _objectWithoutPropertiesLoose(props, ["component", "target", "class", "className", "group", "animate_position", "show_delay", "hide_delay", "active", "position", "arrow", "align"]);

    const content = Tooltip.getContent(this.props);
    const classes = classnames("dnb-tooltip dnb-core-style", createSpacingClasses(props), class_name, className);

    const attributes = _extends({
      className: classes
    }, params);

    validateDOMAttributes(this.props, attributes);

    const newProps = _extends({}, this.props, {
      internal_id: this._id,
      group: this.props.id || group
    });

    if (typeof newProps.active === 'undefined') {
      delete newProps.active;
    }

    return React.createElement(React.Fragment, null, component ? React.createElement(TooltipWithEvents, _extends({
      target: component,
      attributes: attributes
    }, newProps), content) : target && React.createElement(TooltipPortal, _extends({
      target: target,
      attributes: attributes
    }, newProps), content));
  }

}
Tooltip.tagName = 'dnb-tooltip';
Tooltip.contextType = Context;
Tooltip.defaultProps = {
  id: null,
  group: 'main',
  active: undefined,
  position: 'top',
  arrow: 'center',
  align: null,
  animate_position: false,
  show_delay: 300,
  hide_delay: 500,
  class: null,
  className: null,
  children: null,
  onClick: null,
  on_click: null
};
process.env.NODE_ENV !== "production" ? Tooltip.propTypes = _extends({
  id: PropTypes.string,
  group: PropTypes.string,
  active: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  position: PropTypes.oneOf(['top', 'right', 'bottom', 'left']),
  arrow: PropTypes.oneOf([null, 'center', 'top', 'right', 'bottom', 'left']),
  align: PropTypes.oneOf([null, 'center', 'right', 'left']),
  animate_position: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  show_delay: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  hide_delay: PropTypes.oneOfType([PropTypes.string, PropTypes.number])
}, spacingPropTypes, {
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  onClick: PropTypes.func,
  on_click: PropTypes.oneOfType([PropTypes.string, PropTypes.func])
}) : void 0;