import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import TooltipPortal from './TooltipPortal';
export default class TooltipWithEvents extends React.PureComponent {
  constructor(...args) {
    super(...args);
    this.state = {
      isActive: false
    };

    this.onMouseEnter = () => {
      clearTimeout(this._onEnterTimeout);
      this._onEnterTimeout = setTimeout(() => {
        this.setState({
          isActive: true
        });
      }, this.props.show_delay || 1);
    };

    this.onMouseLeave = () => {
      clearTimeout(this._onEnterTimeout);
      this.setState({
        isActive: false
      });
    };
  }

  componentWillUnmount() {
    clearTimeout(this._onEnterTimeout);
    clearTimeout(this._targetTimeout);
  }

  render() {
    const _this$props = this.props,
          {
      children,
      className,
      target
    } = _this$props,
          props = _objectWithoutPropertiesLoose(_this$props, ["children", "className", "target"]);

    if (!this.state.target && typeof target.current !== 'undefined') {
      this._targetTimeout = setTimeout(() => {
        if (!target.current) return;

        try {
          const elem = target.current;
          elem.addEventListener('mouseenter', this.onMouseEnter);
          elem.addEventListener('mouseleave', this.onMouseLeave);
          const describedby = [this.props.internal_id];
          const db = elem.getAttribute('aria-describedby');

          if (db) {
            describedby.unshift(db);
          }

          elem.setAttribute('aria-describedby', describedby.join(' '));
        } catch (e) {
          console.warn('Tooltip: Could not add event listeners', e);
        }

        this.setState({
          target: target.current
        });
      }, 1);
      return null;
    }

    let componentWrapper;

    if (React.isValidElement(target)) {
      const describedby = [this.props.internal_id];

      if (target.props && target.props['aria-describedby']) {
        describedby.unshift(target.props['aria-describedby']);
      }

      componentWrapper = React.createElement("span", {
        key: "target-wrapper",
        ref: target => this.setState({
          target
        }),
        className: className,
        onMouseEnter: this.onMouseEnter,
        onMouseLeave: this.onMouseLeave
      }, React.cloneElement(target, {
        'aria-describedby': describedby.join(' ')
      }));
    }

    return React.createElement(React.Fragment, null, componentWrapper, this.state.target && React.createElement(TooltipPortal, _extends({
      key: "tooltip",
      active: this.state.isActive,
      target: this.state.target
    }, props), children));
  }

}
TooltipWithEvents.defaultProps = {
  className: '',
  internal_id: null,
  show_delay: 1,
  target: null,
  children: null
};
process.env.NODE_ENV !== "production" ? TooltipWithEvents.propTypes = {
  className: PropTypes.string,
  internal_id: PropTypes.string,
  show_delay: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  target: PropTypes.oneOfType([PropTypes.string, PropTypes.object, PropTypes.func, PropTypes.node]),
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node])
} : void 0;