import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import Context from '../shared/Context';
import { validateDOMAttributes, extendPropsWithContext } from '../shared/component-helper';
import { spacingPropTypes, createSpacingClasses } from '../components/space/SpacingHelper';
import { createSkeletonClass, skeletonDOMAttributes } from '../components/skeleton/SkeletonHelper';

class Elem extends React.PureComponent {
  constructor(props) {
    super(props);
  }

  render() {
    var _this$context, _this$context2;

    const props = this.props.skeleton !== false && typeof ((_this$context = this.context) === null || _this$context === void 0 ? void 0 : _this$context.skeleton) !== 'undefined' ? extendPropsWithContext(this.props, Elem.defaultProps, {
      skeleton: (_this$context2 = this.context) === null || _this$context2 === void 0 ? void 0 : _this$context2.skeleton
    }) : this.props;

    const {
      className,
      class: _className,
      internalClass,
      css,
      is: Tag,
      _ref,
      skeleton,
      skeleton_method
    } = props,
          rest = _objectWithoutPropertiesLoose(props, ["className", "class", "internalClass", "css", "is", "_ref", "skeleton", "skeleton_method"]);

    const tagClass = internalClass || `dnb-${Tag}`;
    rest.className = classnames(className, _className, css, createSkeletonClass(skeleton_method, skeleton, this.context), createSpacingClasses(rest, Tag), !new RegExp(`${tagClass}(\\s|$)`).test(String(className)) && tagClass);
    validateDOMAttributes(null, rest);
    skeletonDOMAttributes(rest, skeleton, this.context);
    return React.createElement(Tag, _extends({
      ref: _ref
    }, rest));
  }

}

Elem.contextType = Context;
Elem.defaultProps = {
  skeleton: null,
  skeleton_method: 'font',
  className: null,
  class: null,
  internalClass: null,
  css: null,
  _ref: null,
  children: null
};
process.env.NODE_ENV !== "production" ? Elem.propTypes = _extends({
  is: PropTypes.string.isRequired,
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  skeleton_method: PropTypes.string
}, spacingPropTypes, {
  className: PropTypes.oneOfType([PropTypes.string, PropTypes.object, PropTypes.array]),
  class: PropTypes.string,
  internalClass: PropTypes.string,
  css: PropTypes.oneOfType([PropTypes.string, PropTypes.object]),
  _ref: PropTypes.object,
  children: PropTypes.node
}) : void 0;
const Element = React.forwardRef((props, ref) => {
  return React.createElement(Elem, _extends({
    _ref: ref
  }, props));
});
process.env.NODE_ENV !== "production" ? Element.propTypes = Elem.propTypes : void 0;
Element.defaultProps = Elem.defaultProps;
export default Element;