import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { extendPropsWithContext, registerElement, validateDOMAttributes, processChildren } from '../../shared/component-helper';
import Context from '../../shared/Context';
import { spacingPropTypes, createSpacingClasses } from '../../components/space/SpacingHelper';

class ScrollView extends React.PureComponent {
  static enableWebComponent() {
    registerElement(ScrollView.tagName, ScrollView, ScrollView.defaultProps);
  }

  static getContent(props) {
    if (props.text) return props.text;
    return processChildren(props);
  }

  constructor(props) {
    super(props);
    this.ref = props.innerRef || React.createRef();
  }

  render() {
    const props = extendPropsWithContext(this.props, ScrollView.defaultProps, this.context.FormRow, this.context.ScrollView);

    const {
      className,
      class: _className
    } = props,
          attributes = _objectWithoutPropertiesLoose(props, ["className", "class", "innerRef"]);

    const contentToRender = ScrollView.getContent(props);

    const mainParams = _extends({
      className: classnames('dnb-scroll-view', createSpacingClasses(props), _className, className)
    }, attributes);

    validateDOMAttributes(this.props, mainParams);
    return React.createElement("div", _extends({}, mainParams, {
      ref: this.ref
    }), contentToRender);
  }

}

ScrollView.tagName = 'dnb-scroll-view';
ScrollView.contextType = Context;
ScrollView.defaultProps = {
  className: null,
  children: null,
  innerRef: null,
  class: null
};
process.env.NODE_ENV !== "production" ? ScrollView.propTypes = _extends({}, spacingPropTypes, {
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  innerRef: PropTypes.object,
  class: PropTypes.string
}) : void 0;
export default React.forwardRef(function ScrollViewRef(props, ref) {
  return React.createElement(ScrollView, _extends({
    innerRef: ref
  }, props));
});