import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { registerElement, validateDOMAttributes } from '../../shared/component-helper';
import Button from '../../components/button/Button';
export default class ActionNav extends React.PureComponent {
  static enableWebComponent() {
    registerElement(ActionNav.tagName, ActionNav, ActionNav.defaultProps);
  }

  render() {
    const {
      children,
      prev_href,
      next_href,
      prev_text,
      next_text,
      prev_title,
      next_title,
      className,
      class: _className
    } = this.props;
    const params = {
      className: classnames("dnb-action-nav dnb-belt", className, _className)
    };
    const hasNavAttributes = prev_href && next_href;
    validateDOMAttributes(this.props, params);
    return React.createElement("div", params, React.createElement("div", {
      className: "dnb-width-limit"
    }, React.createElement("div", {
      className: "dnb-action-nav__inner"
    }, hasNavAttributes && React.createElement(React.Fragment, null, React.createElement("div", {
      className: "dnb-action-nav__left"
    }, prev_href && React.createElement("div", {
      className: "dnb-action-nav__item"
    }, React.createElement(Button, {
      type: "button",
      text: prev_text,
      title: prev_title,
      variant: "secondary",
      icon_position: "left",
      icon: "chevron_left",
      href: prev_href
    })), next_href && React.createElement("div", {
      className: "dnb-action-nav__item"
    }, React.createElement(Button, {
      text: next_text,
      title: next_title,
      variant: "primary",
      icon: "chevron_right",
      href: next_href
    }))), React.createElement("div", {
      className: "dnb-action-nav__right"
    }, children)) || children)));
  }

}
ActionNav.tagName = 'dnb-action-nav';
ActionNav.defaultProps = {
  prev_href: null,
  next_href: null,
  prev_text: 'Back',
  next_text: 'Next',
  prev_title: 'Go back',
  next_title: 'Go next',
  class: null,
  className: null,
  children: null
};
process.env.NODE_ENV !== "production" ? ActionNav.propTypes = {
  prev_href: PropTypes.string,
  next_href: PropTypes.string,
  prev_text: PropTypes.string,
  next_text: PropTypes.string,
  prev_title: PropTypes.string,
  next_title: PropTypes.string,
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.func])
} : void 0;