import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { registerElement, validateDOMAttributes, processChildren } from '../../shared/component-helper';
export default class DescriptionList extends React.PureComponent {
  static enableWebComponent() {
    registerElement(DescriptionList.tagName, DescriptionList, DescriptionList.defaultProps);
  }

  static getData(props) {
    let res = [];
    if (props.data) res = props.data;else res = processChildren(props);
    if (typeof res === 'string') return res[0] === '[' ? JSON.parse(res) : [];
    return res || [];
  }

  static getDerivedStateFromProps(props, state) {
    if (state._listenForPropChanges) {
      if (props.data) {
        if (state._data !== props.data) {
          state._data = props.data;
          state.data = DescriptionList.getData(props);
        }
      }
    }

    state._listenForPropChanges = true;
    return state;
  }

  constructor(props) {
    super(props);
    this.state = {
      _listenForPropChanges: true,
      _data: props.data || props.children,
      data: DescriptionList.getData(props)
    };
  }

  render() {
    const {
      info,
      className,
      class: _className
    } = this.props;
    const data = DescriptionList.getData(this.props);
    const params = {
      className: classnames('dnb-description-list', className, _className)
    };
    validateDOMAttributes(this.props, params);
    return React.createElement("div", params, data.length > 0 && React.createElement("dl", {
      className: "dnb-description-list__dl typo-light"
    }, data.map(({
      title,
      value
    }, i) => React.createElement(React.Fragment, {
      key: `dl${i}`
    }, React.createElement("dt", {
      className: "dnb-description-list__dt"
    }, title), React.createElement("dd", {
      className: "dnb-description-list__dd"
    }, value)))), info && React.createElement("p", {
      className: "dnb-description-list__info dnb-typo-regular"
    }, info));
  }

}
DescriptionList.tagName = 'dnb-description-list';
DescriptionList.defaultProps = {
  data: [],
  info: null,
  class: null,
  className: null,
  children: null
};
process.env.NODE_ENV !== "production" ? DescriptionList.propTypes = {
  data: PropTypes.oneOfType([PropTypes.string, PropTypes.arrayOf(PropTypes.exact({
    title: PropTypes.string.isRequired,
    value: PropTypes.string.isRequired
  }))]).isRequired,
  info: PropTypes.string,
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.func])
} : void 0;