import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { registerElement, validateDOMAttributes } from '../../shared/component-helper';
import DescriptionList from '../description-list/DescriptionList';
export default class FormSummary extends React.PureComponent {
  static enableWebComponent() {
    registerElement(FormSummary.tagName, FormSummary, FormSummary.defaultProps);
  }

  render() {
    const {
      title,
      text,
      descriptionListData,
      descriptionListInfo,
      className,
      class: _className
    } = this.props;
    const params = {
      className: classnames('dnb-form-summary', className, _className)
    };
    validateDOMAttributes(this.props, params);
    return React.createElement("div", params, React.createElement("div", {
      className: "dnb-belt"
    }, React.createElement("div", {
      className: "dnb-width-limit"
    }, React.createElement("div", {
      className: "dnb-grid dnb-grid--gutters"
    }, React.createElement("div", {
      className: "dnb-grid__cell u-1of3"
    }, React.createElement("div", {
      className: "dnb-form-summary__header"
    }, React.createElement("h2", {
      className: "dnb-form-summary__title typo-light"
    }, title), React.createElement("p", {
      className: "dnb-form-summary__info dnb-typo-regular"
    }, text))), React.createElement("div", {
      className: "dnb-grid__cell"
    }, React.createElement("div", {
      className: "dnb-form-summary__list"
    }, React.createElement(DescriptionList, {
      data: descriptionListData,
      info: descriptionListInfo
    })))))));
  }

}
FormSummary.tagName = 'dnb-form-summary';
FormSummary.defaultProps = {
  title: null,
  text: null,
  descriptionListData: null,
  descriptionListInfo: null,
  class: null,
  className: null,
  children: null
};
process.env.NODE_ENV !== "production" ? FormSummary.propTypes = {
  title: PropTypes.string,
  text: PropTypes.string,
  descriptionListData: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
  descriptionListInfo: PropTypes.string,
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.func])
} : void 0;