import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import Context from '../../shared/Context';
import Provider from '../../shared/Provider';
import { isTrue, registerElement, validateDOMAttributes, extendPropsWithContext } from '../../shared/component-helper';
import { spacingPropTypes, createSpacingClasses } from '../../components/space/SpacingHelper';
import { skeletonDOMAttributes, createSkeletonClass } from '../../components/skeleton/SkeletonHelper';
import P from '../../elements/P';
import { ProductType, CardType } from './utils/Types';
import Designs, { defaultDesign } from './utils/CardDesigns';
import cardProducts from './utils/cardProducts';
import { ProductLogo, TypeLogo, BankLogo, StatusIcon } from './icons';
export { Designs, ProductType, CardType };
const cardDataTypes = process.env.NODE_ENV !== "production" ? PropTypes.shape({
  productCode: PropTypes.string.isRequired,
  productName: PropTypes.string.isRequired,
  displayName: PropTypes.string.isRequired,
  cardDesign: PropTypes.shape.isRequired,
  cardType: PropTypes.shape.isRequired,
  productType: PropTypes.shape.isRequired
}) : {};
const translationDefaultPropsProps = {
  text_card_number: null,
  text_expired: null,
  text_blocked: null
};
export default class PaymentCard extends React.PureComponent {
  static enableWebComponent() {
    registerElement(PaymentCard.tagName, PaymentCard, PaymentCard.defaultProps);
  }

  render() {
    var _this$context;

    const props = extendPropsWithContext(this.props, PaymentCard.defaultProps, {
      locale: this.context.locale
    }, {
      skeleton: (_this$context = this.context) === null || _this$context === void 0 ? void 0 : _this$context.skeleton
    });

    const {
      product_code,
      card_number,
      card_status,
      variant,
      digits,
      id,
      raw_data,
      locale,
      skeleton,
      className,
      class: _className
    } = props,
          attributes = _objectWithoutPropertiesLoose(props, ["product_code", "card_number", "card_status", "variant", "digits", "id", "raw_data", "locale", "skeleton", "className", "class", "children"]);

    const cardData = raw_data || getCardData(product_code);

    const params = _extends({
      className: classnames(`dnb-payment-card dnb-payment-card--${variant}`, createSkeletonClass(null, skeleton, this.context), createSpacingClasses(props), className, _className)
    }, attributes);

    skeletonDOMAttributes(params, skeleton, this.context);
    validateDOMAttributes(this.props, params);
    return React.createElement(Provider, {
      locale: locale
    }, React.createElement(Context.Consumer, null, ({
      translation
    }) => {
      const translations = extendPropsWithContext(this.props, translationDefaultPropsProps, translation.PaymentCard);
      return React.createElement("figure", params, React.createElement("figcaption", {
        className: "dnb-sr-only dnb-payment-card__figcaption"
      }, cardData.productName), React.createElement(NormalCard, {
        id: id,
        skeleton: isTrue(skeleton),
        data: cardData,
        cardStatus: card_status,
        cardNumber: formatCardNumber(card_number, parseFloat(digits)),
        translations: translations
      }));
    }));
  }

}
PaymentCard.tagName = 'dnb-payment-card';
PaymentCard.contextType = Context;
PaymentCard.defaultProps = _extends({
  digits: 8,
  locale: null,
  card_status: 'active',
  variant: 'normal',
  id: null,
  raw_data: null,
  skeleton: false,
  class: null,
  className: null,
  children: null
}, translationDefaultPropsProps);
process.env.NODE_ENV !== "production" ? PaymentCard.propTypes = _extends({
  product_code: PropTypes.string.isRequired,
  card_number: PropTypes.string.isRequired,
  card_status: PropTypes.oneOf(['active', 'blocked', 'expired']),
  variant: PropTypes.oneOf(['normal', 'compact']),
  digits: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  raw_data: cardDataTypes,
  id: PropTypes.string,
  locale: PropTypes.string,
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool])
}, spacingPropTypes, {
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.func])
}) : void 0;
const formatCardNumberRegex = /(?=(?:....)*$)/g;
export const formatCardNumber = (cardNumber, digits) => digits ? cardNumber.slice(cardNumber.length - digits, cardNumber.length).replace(formatCardNumberRegex, ' ') : cardNumber.replace(formatCardNumberRegex, ' ');
export const getCardData = productCode => {
  const card = cardProducts.find(item => item.productCode === productCode);
  return card || defaultCard(productCode);
};

const defaultCard = productCode => ({
  productCode,
  productName: '',
  displayName: '',
  cardDesign: defaultDesign,
  cardType: CardType.None,
  productType: ProductType.None
});

process.env.NODE_ENV !== "production" ? StatusOverlay.propTypes = {
  cardStatus: PropTypes.string.isRequired,
  translations: PropTypes.object.isRequired,
  skeleton: PropTypes.bool
} : void 0;
StatusOverlay.defaultProps = {
  skeleton: false
};

function StatusOverlay({
  cardStatus,
  translations,
  skeleton
}) {
  switch (cardStatus) {
    case 'blocked':
      return React.createElement("div", {
        className: classnames('dnb-payment-card__blocking__overlay', createSkeletonClass('font', skeleton))
      }, React.createElement("div", {
        className: "dnb-payment-card__blocking__center"
      }, React.createElement(StatusIcon, {
        status: cardStatus
      }), React.createElement(P, {
        top: "xx-small"
      }, translations.text_blocked)));

    case 'expired':
      return React.createElement("div", {
        className: classnames('dnb-payment-card__blocking__overlay', createSkeletonClass('font', skeleton))
      }, React.createElement("div", {
        className: "dnb-payment-card__blocking__center"
      }, React.createElement(StatusIcon, {
        status: cardStatus
      }), React.createElement(P, {
        top: "xx-small"
      }, translations.text_expired)));

    case 'active':
    default:
      return null;
  }
}

process.env.NODE_ENV !== "production" ? CardText.propTypes = {
  cardNumber: PropTypes.string.isRequired,
  translations: PropTypes.object.isRequired,
  skeleton: PropTypes.bool.isRequired
} : void 0;

function CardText({
  cardNumber,
  translations,
  skeleton
}) {
  return React.createElement("span", {
    className: classnames('dnb-payment-card__card__wrapper', createSkeletonClass('font', skeleton))
  }, React.createElement(P, {
    className: "dnb-payment-card__card__holder",
    modifier: "x-small bold"
  }, translations.text_card_number), React.createElement(P, {
    className: "dnb-payment-card__card__numbers"
  }, cardNumber));
}

process.env.NODE_ENV !== "production" ? NormalCard.propTypes = {
  id: PropTypes.string,
  skeleton: PropTypes.bool,
  data: cardDataTypes.isRequired,
  cardStatus: PropTypes.string.isRequired,
  cardNumber: PropTypes.string.isRequired,
  translations: PropTypes.object.isRequired
} : void 0;
NormalCard.defaultProps = {
  id: null,
  skeleton: null
};

function NormalCard({
  data,
  cardStatus,
  cardNumber,
  id,
  skeleton,
  translations
}) {
  return React.createElement("div", {
    id: id,
    className: `dnb-payment-card__card dnb-payment-card__${data.cardDesign.cardStyle}`
  }, React.createElement("div", {
    className: "dnb-payment-card__card__content"
  }, React.createElement("div", {
    className: "dnb-payment-card__card__top"
  }, React.createElement(BankLogo, {
    logoType: data.cardDesign.bankLogo
  }), React.createElement(ProductLogo, {
    productType: data.productType,
    cardDesign: data.cardDesign
  })), React.createElement("div", {
    className: "dnb-payment-card__card__bottom"
  }, React.createElement(CardText, {
    cardNumber: cardNumber,
    displayName: data.displayName,
    translations: translations,
    skeleton: skeleton
  }), React.createElement(TypeLogo, {
    cardType: data.cardType,
    cardDesign: data.cardDesign
  }))), React.createElement(StatusOverlay, {
    skeleton: skeleton,
    cardStatus: cardStatus,
    translations: translations
  }));
}