import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { registerElement, validateDOMAttributes } from '../../shared/component-helper';
import Slider from '../../components/slider/Slider';
import Input from '../../components/input/Input';
import Modal from '../../components/modal/Modal';
import FormLabel from '../../components/form-label/FormLabel';
export default class RangeSlider extends React.PureComponent {
  static enableWebComponent() {
    registerElement(RangeSlider.tagName, RangeSlider, RangeSlider.defaultProps);
  }

  constructor(props) {
    super(props);
    this.state = {
      value: null
    };
    this.state.value = props.range_val;
  }

  render() {
    const {
      range_min,
      range_max,
      range_step,
      label_text,
      range_output_description,
      range_output_extra_information,
      range_trigger_text,
      range_modal_text,
      className,
      class: _className
    } = this.props;
    const params = {
      className: classnames('dnb-range-slider', className, _className)
    };
    validateDOMAttributes(this.props, params);
    return React.createElement("div", params, React.createElement("div", {
      className: "dnb-range-slider__label"
    }, React.createElement("div", {
      className: "dnb-range-slider__label__inner"
    }, React.createElement(FormLabel, {
      for_id: "range-slider",
      text: label_text
    }))), React.createElement("div", {
      className: "dnb-range-slider__slider"
    }, React.createElement("div", {
      className: "dnb-range-slider__input-container"
    }, React.createElement(Slider, {
      id: "range-slider",
      min: range_min,
      max: range_max,
      value: this.state.value,
      step: range_step,
      on_init: ({
        value
      }) => {
        this.setState({
          value
        });
      },
      on_change: ({
        value
      }) => {
        this.setState({
          value
        });
      }
    })), React.createElement("div", {
      className: "dnb-range-slider__output-container"
    }, React.createElement(Input, {
      type: "text",
      size: "large",
      value: this.state.value,
      description: range_output_description,
      status: range_output_extra_information,
      status_state: "info",
      on_change: ({
        value
      }) => {
        this.setState({
          value: parseFloat(value)
        });
      }
    })), React.createElement("div", {
      className: "dnb-range-slider__slider__modal"
    }, React.createElement(Modal, {
      type: "button",
      trigger_text: range_trigger_text,
      modal_content: range_modal_text
    }))));
  }

}
RangeSlider.tagName = 'dnb-range-slider';
RangeSlider.defaultProps = {
  title: null,
  text: null,
  descriptionListData: null,
  descriptionListInfo: null,
  class: null,
  className: null,
  children: null
};
process.env.NODE_ENV !== "production" ? RangeSlider.propTypes = {
  label_text: PropTypes.string.isRequired,
  range_min: PropTypes.number.isRequired,
  range_max: PropTypes.number.isRequired,
  range_val: PropTypes.number.isRequired,
  range_step: PropTypes.number.isRequired,
  range_output_description: PropTypes.string.isRequired,
  range_output_extra_information: PropTypes.string.isRequired,
  range_trigger_text: PropTypes.string.isRequired,
  range_modal_text: PropTypes.string.isRequired,
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.func])
} : void 0;