import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import { LOCALE, CURRENCY, CURRENCY_DISPLAY } from './defaults';
import defaultLocales from './locales';
import { extend } from './component-helper';
export const prepareContext = (props = {}) => {
  const locales = props.locales ? extend(defaultLocales, props.locales) : defaultLocales;

  if (props.__newContext) {
    _extends(props, props.__newContext);

    delete props.__newContext;
  }

  const key = handleLocaleFallbacks(props.locale || LOCALE, locales);
  const translation = locales[key] || defaultLocales[LOCALE] || {};

  if (locales[key]) {
    locales[key] = destruct(locales[key], translation);
  }

  const context = _extends({
    updateTranslation: (locale, translation) => {
      context.translation = context.locales[locale] = translation;
    },
    getTranslation: props => {
      if (props) {
        const lang = props.lang || props.locale;

        if (lang && context.locales[lang] && lang !== key) {
          return context.locales[lang];
        }
      }

      return context.translation;
    },
    locales
  }, props, {
    translation
  });

  return context;
};

function handleLocaleFallbacks(locale, locales) {
  if (!locales[locale]) {
    if (locale === 'en' || locale.split('-')[0] === 'en') {
      return 'en-GB';
    }
  }

  return locale;
}

const Context = React.createContext(prepareContext({
  locale: LOCALE,
  currency: CURRENCY,
  currency_display: CURRENCY_DISPLAY
}));
export default Context;

function destruct(source, validKeys) {
  for (let k in source) {
    if (String(k).includes('.')) {
      const list = k.split('.');

      if (validKeys[list[0]]) {
        const val = source[k];
        const last = list.length - 1;
        list.forEach((k, i) => {
          source[k] = i === last ? val : source[k];
          source = source[k] || {};
        });
      }
    }
  }

  return source;
}