import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import Context, { prepareContext } from './Context';
import { makeUniqueId } from './component-helper';
import { prepareFormRowContext } from '../components/form-row/FormRow';
export default class Provider extends React.PureComponent {
  static getDerivedStateFromProps(props, state) {
    if (state._listenForPropChanges) {
      const updatedProps = _objectWithoutPropertiesLoose(props, ["children"]);

      let newContext = state;

      if (state._startupProps !== updatedProps) {
        let hasChanges = false;

        for (const i in updatedProps) {
          if (state._startupProps[i] !== updatedProps[i] || typeof updatedProps[i] === 'boolean') {
            hasChanges = true;
            break;
          }
        }

        if (hasChanges) {
          newContext = _extends(state, updatedProps);
          state._startupProps = updatedProps;
        }
      }

      if (newContext.FormRow) {
        newContext.FormRow = prepareFormRowContext(newContext.FormRow);
      }

      state = newContext;
    }

    state._listenForPropChanges = true;
    return prepareContext(state);
  }

  constructor(props, context) {
    super(props);

    const {
      value
    } = props,
          startupProps = _objectWithoutPropertiesLoose(props, ["children", "value"]);

    if (typeof startupProps.formRow !== 'undefined') {
      startupProps.FormRow = startupProps.formRow;
    }

    const newContext = _extends({}, value, context, startupProps);

    const isRoot = !(newContext && newContext.__providerId);
    newContext.__providerId = makeUniqueId();
    const pC = isRoot ? prepareContext(newContext) : newContext;

    pC.updateCurrent = props => this.setNewContext(props);

    pC.setCurrentLocale = locale => this.setNewContext({
      locale
    });

    pC.update = props => {
      if (typeof context.update === 'function') {
        context.update(props);
      }

      this.setNewContext(props);
    };

    pC.setLocale = locale => {
      if (typeof context.update === 'function') {
        context.update({
          locale
        });
      }

      this.setNewContext({
        locale
      });
    };

    this.state = pC;
    this.state.isRoot = isRoot;
    this.state._listenForPropChanges = true;
    this.state._startupProps = startupProps;
  }

  setNewContext(__newContext) {
    this.setState({
      _listenForPropChanges: false
    });
    this.setState({
      __newContext
    });
  }

  render() {
    const {
      children
    } = this.props;
    const context = !this.state.isRoot ? _extends({}, this.context, this.state) : this.state;
    this.context.updateTranslation(context.locale, context.translation);
    return React.createElement(Context.Provider, {
      value: context
    }, children);
  }

}
Provider.contextType = Context;
Provider.defaultProps = {};
process.env.NODE_ENV !== "production" ? Provider.propTypes = {
  value: PropTypes.object,
  locale: PropTypes.string,
  skeleton: PropTypes.bool,
  children: PropTypes.node.isRequired
} : void 0;