import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import keycode from 'keycode';
import whatInput from 'what-input';
import { registerElement } from './custom-element';
export { registerElement };
export const PLATFORM_MAC = 'Mac|iPad|iPhone|iPod';
export const PLATFORM_WIN = 'Win';
export const PLATFORM_LINUX = 'Linux';
export const PLATFORM_IOS = 'iOS|iPhone|iPad|iPod';

if (typeof process !== 'undefined' && process.env.NODE_ENV === 'test' && typeof window !== 'undefined') {
  window.IS_TEST = true;
}

whatInput.specificKeys([9]);
defineNavigator();
export function isTouchDevice() {
  if (typeof document !== 'undefined') {
    let intent = false;

    try {
      intent = document.documentElement.getAttribute('data-whatintent');
    } catch (e) {}

    return intent === 'touch';
  }

  return false;
}
export function defineNavigator() {
  const handleNavigator = () => {
    if (typeof document === 'undefined' || typeof window === 'undefined' || typeof navigator === 'undefined') {
      return;
    }

    try {
      if (!(typeof window !== 'undefined' && window.IS_TEST)) {
        if (navigator.platform.match(new RegExp(PLATFORM_MAC)) !== null) {
          document.documentElement.setAttribute('data-os', 'mac');
        } else if (navigator.platform.match(new RegExp(PLATFORM_WIN)) !== null) {
          document.documentElement.setAttribute('data-os', 'win');
        } else if (navigator.platform.match(new RegExp(PLATFORM_LINUX)) !== null) {
          document.documentElement.setAttribute('data-os', 'linux');
        }
      } else {
        document.documentElement.setAttribute('data-os', 'other');
      }
    } catch (e) {
      warn(e);
    }

    document.removeEventListener('DOMContentLoaded', handleNavigator);
  };

  if (typeof document !== 'undefined' && document.readyState === 'loading') {
    document.addEventListener('DOMContentLoaded', handleNavigator);
  } else {
    handleNavigator();
  }
}
export const validateDOMAttributes = (props, params) => {
  if (props && props.attributes) {
    let attr = props.attributes;

    if (attr) {
      if (attr[0] === '{') attr = JSON.parse(attr);

      if (attr && typeof attr === 'object') {
        Object.entries(attr).forEach(([key, value]) => {
          _extends(params, {
            [key]: value
          });
        });
      }
    }
  }

  if (params.disabled === null || params.disabled === 'false') {
    delete params.disabled;
  }

  if (typeof params.space !== 'undefined') {
    delete params.space;
  }

  if (typeof params.top !== 'undefined') {
    delete params.top;
  }

  if (typeof params.right !== 'undefined') {
    delete params.right;
  }

  if (typeof params.bottom !== 'undefined') {
    delete params.bottom;
  }

  if (typeof params.left !== 'undefined') {
    delete params.left;
  }

  if (typeof params.no_collapse !== 'undefined') {
    delete params.no_collapse;
  } else if (params.disabled === 'true') {
      params.disabled = true;
    }

  if (params.disabled === true) {
    params['aria-disabled'] = true;
  }

  if (props && props.tabindex) {
    let tabIndex = props.tabindex;

    if (tabIndex === 'off') {
      tabIndex = '-1';
    }

    params['tabIndex'] = tabIndex;
  }

  if (params && typeof params === 'object') {
    for (const i in params) {
      if (typeof params[i] === 'function' && !/(^[a-z]{1,}[A-Z]{1})/.test(i)) {
        delete params[i];
      } else if (params[i] === null || /[^a-z-]/i.test(i)) {
          delete params[i];
        }
    }
  }

  return params;
};
export const processChildren = props => {
  if (!props) {
    return null;
  }

  if (typeof global !== 'undefined' && Array.isArray(global.registeredElements) && global.registeredElements.length > 0) {
    let cache = null;
    Object.entries(props).reverse().map(([key, cb]) => {
      if (key.includes('render_') && /^render_/.test(key)) {
        if (typeof cb === 'function') {
          if (cache) {
            if (Object.isFrozen(props)) {
              props = _extends({}, props);
            }

            props.children = cache;
          }

          return cache = React.createElement(React.Fragment, {
            key: key
          }, cb(props));
        }
      }

      return null;
    }).filter(Boolean);

    if (cache) {
      return cache;
    }
  }

  const res = typeof props.children === 'function' ? props.children(props) : props.children;

  if (Array.isArray(res)) {
    const onlyTexts = res.reduce((pV, cV) => {
      if (typeof cV === 'string' || typeof cV === 'number') {
        pV.push(cV);
      }

      return pV;
    }, []);

    if (onlyTexts.length === res.length && onlyTexts.length > 0) {
      return onlyTexts.join('');
    }
  }

  return res;
};
export const extend = (...objects) => {
  let first = {};
  const keepRef = objects[0];

  if (keepRef === true || keepRef === false) {
    objects.shift();

    if (keepRef) {
      first = objects.shift();
    }
  }

  return objects.reduce((acc1, object) => {
    if (object) {
      acc1 = _extends(acc1, Object.entries(object).reduce((acc2, [key, value]) => {
        if (value !== null) {
          if (typeof value === 'object') {
            value = extend(acc1[key] || {}, value);

            if (Object.keys(value).length > 0) {
              acc2[key] = value;
            }
          } else {
            acc2[key] = value;
          }
        }

        return acc2;
      }, {}));
    }

    return acc1;
  }, first);
};
export const extendPropsWithContext = (props, defaults = {}, ...contexts) => {
  const context = contexts.reduce((acc, cur) => {
    if (cur) {
      acc = _extends({}, acc, cur);
    }

    return acc;
  }, {});
  return _extends({}, props, Object.entries(context).reduce((acc, [key, value]) => {
    if (typeof props[key] !== 'undefined' && props[key] === defaults[key]) {
      acc[key] = value;
    }

    return acc;
  }, {}));
};
export const isTrue = value => {
  if (value !== null && typeof value !== 'undefined' && (String(value) === 'true' || String(value) === '1')) {
    return true;
  }

  return false;
};
export const dispatchCustomElementEvent = (src, eventName, eventObjectOrig) => {
  let ret = undefined;

  const eventObject = _extends({}, eventObjectOrig.event || {}, eventObjectOrig);

  if (eventObject && eventObject.attributes && eventObject.event) {
    const currentTarget = eventObject.event.currentTarget;

    if (currentTarget) {
      try {
        const dataset = _extends({}, currentTarget.dataset || {});

        const attributes = _extends({}, eventObject.attributes);

        for (const i in attributes) {
          if (/^data-/.test(i)) {
            dataset[String(i).replace(/^data-/, '')] = attributes[i];
          }
        }

        for (const i in dataset) {
          if (eventObject.event.currentTarget.dataset) {
            eventObject.event.currentTarget.dataset[i] = dataset[i];
          }

          if (eventObject.event.target && eventObject.event.target.dataset) {
            eventObject.event.target.dataset[i] = dataset[i];
          }
        }
      } catch (e) {
        warn('Error on handling dataset:', e);
      }
    }
  }

  const props = src && src.props || src;

  if (props.custom_element) {
    if (typeof props.custom_element.fireEvent === 'function') {
      ret = props.custom_element.fireEvent(eventName, eventObject);
    }
  }

  if (eventName.includes('_')) {
    if (typeof props[eventName] === 'function') {
      const r = props[eventName].apply(src, [eventObject]);

      if (typeof r !== 'undefined') {
        ret = r;
      }
    }

    eventName = toCamelCase(eventName);

    if (typeof props[eventName] === 'function') {
      const r = props[eventName].apply(src, [eventObject]);

      if (typeof r !== 'undefined') {
        ret = r;
      }
    }
  } else {
    if (typeof props[eventName] === 'function') {
      const r = props[eventName].apply(src, [eventObject]);

      if (typeof r !== 'undefined') {
        ret = r;
      }
    }

    eventName = toSnakeCase(eventName);

    if (typeof props[eventName] === 'function') {
      const r = props[eventName].apply(src, [eventObject]);

      if (typeof r !== 'undefined') {
        ret = r;
      }
    }
  }

  return ret;
};
export const toCamelCase = s => s.split(/_/g).reduce((acc, cur, i) => acc + (i === 0 ? cur : cur.replace(/(\w)(\w*)/g, (g0, g1, g2) => g1.toUpperCase() + g2.toLowerCase())), '');
export const toPascalCase = s => s.split(/_/g).reduce((acc, cur) => acc + cur.replace(/(\w)(\w*)/g, (g0, g1, g2) => g1.toUpperCase() + g2.toLowerCase()), '');
export const toSnakeCase = str => str.replace(/\B[A-Z]/g, letter => `_${letter}`).toLowerCase();
export const toKebabCase = str => str.replace(/\B[A-Z]/g, letter => `-${letter}`).toLowerCase();
export const detectOutsideClick = (ignoreElements, onSuccess, options) => new DetectOutsideClickClass(ignoreElements, onSuccess, options);
export class DetectOutsideClickClass {
  constructor(_ignoreElements, _onSuccess, options = {}) {
    this.checkOutsideClick = ({
      currentElement,
      ignoreElements
    }, onSuccess = null) => {
      try {
        if (typeof currentElement.tagName === 'undefined' || /html/i.test(currentElement.tagName)) {
            return;
          }

        if (checkIfHasScrollbar(currentElement)) {
          return;
        }

        for (let i = 0, elem, l = ignoreElements.length; i < l; ++i) {
          elem = currentElement;

          if (!ignoreElements[i]) {
            continue;
          }

          do {
            if (elem === ignoreElements[i]) {
              return;
            }

            elem = elem && elem.parentNode;
          } while (elem);
        }

        if (typeof onSuccess === 'function') {
          onSuccess();
        }
      } catch (e) {
        warn(e);
      }
    };

    if (!this.handleClickOutside && typeof document !== 'undefined' && typeof window !== 'undefined') {
      if (!Array.isArray(_ignoreElements)) {
        _ignoreElements = [_ignoreElements];
      }

      this.handleClickOutside = event => {
        this.checkOutsideClick({
          currentElement: event.target,
          ignoreElements: _ignoreElements
        }, () => typeof _onSuccess === 'function' && _onSuccess({
          event
        }));
      };

      document.addEventListener('mousedown', this.handleClickOutside);

      this.keydownCallback = event => {
        const keyCode = keycode(event);

        if (keyCode === 'esc') {
          window.removeEventListener('keydown', this.keydownCallback);

          if (typeof _onSuccess === 'function') {
            _onSuccess({
              event
            });
          }
        }
      };

      window.addEventListener('keydown', this.keydownCallback);

      if (options.includedKeys) {
        this.keyupCallback = event => {
          const keyCode = keycode(event);

          if (options.includedKeys.includes(keyCode) && typeof this.handleClickOutside === 'function') {
            this.handleClickOutside(event, () => {
              if (this.keyupCallback) window.removeEventListener('keyup', this.keyupCallback);
            });
          }
        };

        window.addEventListener('keyup', this.keyupCallback);
      }
    }
  }

  remove() {
    if (this.handleClickOutside && typeof document !== 'undefined') {
      document.removeEventListener('mousedown', this.handleClickOutside);
      this.handleClickOutside = null;
    }

    if (this.keydownCallback && typeof window !== 'undefined') {
      window.removeEventListener('keydown', this.keydownCallback);
      this.keydownCallback = null;
    }

    if (this.keyupCallback && typeof window !== 'undefined') {
      window.removeEventListener('keyup', this.keyupCallback);
      this.keyupCallback = null;
    }
  }

}
export const checkIfHasScrollbar = elem => {
  return elem && (elem.scrollHeight > elem.offsetHeight || elem.scrollWidth > elem.offsetWidth) && overflowIsScrollable(elem);
};

const overflowIsScrollable = elem => {
  const style = window.getComputedStyle(elem);
  return /scroll|auto/i.test(style.overflow + (style.overflowX || '') + (style.overflowY || ''));
};

export const filterProps = (props, remove = null, allowed = null) => {
  if (Array.isArray(remove)) {
    remove = remove.reduce((acc, key) => {
      acc[key] = true;
      return acc;
    }, {});
  }

  if (Array.isArray(allowed)) {
    allowed = allowed.reduce((acc, key) => {
      acc[key] = true;
      return acc;
    }, {});
  }

  return Object.entries(props).reduce((acc, [k, v]) => {
    if (remove && typeof remove[k] === 'undefined' || allowed && typeof allowed[k] !== 'undefined') {
      acc[k] = v;
    }

    return acc;
  }, {});
};
export const makeUniqueId = (prefix = '', length = 8) => prefix + String(Math.random().toString(36).substr(2, length) + idIncrement++).slice(-length);
let idIncrement = 0;
export const slugify = s => String(s).toLowerCase().replace(/[^\w\s-]/g, '').replace(/[\s_-]+/g, '-').replace(/^-+|-+$/g, '');
export const matchAll = (string, regex) => {
  if (typeof string.matchAll === 'function') {
    return Array.from(string.matchAll(regex));
  }

  const matches = [];
  let match;

  while (match = regex.exec(string)) {
    matches.push(match);
  }

  return matches;
};
export const getPreviousSibling = (className, element) => {
  try {
    const contains = element => element && element.classList.contains(className);

    if (contains(element)) {
      return element;
    }

    while ((element = element && element.parentElement) && !contains(element));
  } catch (e) {
    warn(e);
  }

  return element;
};
export const isChildOfElement = (element, target, cb = null) => {
  try {
    const contains = element => {
      if (cb) {
        const res = cb(element);

        if (typeof res === 'boolean') {
          return res;
        }
      }

      return element && element === target;
    };

    if (contains(element)) {
      return element;
    }

    while ((element = element && element.parentElement) && !contains(element));
  } catch (e) {}

  return element;
};
export const roundToNearest = (num, target) => {
  const diff = num % target;
  return diff > target / 2 ? num - diff + target : num - diff;
};
export const isInsideScrollView = (currentElement, returnElement = false) => {
  const elem = getPreviousSibling('dnb-scroll-view', currentElement);

  if (returnElement) {
    return elem == window ? null : elem;
  }

  return elem == window ? false : Boolean(elem);
};
export const warn = (...e) => {
  if (typeof process !== 'undefined' && typeof console !== 'undefined' && process.env.NODE_ENV !== 'production' && typeof console.log === 'function') {
    console.log('Eufemia:', ...e);
  }
};
export const convertJsxToString = (elements, separator = undefined) => {
  if (!Array.isArray(elements)) {
    elements = [elements];
  }

  const process = word => {
    if (React.isValidElement(word)) {
      if (typeof word.props.children === 'string') {
        word = word.props.children;
      } else if (Array.isArray(word.props.children)) {
        word = word.props.children.reduce((acc, word) => {
          if (typeof word !== 'string') {
            word = process(word);
          }

          if (typeof word === 'string') {
            acc = acc + word;
          }

          return acc;
        }, '');
      } else {
        return null;
      }
    }

    return word;
  };

  return elements.map(word => process(word)).filter(Boolean).join(separator);
};
export class InteractionInvalidation {
  constructor() {
    this.bypassElement = null;
    this.bypassSelectors = [];
    return this;
  }

  setBypassElement(bypassElement) {
    if (bypassElement instanceof HTMLElement) {
      this.bypassElement = bypassElement;
    }

    return this;
  }

  setBypassSelector(bypassSelector) {
    if (!Array.isArray(bypassSelector)) {
      bypassSelector = [bypassSelector];
    }

    this.bypassSelectors = bypassSelector;
    return this;
  }

  activate(targetElement = null) {
    if (!this.nodesToInvalidate) {
      this._runInvalidaiton(targetElement);
    }
  }

  revert() {
    this._revertInvalidation();

    this.nodesToInvalidate = null;
  }

  _runInvalidaiton(targetElement) {
    if (typeof document === 'undefined') {
        return;
      }

    this.nodesToInvalidate = this.getNodesToInvalidate(targetElement);

    if (Array.isArray(this.nodesToInvalidate)) {
      this.nodesToInvalidate.forEach(node => {
        try {
          if (node && typeof node._orig_tabindex === 'undefined' && node.hasAttribute('tabindex')) {
            node._orig_tabindex = node.getAttribute('tabindex');
          }

          if (node && typeof node._orig_ariahidden === 'undefined' && node.hasAttribute('aria-hidden')) {
            node._orig_ariahidden = node.getAttribute('aria-hidden');
          }

          node.setAttribute('tabindex', '-1');
          node.setAttribute('aria-hidden', 'true');
        } catch (e) {}
      });
    }
  }

  _revertInvalidation() {
    if (!this.nodesToInvalidate) {
      return;
    }

    this.nodesToInvalidate.forEach(node => {
      try {
        if (node && typeof node._orig_tabindex !== 'undefined') {
          node.setAttribute('tabindex', node._orig_tabindex);
          node._orig_tabindex = null;
          delete node._orig_tabindex;
        } else {
          node.removeAttribute('tabindex');
        }

        if (node && typeof node._orig_ariahidden !== 'undefined') {
          node.setAttribute('aria-hidden', node._orig_ariahidden);
          node._orig_ariahidden = null;
          delete node._orig_ariahidden;
        } else {
          node.removeAttribute('aria-hidden');
        }
      } catch (e) {}
    });
  }

  getNodesToInvalidate(targetElement = null) {
    if (typeof document === 'undefined') {
      return;
    }

    if (typeof targetElement === 'string') {
      targetElement = document.querySelector(targetElement);
    }

    const skipTheseNodes = this.bypassSelectors && this.bypassSelectors.length > 0 ? Array.from((this.bypassElement || document).querySelectorAll(this.bypassSelectors ? this.bypassSelectors.map(s => `${s} *`).join(', ') : '*')) : [];
    const rootSelector = targetElement ? '*' : 'html *';
    const elementSelector = this.bypassSelectors.map(s => `:not(${s})`).join('');
    const selector = `${rootSelector} ${elementSelector}:not(script):not(style):not(path)`;
    return Array.from((targetElement || document.documentElement).querySelectorAll(selector)).filter(node => !skipTheseNodes.includes(node));
  }

}
export class AnimateHeight {
  constructor(opts = {}) {
    this.state = 'init';
    this.onStartStack = [];
    this.onEndStack = [];
    this.opts = opts;
  }

  setElem(elem, container = null) {
    var _this$elem;

    this.elem = elem || typeof document !== 'undefined' && document.createElement('div');

    if (String((_this$elem = this.elem) === null || _this$elem === void 0 ? void 0 : _this$elem.nodeName).toLowerCase() === 'td') {
      this.elem = this.elem.parentElement;
    }

    this.container = container;

    if (this.container) {
      this.onResize = () => {
        clearTimeout(this.resizeTimeout);
        this.resizeTimeout = setTimeout(() => this.setContainerHeight(), 300);
      };

      window.addEventListener('resize', this.onResize);
    }
  }

  removeEndEvents() {
    if (this.onOpenEnd) {
      this.elem.removeEventListener('transitionend', this.onOpenEnd);
      this.onOpenEnd = null;
    }

    if (this.onAdjustEnd) {
      this.elem.removeEventListener('transitionend', this.onAdjustEnd);
      this.onAdjustEnd = null;
    }

    if (this.onCloseEnd) {
      this.elem.removeEventListener('transitionend', this.onCloseEnd);
      this.onCloseEnd = null;
    }
  }

  remove() {
    this.removeEndEvents();
    this.isAnimating = false;
    this.onStartStack = null;
    this.onEndStack = null;
    this.stop();
    this.elem = null;
    this.state = 'init';

    if (this.onResize) {
      clearTimeout(this.resizeTimeout);
      window.removeEventListener('resize', this.onResize);
    }
  }

  getCloseHeight() {
    return parseFloat(this.elem.clientHeight);
  }

  getOpenHeight(state) {
    const currentHeight = window.getComputedStyle(this.elem).height;
    this.elem.parentElement.style.position = 'relative';
    this.elem.style.position = 'absolute';
    this.elem.style.visibility = 'hidden';
    this.elem.style.height = 'auto';
    const height = parseFloat(this.elem.clientHeight);
    this.elem.parentElement.style.position = '';
    this.elem.style.position = '';
    this.elem.style.visibility = '';

    switch (state) {
      case 'open':
        this.elem.style.height = this.state === 'init' ? '0' : currentHeight;
        break;
    }

    return height;
  }

  onStart(fn) {
    this.onStartStack.push(fn);
  }

  onEnd(fn) {
    this.onEndStack.push(fn);
  }

  callOnStart() {
    this.onStartStack.forEach(fn => {
      if (typeof fn === 'function') {
        fn(this.state);
      }
    });
  }

  callOnEnd() {
    this.isAnimating = false;
    this.removeEndEvents();
    this.resetSuppressAnimation();
    this.onEndStack.forEach(fn => {
      if (typeof fn === 'function') {
        fn(this.state);
      }
    });
  }

  start(fromHeight, toHeight, {
    animate
  }) {
    var _this$opts;

    if (animate === false || ((_this$opts = this.opts) === null || _this$opts === void 0 ? void 0 : _this$opts.animate) === false) {
      this.elem.style.height = `${toHeight}px`;
      this.callOnStart();

      try {
        const event = new CustomEvent('transitionend');
        this.elem.dispatchEvent(event);
      } catch (e) {
        warn(e);
      }

      return;
    }

    if (typeof window !== 'undefined' && window.requestAnimationFrame) {
      this.stop();
      this.isAnimating = true;

      if (animate === false) {
        this.suppressAnimation();
      }

      this.callOnStart();
      this.reqId1 = window.requestAnimationFrame(() => {
        this.elem.style.height = `${fromHeight}px`;

        if (this.container) {
          this.container.style.minHeight = `${fromHeight}px`;
        }

        this.reqId2 = window.requestAnimationFrame(() => {
          this.elem.style.height = `${toHeight}px`;
          this.setContainerHeight();
        });
      });
    }
  }

  setContainerHeight() {
    if (this.container) {
      const contentElem = this.elem;

      if (contentElem.offsetHeight > 0) {
        this.container.style.minHeight = `${contentElem.offsetHeight + contentElem.offsetTop}px`;
      }
    }
  }

  stop() {
    if (typeof window !== 'undefined' && window.requestAnimationFrame) {
      window.cancelAnimationFrame(this.reqId1);
      window.cancelAnimationFrame(this.reqId2);
    }
  }

  suppressAnimation() {
    this.transitionDuration = window.getComputedStyle(this.elem).transitionDuration;
    this.elem.style.transitionDuration = '1ms';
  }

  resetSuppressAnimation() {
    if (this.transitionDuration) {
      this.elem.style.transitionDuration = this.transitionDuration;
      this.transitionDuration = null;
    }
  }

  adjustFrom() {
    if (!this.elem) {
      return;
    }

    const height = this.getCloseHeight();
    this.elem.style.height = `${height}px`;
    return height;
  }

  adjustTo(fromHeight, {
    animate = true
  } = {}) {
    if (!this.elem) {
      return;
    }

    const toHeight = this.getOpenHeight('open');
    this.state = 'adjusting';
    this.removeEndEvents();

    if (!this.onAdjustEnd) {
      this.elem.addEventListener('transitionend', this.onAdjustEnd = () => {
        if (this.elem) {
          this.elem.style.height = 'auto';
        }

        this.state = 'adjusted';
        this.callOnEnd();
        this.setContainerHeight();
      });
    }

    this.start(fromHeight, toHeight, {
      animate
    });
  }

  open({
    animate = true
  } = {}) {
    if (!this.elem || this.state === 'opened' || this.state === 'opening') {
      return;
    }

    const height = this.getOpenHeight('open');
    this.state = 'opening';
    this.removeEndEvents();

    if (!this.onOpenEnd) {
      this.elem.addEventListener('transitionend', this.onOpenEnd = () => {
        if (this.elem) {
          this.elem.style.height = 'auto';
        }

        this.state = 'opened';
        this.callOnEnd();
        this.setContainerHeight();
      });
    }

    this.start(0, height, {
      animate
    });
  }

  close({
    animate = true
  } = {}) {
    if (!this.elem || this.state === 'closed' || this.state === 'closing') {
      return;
    }

    let height = this.getCloseHeight();

    if (this.state === 'init') {
      height = this.getOpenHeight();
    }

    this.state = 'closing';
    this.removeEndEvents();

    if (!this.onCloseEnd) {
      this.elem.addEventListener('transitionend', this.onCloseEnd = () => {
        if (this.elem) {
          this.elem.style.visibility = 'hidden';
        }

        this.state = 'closed';
        this.callOnEnd();
      });
    }

    this.start(height, 0, {
      animate
    });
  }

}
export function convertStatusToStateOnly(status, state) {
  return status ? state : null;
}
export function getStatusState(status) {
  return status && status !== 'error' && status !== 'warn' && status !== 'info';
}
export function combineLabelledBy(...params) {
  return combineAriaBy('aria-labelledby', params);
}
export function combineDescribedBy(...params) {
  return combineAriaBy('aria-describedby', params);
}

function combineAriaBy(type, params) {
  params = params.map(cur => {
    if (cur && typeof cur[type] !== 'undefined') {
      cur = cur[type];
    }

    if (typeof cur !== 'string') {
      cur = null;
    }

    return cur;
  });
  params = params.filter(Boolean).join(' ');

  if (params === '') {
    params = undefined;
  }

  return params;
}