import { warn, PLATFORM_MAC, PLATFORM_WIN, PLATFORM_LINUX, PLATFORM_IOS } from './component-helper';
export let IS_IE11 = false;
export let IS_EDGE = false;
export let IS_IOS = false;
export let IS_SAFARI = false;
export let IS_WIN = false;
export let IS_MAC = false;
export let IS_LINUX = false;
export const isMac = () => {
  var _navigator;

  return IS_MAC = typeof navigator !== 'undefined' && new RegExp(PLATFORM_MAC, 'i').test((_navigator = navigator) === null || _navigator === void 0 ? void 0 : _navigator.platform);
};
export const isWin = () => {
  var _navigator2;

  return IS_WIN = typeof navigator !== 'undefined' && new RegExp(PLATFORM_WIN, 'i').test((_navigator2 = navigator) === null || _navigator2 === void 0 ? void 0 : _navigator2.platform);
};
export const isLinux = () => {
  var _navigator3;

  return IS_LINUX = typeof navigator !== 'undefined' && new RegExp(PLATFORM_LINUX, 'i').test((_navigator3 = navigator) === null || _navigator3 === void 0 ? void 0 : _navigator3.platform);
};
export const isiOS = () => {
  var _navigator4;

  return IS_IOS = typeof navigator !== 'undefined' && new RegExp(PLATFORM_IOS, 'i').test((_navigator4 = navigator) === null || _navigator4 === void 0 ? void 0 : _navigator4.platform);
};
export const isSafari = () => {
  var _navigator5, _navigator6;

  return IS_SAFARI = typeof navigator !== 'undefined' && /safari/i.test((_navigator5 = navigator) === null || _navigator5 === void 0 ? void 0 : _navigator5.userAgent) && !/chrome/i.test((_navigator6 = navigator) === null || _navigator6 === void 0 ? void 0 : _navigator6.userAgent);
};
export const isIE11 = () => IS_IE11 = typeof window !== 'undefined' && typeof document !== 'undefined' ? !!window.MSInputMethodContext && !!document.documentMode : false;
export const isEdge = () => {
  var _navigator7;

  return IS_EDGE = typeof navigator !== 'undefined' && /edge/i.test((_navigator7 = navigator) === null || _navigator7 === void 0 ? void 0 : _navigator7.userAgent);
};
isIE11();
isEdge();
isiOS();
isSafari();
isWin();
isMac();
isLinux();
const pageFocusElements = {};
export function setPageFocusElement(selectorOrElement, key = 'default') {
  return pageFocusElements[key] = selectorOrElement;
}
export function applyPageFocus(key = 'default', callback = null) {
  try {
    let element = pageFocusElements[key];

    if (typeof element === 'string' && typeof document !== 'undefined') {
      element = document.querySelector(element);
    } else if (!element && typeof document !== 'undefined') {
      element = document.querySelector('.dnb-no-focus');
    }

    if (element instanceof HTMLElement) {
      if (['h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'p', 'div', 'main', 'nav', 'header', 'footer', 'aside', 'section', 'article'].includes(String(element.nodeName).toLowerCase())) {
        if (!element.hasAttribute('tabindex')) {
          element.setAttribute('tabindex', '-1');
        }

        if (element.classList && !element.classList.contains('dnb-no-focus')) {
          element.classList.add('dnb-no-focus');
        }
      }

      element.focus({
        preventScroll: true
      });

      if (typeof callback === 'function') {
        callback(element);
      }
    }
  } catch (e) {
    warn('Error on applyPageFocus:', e);
  }
}
export function getOffsetTop(elem) {
  let offsetTop = 0;

  do {
    if (!isNaN(elem.offsetTop)) {
      offsetTop += elem.offsetTop;
    }
  } while (elem = elem.offsetParent);

  return offsetTop;
}
export function getOffsetLeft(elem) {
  let offsetLeft = 0;

  do {
    if (!isNaN(elem.offsetLeft)) {
      offsetLeft += elem.offsetLeft;
    }
  } while (elem = elem.offsetParent);

  return offsetLeft;
}
export function scrollToLocationHashId({
  offset = 0,
  delay = null,
  onCompletion = null
} = {}) {
  if (typeof document !== 'undefined' && typeof window !== 'undefined' && window.location) {
    try {
      let _timeout;

      const id = String(window.location.hash).replace('#', '');

      if (id.length > 0) {
        const handleScroll = () => {
          const runScroll = () => {
            const top = getOffsetTop(elem) - offset;

            try {
              if (typeof IntersectionObserver !== 'undefined') {
                const intersectionObserver = new IntersectionObserver(entries => {
                  const [entry] = entries;

                  if (entry.isIntersecting) {
                    intersectionObserver.unobserve(elem);

                    if (typeof onCompletion === 'function') {
                      onCompletion(elem);
                    }
                  }
                });
                intersectionObserver.observe(elem);
              }

              if (window.scrollTo) {
                window.scrollTo({
                  top,
                  behavior: 'smooth'
                });
              } else {
                window.scrollTop = top;
              }
            } catch (e) {
              warn('Error on scrollToLocationHashId:', e);
            }
          };

          if (delay > 0) {
            clearTimeout(_timeout);
            _timeout = setTimeout(runScroll, delay);
          } else {
            runScroll();
          }
        };

        const elem = document.getElementById(id);

        if (elem instanceof HTMLElement) {
          window.addEventListener('beforeunload', () => clearTimeout(_timeout));

          if (document.readyState === 'loading') {
            document.addEventListener('DOMContentLoaded', handleScroll);
          } else {
            handleScroll();
          }
        }

        return elem;
      }
    } catch (e) {
      warn('Error on scrollToLocationHashId:', e);
    }
  }
}
export function debounce(func, wait = 250, {
  immediate = false,
  context = null
} = {}) {
  let timeout;
  let recall;
  return function executedFunction() {
    const ctx = context || this;
    const args = arguments;

    if (typeof recall === 'function') {
      recall();
    }

    const later = () => {
      timeout = null;

      if (!immediate) {
        recall = func.apply(ctx, args);
      }
    };

    const callNow = immediate && !timeout;
    clearTimeout(timeout);
    timeout = setTimeout(later, wait);

    if (callNow) {
      recall = func.apply(ctx, args);
    }
  };
}
export function insertElementBeforeSelection(elem) {
  try {
    const selection = window.getSelection();
    const range = selection.getRangeAt(0);
    range.cloneRange().insertNode(elem);
    selection.addRange(range);
  } catch (e) {}
}
export function getSelectedText() {
  try {
    return window.getSelection().toString();
  } catch (e) {}

  return '';
}
export function hasSelectedText() {
  return getSelectedText().length > 0;
}
export function getSelectedElement() {
  try {
    const selection = window.getSelection();

    if (selection.rangeCount > 0) {
      let elem = selection.getRangeAt(0).startContainer;

      if (elem && typeof elem === 'object') {
        elem = elem.parentNode;
      }

      return elem;
    }
  } catch (e) {}

  return null;
}
export async function copyToClipboard(string) {
  var _navigator8;

  if (typeof window === 'undefined' || typeof document === 'undefined') {
    return false;
  }

  const selection = window.getSelection();
  const range = selection.rangeCount > 0 ? selection.getRangeAt(0) : false;

  const resetSelection = () => {
    try {
      selection.removeAllRanges();
      selection.addRange(range);
    } catch (e) {}
  };

  const copyFallback = () => {
    try {
      const elem = document.createElement('textarea');
      elem.value = String(string);
      elem.contentEditable = true;
      elem.readOnly = false;
      elem.style.position = 'fixed';
      elem.style.top = '-1000px';
      document.body.appendChild(elem);
      const success = document.execCommand('copy');
      document.body.removeChild(elem);
      resetSelection();

      if (success) {
        return true;
      }
    } catch (e) {
      return e;
    }

    return `Could not copy! Unknown reason. ${string}`;
  };

  let success;

  if (typeof navigator !== 'undefined' && (_navigator8 = navigator) !== null && _navigator8 !== void 0 && _navigator8.clipboard) {
    try {
      await navigator.clipboard.writeText(String(string));
      success = true;
      resetSelection();
    } catch (e) {
      success = e;
      const newTry = copyFallback();

      if (newTry === true) {
        success = newTry;
      }
    }
  } else {
    success = copyFallback();
  }

  return success;
}