import "core-js/modules/es.array.concat.js";
import "core-js/modules/es.array.fill.js";
import "core-js/modules/es.array.index-of.js";
import "core-js/modules/es.array.map.js";
import "core-js/modules/es.regexp.exec.js";
import "core-js/modules/es.string.replace.js";
import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import subMonths from 'date-fns/subMonths';
import addMonths from 'date-fns/addMonths';
import isWithinInterval from 'date-fns/isWithinInterval';
import isValid from 'date-fns/isValid';
import isAfter from 'date-fns/isAfter';
import isBefore from 'date-fns/isBefore';
import isSameDay from 'date-fns/isSameDay';
import isToday from 'date-fns/isToday';
import isSameMonth from 'date-fns/isSameMonth';
import addDays from 'date-fns/addDays';
import startOfWeek from 'date-fns/startOfWeek';
import getDay from 'date-fns/getDay';
import startOfMonth from 'date-fns/startOfMonth';
import getDaysInMonth from 'date-fns/getDaysInMonth';
import toDate from 'date-fns/toDate';
import parseISO from 'date-fns/parseISO';
import parse from 'date-fns/parse';
import { warn } from '../../shared/component-helper';
export var makeDayObject = function makeDayObject(date, _ref) {
  var startDate = _ref.startDate,
      endDate = _ref.endDate,
      hoverDate = _ref.hoverDate,
      minDate = _ref.minDate,
      maxDate = _ref.maxDate,
      month = _ref.month;
  var range = getRange(startDate, endDate, hoverDate);
  var isLastMonth = isSameMonth(subMonths(date, 1), month);
  var isNextMonth = isSameMonth(addMonths(date, 1), month);
  var isStartDate = isStartDateCalc(date, range);
  var isEndDate = isEndDateCalc(date, range);
  var isWithinSelection = isWithinSelectionCalc(date, startDate, endDate);
  var isPreview = isPreviewCalc(date, startDate, endDate, hoverDate);
  var isDisabled = isDisabledCalc(date, minDate, maxDate);
  return {
    date: date,
    isToday: isToday(date),
    isLastMonth: isLastMonth,
    isNextMonth: isNextMonth,
    isStartDate: isStartDate,
    isEndDate: isEndDate,
    isWithinSelection: isWithinSelection,
    isPreview: isPreview,
    isDisabled: isDisabled,
    isSelectable: !isLastMonth && !isNextMonth && !isDisabled,
    isInactive: isLastMonth || isNextMonth || isDisabled
  };
};
export var getCalendar = function getCalendar(month) {
  var weekStartsOn = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

  var _ref2 = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
      _ref2$onlyMonth = _ref2.onlyMonth,
      onlyMonth = _ref2$onlyMonth === void 0 ? false : _ref2$onlyMonth,
      _ref2$hideNextMonthWe = _ref2.hideNextMonthWeek,
      hideNextMonthWeek = _ref2$hideNextMonthWe === void 0 ? false : _ref2$hideNextMonthWe;

  var thisMonth = getMonth(month);

  if (onlyMonth) {
    return calendarCache[month] = _toConsumableArray(thisMonth);
  }

  if (calendarCache[month]) {
    return calendarCache[month];
  }

  var firstDay = (7 + getDay(startOfMonth(month)) - weekStartsOn) % 7;
  var lastMonth = getMonth(subMonths(month, 1), getDaysInMonth(subMonths(month, 1)) - firstDay);
  var fillCount = -1;

  if (hideNextMonthWeek) {
    fillCount = 35 - (thisMonth.length + firstDay);
  }

  if (fillCount < 0) {
    fillCount = 42 - (thisMonth.length + firstDay);
  }

  var nextMonth = getMonth(addMonths(month, 1), 0, fillCount > -1 ? fillCount : 0);
  return calendarCache[month] = [].concat(_toConsumableArray(lastMonth), _toConsumableArray(thisMonth), _toConsumableArray(nextMonth));
};
var calendarCache = {};
export var dayOffset = function dayOffset(dayName) {
  var week = ['sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday'];
  return dayName ? week.indexOf(dayName.toLowerCase()) : 0;
};
export var toRange = function toRange(startDate, endDate) {
  if (isBefore(endDate, startDate)) {
    var _startDate = startDate;
    startDate = endDate;
    endDate = _startDate;
  }

  return {
    startDate: startDate,
    endDate: endDate
  };
};

var getRange = function getRange(startDate, endDate, hoverDate) {
  if (startDate && endDate) {
    return toRange(startDate, endDate);
  } else if (startDate && hoverDate) {
    return toRange(startDate, hoverDate);
  } else {
    return toRange(startDate, startDate);
  }
};

export var getWeek = function getWeek(weekStartsOn) {
  var startDay = startOfWeek(new Date(), {
    weekStartsOn: weekStartsOn
  });
  return Array(7).fill(1).map(function (value, i) {
    return addDays(startDay, i);
  });
};
export var getMonth = function getMonth(month) {
  var skip = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  var limit = arguments.length > 2 ? arguments[2] : undefined;
  var startDay = startOfMonth(month);
  var size = getDaysInMonth(month) - skip;
  size = Math.min(Math.max(parseInt(size), 0), limit > -1 ? limit : size);
  size = size < 0 ? 0 : size;
  return Array(size).fill(1).map(function (value, i) {
    return addDays(startDay, i + skip);
  });
};

var isWithinSelectionCalc = function isWithinSelectionCalc(date, startDate, endDate) {
  var _toRange = toRange(startDate, endDate),
      start = _toRange.startDate,
      end = _toRange.endDate;

  return startDate && endDate ? isValid(start) && isValid(end) && isWithinInterval(date, {
    start: start,
    end: end
  }) : false;
};

var isDisabledCalc = function isDisabledCalc(date, minDate, maxDate) {
  return minDate && isBefore(date, minDate) || maxDate && isAfter(date, maxDate);
};

export { isDisabledCalc as isDisabled };

var isStartDateCalc = function isStartDateCalc(date, range) {
  return range.startDate && isSameDay(date, range.startDate);
};

var isEndDateCalc = function isEndDateCalc(date, range) {
  return range.endDate && isSameDay(date, range.endDate);
};

var isPreviewCalc = function isPreviewCalc(date, startDate, endDate, hoverDate) {
  var _toRange2 = toRange(startDate, hoverDate),
      start = _toRange2.startDate,
      end = _toRange2.endDate;

  return startDate && !endDate && isValid(start) && isValid(end) && isWithinInterval(date, {
    start: start,
    end: end
  });
};

export var correctV1Format = function correctV1Format(date) {
  if (/YYYY/.test(date) && /DD/.test(date)) {
    warn('You are using "YYYY-MM-DD" as the date_format or return_format? Please use "yyyy-MM-dd" instead!');
    date = date.replace(/DD/, 'dd').replace(/YYYY/, 'yyyy');
  }

  return date;
};
export var convertStringToDate = function convertStringToDate(date) {
  var _ref3 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref3$date_format = _ref3.date_format,
      date_format = _ref3$date_format === void 0 ? null : _ref3$date_format;

  if (date === null) {
    return null;
  }

  var dateObject;
  dateObject = typeof date === 'string' ? parseISO(date) : toDate(date);

  if (typeof date === 'string' && date_format && !isValid(dateObject)) {
    date_format = correctV1Format(date_format);
    dateObject = parse(date, date_format, new Date());
  }

  if (!isValid(dateObject)) {
    warn('convertStringToDate got invalid date:', date);
    return null;
  }

  return dateObject;
};