import "core-js/modules/es.array.concat.js";
import "core-js/modules/es.array.filter.js";
import "core-js/modules/es.array.find.js";
import "core-js/modules/es.array.join.js";
import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import React from 'react';
import { warn, convertJsxToString } from '../../shared/component-helper';
export var globalSyncCounter = React.createRef();
export var globalHeadingCounter = React.createRef(null);
var refs = React.createRef();
export var correctHeadingLevel = function correctHeadingLevel(_ref) {
  var counter = _ref.counter,
      level = _ref.level,
      _ref$ref = _ref.ref,
      ref = _ref$ref === void 0 ? null : _ref$ref,
      _ref$reset = _ref.reset,
      reset = _ref$reset === void 0 ? null : _ref$reset,
      _ref$inherit = _ref.inherit,
      inherit = _ref$inherit === void 0 ? null : _ref$inherit,
      _ref$increase = _ref.increase,
      increase = _ref$increase === void 0 ? false : _ref$increase,
      _ref$decrease = _ref.decrease,
      decrease = _ref$decrease === void 0 ? false : _ref$decrease,
      _ref$source = _ref.source,
      source = _ref$source === void 0 ? null : _ref$source,
      _ref$bypassChecks = _ref.bypassChecks,
      bypassChecks = _ref$bypassChecks === void 0 ? false : _ref$bypassChecks,
      _ref$debug = _ref.debug,
      debug = _ref$debug === void 0 ? null : _ref$debug;

  if (ref && refs.current) {
    var foundRef = refs.current.find(function (cur) {
      return cur.ref === ref;
    });

    if (foundRef) {
      return foundRef.counter;
    }
  }

  if (bypassChecks) {
    counter.enableBypassChecks();
  }

  var update = function update(level) {
    var _globalHeadingCounter;

    counter.makeMeReady();

    if (inherit) {
      if (globalSyncCounter.current) {
        counter.entry = globalSyncCounter.current.contextCounter.entry;
        counter.level = globalSyncCounter.current.level;
      } else {
        report(debug, source, 'Heading got inherit, but there was noting to inherit.');
      }
    }

    if (level >= 1) {
      counter.setLevel(level);
    } else if (decrease) {
      counter.decrement();
    } else if (increase) {
      counter.increment();
    }

    if (counter.level > 0 && !((_globalHeadingCounter = globalHeadingCounter.current) !== null && _globalHeadingCounter !== void 0 && _globalHeadingCounter.hasEntryLevel())) {
      var _globalHeadingCounter2;

      (_globalHeadingCounter2 = globalHeadingCounter.current) === null || _globalHeadingCounter2 === void 0 ? void 0 : _globalHeadingCounter2.setEntryLevel();
    }
  };

  var skipThisTime = false;

  var canBeManipulatedNextTime = function canBeManipulatedNextTime(overwriteContext) {
    return counter.contextCounter.isGlobal || counter.contextCounter.entry === 1 || overwriteContext;
  };

  if (globalResetNextTime.current) {
    var _globalResetNextTime$ = globalResetNextTime.current,
        resetLevel = _globalResetNextTime$.level,
        overwriteContext = _globalResetNextTime$.overwriteContext;
    globalResetNextTime.current = null;

    if (canBeManipulatedNextTime(overwriteContext) || counter.lastResetLevel === resetLevel) {
      counter.makeMeReady();
      counter.reset(resetLevel);
      skipThisTime = true;
    }
  } else if (globalNextLevel.current) {
    var _globalNextLevel$curr = globalNextLevel.current,
        nextLevel = _globalNextLevel$curr.level,
        _overwriteContext = _globalNextLevel$curr.overwriteContext;
    globalNextLevel.current = null;

    if (canBeManipulatedNextTime(_overwriteContext)) {
      counter.enableBypassChecks();
      update(nextLevel);
      counter.disableBypassChecks();
      skipThisTime = true;
    }
  }

  if (!skipThisTime) {
    if (reset === true || reset === 'true' || parseFloat(reset) > -1) {
      counter.reset(reset);
    } else {
      update(level);
    }
  }

  var hasReport = counter.useLastReport();

  if (hasReport) {
    report.apply(void 0, [debug, source].concat(_toConsumableArray(hasReport)));
  }

  if (bypassChecks) {
    counter.disableBypassChecks();
  }

  if (ref) {
    refs.current = refs.current || [];
    refs.current.push({
      ref: ref,
      counter: counter
    });
  }

  return counter;
};

function report(debug, source) {
  for (var _len = arguments.length, reports = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    reports[_key - 2] = arguments[_key];
  }

  if (source) {
    var props = source.props || {};
    var ideintifiers = [props.id, props['class'], props.className].filter(Boolean);
    reports.push('\nNB: This warning was triggered by:', ideintifiers.length > 0 ? ideintifiers.join(', ') : '', convertJsxToString(source));
  }

  if (typeof debug === 'function') {
    debug.apply(void 0, reports);
  } else {
    warn.apply(void 0, reports);
  }
}

export function resetAllLevels() {
  countHeadings = 0;
  resetLevels(1, {
    overwriteContext: false
  });
  teardownHeadings();
}
export var globalResetNextTime = React.createRef(false);
export function resetLevels(level) {
  var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref2$overwriteContex = _ref2.overwriteContext,
      overwriteContext = _ref2$overwriteContex === void 0 ? false : _ref2$overwriteContex;

  globalResetNextTime.current = {
    level: level,
    overwriteContext: overwriteContext
  };
}
export var globalNextLevel = React.createRef(null);
export function setNextLevel(level) {
  var _ref3 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref3$overwriteContex = _ref3.overwriteContext,
      overwriteContext = _ref3$overwriteContex === void 0 ? false : _ref3$overwriteContex;

  globalNextLevel.current = {
    level: level,
    overwriteContext: overwriteContext
  };
}
var countHeadings = 0;
export function windupHeadings() {
  countHeadings++;
  globalSyncCounter.current = null;
}
export function teardownHeadings() {
  countHeadings--;

  if (countHeadings === 0) {
    globalHeadingCounter.current = null;
    globalSyncCounter.current = null;
    globalResetNextTime.current = null;
    globalNextLevel.current = null;
  }
}
export function debugCounter(counter) {
  return JSON.stringify({
    group: counter.group || counter.contextCounter.group,
    level: counter.level,
    entry: counter.entry,
    contextLevel: counter.contextCounter.level,
    contextLEntry: counter.contextCounter.entry
  }, null, 2);
}