import * as React from 'react';
export type SliderLabel =
  | string
  | ((...args: any[]) => any)
  | React.ReactNode;
export type SliderLabelDirection = 'horizontal' | 'vertical';
export type SliderLabelSrOnly = string | boolean;
export type SliderStatus =
  | string
  | boolean
  | ((...args: any[]) => any)
  | React.ReactNode;
export type SliderSuffix =
  | string
  | ((...args: any[]) => any)
  | React.ReactNode;
export type SliderMin = string | number;
export type SliderMax = string | number;
export type SliderValue = string | number;
export type SliderStep = string | number;
export type SliderVertical = string | boolean;
export type SliderReverse = string | boolean;
export type SliderStretch = string | boolean;
export type SliderNumberFormat = string | Object;
export type SliderDisabled = string | boolean;
export type SliderHideButtons = string | boolean;
export type SliderUseScrollwheel = string | boolean;
export type SliderSkeleton = string | boolean;
export type SliderSpace =
  | string
  | number
  | boolean
  | {
      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. Will use `margin-top`.
       */
      top?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-right`.
       */
      right?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-bottom`.
       */
      bottom?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-left`.
       */
      left?: string | number | boolean;
    };
export type SliderTop = string | number | boolean;
export type SliderRight = string | number | boolean;
export type SliderBottom = string | number | boolean;
export type SliderLeft = string | number | boolean;
export type SliderChildren = string | ((...args: any[]) => any);

/**
 * NB: Do not change the docs (comments) in here. The docs are updated during build time by "generateTypes.js" and "fetchPropertiesFromDocs.js".
 */
export interface SliderProps extends React.HTMLProps<HTMLElement> {
  id?: string;

  /**
   * Prepends the Form Label component. If no ID is provided, a random ID is created.
   */
  label?: SliderLabel;

  /**
   * Use `label_direction="vertical"` to change the label layout direction. Defaults to `horizontal`.
   */
  label_direction?: SliderLabelDirection;

  /**
   * Use `true` to make the label only readable by screen readers.
   */
  label_sr_only?: SliderLabelSrOnly;

  /**
   * Text with a status message. The style defaults to an error message. You can use `true` to only get the status color, without a message.
   */
  status?: SliderStatus;

  /**
   * Defines the state of the status. Currently there are two statuses `[error, info]`. Defaults to `error`.
   */
  status_state?: string;
  status_animation?: string;

  /**
   * The `status_id` used for the target <a href="/uilib/components/global-status">GlobalStatus</a>.
   */
  global_status_id?: string;

  /**
   * Text describing the content of the Slider more than the label. You can also send in a React component, so it gets wrapped inside the Slider component.
   */
  suffix?: SliderSuffix;

  /**
   * Give the slider thump button a title for accessibility reason. Defaults to `null`.
   */
  thump_title?: string;

  /**
   * Give the add button a title for accessibility reason. Defaults to `+`.
   */
  add_title?: string;

  /**
   * Give the subtract button a title for accessibility reason. Defaults to `−`.
   */
  subtract_title?: string;

  /**
   * The minimum value. Defaults to `0`.
   */
  min?: SliderMin;

  /**
   * The maxium value. Defaults to `100`
   */
  max?: SliderMax;

  /**
   * The `vlaue` of the slider. Also the event callback result.
   */
  value?: SliderValue;

  /**
   * The steps the slider takes on changing the value. Defaults to `null`
   */
  step?: SliderStep;

  /**
   * Show the slider vertically. Defaults to `false`.
   */
  vertical?: SliderVertical;

  /**
   * Show the slider reversed. Defaults to `false`.
   */
  reverse?: SliderReverse;

  /**
   * If set to `true`, then the slider will be 100% in `width`.
   */
  stretch?: SliderStretch;

  /**
   * Will extend the return object with a `number` property (from `on_change` event). You can use all the options from the <a href="/uilib/components/number/properties">Number</a> component. It also will use that formatted number in the increase/decrease buttons. If it has to represent a currency, then use e.g. `number_format={{ currency: true, decimals: 0 }}`
   */
  number_format?: SliderNumberFormat;
  disabled?: SliderDisabled;

  /**
   * Removes the helper buttons. Defaults to `false`.
   */
  hide_buttons?: SliderHideButtons;

  /**
   * Enable mouse scroll-wheel support. Defaults to `false`.
   */
  use_scrollwheel?: SliderUseScrollwheel;
  skeleton?: SliderSkeleton;

  /**
   * Has to be an object with either: `top`, `right`, `bottom` or `left`. Use spacing values like: `small`, `1rem`, `1` or , `16px`.
   */
  space?: SliderSpace;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. Will use `margin-top`.
   */
  top?: SliderTop;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-right`.
   */
  right?: SliderRight;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-bottom`.
   */
  bottom?: SliderBottom;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-left`.
   */
  left?: SliderLeft;
  class?: string;
  className?: string;
  children?: SliderChildren;

  /**
   * Will be called once the component is ready to use. The callback `value` is a number `{ value }`.
   */
  on_init?: (...args: any[]) => any;

  /**
   * Will be called on state changes made by the user. The callback `value` and `rawValue` is a number `{ value, rawValue, event }`. But if the prop `number_format` is given, then it will return an additional `number` with the given format `{ value, number, rawValue, event }`.
   */
  on_change?: (...args: any[]) => any;

  /**
   * Will be called once the user stops dragging. Returns `{ event }`.
   */
  on_drag_start?: (...args: any[]) => any;

  /**
   * Will be called once the user starts dragging. Returns `{ event }`.
   */
  on_drag_end?: (...args: any[]) => any;
  on_state_update?: (...args: any[]) => any;
}
export default class Slider extends React.Component<SliderProps, any> {
  static defaultProps: object;
  render(): JSX.Element;
}
