import * as React from 'react';
export type SpaceInline = string | boolean;
export type SpaceNoCollapse = string | boolean;
export type SpaceSpace =
  | string
  | number
  | boolean
  | {
      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. Will use `margin-top`.
       */
      top?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-right`.
       */
      right?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-bottom`.
       */
      bottom?: string | number | boolean;

      /**
       * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-left`.
       */
      left?: string | number | boolean;
    };
export type SpaceTop = string | number | boolean;
export type SpaceRight = string | number | boolean;
export type SpaceBottom = string | number | boolean;
export type SpaceLeft = string | number | boolean;
export type SpaceSkeleton = string | boolean;
export type SpaceChildren =
  | string
  | ((...args: any[]) => any)
  | React.ReactNode;

/**
 * NB: Do not change the docs (comments) in here. The docs are updated during build time by "generateTypes.js" and "fetchPropertiesFromDocs.js".
 */
export interface SpaceProps extends React.HTMLProps<HTMLElement> {
  id?: string;
  element?: string;
  inline?: SpaceInline;
  no_collapse?: SpaceNoCollapse;

  /**
   * Has to be an object with either: `top`, `right`, `bottom` or `left`. Use spacing values like: `small`, `1rem`, `1` or , `16px`.
   */
  space?: SpaceSpace;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. Will use `margin-top`.
   */
  top?: SpaceTop;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-right`.
   */
  right?: SpaceRight;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-bottom`.
   */
  bottom?: SpaceBottom;

  /**
   * Use spacing values like: `small`, `1rem`, `1` or , `16px`. will use `margin-left`.
   */
  left?: SpaceLeft;
  skeleton?: SpaceSkeleton;
  class?: string;
  className?: string;
  children?: SpaceChildren;
}
export default class Space extends React.Component<SpaceProps, any> {
  static defaultProps: object;
  render(): JSX.Element;
}
