import "core-js/modules/es.array.reduce.js";
import "core-js/modules/es.object.keys.js";
import _extends from "@babel/runtime/helpers/esm/extends";
import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import _assertThisInitialized from "@babel/runtime/helpers/esm/assertThisInitialized";
import _inherits from "@babel/runtime/helpers/esm/inherits";
import _possibleConstructorReturn from "@babel/runtime/helpers/esm/possibleConstructorReturn";
import _getPrototypeOf from "@babel/runtime/helpers/esm/getPrototypeOf";

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

import React from 'react';
import PropTypes from 'prop-types';
import keycode from 'keycode';
import Context from '../../shared/Context';
import { warn, isTrue, roundToNearest, isInsideScrollView, detectOutsideClick, dispatchCustomElementEvent } from '../../shared/component-helper';
import { getOffsetTop, getOffsetLeft, hasSelectedText } from '../../shared/helpers';
import { getData, normalizeData, findClosest, getSelectedItemValue, parseContentTitle, getEventData, prepareStartupState, prepareDerivedState } from './DrawerListHelpers';
import DrawerListContext from './DrawerListContext';
import { disableBodyScroll, enableBodyScroll } from '../../shared/libs/bodyScrollLock';

var DrawerListProvider = function (_React$PureComponent) {
  _inherits(DrawerListProvider, _React$PureComponent);

  var _super = _createSuper(DrawerListProvider);

  function DrawerListProvider(props) {
    var _this;

    _classCallCheck(this, DrawerListProvider);

    _this = _super.call(this, props);

    _this.enableBodyLock = function () {
      if (_this._refUl.current) {
        _this._bodyLockIsEnabled = true;
        disableBodyScroll(_this._refUl.current);
      }
    };

    _this.disableBodyLock = function () {
      if (_this._bodyLockIsEnabled && _this._refUl.current) {
        _this._bodyLockIsEnabled = null;
        enableBodyScroll(_this._refUl.current);
      }
    };

    _this.scrollToItem = function (active_item) {
      var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          _ref$scrollTo = _ref.scrollTo,
          scrollTo = _ref$scrollTo === void 0 ? true : _ref$scrollTo;

      clearTimeout(_this._scrollTimeout);
      _this._scrollTimeout = setTimeout(function () {
        if (_this._refUl.current && parseFloat(active_item) > -1) {
          try {
            var ulElement = _this._refUl.current;

            var liElement = _this.getActiveElement();

            if (liElement) {
              var top = liElement.offsetTop;

              if (ulElement.scrollTo) {
                if (scrollTo === false || window.IS_TEST) {
                  ulElement.style.scrollBehavior = 'auto';
                }

                ulElement.scrollTo({
                  top: top,
                  behavior: scrollTo ? 'smooth' : 'auto'
                });

                if (scrollTo === false) {
                  ulElement.style.scrollBehavior = 'smooth';
                }
              } else if (ulElement.scrollTop) {
                ulElement.scrollTop = top;
              }

              if (!isTrue(_this.props.prevent_focus) && liElement) {
                liElement.focus();
                dispatchCustomElementEvent(_assertThisInitialized(_this), 'on_show_focus', {
                  element: liElement
                });
              }
            }
          } catch (e) {
            warn('List could not scroll into element:', e);
          }
        }
      }, 1);
    };

    _this.scrollToAndSetActiveItem = function (active_item) {
      var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          _ref2$fireSelectEvent = _ref2.fireSelectEvent,
          fireSelectEvent = _ref2$fireSelectEvent === void 0 ? false : _ref2$fireSelectEvent,
          _ref2$scrollTo = _ref2.scrollTo,
          scrollTo = _ref2$scrollTo === void 0 ? true : _ref2$scrollTo,
          _ref2$event = _ref2.event,
          event = _ref2$event === void 0 ? null : _ref2$event;

      if (parseFloat(active_item) > -1) {
        _this.setState({
          active_item: active_item,
          _listenForPropChanges: false
        }, function () {
          var selected_item = _this.state.selected_item;

          if (fireSelectEvent) {
            var attributes = _this.attributes;
            var ret = dispatchCustomElementEvent(_this.state, 'on_select', {
              active_item: active_item,
              value: getSelectedItemValue(selected_item, _this.state),
              data: getEventData(active_item, _this.state.data),
              event: event,
              attributes: attributes
            });

            if (ret === false) {
              return;
            }
          }

          _this.scrollToItem(active_item, {
            scrollTo: scrollTo
          });
        });
      } else if (!isTrue(_this.props.prevent_focus)) {
        if (_this._refUl.current) {
          _this._refUl.current.focus({
            preventScroll: true
          });

          dispatchCustomElementEvent(_assertThisInitialized(_this), 'on_show_focus', {
            element: _this._refUl.current
          });
        }
      }
    };

    _this.setWrapperElement = function () {
      var wrapper_element = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _this.props.wrapper_element;

      if (typeof wrapper_element === 'string' && typeof document !== 'undefined') {
        wrapper_element = document.querySelector(wrapper_element);
      }

      if (wrapper_element) {
        _this.setState({
          wrapper_element: wrapper_element,
          _listenForPropChanges: false
        });
      }

      return _assertThisInitialized(_this);
    };

    _this.anchorKeyDownHandler = function (e) {
      var key = keycode(e);

      switch (key) {
        case 'tab':
          try {
            var nextEl = _this.meta.shift ? e.target.previousSibling : e.target.nextSibling;
            e.stopPropagation();
            e.preventDefault();

            if (nextEl) {
              _this.focusAnchorElem(nextEl);
            } else {
              _this.getActiveElement().focus();
            }
          } catch (e) {}

          break;

        case 'enter':
        case 'space':
          e.stopPropagation();
          break;

        default:
          break;
      }
    };

    _this.setMetaKey = function (e) {
      var we = typeof window !== 'undefined' && window.event ? window.event : e;
      _this.meta = {
        cmd: we.metaKey,
        ctrl: we.ctrlKey,
        shift: we.shiftKey,
        alt: we.altKey
      };
    };

    _this.onKeyUpHandler = function (e) {
      _this.setMetaKey(e);
    };

    _this.onKeyDownHandler = function (e) {
      var key = keycode(e);

      if (/command|alt|shift|ctrl/.test(key)) {
        _this.setMetaKey(e);
      }

      if (isTrue(_this.props.prevent_close)) {
          var isSameDrawer = false;

          try {
            isSameDrawer = typeof document !== 'undefined' && "".concat(document.activeElement.getAttribute('id'), "-ul") === _this._refUl.current.getAttribute('id');
          } catch (e) {
            warn(e);
          }

          if (!isSameDrawer || key === 'tab') {
            return;
          }
        }

      if (isTrue(_this.state.ignore_events) && key !== 'tab') {
        return;
      }

      if (!_this.state.isOpen) {
        return;
      }

      var active_item = parseFloat(_this.state.active_item);

      if (isNaN(active_item)) {
        active_item = -1;
      }

      var total = _this.state.data && _this.state.data.length - 1;

      switch (key) {
        case 'up':
          {
            e.preventDefault();
            active_item = _this.getPrevActiveItem();

            if (isNaN(active_item)) {
              active_item = _this.getFirstItem() || 0;
            }
          }
          break;

        case 'down':
          {
            e.preventDefault();

            var activeItem = _this.getCurrentActiveItem();

            if (active_item === -1 || isNaN(activeItem)) {
              active_item = _this.getFirstItem() || 0;
            } else {
              active_item = _this.getNextActiveItem();
            }
          }
          break;

        case 'page up':
        case 'home':
          {
            e.preventDefault();
            active_item = _this.getFirstItem() || 0;
          }
          break;

        case 'page down':
        case 'end':
          {
            e.preventDefault();
            active_item = _this.getLastItem();

            if (isNaN(active_item)) {
              active_item = total;
            }
          }
          break;

        case 'enter':
        case 'space':
          {
            active_item = _this.getCurrentActiveItem();

            if (isTrue(_this.props.skip_keysearch) ? active_item > -1 && key !== 'space' : true) {
              e.preventDefault();

              _this.selectItem(active_item, {
                fireSelectEvent: true,
                event: e
              });
            }
          }
          break;

        case 'esc':
          {
            _this.setHidden({
              event: e
            });

            e.preventDefault();
          }
          break;

        case 'tab':
          {
            if (active_item > -1) {
              var anchorElem = _this.getAnchorElem();

              if (anchorElem) {
                e.preventDefault();

                _this.focusAnchorElem(anchorElem);

                return;
              }
            }

            _this.setHidden({
              event: e
            });
          }
          break;

        default:
          active_item = _this.findItemByValue(keycode(e));
          break;
      }

      if (parseFloat(active_item) > -1 && active_item !== _this.state.active_item) {
        _this.scrollToAndSetActiveItem(active_item, {
          fireSelectEvent: true,
          event: e
        });
      }
    };

    _this.getSelectedElement = function () {
      var _this$_refUl$current;

      return ((_this$_refUl$current = _this._refUl.current) === null || _this$_refUl$current === void 0 ? void 0 : _this$_refUl$current.querySelector('li.dnb-drawer-list__option--selected')) || _this._refUl.current || {
        getAttribute: function getAttribute() {
          return null;
        }
      };
    };

    _this.getCurrentSelectedItem = function () {
      var elem = _this.getSelectedElement();

      return parseFloat(elem && elem.getAttribute('data-item'));
    };

    _this.getActiveElement = function () {
      var _this$_refUl$current2;

      return ((_this$_refUl$current2 = _this._refUl.current) === null || _this$_refUl$current2 === void 0 ? void 0 : _this$_refUl$current2.querySelector('li.dnb-drawer-list__option--focus')) || _this.getSelectedElement();
    };

    _this.getCurrentActiveItem = function () {
      var elem = _this.getActiveElement();

      return parseFloat(elem && elem.getAttribute('data-item'));
    };

    _this.getNextActiveItem = function () {
      var elem = _this.getActiveElement().nextSibling;

      return parseFloat(elem && elem.getAttribute('data-item'));
    };

    _this.getPrevActiveItem = function () {
      var elem = _this.getActiveElement().previousSibling;

      return parseFloat(elem && elem.getAttribute('data-item'));
    };

    _this.getFirstItem = function () {
      var _this$_refUl$current3;

      var elem = (_this$_refUl$current3 = _this._refUl.current) === null || _this$_refUl$current3 === void 0 ? void 0 : _this$_refUl$current3.querySelector('li.dnb-drawer-list__option.first-of-type');
      return parseFloat(elem && elem.getAttribute('data-item'));
    };

    _this.getLastItem = function () {
      var _this$_refUl$current4;

      var elem = (_this$_refUl$current4 = _this._refUl.current) === null || _this$_refUl$current4 === void 0 ? void 0 : _this$_refUl$current4.querySelector('li.dnb-drawer-list__option.last-of-type');
      return parseFloat(elem && elem.getAttribute('data-item'));
    };

    _this.setOutsideClickObserver = function () {
      _this.removeOutsideClickObserver();

      _this.outsideClick = detectOutsideClick([_this.state.wrapper_element, _this._refRoot.current, _this._refUl.current], _this.setHidden);

      if (typeof document !== 'undefined') {
        document.addEventListener('keydown', _this.onKeyDownHandler);
        document.addEventListener('keyup', _this.onKeyUpHandler);
      }
    };

    _this.assignObservers = function () {
      _this.setDirectionObserver();

      _this.setScrollObserver();

      _this.setOutsideClickObserver();
    };

    _this.setVisible = function () {
      clearTimeout(_this._hideTimeout);

      if (_this.state.opened && _this.state.hidden === false) {
        return;
      }

      _this.searchCache = null;

      _this.setState({
        hidden: false,
        opened: true,
        _listenForPropChanges: false
      }, _this.assignObservers);

      var delayHandler = function delayHandler() {
        _this.setState({
          isOpen: true,
          _listenForPropChanges: false
        });
      };

      if (isTrue(_this.props.no_animation)) {
        delayHandler();
      } else {
        _this._hideTimeout = setTimeout(delayHandler, DrawerListProvider.blurDelay);
      }

      var _this$state = _this.state,
          selected_item = _this$state.selected_item,
          active_item = _this$state.active_item;
      dispatchCustomElementEvent(_this.state, 'on_show', {
        data: getEventData(parseFloat(selected_item) > -1 ? selected_item : active_item, _this.state.data),
        attributes: _this.attributes,
        ulElement: _this._refUl.current
      });
    };

    _this.setHidden = function () {
      var args = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var onStateComplete = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

      if (!_this.state.opened || isTrue(_this.props.prevent_close)) {
        if (typeof onStateComplete === 'function') {
          onStateComplete();
        }

        return;
      }

      clearTimeout(_this._hideTimeout);
      var _this$state2 = _this.state,
          selected_item = _this$state2.selected_item,
          active_item = _this$state2.active_item;
      var res = dispatchCustomElementEvent(_this.state, 'on_hide', _extends({}, args, {
        data: getEventData(parseFloat(selected_item) > -1 ? selected_item : active_item, _this.state.data),
        attributes: _this.attributes
      }));

      if (res !== false) {
        _this.setState({
          opened: false,
          _listenForPropChanges: false
        });

        var delayHandler = function delayHandler() {
          _this.setState({
            hidden: true,
            isOpen: false,
            _listenForPropChanges: false
          });

          if (typeof onStateComplete === 'function') {
            onStateComplete();
          }
        };

        if (isTrue(_this.props.no_animation)) {
          delayHandler();
        } else {
          _this._hideTimeout = setTimeout(delayHandler, DrawerListProvider.blurDelay);
        }

        _this.waitUntilUlIsReady = false;

        _this.removeDirectionObserver();

        _this.removeScrollObserver();

        _this.removeOutsideClickObserver();
      }
    };

    _this.setDataHandler = function (data) {
      var cb = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

      var _ref3 = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
          _ref3$overwriteOrigin = _ref3.overwriteOriginalData,
          overwriteOriginalData = _ref3$overwriteOrigin === void 0 ? false : _ref3$overwriteOrigin;

      if (!data) {
        return;
      }

      if (typeof data === 'function') {
        data = getData(data);
      }

      data = normalizeData(data);

      _this.setState({
        data: data,
        original_data: overwriteOriginalData ? data : _this.state.original_data,
        _listenForPropChanges: false
      }, function () {
        _this.refreshScrollObserver();

        typeof cb === 'function' && cb(data);
      });

      return _assertThisInitialized(_this);
    };

    _this.setStateHandler = function (state) {
      var cb = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

      _this.setState(_extends({}, state, {
        _listenForPropChanges: false
      }), cb);

      return _assertThisInitialized(_this);
    };

    _this.selectItem = function (itemToSelect) {
      var _ref4 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          _ref4$fireSelectEvent = _ref4.fireSelectEvent,
          fireSelectEvent = _ref4$fireSelectEvent === void 0 ? false : _ref4$fireSelectEvent,
          _ref4$event = _ref4.event,
          event = _ref4$event === void 0 ? null : _ref4$event;

      if (event && typeof event.persist === 'function') {
        event.persist();
      }

      if (itemToSelect === -1) {
        itemToSelect = null;
      }

      var data = getEventData(itemToSelect, _this.state.data) || null;
      var value = getSelectedItemValue(itemToSelect, _this.state);
      var attributes = _this.attributes;
      var attr = {
        selected_item: itemToSelect,
        value: value,
        data: data,
        event: event,
        attributes: attributes
      };
      var res = dispatchCustomElementEvent(_this.state, 'on_pre_change', attr);

      if (res === false || hasSelectedText()) {
        return;
      }

      var _this$props = _this.props,
          keep_open = _this$props.keep_open,
          no_animation = _this$props.no_animation,
          prevent_selection = _this$props.prevent_selection;
      var doCallOnChange = _this.state.selected_item !== itemToSelect;

      var onSelectionIsComplete = function onSelectionIsComplete() {
        var delayHandler = function delayHandler() {
          if (doCallOnChange) {
            dispatchCustomElementEvent(_this.state, 'on_change', attr);
          }

          if (fireSelectEvent) {
            dispatchCustomElementEvent(_this.state, 'on_select', _extends({}, attr, {
              active_item: itemToSelect
            }));
          }

          if (!isTrue(keep_open)) {
            _this.setHidden();
          }
        };

        if (isTrue(no_animation)) {
          delayHandler();
        } else {
          clearTimeout(_this._selectTimeout);
          _this._selectTimeout = setTimeout(delayHandler, DrawerListProvider.blurDelay / 2);
        }
      };

      if (isTrue(prevent_selection)) {
        onSelectionIsComplete();
      } else {
        _this.setState({
          _listenForPropChanges: false,
          selected_item: itemToSelect,
          active_item: itemToSelect
        }, onSelectionIsComplete);
      }
    };

    _this.attributes = {};
    _this.state = _extends({
      tagName: 'dnb-drawer-list',
      cache_hash: '',
      active_item: null,
      selected_item: null,
      ignore_events: false
    }, prepareStartupState(props), {
      _listenForPropChanges: true
    });
    _this._refRoot = React.createRef();
    _this._refShell = React.createRef();
    _this._refUl = React.createRef();
    _this._refTriangle = React.createRef();
    return _this;
  }

  _createClass(DrawerListProvider, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      if (isTrue(this.props.opened)) {
        this.setVisible();
      }
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate(prevProps) {
      if (this.props.opened !== null && this.props.opened !== prevProps.opened) {
        if (isTrue(this.props.opened)) {
          this.setVisible();
        } else if (isTrue(this.props.opened) === false) {
          this.setHidden();
        }
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      clearInterval(this._outsideClickTimeout);
      clearTimeout(this._hideTimeout);
      clearTimeout(this._selectTimeout);
      clearTimeout(this._scrollTimeout);
      clearTimeout(this._ddTimeout);
      clearTimeout(this._doTimeout);
      this.setState({
        opened: false,
        _listenForPropChanges: false
      });
      this.removeDirectionObserver();
      this.removeScrollObserver();
      this.removeOutsideClickObserver();
    }
  }, {
    key: "refreshScrollObserver",
    value: function refreshScrollObserver() {
      var _this2 = this;

      if (typeof window === 'undefined' || !this._refUl.current) {
        return;
      }

      this.itemSpots = this.state.data.reduce(function (acc, cur, i) {
        var _this2$_refUl$current;

        var element = (_this2$_refUl$current = _this2._refUl.current) === null || _this2$_refUl$current === void 0 ? void 0 : _this2$_refUl$current.querySelector("li.dnb-drawer-list__option:nth-of-type(".concat(i + 1, ")"));

        if (element) {
          acc[element.offsetTop] = {
            i: i
          };
        }

        return acc;
      }, {});
      this.itemSpotsCount = Object.keys(this.itemSpots).length;
    }
  }, {
    key: "setScrollObserver",
    value: function setScrollObserver() {
      var _this3 = this;

      if (!isTrue(this.props.enable_closest_observer) || typeof window === 'undefined' || !this._refUl.current) {
        return;
      }

      this.removeScrollObserver();
      this.itemSpotsCount = 1;

      try {
        var closestToTop = null,
            closestToBottom = null,
            tmpToTop,
            tmpToBottom;

        this.setOnScroll = function () {
          if (!_this3._refUl.current) {
            return;
          }

          if (_this3.itemSpotsCount <= 1) {
            _this3.refreshScrollObserver();
          }

          var counts = Object.keys(_this3.itemSpots);
          closestToBottom = findClosest(counts, _this3._refUl.current.scrollTop + _this3._refUl.current.offsetHeight);
          closestToTop = findClosest(counts, _this3._refUl.current.scrollTop);

          if (_this3.itemSpots[closestToTop] && closestToTop !== tmpToTop) {
            _this3.setState({
              closestToTop: _this3.itemSpots[closestToTop].i,
              _listenForPropChanges: false
            });
          }

          if (closestToBottom !== tmpToBottom && _this3.itemSpots[closestToBottom]) {
            _this3.setState({
              closestToBottom: _this3.itemSpots[closestToBottom].i,
              _listenForPropChanges: false
            });
          }

          tmpToTop = closestToTop;
          tmpToBottom = closestToBottom;
        };

        this._refUl.current.addEventListener('scroll', this.setOnScroll);

        this.setOnScroll();
      } catch (e) {
        warn('List could not set onScroll:', e);
      }
    }
  }, {
    key: "removeScrollObserver",
    value: function removeScrollObserver() {
      if (typeof window !== 'undefined' && this.setOnScroll) {
        window.removeEventListener('resize', this.setOnScroll);
        this.setOnScroll = null;
      }
    }
  }, {
    key: "setDirectionObserver",
    value: function setDirectionObserver() {
      var _this4 = this;

      if (typeof window === 'undefined' || typeof document === 'undefined' || !(this.state.wrapper_element || this._refRoot.current)) {
        return;
      }

      var _this$props2 = this.props,
          enable_body_lock = _this$props2.enable_body_lock,
          use_drawer_on_mobile = _this$props2.use_drawer_on_mobile,
          scrollable = _this$props2.scrollable,
          min_height = _this$props2.min_height,
          max_height = _this$props2.max_height,
          on_resize = _this$props2.on_resize,
          page_offset = _this$props2.page_offset,
          observer_element = _this$props2.observer_element;
      var useBodyLock = isTrue(enable_body_lock);
      var useDrawer = isTrue(use_drawer_on_mobile);
      var isScrollable = isTrue(scrollable);
      var customMinHeight = parseFloat(min_height) * 16;
      var customMaxHeight = parseFloat(max_height) || 0;
      var customElem = typeof observer_element === 'string' ? document.querySelector(observer_element) : null;

      if (!customElem) {
        customElem = isInsideScrollView(this._refRoot.current, true);
      }

      this.removeDirectionObserver();
      var directionOffset = 96;
      var spaceToTopOffset = 2 * 16;
      var spaceToBottomOffset = 2 * 16;
      var elem = this.state.wrapper_element || this._refRoot.current;

      var renderDirection = function renderDirection() {
        try {
          var rootElem = customElem || document.documentElement;
          var pageYOffset = !isNaN(parseFloat(page_offset)) ? parseFloat(page_offset) : rootElem.scrollTop;
          var spaceToTop = getOffsetTop(elem) + elem.offsetHeight - pageYOffset;
          var spaceToBottom = rootElem.clientHeight - (getOffsetTop(elem) + elem.offsetHeight) + pageYOffset;
          var direction = Math.max(spaceToBottom - directionOffset, directionOffset) < customMinHeight && spaceToTop > customMinHeight ? 'top' : 'bottom';
          var _max_height = customMaxHeight;

          if (!(_max_height > 0)) {
            _max_height = direction === 'top' ? spaceToTop - ((_this4.state.wrapper_element || _this4._refRoot.current).offsetHeight || 0) - spaceToTopOffset : spaceToBottom - spaceToBottomOffset;
            var vh = 0;

            if (typeof window.visualViewport !== 'undefined') {
              vh = window.visualViewport.height;
            } else {
              vh = Math.max(document.documentElement.clientHeight, window.innerHeight || 0);
            }

            vh = vh * (isScrollable ? 0.7 : 0.9);

            if (_max_height > vh) {
              _max_height = vh;
            }

            _max_height = roundToNearest(_max_height, 8) / 16;
          }

          if (_this4.props.direction === 'auto') {
            _this4.setState({
              direction: direction,
              _listenForPropChanges: false
            });
          }

          _this4.setState({
            max_height: _max_height,
            _listenForPropChanges: false
          });

          if (on_resize) {
            dispatchCustomElementEvent(_this4.state, 'on_resize', {
              direction: direction,
              max_height: _max_height
            });
          }
        } catch (e) {
          warn('List could not set onResize:', e);
        }
      };

      this.setDirection = function (e) {
        clearTimeout(_this4._ddTimeout);
        _this4._ddTimeout = setTimeout(renderDirection, 30);

        if (useDrawer && e.type === 'resize') {
          if (!_this4._bodyLockIsEnabled && (window.innerWidth / 16 <= 40 || window.innerHeight / 16 <= 40)) {
            _this4.enableBodyLock();
          } else if (_this4._bodyLockIsEnabled && !useBodyLock) {
            _this4.disableBodyLock();
          }
        }

        if (e.type === 'resize') {
          _this4.correctHiddenView();
        }
      };

      this._rootElem = customElem || window;

      this._rootElem.addEventListener('scroll', this.setDirection);

      if (typeof window.visualViewport !== 'undefined') {
        window.visualViewport.addEventListener('scroll', this.setDirection);
        window.visualViewport.addEventListener('resize', this.setDirection);
      } else {
        window.addEventListener('resize', this.setDirection);
      }

      clearTimeout(this._doTimeout);
      this._doTimeout = setTimeout(function () {
        if (useBodyLock || useDrawer && (window.innerWidth / 16 <= 40 || window.innerHeight / 16 <= 40)) {
          _this4.enableBodyLock();
        }

        _this4.correctHiddenView();

        _this4.refreshScrollObserver();

        var _this4$state = _this4.state,
            selected_item = _this4$state.selected_item,
            active_item = _this4$state.active_item;

        _this4.scrollToAndSetActiveItem(parseFloat(active_item) > -1 ? active_item : selected_item, {
          scrollTo: false
        });
      }, 1);
      renderDirection();
    }
  }, {
    key: "correctHiddenView",
    value: function correctHiddenView() {
      try {
        var ui = this._refUl.current;
        var spaceToLeft = getOffsetLeft(ui);
        var spaceToRight = window.innerWidth - (getOffsetLeft(ui) + ui.offsetWidth);
        var tri = this._refTriangle.current.style;
        var shell = this._refShell.current.style;

        if (spaceToLeft < 0) {
          shell.transform = "translateX(".concat(Math.abs(spaceToLeft), "px)");
          tri.right = "".concat(Math.abs(spaceToLeft), "px");
        } else if (spaceToRight < 0) {
          shell.transform = "translateX(".concat(spaceToRight, "px)");
          tri.left = "".concat(Math.abs(spaceToRight), "px");
        } else {
          if (shell.transform) {
            shell.transform = '';
            tri.left = 'auto';
            tri.right = 'auto';
          }
        }
      } catch (e) {}
    }
  }, {
    key: "findItemByValue",
    value: function findItemByValue(value) {
      if (isTrue(this.props.skip_keysearch)) {
        return;
      }

      var index = -1;

      try {
        value = String(value).toLowerCase();

        if (this.changedOrderFor !== value) {
          this.searchCache = null;
          this.changedOrderFor = null;
        }

        this.searchCache = this.searchCache || this.state.data.reduce(function (acc, itemData, i) {
          var str = String(parseContentTitle(itemData, {
            separator: ' ',
            removeNumericOnlyValues: true
          }));
          var firstLetter = String(str[0]).toLowerCase();
          acc[firstLetter] = acc[firstLetter] || [];
          acc[firstLetter].push({
            i: i
          });
          return acc;
        }, {});
        var found = this.searchCache[value];
        index = found && found[0] && found[0].i > -1 ? found[0].i : -1;

        if (found && found.length > 1) {
          found.push(found.shift());
          this.changedOrderFor = value;
        }
      } catch (e) {
        warn('List could not findItemByValue:', e);
      }

      return index;
    }
  }, {
    key: "removeDirectionObserver",
    value: function removeDirectionObserver() {
      this.disableBodyLock();
      clearTimeout(this._ddTimeout);

      if (typeof window !== 'undefined' && this.setDirection) {
        var _this$_rootElem;

        (_this$_rootElem = this._rootElem) === null || _this$_rootElem === void 0 ? void 0 : _this$_rootElem.removeEventListener('scroll', this.setDirection);

        if (typeof window.visualViewport !== 'undefined') {
          window.visualViewport.removeEventListener('scroll', this.setDirection);
          window.visualViewport.removeEventListener('resize', this.setDirection);
        } else {
          window.removeEventListener('resize', this.setDirection);
        }

        this.setDirection = null;
      }
    }
  }, {
    key: "getAnchorElem",
    value: function getAnchorElem() {
      try {
        return this.getActiveElement().querySelector('a:not(:focus):first-of-type');
      } catch (e) {
        return null;
      }
    }
  }, {
    key: "focusAnchorElem",
    value: function focusAnchorElem(elem) {
      if (elem) {
        elem.focus({
          preventScroll: true
        });

        if (!elem._hkh) {
          elem._hkh = true;
          elem.addEventListener('keydown', this.anchorKeyDownHandler);
        }
      }
    }
  }, {
    key: "removeOutsideClickObserver",
    value: function removeOutsideClickObserver() {
      if (this.outsideClick) {
        this.outsideClick.remove();
      }

      if (typeof document !== 'undefined') {
        document.removeEventListener('keydown', this.onKeyDownHandler);
        document.removeEventListener('keyup', this.onKeyUpHandler);
      }
    }
  }, {
    key: "render",
    value: function render() {
      return React.createElement(DrawerListContext.Provider, {
        value: _extends({}, this.context, {
          drawerList: _extends({
            attributes: this.attributes,
            _refRoot: this._refRoot,
            _refShell: this._refShell,
            _refUl: this._refUl,
            _refTriangle: this._refTriangle,
            _rootElem: this._rootElem,
            getActiveElement: this.getActiveElement,
            setData: this.setDataHandler,
            setState: this.setStateHandler,
            setWrapperElement: this.setWrapperElement,
            assignObservers: this.assignObservers,
            setVisible: this.setVisible,
            setHidden: this.setHidden,
            selectItem: this.selectItem,
            scrollToItem: this.scrollToItem,
            scrollToAndSetActiveItem: this.scrollToAndSetActiveItem
          }, this.state)
        })
      }, this.props.children);
    }
  }], [{
    key: "getDerivedStateFromProps",
    value: function getDerivedStateFromProps(props, state) {
      return prepareDerivedState(props, state);
    }
  }]);

  return DrawerListProvider;
}(React.PureComponent);

DrawerListProvider.contextType = Context;
DrawerListProvider.defaultProps = {
  no_animation: false,
  prevent_selection: false,
  direction: 'auto',
  wrapper_element: null,
  prevent_close: false,
  keep_open: false,
  prevent_focus: false,
  skip_keysearch: false,
  use_drawer_on_mobile: null,
  enable_body_lock: null,
  page_offset: null,
  observer_element: null,
  enable_closest_observer: null,
  opened: null,
  scrollable: null,
  min_height: 10,
  max_height: null,
  on_resize: null,
  children: null
};
DrawerListProvider.blurDelay = 201;
export { DrawerListProvider as default };
process.env.NODE_ENV !== "production" ? DrawerListProvider.propTypes = {
  no_animation: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  prevent_selection: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  direction: PropTypes.oneOf(['auto', 'top', 'bottom']),
  wrapper_element: PropTypes.oneOfType([PropTypes.object, PropTypes.func, PropTypes.node]),
  prevent_close: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  keep_open: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  prevent_focus: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  skip_keysearch: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  use_drawer_on_mobile: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  enable_body_lock: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  page_offset: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  observer_element: PropTypes.oneOfType([PropTypes.string, PropTypes.node]),
  enable_closest_observer: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  opened: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  scrollable: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  min_height: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  max_height: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  on_resize: PropTypes.func,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node, PropTypes.object, PropTypes.array])
} : void 0;