import "core-js/modules/es.array.find.js";
import "core-js/modules/es.array.slice.js";
import "core-js/modules/es.regexp.exec.js";
import "core-js/modules/es.string.replace.js";
import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";
import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import _inherits from "@babel/runtime/helpers/esm/inherits";
import _possibleConstructorReturn from "@babel/runtime/helpers/esm/possibleConstructorReturn";
import _getPrototypeOf from "@babel/runtime/helpers/esm/getPrototypeOf";

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import Context from '../../shared/Context';
import Provider from '../../shared/Provider';
import { isTrue, registerElement, validateDOMAttributes, extendPropsWithContext } from '../../shared/component-helper';
import { spacingPropTypes, createSpacingClasses } from '../../components/space/SpacingHelper';
import { skeletonDOMAttributes, createSkeletonClass } from '../../components/skeleton/SkeletonHelper';
import P from '../../elements/P';
import { ProductType, CardType } from './utils/Types';
import Designs, { defaultDesign } from './utils/CardDesigns';
import cardProducts from './utils/cardProducts';
import { ProductLogo, TypeLogo, BankLogo, StatusIcon } from './icons';
export { Designs, ProductType, CardType };
var cardDataTypes = process.env.NODE_ENV !== "production" ? PropTypes.shape({
  productCode: PropTypes.string.isRequired,
  productName: PropTypes.string.isRequired,
  displayName: PropTypes.string.isRequired,
  cardDesign: PropTypes.shape.isRequired,
  cardType: PropTypes.shape.isRequired,
  productType: PropTypes.shape.isRequired
}) : {};
var translationDefaultPropsProps = {
  text_card_number: null,
  text_expired: null,
  text_blocked: null
};

var PaymentCard = function (_React$PureComponent) {
  _inherits(PaymentCard, _React$PureComponent);

  var _super = _createSuper(PaymentCard);

  function PaymentCard() {
    _classCallCheck(this, PaymentCard);

    return _super.apply(this, arguments);
  }

  _createClass(PaymentCard, [{
    key: "render",
    value: function render() {
      var _this$context,
          _this = this;

      var props = extendPropsWithContext(this.props, PaymentCard.defaultProps, {
        locale: this.context.locale
      }, {
        skeleton: (_this$context = this.context) === null || _this$context === void 0 ? void 0 : _this$context.skeleton
      });

      var product_code = props.product_code,
          card_number = props.card_number,
          card_status = props.card_status,
          variant = props.variant,
          digits = props.digits,
          id = props.id,
          raw_data = props.raw_data,
          locale = props.locale,
          skeleton = props.skeleton,
          className = props.className,
          _className = props.class,
          children = props.children,
          attributes = _objectWithoutProperties(props, ["product_code", "card_number", "card_status", "variant", "digits", "id", "raw_data", "locale", "skeleton", "className", "class", "children"]);

      var cardData = raw_data || getCardData(product_code);

      var params = _extends({
        className: classnames("dnb-payment-card dnb-payment-card--".concat(variant), createSkeletonClass(null, skeleton, this.context), createSpacingClasses(props), className, _className)
      }, attributes);

      skeletonDOMAttributes(params, skeleton, this.context);
      validateDOMAttributes(this.props, params);
      return React.createElement(Provider, {
        locale: locale
      }, React.createElement(Context.Consumer, null, function (_ref) {
        var translation = _ref.translation;
        var translations = extendPropsWithContext(_this.props, translationDefaultPropsProps, translation.PaymentCard);
        return React.createElement("figure", params, React.createElement("figcaption", {
          className: "dnb-sr-only dnb-payment-card__figcaption"
        }, cardData.productName), React.createElement(NormalCard, {
          id: id,
          skeleton: isTrue(skeleton),
          data: cardData,
          cardStatus: card_status,
          cardNumber: formatCardNumber(card_number, parseFloat(digits)),
          translations: translations
        }));
      }));
    }
  }], [{
    key: "enableWebComponent",
    value: function enableWebComponent() {
      registerElement(PaymentCard.tagName, PaymentCard, PaymentCard.defaultProps);
    }
  }]);

  return PaymentCard;
}(React.PureComponent);

PaymentCard.tagName = 'dnb-payment-card';
PaymentCard.contextType = Context;
PaymentCard.defaultProps = _extends({
  digits: 8,
  locale: null,
  card_status: 'active',
  variant: 'normal',
  id: null,
  raw_data: null,
  skeleton: false,
  class: null,
  className: null,
  children: null
}, translationDefaultPropsProps);
export { PaymentCard as default };
process.env.NODE_ENV !== "production" ? PaymentCard.propTypes = _extends({
  product_code: PropTypes.string.isRequired,
  card_number: PropTypes.string.isRequired,
  card_status: PropTypes.oneOf(['active', 'blocked', 'expired']),
  variant: PropTypes.oneOf(['normal', 'compact']),
  digits: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  raw_data: cardDataTypes,
  id: PropTypes.string,
  locale: PropTypes.string,
  skeleton: PropTypes.oneOfType([PropTypes.string, PropTypes.bool])
}, spacingPropTypes, {
  class: PropTypes.string,
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.node, PropTypes.func])
}) : void 0;
var formatCardNumberRegex = /(?=(?:....)*$)/g;
export var formatCardNumber = function formatCardNumber(cardNumber, digits) {
  return digits ? cardNumber.slice(cardNumber.length - digits, cardNumber.length).replace(formatCardNumberRegex, ' ') : cardNumber.replace(formatCardNumberRegex, ' ');
};
export var getCardData = function getCardData(productCode) {
  var card = cardProducts.find(function (item) {
    return item.productCode === productCode;
  });
  return card || defaultCard(productCode);
};

var defaultCard = function defaultCard(productCode) {
  return {
    productCode: productCode,
    productName: '',
    displayName: '',
    cardDesign: defaultDesign,
    cardType: CardType.None,
    productType: ProductType.None
  };
};

process.env.NODE_ENV !== "production" ? StatusOverlay.propTypes = {
  cardStatus: PropTypes.string.isRequired,
  translations: PropTypes.object.isRequired,
  skeleton: PropTypes.bool
} : void 0;
StatusOverlay.defaultProps = {
  skeleton: false
};

function StatusOverlay(_ref2) {
  var cardStatus = _ref2.cardStatus,
      translations = _ref2.translations,
      skeleton = _ref2.skeleton;

  switch (cardStatus) {
    case 'blocked':
      return React.createElement("div", {
        className: classnames('dnb-payment-card__blocking__overlay', createSkeletonClass('font', skeleton))
      }, React.createElement("div", {
        className: "dnb-payment-card__blocking__center"
      }, React.createElement(StatusIcon, {
        status: cardStatus
      }), React.createElement(P, {
        top: "xx-small"
      }, translations.text_blocked)));

    case 'expired':
      return React.createElement("div", {
        className: classnames('dnb-payment-card__blocking__overlay', createSkeletonClass('font', skeleton))
      }, React.createElement("div", {
        className: "dnb-payment-card__blocking__center"
      }, React.createElement(StatusIcon, {
        status: cardStatus
      }), React.createElement(P, {
        top: "xx-small"
      }, translations.text_expired)));

    case 'active':
    default:
      return null;
  }
}

process.env.NODE_ENV !== "production" ? CardText.propTypes = {
  cardNumber: PropTypes.string.isRequired,
  translations: PropTypes.object.isRequired,
  skeleton: PropTypes.bool.isRequired
} : void 0;

function CardText(_ref3) {
  var cardNumber = _ref3.cardNumber,
      translations = _ref3.translations,
      skeleton = _ref3.skeleton;
  return React.createElement("span", {
    className: classnames('dnb-payment-card__card__wrapper', createSkeletonClass('font', skeleton))
  }, React.createElement(P, {
    className: "dnb-payment-card__card__holder",
    modifier: "x-small bold"
  }, translations.text_card_number), React.createElement(P, {
    className: "dnb-payment-card__card__numbers"
  }, cardNumber));
}

process.env.NODE_ENV !== "production" ? NormalCard.propTypes = {
  id: PropTypes.string,
  skeleton: PropTypes.bool,
  data: cardDataTypes.isRequired,
  cardStatus: PropTypes.string.isRequired,
  cardNumber: PropTypes.string.isRequired,
  translations: PropTypes.object.isRequired
} : void 0;
NormalCard.defaultProps = {
  id: null,
  skeleton: null
};

function NormalCard(_ref4) {
  var data = _ref4.data,
      cardStatus = _ref4.cardStatus,
      cardNumber = _ref4.cardNumber,
      id = _ref4.id,
      skeleton = _ref4.skeleton,
      translations = _ref4.translations;
  return React.createElement("div", {
    id: id,
    className: "dnb-payment-card__card dnb-payment-card__".concat(data.cardDesign.cardStyle)
  }, React.createElement("div", {
    className: "dnb-payment-card__card__content"
  }, React.createElement("div", {
    className: "dnb-payment-card__card__top"
  }, React.createElement(BankLogo, {
    logoType: data.cardDesign.bankLogo
  }), React.createElement(ProductLogo, {
    productType: data.productType,
    cardDesign: data.cardDesign
  })), React.createElement("div", {
    className: "dnb-payment-card__card__bottom"
  }, React.createElement(CardText, {
    cardNumber: cardNumber,
    displayName: data.displayName,
    translations: translations,
    skeleton: skeleton
  }), React.createElement(TypeLogo, {
    cardType: data.cardType,
    cardDesign: data.cardDesign
  }))), React.createElement(StatusOverlay, {
    skeleton: skeleton,
    cardStatus: cardStatus,
    translations: translations
  }));
}