import "core-js/modules/es.array.for-each.js";
import "core-js/modules/es.object.keys.js";
import "core-js/modules/es.object.to-string.js";
import "core-js/modules/es.regexp.to-string.js";
import "core-js/modules/web.dom-collections.for-each.js";
import _typeof from "@babel/runtime/helpers/esm/typeof";
var daggy = createDaggy();
var DNB = daggy.taggedSum('DNB', {
  Colored: ['color'],
  Metalic: []
});
var Visa = daggy.taggedSum('Visa', {
  Colored: ['color'],
  Metalic: []
});
var Mastercard = daggy.taggedSum('Mastercard', {
  Default: [],
  DefaultWhite: [],
  Metalic: [],
  BlackMetalic: []
});
var CardType = daggy.taggedSum('CardType', {
  Visa: [],
  Mastercard: [],
  None: []
});
var BankAxept = daggy.taggedSum('BankAxept', {
  White: [],
  Black: []
});
var Saga = daggy.taggedSum('Saga', {
  Gold: [],
  Platinum: [],
  VisaPlatinum: [],
  None: []
});
var PB = daggy.taggedSum('PB', {
  Default: [],
  Platinum: [],
  None: []
});
var ProductType = daggy.taggedSum('ProductType', {
  BankAxept: [],
  Saga: [],
  Pluss: [],
  PrivateBanking: [],
  None: []
});
export { DNB, Saga, PB, Mastercard, ProductType, CardType, BankAxept, Visa };
var Types = {
  DNB: DNB,
  Saga: Saga,
  PB: PB,
  Mastercard: Mastercard,
  ProductType: ProductType,
  CardType: CardType,
  BankAxept: BankAxept,
  Visa: Visa
};
export default Types;

function createDaggy() {
  var TAG = '@@tag';
  var VALUES = '@@values';
  var TYPE = '@@type';
  var RET_TYPE = '@@ret_type';
  var TAGS = '@@tags';

  function taggedSum(typeName, constructors) {
    var proto = {
      cata: sum$cata,
      toString: sum$toString
    };
    var tags = Object.keys(constructors);
    var typeRep = proto.constructor = {
      toString: typeRepToString,
      prototype: proto,
      is: isType(typeName),
      '@@type': typeName,
      '@@tags': tags
    };
    tags.forEach(function (tag) {
      var fields = constructors[tag];
      var tagProto = Object.create(proto);
      defProp(tagProto, TAG, tag);

      if (fields.length === 0) {
        typeRep[tag] = makeValue(fields, tagProto, [], 0);
        typeRep[tag].is = sum$isUnit(typeRep[tag]);
        return;
      }

      typeRep[tag] = makeConstructor(fields, tagProto);
      typeRep[tag].is = sum$isVariant(typeRep[tag]);
      typeRep[tag][TAG] = tag;
      typeRep[tag][RET_TYPE] = typeName;
      typeRep[tag].toString = sum$ctrToString;
      typeRep[tag].from = makeConstructorFromObject(fields, tagProto);
    });
    return typeRep;
  }

  function sum$cata(fs) {
    var tags = this.constructor[TAGS];
    var tag;

    for (var idx = 0; idx < tags.length; idx += 1) {
      tag = tags[idx];

      if (!fs[tag]) {
        throw new TypeError("Constructors given to cata didn't include: " + tag);
      }
    }

    return fs[this[TAG]].apply(fs, this[VALUES]);
  }

  function sum$ctrToString() {
    return this[RET_TYPE] + '.' + this[TAG];
  }

  function sum$toString() {
    return this.constructor[TYPE] + '.' + this[TAG] + arrToString(this[VALUES]);
  }

  function typeRepToString() {
    return this[TYPE];
  }

  function sum$isVariant(variant) {
    return function $sum$isVariant(val) {
      return Boolean(val) && variant[TAG] === val[TAG] && variant[RET_TYPE] === _typeof(val);
    };
  }

  function sum$isUnit(unit) {
    return function $sum$isUnit(val) {
      return unit === val || Boolean(val) && unit[TAG] === val[TAG] && _typeof(unit) === _typeof(val);
    };
  }

  function isType(typeName) {
    return function $isType(val) {
      return typeName === _typeof(val);
    };
  }

  function makeValue(fields, proto, values, argumentsLength) {
    if (argumentsLength !== fields.length) {
      throw new TypeError('Expected ' + fields.length + ' arguments, got ' + argumentsLength);
    }

    var obj = Object.create(proto);
    defProp(obj, VALUES, values);

    for (var idx = 0; idx < fields.length; idx += 1) {
      obj[fields[idx]] = values[idx];
    }

    return obj;
  }

  function defProp(obj, prop, val) {
    var desc = defProp.desc || (defProp.desc = {
      enumerable: false,
      writable: false,
      configurable: false,
      value: null
    });
    desc.value = val;
    Object.defineProperty(obj, prop, desc);
  }

  function arrToString(arr) {
    if (arr.length === 0) return '';
    var str = '(' + String(arr[0]);

    for (var idx = 1; idx < arr.length; idx += 1) {
      str = str + ', ' + String(arr[idx]);
    }

    return str + ')';
  }

  function makeConstructor(fields, proto) {
    switch (fields.length) {
      case 1:
        return function (a) {
          return makeValue(fields, proto, [a], arguments.length);
        };

      case 2:
        return function (a, b) {
          return makeValue(fields, proto, [a, b], arguments.length);
        };

      case 3:
        return function (a, b, c) {
          return makeValue(fields, proto, [a, b, c], arguments.length);
        };

      case 4:
        return function (a, b, c, d) {
          return makeValue(fields, proto, [a, b, c, d], arguments.length);
        };

      case 5:
        return function (a, b, c, d, e) {
          return makeValue(fields, proto, [a, b, c, d, e], arguments.length);
        };

      case 6:
        return function (a, b, c, d, e, f) {
          return makeValue(fields, proto, [a, b, c, d, e, f], arguments.length);
        };

      case 7:
        return function (a, b, c, d, e, f, g) {
          return makeValue(fields, proto, [a, b, c, d, e, f, g], arguments.length);
        };

      case 8:
        return function (a, b, c, d, e, f, g, h) {
          return makeValue(fields, proto, [a, b, c, d, e, f, g, h], arguments.length);
        };

      case 9:
        return function (a, b, c, d, e, f, g, h, i) {
          return makeValue(fields, proto, [a, b, c, d, e, f, g, h, i], arguments.length);
        };

      case 10:
        return function (a, b, c, d, e, f, g, h, i, j) {
          return makeValue(fields, proto, [a, b, c, d, e, f, g, h, i, j], arguments.length);
        };

      default:
        return Object.defineProperty(function () {
          return makeValue(fields, proto, arguments, arguments.length);
        }, 'length', {
          value: fields.length
        });
    }
  }

  function makeConstructorFromObject(fields, proto) {
    return function (obj) {
      var values = [];

      for (var idx = 0; idx < fields.length; idx += 1) {
        var field = fields[idx];

        if (!Object.prototype.hasOwnProperty.call(obj, field)) {
          throw new TypeError('Missing field: ' + field);
        }

        values.push(obj[field]);
      }

      return makeValue(fields, proto, values, values.length);
    };
  }

  return {
    taggedSum: taggedSum
  };
}