import "core-js/modules/es.array.for-each.js";
import "core-js/modules/es.array.includes.js";
import "core-js/modules/es.regexp.exec.js";
import "core-js/modules/es.string.includes.js";
import "core-js/modules/es.string.split.js";
import "core-js/modules/web.dom-collections.for-each.js";
import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import { LOCALE, CURRENCY, CURRENCY_DISPLAY } from './defaults';
import defaultLocales from './locales';
import { extend } from './component-helper';
export var prepareContext = function prepareContext() {
  var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var locales = props.locales ? extend(defaultLocales, props.locales) : defaultLocales;

  if (props.__newContext) {
    _extends(props, props.__newContext);

    delete props.__newContext;
  }

  var key = handleLocaleFallbacks(props.locale || LOCALE, locales);
  var translation = locales[key] || defaultLocales[LOCALE] || {};

  if (locales[key]) {
    locales[key] = destruct(locales[key], translation);
  }

  var context = _extends({
    updateTranslation: function updateTranslation(locale, translation) {
      context.translation = context.locales[locale] = translation;
    },
    getTranslation: function getTranslation(props) {
      if (props) {
        var lang = props.lang || props.locale;

        if (lang && context.locales[lang] && lang !== key) {
          return context.locales[lang];
        }
      }

      return context.translation;
    },
    locales: locales
  }, props, {
    translation: translation
  });

  return context;
};

function handleLocaleFallbacks(locale, locales) {
  if (!locales[locale]) {
    if (locale === 'en' || locale.split('-')[0] === 'en') {
      return 'en-GB';
    }
  }

  return locale;
}

var Context = React.createContext(prepareContext({
  locale: LOCALE,
  currency: CURRENCY,
  currency_display: CURRENCY_DISPLAY
}));
export default Context;

function destruct(source, validKeys) {
  for (var k in source) {
    if (String(k).includes('.')) {
      var list = k.split('.');

      if (validKeys[list[0]]) {
        (function () {
          var val = source[k];
          var last = list.length - 1;
          list.forEach(function (k, i) {
            source[k] = i === last ? val : source[k];
            source = source[k] || {};
          });
        })();
      }
    }
  }

  return source;
}